#
# Copyright (c) 1994 Sun Microsystems, Inc.  All Rights Reserved.
#
# @(#)Native.mk	1.6 95/01/31
#
# Makefile for building javai.
# builds javac and javah automatically
#

TOPDIR=../../../..

PROGRAM=javai

FILES.c = util.c utf.c globals.c classloader.c interpreter.c \
    classruntime.c common_exceptions.c threadruntime.c profiler.c \
    classresolver.c classinitialize.c gc.c exceptions.c \
    counter.c debug.c interrupt.c monitor.c monitor_cache.c \
    cruft_md.c object.c class.c objectscope.c \
    thread.c threads.c exception.c string.c system.c math.c linker.c \
    classstubs.c io.c \
    CompiledClasses.c CompSupport.c filesystem.c \
    path_md.c import_md.c seenfile_md.c cruft_md.c linker_md.c  \
    check_code.c check_class.c synch.c \
    native_javai.c

GREENTHREADS.c =  \
    interrupt_md.c \
    system_md.c threads_md.c \
    monitor_md.c clock.c schedule.c itimer.c signals.c iomgr.c \
    context.c 

NATIVETHREADS.c = \
    threads_md.c condvar_md.c \
    interrupt_md.c monitor_md.c \
    mutex_md.c system_md.c

FILES.c += $(NATIVETHREADS.c)

obj$${VARIANT}/%.o : $$(SOL_JAVA)/native_threads/src/%.c
	$(COMPILE.c) -o obj${VARIANT}/$(@F) $<

obj$${VARIANT}/%.o : $$(SHARE_JAVA)/native_threads/src/%.c
	$(COMPILE.c) -o obj${VARIANT}/$(@F) $<

# javai_g := FILES.c += print.c tradeoff.c

FILES.s = \
	machgc.s 

CLASSHEADERS = classheaders

#
# The next two included files define
# JAVA_CORE_LIST and CLASSES_EXPORTED_TO_C 
#
include classfiles.mk
include exportedfiles.mk

CLASS_EXPORT_FILES = $(CLASSES_EXPORTED_TO_C:%=$(CLASSES)/%.class)

FILES.java = $(JAVA_CORE_LIST)

NATIVE_INCLUDE =  -I$(SOL_JAVA)/native_threads/include 

include $(TOPDIR)/build/solaris/makefiles/program.mk

LDLIBS = -lm -ldl -lthread

$(OBJDIR)/gram.o := CFLAGS += -DYYSTACKSIZE=1000
obj_g/javai.o := CFLAGS += -DNATIVE
obj/javai.o := CFLAGS += -DNATIVE

classes: $(CLASSES) $(CLASSES)/StandardRules.javah $(JAVAC) $$(FILES.class)

$(CLASSES)/StandardRules.javah : $(SHARE_JAVA)/lang/StandardRules.javah 
	@-rm -f $@
	cp $(SHARE_JAVA)/lang/StandardRules.javah $@

classheaders: $(JAVAH) classes $(FILES.class)
	for i in $(CLASSES_EXPORTED_TO_C) ;				\
	do								\
		$(JAVAH) -classpath $(CLASSES) -d CClassHeaders/ $$i ;	\
	done

$(JAVAC) : FORCE
	@echo ">>>Recursively making javac..."
	@cd ../JAVAC; $(MAKE) $(@)
	@echo "<<<Finished Recursively making javac."

$(JAVAH) : FORCE
	@echo ">>>Recursively making javah..."
	@cd ../JAVAH; $(MAKE) $(@)
	@echo "<<<Finished Recursively making javah."

clean :: FORCE
	rm -rf CClassHeaders/ stubs/
	rm -f classstubs.c opcodes.h opcodes.init 

.INIT :: opcodes.h opcodes.init

