/*
 * @(#)awt.h	1.74 95/04/10 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Platform-independent api for the AWT toolkit.
 *
 */
#ifndef _AWT_H_
#define _AWT_H_

#ifdef WIN32
#define AWTEXPORT __declspec(dllexport)
extern "C" {
#else
#define AWTEXPORT
#endif

#include <oobj.h>
#include <interpreter.h>
#include <tree.h>
#include <common_exceptions.h>
#include <javaString.h>
#include <monitor.h>

/*
 * Java-generated header files. These define all the structs which
 * correspond to the Java objects. By convention, the names of these
 * structs all begin with "H".
 */
#include "CClassHeaders/awt_Dimension.h"
#include "CClassHeaders/awt_Component.h"
#include "CClassHeaders/awt_Font.h"
#include "CClassHeaders/awt_WSFontMetrics.h"
#include "CClassHeaders/awt_RadioGroup.h"
#include "CClassHeaders/awt_DialogHandler.h"
#include "CClassHeaders/awt_MessageDialog.h"
#include "CClassHeaders/awt_FontTable.h"
#include "CClassHeaders/awt_Scrollable.h"
#include "CClassHeaders/awt_Scrollbarable.h"
#include "CClassHeaders/awt_Button.h"
#include "CClassHeaders/awt_Label.h"
#include "CClassHeaders/awt_DIBitmap.h"
#include "CClassHeaders/awt_Graphics.h"
#include "CClassHeaders/awt_Image.h"
#include "CClassHeaders/awt_TextField.h"
#include "CClassHeaders/awt_TextArea.h"
#include "CClassHeaders/awt_Toggle.h"
#include "CClassHeaders/awt_Scrollbar.h"
#include "CClassHeaders/awt_List.h"
#include "CClassHeaders/awt_ChoiceHandler.h"
#include "CClassHeaders/awt_Menu.h"
#include "CClassHeaders/awt_MenuBar.h"
#include "CClassHeaders/awt_OptionMenu.h"
#include "CClassHeaders/awt_Window.h"
#include "CClassHeaders/awt_Frame.h"
#include "CClassHeaders/awt_Color.h"
#include "CClassHeaders/awt_FileDialog.h"
#include "CClassHeaders/awt_MenuItem.h"
#include "CClassHeaders/awt_WServer.h"

/*
 * Button methods
 */
AWTEXPORT void awt_WServer_buttonCreate(Hawt_WServer *ws,
					Hawt_Button *self,
					Hjava_lang_String *label,
					Hawt_Image *normalImage,
					Hawt_Image *pressedImage,
					Hawt_Image *disabledImage,
					Hawt_Window *parent);

AWTEXPORT void awt_WServer_buttonMoveTo(Hawt_WServer *ws,
					Hawt_Button *self,
					long x,
					long y);

AWTEXPORT void awt_WServer_buttonReshape(Hawt_WServer *ws,
					 Hawt_Button *self,
					 long x,
					 long y,
					 long w,
					 long h);

AWTEXPORT void  awt_WServer_buttonDispose(Hawt_WServer *ws,
					  Hawt_Button *self);

AWTEXPORT void  awt_WServer_buttonShow(Hawt_WServer *ws,
				       Hawt_Button *self);

AWTEXPORT void  awt_WServer_buttonHide(Hawt_WServer *ws,
				       Hawt_Button *self);

AWTEXPORT void  awt_WServer_buttonEnable(Hawt_WServer *ws,
					 Hawt_Button *self);

AWTEXPORT void  awt_WServer_buttonDisable(Hawt_WServer *ws,
					  Hawt_Button *self);


/* 
 * Color methods 
 */
AWTEXPORT void awt_WServer_colorCreate(Hawt_WServer *ws, 
			       Hawt_Color *self);

/*
 * Image methods
 */
AWTEXPORT void awt_WServer_imageCreate(Hawt_WServer *ws,
				       Hawt_Image *self,
				       Hawt_DIBitmap *bm);

AWTEXPORT void awt_WServer_scaledImageCreate(Hawt_WServer *ws,
					     Hawt_Image *self,
					     Hawt_DIBitmap *bm,
					     long srcx, long srcy,
					     long srcw, long srch,
					     long dstw, long dsth);

AWTEXPORT void awt_WServer_bitmapRetrieve(Hawt_WServer *ws,
					  Hawt_Image *self,
					  Hawt_DIBitmap *bm);

AWTEXPORT void awt_WServer_imageDispose(Hawt_WServer *ws,
				       Hawt_Image *self);

/*
 * OptionMenu methods
 */
AWTEXPORT void awt_WServer_optionMenuCreate(Hawt_WServer *,
					    Hawt_OptionMenu *,
					    Hawt_Window *,
					    Hjava_lang_String *);
AWTEXPORT void awt_WServer_optionMenuDispose(Hawt_WServer *,
					     Hawt_OptionMenu *);
AWTEXPORT void awt_WServer_optionMenuMoveTo(Hawt_WServer *,
					    Hawt_OptionMenu *,
					    long,
					    long);
AWTEXPORT void awt_WServer_optionMenuShow(Hawt_WServer *,
					  Hawt_OptionMenu *);
AWTEXPORT void awt_WServer_optionMenuHide(Hawt_WServer *,
					  Hawt_OptionMenu *);
AWTEXPORT void awt_WServer_optionMenuAddItem(Hawt_WServer *,
					     Hawt_OptionMenu *,
					     Hjava_lang_String *,
					     long);
AWTEXPORT void awt_WServer_optionMenuAddSeparator(Hawt_WServer *,
						  Hawt_OptionMenu *,
						  long);

AWTEXPORT void awt_WServer_optionMenuReshape(Hawt_WServer *,
					     Hawt_OptionMenu *,
					     long,long,long,long);

AWTEXPORT void awt_WServer_optionMenuDimensions(struct Hawt_WServer *,struct Hawt_OptionMenu *);

AWTEXPORT void awt_WServer_optionMenuSelect(Hawt_WServer *ws,
					    Hawt_OptionMenu *self,
					    long index);
/*
 * Scrollbar methods
 */
AWTEXPORT void  awt_WServer_scrollbarCreate(Hawt_WServer *ws,
				 Hawt_Scrollbar *self,
				 Hawt_Window *parent,
				 long orientation,
				 long manageScrollbar);

AWTEXPORT void  awt_WServer_scrollbarMoveTo(Hawt_WServer *ws,
				 Hawt_Scrollbar *self,
				 long x, long y);
     
AWTEXPORT void  awt_WServer_scrollbarReshape(Hawt_WServer *ws,
					     Hawt_Scrollbar *self,
					     long x, long y, long w, long h);

AWTEXPORT long awt_WServer_scrollbarMinimum(Hawt_WServer *ws,
					    Hawt_Scrollbar *self);

AWTEXPORT long awt_WServer_scrollbarMaximum(Hawt_WServer *ws,
					    Hawt_Scrollbar *self);

AWTEXPORT void awt_WServer_scrollbarSetValues(Hawt_WServer *ws,
					      Hawt_Scrollbar *self,
					      long newValue,
					      long visible,
					      long minimum,
					      long maximum);

AWTEXPORT long awt_WServer_scrollbarValue(Hawt_WServer *ws,
					  Hawt_Scrollbar *self);

AWTEXPORT void  awt_WServer_scrollbarDispose(Hawt_WServer *ws,
					     Hawt_Scrollbar *self);

AWTEXPORT void  awt_WServer_scrollbarShow(Hawt_WServer *ws,
					  Hawt_Scrollbar *self);

AWTEXPORT void  awt_WServer_scrollbarHide(Hawt_WServer *ws,
					  Hawt_Scrollbar *self);

/*
 * TextArea methods
 */
AWTEXPORT void awt_WServer_textAreaCreate(Hawt_WServer *ws,
					  Hawt_TextArea *self,
					  Hawt_Window *p,
					  Hawt_Font *font,
					  long columns,
					  long rows);

AWTEXPORT void awt_WServer_textAreaSetEditable(Hawt_WServer *ws,
					       Hawt_TextArea *self,
					       /*boolean*/ long e);

AWTEXPORT long awt_WServer_textAreaCursorPos(Hawt_WServer *ws,
					     Hawt_TextArea *self);

AWTEXPORT void awt_WServer_textAreaSetCursorPos(Hawt_WServer *ws,
						Hawt_TextArea *self,
						long pos);

AWTEXPORT long awt_WServer_textAreaEndPos(Hawt_WServer *ws,
					  Hawt_TextArea *self);

AWTEXPORT void awt_WServer_textAreaSetText(Hawt_WServer *ws,
					   Hawt_TextArea *self,
					   Hjava_lang_String *txt);

AWTEXPORT Hjava_lang_String *awt_WServer_textAreaGetText(Hawt_WServer *ws,
							Hawt_TextArea *self);

AWTEXPORT void awt_WServer_textAreaInsertText(Hawt_WServer *ws,
					      Hawt_TextArea *self,
					      Hjava_lang_String *,
					      long pos);

AWTEXPORT void awt_WServer_textAreaReplaceText(Hawt_WServer *ws,
					       Hawt_TextArea *self,
					       Hjava_lang_String *txt,
					       long start,
					       long end);

AWTEXPORT void awt_WServer_textAreaDispose(Hawt_WServer *ws,
					   Hawt_TextArea *self);

AWTEXPORT void awt_WServer_textAreaMoveTo(Hawt_WServer *,
					  Hawt_TextArea *self,
					  long x,
					  long y);

AWTEXPORT void awt_WServer_textAreaReshape(Hawt_WServer *ws,
					   Hawt_TextArea *self,
					   long x,
					   long y,
					   long w,
					   long h);

AWTEXPORT void awt_WServer_textAreaShow(Hawt_WServer *ws,
					Hawt_TextArea *self);

AWTEXPORT void awt_WServer_textAreaHide(Hawt_WServer *ws,
					Hawt_TextArea *self);

/* 
 * TextField methods 
 */
AWTEXPORT void awt_WServer_textFieldCreate(Hawt_WServer *ws, 
					   Hawt_TextField *self,
					   Hjava_lang_String *initValue,
					   Hawt_Window *parent,
					   long editable);

AWTEXPORT void awt_WServer_textFieldSetText(Hawt_WServer *ws,
					    Hawt_TextField *self,
					    Hjava_lang_String *l);

AWTEXPORT void awt_WServer_textFieldSetEditable(Hawt_WServer *ws, 
						Hawt_TextField *self,
						long editable);

AWTEXPORT void awt_WServer_textFieldDispose(Hawt_WServer *ws, 
					    Hawt_TextField *self);

AWTEXPORT void awt_WServer_textFieldMoveTo(Hawt_WServer *ws, 
					   Hawt_TextField *self,
					   long x,
					   long y);

AWTEXPORT Hjava_lang_String *awt_WServer_textFieldGetText(Hawt_WServer *ws, 
					      Hawt_TextField *self);

AWTEXPORT void awt_WServer_textFieldSetColor(Hawt_WServer *ws, 
					     Hawt_TextField *self, 
					     Hawt_Color *c);

AWTEXPORT void awt_WServer_textFieldSetFont(Hawt_WServer *ws, 
					    Hawt_TextField *self, 
					    Hawt_Font *f);

AWTEXPORT void awt_WServer_textFieldSetEchoCharacter(Hawt_WServer *ws, 
						     Hawt_TextField *self, 
						     unicode c);

AWTEXPORT void awt_WServer_textFieldReshape(Hawt_WServer *ws, 
					    Hawt_TextField *self,
					    long x,
					    long y,
					    long w,
					    long h);

AWTEXPORT void awt_WServer_textFieldShow(Hawt_WServer *ws, 
					 Hawt_TextField *self);

AWTEXPORT void awt_WServer_textFieldHide(Hawt_WServer *ws, 
					 Hawt_TextField *self);

/*
 * Frame methods
 */
AWTEXPORT void  awt_WServer_frameCreate(Hawt_WServer *ws,
					Hawt_Frame *self,
					long hasTitleBar,
					long isModal,
					Hawt_Frame *parentFrame,
					long width,
					long height,
					Hawt_Color *bg);

AWTEXPORT void  awt_WServer_frameSetTitle(Hawt_WServer *ws,
			       Hawt_Frame *self,
			       Hjava_lang_String *title);

AWTEXPORT void  awt_WServer_frameShow(Hawt_WServer *ws,
			   Hawt_Frame *self);

AWTEXPORT void  awt_WServer_frameHide(Hawt_WServer *ws,
			   Hawt_Frame *self);

AWTEXPORT void  awt_WServer_frameSetTitle(Hawt_WServer *ws,
			       Hawt_Frame *self,
			       Hjava_lang_String *title);

AWTEXPORT void  awt_WServer_frameSetIconImage(Hawt_WServer *ws,
			       Hawt_Frame *self,
			       Hawt_Image *image);

AWTEXPORT void  awt_WServer_frameDispose(Hawt_WServer *ws,
			      Hawt_Frame *self);

AWTEXPORT void  awt_WServer_frameSetMinSize(Hawt_WServer *ws,
				 Hawt_Frame *self,
				 long width, long height);

AWTEXPORT void  awt_WServer_frameReshape(Hawt_WServer *ws,
			      Hawt_Frame *self,
			      long x, long y,
			      long w, long h);

AWTEXPORT void  awt_WServer_frameSetDefaultFont(Hawt_WServer *ws,
						Hawt_Frame *self,
						Hawt_Font *font);

AWTEXPORT long  awt_WServer_frameHasStatusBar(Hawt_WServer *ws,
				 Hawt_Frame *self);

AWTEXPORT void  awt_WServer_frameShowStatusBar(Hawt_WServer *ws,
					       Hawt_Frame *self,
					       long show);

AWTEXPORT long  awt_WServer_frameShowingStatusBar(Hawt_WServer *ws,
						  Hawt_Frame *self);

AWTEXPORT void  awt_WServer_frameSetStatusMessage(Hawt_WServer *ws,
						  Hawt_Frame *self,
						  Hjava_lang_String *label);


/*
 * Label methods 
 */
AWTEXPORT void awt_WServer_labelMoveTo(Hawt_WServer *ws, 
				       Hawt_Label *self,
					long x,
					long y);

AWTEXPORT void awt_WServer_labelSetText(Hawt_WServer *ws, 
					Hawt_Label *self,
					Hjava_lang_String *label);

AWTEXPORT void awt_WServer_labelCreate(Hawt_WServer *ws, 
				       Hawt_Label *self,
				       Hjava_lang_String *label,
				       Hawt_Window *parent,
				       Hawt_Font *font);

AWTEXPORT void awt_WServer_labelDispose(Hawt_WServer *ws, 
					Hawt_Label *self);    

AWTEXPORT void awt_WServer_labelSetColor(Hawt_WServer *ws, 
					 Hawt_Label *self, 
					 Hawt_Color *c);    

AWTEXPORT void awt_WServer_labelSetFont(Hawt_WServer *ws, 
					Hawt_Label *self, 
					Hawt_Font *f);    

AWTEXPORT void awt_WServer_labelReshape(Hawt_WServer *ws, Hawt_Label *self,
					  long x,
					  long y,
					  long w,
					  long h);

AWTEXPORT void awt_WServer_labelDimensions(struct Hawt_WServer *,struct Hawt_Label *);

AWTEXPORT void awt_WServer_labelShow(Hawt_WServer *ws, 
				     Hawt_Label *self);    

AWTEXPORT void awt_WServer_labelHide(Hawt_WServer *ws, 
				     Hawt_Label *self);    

/*
 * FileDialog methods
 */

AWTEXPORT void  awt_WServer_fileDialogCreate(Hawt_WServer *ws,
					     Hawt_FileDialog *self,
					     Hjava_lang_String *title,
					     Hawt_Frame *parent);

AWTEXPORT void  awt_WServer_fileDialogDispose(Hawt_WServer *ws,
					      Hawt_FileDialog *self);

AWTEXPORT Hjava_lang_String  *awt_WServer_fileDialogChooseFile(Hawt_WServer *ws,
							      Hawt_FileDialog *self,
							      Hjava_lang_String *initValue);

/*
 * Font methods
 */
AWTEXPORT void awt_WServer_fontCreate(Hawt_WServer *ws,
				      Hawt_Font *self,
				      Hjava_lang_String *_name,
				      long style,
				      long height);

AWTEXPORT long awt_WServer_fontStringWidth(Hawt_WServer *ws,
					   Hawt_WSFontMetrics *self,
					   Hjava_lang_String *s);

AWTEXPORT long awt_WServer_fontCharsWidth(Hawt_WServer *ws, 
					  Hawt_WSFontMetrics *self, 
					  HArrayOfChar *chars,
					  long off,
					  long len);

AWTEXPORT void awt_WServer_fontDispose(Hawt_WServer *ws,
				       Hawt_Font *self);

/*
 * FontTable methods 
 */

AWTEXPORT Hjava_lang_String*
  awt_FontTable_mapDefaultName(Hawt_FontTable *fontTable, 
			       Hjava_lang_String *s);

/*
 * Menu methods
 */
AWTEXPORT void  awt_WServer_menuCreate(Hawt_WServer *ws,
				       Hawt_Menu *self,
				       Hjava_lang_String *title,
				       Hawt_MenuBar *menuBar,
				       long canTearOff);

AWTEXPORT void  awt_WServer_menuDispose(Hawt_WServer *ws,
			     Hawt_Menu *self);

AWTEXPORT void awt_WServer_menuShow(Hawt_WServer *ws,
				    Hawt_Menu *self);

AWTEXPORT void awt_WServer_menuHide(Hawt_WServer *ws,
				    Hawt_Menu *self);

AWTEXPORT void awt_WServer_menuAddSeparator(Hawt_WServer *ws,
						Hawt_Menu *self);


/*
 * MenuBar methods
 */
AWTEXPORT void  awt_WServer_menuBarCreate(Hawt_WServer *ws,
			       Hawt_MenuBar *self,
			       Hawt_Frame *frame);

AWTEXPORT void  awt_WServer_menuBarDispose(Hawt_WServer *ws,
				Hawt_MenuBar *self);

AWTEXPORT void  awt_WServer_menuBarReshape(Hawt_WServer *ws,
				Hawt_MenuBar *self,
				long x, long y,
				long w, long h);

AWTEXPORT void  awt_WServer_menuBarAddMenu(Hawt_WServer *ws,
				Hawt_MenuBar *self,
				Hawt_Menu *m);

AWTEXPORT void  awt_WServer_menuBarDelMenu(Hawt_WServer *ws,
				Hawt_MenuBar *self,
				Hawt_Menu *m);


/*
 * MenuItem methods
 */
AWTEXPORT void  awt_WServer_menuItemCreate(Hawt_WServer *ws,
					   Hawt_MenuItem *self,
					   Hjava_lang_String *label,
					   Hawt_Menu *menu,
					   long isToggle);

AWTEXPORT void  awt_WServer_menuItemEnable(Hawt_WServer *ws,
				Hawt_MenuItem *self);

AWTEXPORT void  awt_WServer_menuItemDisable(Hawt_WServer *ws,
				 Hawt_MenuItem *self);

AWTEXPORT void  awt_WServer_menuItemDispose(Hawt_WServer *ws,
				 Hawt_MenuItem *self);

AWTEXPORT void awt_WServer_menuItemSetMark(Hawt_WServer *ws,
					   Hawt_MenuItem *self,
					   long state);

AWTEXPORT long awt_WServer_menuItemGetMark(Hawt_WServer *ws,
					   Hawt_MenuItem *self);

/*
 * List methods
 */

AWTEXPORT void awt_WServer_graphicsCreate(Hawt_WServer *ws, 
					  Hawt_Graphics *self, 
					  Hawt_Window *w);

AWTEXPORT void awt_WServer_graphicsDispose(Hawt_WServer *ws, 
					   Hawt_Graphics *self);

AWTEXPORT void awt_WServer_graphicsSetFont(Hawt_WServer *ws, 
					   Hawt_Graphics *self, 
					   Hawt_Font *f);

AWTEXPORT void awt_WServer_graphicsSetForeground(Hawt_WServer *ws, 
						 Hawt_Graphics *self, 
						 Hawt_Color *c);

AWTEXPORT void awt_WServer_graphicsSetBackground(Hawt_WServer *ws, 
						 Hawt_Graphics *self, 
						 Hawt_Color *c);

AWTEXPORT void awt_WServer_graphicsClearRect(Hawt_WServer *ws, 
					     Hawt_Graphics *self,
					     long X, 
					     long Y, 
					     long W, 
					     long H);

AWTEXPORT void awt_WServer_graphicsFillRect(Hawt_WServer *ws, 
					    Hawt_Graphics *self,
					    long X, 
					    long Y, 
					    long W, 
					    long H);

AWTEXPORT void awt_WServer_graphicsDrawRect(Hawt_WServer *ws, 
					    Hawt_Graphics *self,
					    long X, 
					    long Y, 
					    long W, 
					    long H);

AWTEXPORT void awt_WServer_graphicsDrawString(Hawt_WServer *ws, 
					      Hawt_Graphics *self,
					      Hjava_lang_String *str, 
					      long x, 
					      long y);

AWTEXPORT void awt_WServer_graphicsDrawChars(Hawt_WServer *ws, 
					     Hawt_Graphics *self, 
					     HArrayOfChar *chars,
					     long offset, 
					     long length,
					     long x, 
					     long y);

AWTEXPORT long awt_WServer_graphicsDrawStringWidth(Hawt_WServer *ws, 
					      Hawt_Graphics *self,
					      Hjava_lang_String *str, 
					      long x, 
					      long y);

AWTEXPORT long awt_WServer_graphicsDrawCharsWidth(Hawt_WServer *ws, 
					     Hawt_Graphics *self, 
					     HArrayOfChar *chars,
					     long offset, 
					     long length,
					     long x, 
					     long y);

AWTEXPORT void awt_WServer_graphicsDrawLine(Hawt_WServer *ws, 
					    Hawt_Graphics *self,
					    long x1, 
					    long y1, 
					    long x2, 
					    long y2);

AWTEXPORT void awt_WServer_graphicsClipRect(Hawt_WServer *ws, 
					    Hawt_Graphics *self,
					    long x, 
					    long y, 
					    long w, 
					    long h);

AWTEXPORT void awt_WServer_graphicsClearClip(Hawt_WServer *ws, 
					     Hawt_Graphics *self);

AWTEXPORT void awt_WServer_graphicsSetOrigin(Hawt_WServer *ws, 
					     Hawt_Graphics *self, 
					     long x, 
					     long y);

AWTEXPORT void awt_WServer_graphicsSetScaling(Hawt_WServer *ws, 
					      Hawt_Graphics *self, 
					      float sx, 
					      float sy);

AWTEXPORT void awt_WServer_graphicsCopyArea(Hawt_WServer *ws, 
					    Hawt_Graphics *self,
					    long x,
					    long y,
					    long width,
					    long height,
					    long dx,
					    long dy);

AWTEXPORT void awt_WServer_graphicsDrawImage(Hawt_WServer *ws, 
					     Hawt_Graphics *self,
					     Hawt_Image *I,
					     long X, 
					     long Y);


/*
 * List methods
 */

AWTEXPORT void
awt_WServer_listCreate(Hawt_WServer *ws,
		       Hawt_List *self,
		       Hawt_Window *parent,
		       Hawt_ChoiceHandler *handler,
		       long visibleLines,
		       long multipleSelections,
		       long resizable);

AWTEXPORT long
awt_WServer_listIsSelected(Hawt_WServer *ws,
			   Hawt_List *self,
			   long pos);

AWTEXPORT void
awt_WServer_listAddItem(Hawt_WServer *ws,
			Hawt_List *self,
			Hjava_lang_String *item);
    
AWTEXPORT void
awt_WServer_listMakeVisible(Hawt_WServer *ws,
			    Hawt_List *self,
			    long pos);

AWTEXPORT void
awt_WServer_listSetNVisible(Hawt_WServer *ws,
			    Hawt_List *self,
			    long nLines);

AWTEXPORT void
awt_WServer_listSelect(Hawt_WServer *ws,
		       Hawt_List *self,
		       long pos);

AWTEXPORT void
awt_WServer_listDeselect(Hawt_WServer *ws,
			 Hawt_List *self,
			 long pos);

AWTEXPORT void
awt_WServer_listDelItems(Hawt_WServer *ws,
			 Hawt_List *self,
			 long start,
			 long end);
AWTEXPORT void
awt_WServer_listMoveTo(Hawt_WServer *ws,
		       Hawt_List *self,
		       long x,
		       long y);

AWTEXPORT void
awt_WServer_listReshape(Hawt_WServer *ws,
			Hawt_List *self,
			long x,
			long y,
			long w,
			long h);

AWTEXPORT void awt_WServer_listDispose(Hawt_WServer *ws,
				       Hawt_List *self);

AWTEXPORT void awt_WServer_listShow(Hawt_WServer *ws,
				    Hawt_List *self);

AWTEXPORT void awt_WServer_listHide(Hawt_WServer *ws,
				    Hawt_List *self);

AWTEXPORT void awt_WServer_listDimensions(struct Hawt_WServer *,struct Hawt_List *);

/*
 * Toggle button methods
 */
AWTEXPORT void  awt_WServer_toggleCreate(Hawt_WServer *ws,
					 Hawt_Toggle *self,
					 Hjava_lang_String *label,
					 Hawt_Window *parent,
					 Hawt_RadioGroup *group,
					 long initialState);

AWTEXPORT void  awt_WServer_toggleSetState(Hawt_WServer *ws,
				Hawt_Toggle *self,
				long state);

AWTEXPORT long awt_WServer_toggleGetState(Hawt_WServer *ws,
					  Hawt_Toggle *self);

AWTEXPORT void  awt_WServer_toggleMoveTo(Hawt_WServer *ws,
				   Hawt_Toggle *self,
				   long x,
				   long y);

AWTEXPORT void  awt_WServer_toggleDispose(Hawt_WServer *ws, Hawt_Toggle *self);


/*
 * WServer methods
 */
AWTEXPORT void  awt_WServer_pInit(Hawt_WServer *self);

AWTEXPORT void  awt_WServer_run(Hawt_WServer *self);

AWTEXPORT void  awt_WServer_eventLoop(Hawt_WServer *self);

/*
 * Flush any requests to WServer. Can be ignored if not relevant.
 */
AWTEXPORT void  awt_WServer_sync(Hawt_WServer *self);

/*
 * Window methods
 */
AWTEXPORT void  awt_WServer_windowEnablePointerMotionEvents(Hawt_WServer *ws,
						      Hawt_Window *self);

AWTEXPORT void  awt_WServer_windowDisablePointerMotionEvents(Hawt_WServer *ws,
						       Hawt_Window *self);

AWTEXPORT void awt_WServer_windowDimensions(struct Hawt_WServer *,struct Hawt_Window *);

AWTEXPORT void  awt_WServer_windowCreateInFrame(Hawt_WServer *ws,
					  Hawt_Window *self,
					  Hawt_Frame *container,
					  long trackMotion,
					  Hawt_Color *bg,
					  long width,
					  long height);

AWTEXPORT void awt_WServer_windowCopyArea(Hawt_WServer *ws,
					  Hawt_Window *self,
					  long x,
					  long y,
					  long width,
					  long height,
					  long dx,
					  long dy);

AWTEXPORT void  awt_WServer_windowCreate(Hawt_WServer *ws,
				   Hawt_Window *self,
				   Hawt_Window *container,
				   long trackMotion,
				   Hawt_Color *bg,
				   long width,
				   long height);

AWTEXPORT void  awt_WServer_windowDispose(Hawt_WServer *ws, Hawt_Window *self);

AWTEXPORT void  awt_WServer_windowShow(Hawt_WServer *ws,
				 Hawt_Window *self);

AWTEXPORT void  awt_WServer_windowHide(Hawt_WServer *ws,
				 Hawt_Window *self);

AWTEXPORT void  awt_WServer_windowMoveTo(Hawt_WServer *ws,
				   Hawt_Window *self,
				   long x,
				   long y);

AWTEXPORT void  awt_WServer_windowReshape(Hawt_WServer *ws,
				    Hawt_Window *self,
				    long x, long y,
				    long w, long h);

AWTEXPORT long awt_WServer_windowWidth(Hawt_WServer *ws,
				       Hawt_Window *self);

AWTEXPORT long awt_WServer_windowHeight(Hawt_WServer *ws,
					Hawt_Window *self);

AWTEXPORT void  awt_WServer_windowSetMargin(Hawt_WServer *ws,
				      Hawt_Window *self,
				      long margin);

AWTEXPORT void  awt_WServer_windowScrollWindow(Hawt_WServer *ws,
					       Hawt_Window *self,
					       long dx, long y);
/*
 * MessageDialog methods
 */
AWTEXPORT void awt_WServer_messageDialogCreate(Hawt_WServer *ws,
					       Hawt_MessageDialog *self,
					       Hawt_Frame *frame,
					       Hjava_lang_String *title,
					       Hjava_lang_String *message,
					       long dialogType,
					       long nButtons,
					       /*boolean*/ long isModal,
					       Hjava_lang_String *okLabel,
					       Hjava_lang_String *cancelLabel,
					       Hjava_lang_String *helpLabel);

AWTEXPORT void awt_WServer_messageDialogSetMessage(Hawt_WServer *ws,
						   Hawt_MessageDialog *self,
						   Hjava_lang_String *message);

AWTEXPORT long awt_WServer_mesageDialogShow(Hawt_WServer *ws,
					    Hawt_MessageDialog *self);

AWTEXPORT void awt_WServer_mesageDialogHide(Hawt_WServer *ws,
					    Hawt_MessageDialog *self);

AWTEXPORT void awt_WServer_messageDialogDispose(Hawt_WServer *ws,
						Hawt_MessageDialog *self);

#ifdef WIN32
}               /* extern "C" { */
#endif

#endif           /* _AWT_H_ */
