/*
 * @(#)math.h	1.4 95/01/31  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Math library definitions for all the public functions implemented in libm.a.
 */

#ifndef	_MATH_H_
#define	_MATH_H_

/*
 * Posix (actually ansi C) section
 */
#define	HUGE_VAL	(__infinity())	/* Produces IEEE Infinity. */


extern double	__infinity();
extern double	acos(/* double x */);
extern double	asin(/* double x */);
extern double	atan(/* double x */);
extern double	atan2(/* double y, double x */);
extern double	ceil(/* double x */);
extern double	cos(/* double x */);
extern double	cosh(/* double x */);
extern double	exp(/* double x */);
extern double	fabs(/* double x */);
extern double	floor(/* double x */);
extern double	fmod(/* double x, double y */);
extern double	frexp(/* double value, int *exp */);
extern double	ldexp(/* double value, int exp */);
extern double	log(/* double x */);
extern double	log10(/* double x */);
extern double	modf(/* double value, double *iptr */);
extern double	pow(/* double x, double y */);
extern double	sin(/* double x */);
extern double	sinh(/* double x */);
extern double	sqrt(/* double x */);
extern double	tan(/* double x */);
extern double	tanh(/* double x */);

#ifndef	_POSIX_SOURCE			/* the rest of the file is !POSIX */

extern double	acosh();
extern double	asinh();
extern double	atanh();
extern double	cbrt();
extern double	copysign();
extern double	erf();
extern double	erfc();
extern double	expm1();
extern int	finite();
extern double	hypot();
extern double	j0();
extern double	j1();
extern double	jn();
extern double	lgamma();
extern double	log1p();
extern double	rint();
extern double	y0();
extern double	y1();
extern double	yn();

/*
 * Sun definitions.
 */

/*
 * Implemented precisions for trigonometric argument reduction.
 */
enum fp_pi_type {
	fp_pi_infinite	= 0,	/* Infinite-precision approximation to pi. */
	fp_pi_66	= 1,	/* 66-bit approximation to pi. */
	fp_pi_53	= 2	/* 53-bit approximation to pi. */
};

/*
 * Pi precision to use for trigonometric argument reduction.
 */
extern enum	fp_pi_type fp_pi;

/*
 * Functions callable from C, intended to support IEEE arithmetic.
 */
extern int	ieee_flags();
extern int	ieee_handler();
extern void	ieee_retrospective();
extern int	ilogb();
extern double	infinity();
extern int	irint();
extern int	isinf();
extern int	isnan();
extern int	isnormal();
extern int	issubnormal();
extern int	iszero();
extern double	logb();
extern double	max_normal();
extern double	max_subnormal();
extern double	min_normal();
extern double	min_subnormal();
extern double	nextafter();
extern void	nonstandard_arithmetic();
extern double	quiet_nan();
extern double	remainder();
extern double	scalb();
extern double	scalbn();
extern double	signaling_nan();
extern int	signbit();
extern double	significand();
extern void	standard_arithmetic();

/*
 * Other functions for C programmers.
 */
extern double	acospi();
extern double	aint();
extern double	anint();
extern double	annuity();
extern double	asinpi();
extern double	atan2pi();
extern double	atanpi();
extern double	compound();
extern double	cospi();
extern double	exp10();
extern double	exp2();
extern double	log2();
extern int	nint();
extern void	sincos();
extern void	sincospi();
extern double	sinpi();
extern double	tanpi();
extern int	matherr();

#endif

#endif /* !_MATH_H_ */
