/*
 * @(#)sys_api.h	1.18 95/05/10  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * System or Host dependent API.  This defines the "porting layer" for
 * POSIX.1 compliant operating systems.
 */

#ifndef _SYS_API_H_
#define _SYS_API_H_

#include "typedefs_md.h"

/*
 * Miscellaneous system utility APIs that fall outside the POSIX.1
 * spec.
 *
 * Until POSIX (P1003.1a) formerly P.4 is standard, we'll use our
 * time struct definitions found in timeval.h.
 */
#include <timeval.h>
extern	long 	  sysGetMilliTicks();
extern	long      sysTime(long *);
extern	int64_t   sysTimeMillis();

/*
 * System API for dynamic linking libraries into the interpreter
 */
extern	char *  sysInitializeLinker();
extern	int     sysAddDLSegment(char *);
extern	void    sysSaveLDPath(char *);
extern	long    sysDynamicLink(char *);

/*
 * System API for file and directory parsing
 */
extern	int	sysGetFileSeparator();

/*
 * System API for threads
 */
#include <threads_md.h>
extern int sysThreadBootstrap(sys_thread_t **);
extern int sysThreadCreate(long, uint_t flags, 
				     void *(*)(void *), 
				     sys_thread_t **,
	 			     void *);
extern	int	 sysThreadKill(sys_thread_t *);
extern	void	 sysThreadExit();
extern	sys_thread_t *      sysThreadSelf();
extern	void	 sysThreadYield();
extern	int	 sysThreadSuspend(sys_thread_t *);
extern	int	 sysThreadResume(sys_thread_t *);
extern	int	 sysThreadSetPriority(sys_thread_t *, int);
extern	int	 sysThreadGetPriority(sys_thread_t *, int *);
extern  void *   sysThreadStackPointer(sys_thread_t *); 
extern  stackp_t sysThreadStackBase(sys_thread_t *); 
extern  int	 sysThreadSingle();
extern  void	 sysThreadMulti();
extern  int      sysThreadEnumerateOver(int (*)(sys_thread_t *, void *), void *);
extern  void	 sysThreadInit(sys_thread_t *, stackp_t);


/*
 * System API for monitors
 */
#include <monitor_md.h>
extern int    	sysMonitorInit(sys_mon_t *);
extern int    	sysMonitorEnter(sys_mon_t *);
extern bool_t 	sysMonitorEntered(sys_mon_t *);
extern int    	sysMonitorExit(sys_mon_t *);
extern int    	sysMonitorNotify(sys_mon_t *);
extern int    	sysMonitorNotifyAll(sys_mon_t *);
extern int 	sysMonitorWait(sys_mon_t *, int);
extern void   	sysMonitorDumpInfo(sys_mon_t *);

#define SYS_OK	        0
#define SYS_TIMEOUT     1
#define SYS_ERR	       -1
#define SYS_INVALID    -2    /* invalid arguments */
#define SYS_NOTHREAD   -3    /* no such thread */
#define SYS_MINSTACK   -4    /* supplied stack size less than min stack size */
#define SYS_NOMEM      -5
#define SYS_NORESOURCE -6
#define SYS_DESTROY    -7

/*
 * System API for raw memory allocation
 */
extern void *sysAllocateMem(long);
extern int sysFreeMem(char *, long);

#endif /* !_SYS_API_H_ */
