/*
 * @(#)threads.h	1.18 95/01/31  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#ifndef _THREADS_H_
#define _THREADS_H_

#include <oobj.h>
#include <interpreter.h>
#include <timeval.h>
#include <threads_md.h>
#include <monitor.h>

#include <java_lang_Thread.h>

#define MinimumPriority	    java_lang_Thread_MIN_PRIORITY
#define MaximumPriority	    java_lang_Thread_MAX_PRIORITY
#define NormalPriority	    java_lang_Thread_NORM_PRIORITY

/*
 * Thread-related data structures
 */

typedef struct Hjava_lang_Thread HThread;
typedef struct Hjava_lang_Thread *TID;


typedef sys_thread_t ThreadPrivate;

/* Access to thread data structures */
#define THREAD(tid)	((struct Classjava_lang_Thread *) unhand(tid))
#define SYSTHREAD(tid)	((sys_thread_t *)THREAD(tid)->PrivateInfo)

#define THR_SYSTEM 0		/* System thread */
#define THR_USER   1		/* User thread */

extern TID *ActiveThreadQ;
extern int ActiveThreadCount;		/* All threads */
extern int UserThreadCount;		/* User threads */

#define PROCSTACKSIZE (64 * 1024)     /* Default size of a thread C stack */
#define JAVASTACKSIZE (400 * 1024)     /* Default size of a thread java stack */

extern int ProcStackSize;             /* Actual size of thread C stack */
extern int JavaStackSize;              /* Actual maximum size of java stack */

/*
 * We no longer want to maintain a stack base pointer, as we aren't in
 * as tight control of thread scheduling as before.  So we have to get
 * the stack base from threadSelf().  Furthermore, threadSelf() may
 * not be initialized yet, as we run the static initializers before we
 * can initialize the thread's private data.  So we've got to check for
 * a null result from threadSelf() too.
 */
#define threadCheckStack(redzone, var) \
	sysThreadCheckStack(redzone, var)

void *current_stack_pointer(ThreadPrivate *);

/*
 * External interface to threads support
 */

void threadBootstrap();
int  threadCreate(TID, unsigned int, size_t, void *(*)());
TID  threadSelf();
void threadSleep(int);
int  threadCountUser();
int  threadCountSystem();
int  threadEnumerate(TID*, int);

void threadDumpInfo(TID);                /* Debugging help in debug.c */

/*
 * There may be certain initialization that can't be done except by the
 * thread on itself, e.g. setting thread-local data in Solaris threads.
 * This function is called from ThreadRT0() when the thread starts up
 * to take care of such things.
 */
#define threadInit(tid, sb)		sysThreadInit(SYSTHREAD(tid), sb)

/*
 * Exit the current thread.  This function is not expected to return.
 */
#define threadExit()			sysThreadExit()
#define threadKill(tid)			sysThreadKill(SYSTHREAD(tid))

/*
 * Note that we do not check that priorities are within Java's limits down here.
 * In fact, we make use of that for things like the idle and clock threads.
 * This may change once we work out a portable priority model.
 */
#define threadSetPriority(tid, pri)	sysThreadSetPriority(SYSTHREAD(tid), pri)
#define threadGetPriority(tid, prip)	sysThreadGetPriority(SYSTHREAD(tid), prip)

#define threadYield()			sysThreadYield()
#define threadResume(tid)		sysThreadResume(SYSTHREAD(tid))
#define threadSuspend(tid)		sysThreadSuspend(SYSTHREAD(tid))

/*
 * Return information about this thread's stack.  This is used by
 * Garbage Collection code that needs to inspect the stack.
 *
 * It is permissable to return a null stack_base for those threads
 * that don't have a known stack (e.g. not allocated by the threads
 * package).  It is also permissable to return a somewhat bogus
 * stack_pointer for the current thread.
 */
#define threadStackBase(tid)		sysThreadStackBase(SYSTHREAD(tid))
#define threadStackPointer(tid)		sysThreadStackPointer(SYSTHREAD(tid))

#endif /* !_THREADS_H_ */
