/*
 * @(#)date.c	1.9 95/05/10 Arthur van Hoff
 * 
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies. Please
 * refer to the file "copyright.html" for further important copyright and
 * licensing information.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
 * OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR
 * ITS DERIVATIVES.
 */

#include <oobj.h>
#include <interpreter.h>
#include <javaString.h>

#include <java_util_Date.h>

#include <time.h>

void
java_util_Date_expand(struct Hjava_util_Date * this)
{
    struct tm tm;
    Classjava_util_Date *thisptr = unhand(this);
    time_t t;
    t = ll2int(ll_div(thisptr->value, int2ll(1000)));
    localtime_r(&t, &tm);
    thisptr->tm_millis = ll2int(ll_mod(thisptr->value, int2ll(1000)));
    thisptr->tm_sec = tm.tm_sec;
    thisptr->tm_min = tm.tm_min;
    thisptr->tm_hour = tm.tm_hour;
    thisptr->tm_mday = tm.tm_mday;
    thisptr->tm_mon = tm.tm_mon;
    thisptr->tm_wday = tm.tm_wday;
    thisptr->tm_yday = tm.tm_yday;
    thisptr->tm_year = tm.tm_year;
    thisptr->tm_isdst = tm.tm_isdst;
    thisptr->expanded = TRUE;
}

void
java_util_Date_computeValue(struct Hjava_util_Date * this)
{
    struct tm tm;
    Classjava_util_Date *thisptr = unhand(this);
    tm.tm_sec = thisptr->tm_sec;
    tm.tm_min = thisptr->tm_min;
    tm.tm_hour = thisptr->tm_hour;
    tm.tm_mday = thisptr->tm_mday;
    tm.tm_mon = thisptr->tm_mon;
    tm.tm_year = thisptr->tm_year;
    tm.tm_isdst = -1;
    thisptr->value = ll_mul(int2ll(mktime(&tm)), int2ll(1000));
    thisptr->valueValid = TRUE;
}

/*
 * Convert the date to a string
 */
HString *
java_util_Date_toString(struct Hjava_util_Date * this)
{
    struct tm tm;
    Classjava_util_Date *thisptr = unhand(this);
    char buf[100];
    if (!thisptr->expanded)
	java_util_Date_expand(this);
    tm.tm_sec = thisptr->tm_sec;
    tm.tm_min = thisptr->tm_min;
    tm.tm_hour = thisptr->tm_hour;
    tm.tm_mday = thisptr->tm_mday;
    tm.tm_wday = thisptr->tm_wday;
    tm.tm_yday = thisptr->tm_yday;
    tm.tm_mon = thisptr->tm_mon;
    tm.tm_year = thisptr->tm_year;
    tm.tm_isdst = thisptr->tm_isdst;
    strftime(buf, sizeof buf, "%a %b %d %H:%M:%S %Z %Y", &tm);
    return makeJavaString(buf, strlen(buf));
}

/*
 * Convert the date to a string in local-specific form.
 */
HString *
java_util_Date_toLocaleString(struct Hjava_util_Date * this)
{
    struct tm tm;
    Classjava_util_Date *thisptr = unhand(this);
    char buf[100];
    if (!thisptr->expanded)
	java_util_Date_expand(this);
    tm.tm_sec = thisptr->tm_sec;
    tm.tm_min = thisptr->tm_min;
    tm.tm_hour = thisptr->tm_hour;
    tm.tm_mday = thisptr->tm_mday;
    tm.tm_wday = thisptr->tm_wday;
    tm.tm_yday = thisptr->tm_yday;
    tm.tm_mon = thisptr->tm_mon;
    tm.tm_year = thisptr->tm_year;
    tm.tm_isdst = thisptr->tm_isdst;
    strftime(buf, sizeof buf, 0, &tm);
    return makeJavaString(buf, strlen(buf));
}

/*
 * Convert the date to a string in GMT form,
 * which should be universally parse-able.
 */
HString *
java_util_Date_toGMTString(struct Hjava_util_Date * this)
{
    struct tm tm;
    Classjava_util_Date *thisptr = unhand(this);
    char buf[100];
    time_t t;
    if (!thisptr->valueValid)
	java_util_Date_computeValue(this);
    t = ll2int(ll_div(thisptr->value, int2ll(1000)));
    gmtime_r(&t, &tm);
    strftime(buf, sizeof buf, "%a, %d %b %Y %H:%M:%S GMT", &tm);
    return makeJavaString(buf, strlen(buf));
}
