/*
 * @(#)filesystem.c	1.19 95/03/21 Jonathan Payne
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/* This is a system-independent level of file system operations. */
#include "io_md.h"
#include <unistd.h>
#include <oobj.h>
#include <typecodes.h>
#include <interpreter.h>
#include <javaString.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <java_io_AccessErrorHandler.h>
#include <java_io_File.h>
#include "sys_api.h"
#include "file_acl.h"

HString *
java_io_File_getSeparator(Hjava_io_File *this) {
    char	tmp[2];

    tmp[0] = (char)sysGetFileSeparator();
    tmp[1] = '\0';
    return makeJavaString(tmp, 1);
}


HString *
java_io_File_getReadACL(Hjava_io_File *self) {
    char *acl = getReadACL();

    return makeJavaString(acl, strlen(acl));
}

HString *
java_io_File_getWriteACL(Hjava_io_File *self) {
    char *acl = getWriteACL();

    return makeJavaString(acl, strlen(acl));
}

void
java_io_File_setReadACL(Hjava_io_File *self, Hjava_lang_String *readPath)
{
    initReadACL(readPath);
}

void
java_io_File_setWriteACL(Hjava_io_File *self, Hjava_lang_String *writePath)
{
    initWriteACL(writePath);
}

void
java_io_File_setExecACL(Hjava_io_File *self, Hjava_lang_String *execPath)
{
    initExecACL(execPath);
}

void
java_io_File_setAccessErrorHandler(Hjava_io_File *self,
				  Hjava_io_AccessErrorHandler *h)
{
    setAccessErrorHandler(h);
}


long
java_io_File_exists(Hjava_io_File *this) 
{
    Classjava_io_File *thisptr = unhand(this);
    char path[MAXPATHLEN];

    if (thisptr->path == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(thisptr->path, path, sizeof (path));

    if (! checkForReadAccess(path, thisptr->path)) {
	return 0;
    }

    return sysAccess(path, F_OK) ? 0 : 1;
}

long
java_io_File_canRead(Hjava_io_File *this) 
{
    Classjava_io_File *thisptr = unhand(this);
    char path[MAXPATHLEN];

    if (thisptr->path == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(thisptr->path, path, sizeof (path));

    if (! checkForReadAccess(path, thisptr->path)) {
	return 0;
    }

    return sysAccess(path, R_OK) ? 0 : 1;
}

long
java_io_File_canWrite(Hjava_io_File *this) 
{
    Classjava_io_File *thisptr = unhand(this);
    char path[MAXPATHLEN];

    if (thisptr->path == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(thisptr->path, path, sizeof (path));

    if (! checkForWriteAccess(path, thisptr->path)) {
	return 0;
    }

    return sysAccess(path, W_OK) ? 0 : 1;
}

long
java_io_File_isFile(Hjava_io_File *this) 
{
    Classjava_io_File *thisptr = unhand(this);
    char path[MAXPATHLEN];
    struct stat buf;

    if (thisptr->path == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(thisptr->path, path, sizeof (path));

    if (! checkForReadAccess(path, thisptr->path)) {
	return 0;
    }

    return ((sysStat(path, &buf) != -1) && (buf.st_mode & S_IFREG)) ? 1 : 0;
}

long
java_io_File_isDirectory(Hjava_io_File *this) 
{
    Classjava_io_File *thisptr = unhand(this);
    char path[MAXPATHLEN];
    struct stat buf;

    if (thisptr->path == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(thisptr->path, path, sizeof (path));

/*
  XXX: I'm removing this check because it is sometimes used to
       verify whether an applet is attempting to redefine a 
       package in the CLASSPATH.

    if (! checkForReadAccess(path, thisptr->path)) {
	return 0;
    }
*/

    return ((sysStat(path, &buf) != -1) && (buf.st_mode & S_IFDIR)) ? 1 : 0;
}

long
java_io_File_lastModified(Hjava_io_File *this) 
{
    Classjava_io_File *thisptr = unhand(this);
    char path[MAXPATHLEN];
    struct stat buf;

    if (thisptr->path == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(thisptr->path, path, sizeof (path));

    if (! checkForReadAccess(path, thisptr->path)) {
	return 0;
    }

    return (sysStat(path, &buf) != -1) ? buf.st_mtime : 0;
}

long
java_io_File_mkdir(Hjava_io_File *this) 
{
    Classjava_io_File *thisptr = unhand(this);
    char path[MAXPATHLEN];

    if (thisptr->path == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(thisptr->path, path, sizeof (path));

    if (! checkForWriteAccess(path, thisptr->path)) {
	return 0;
    }

    return mkdir(path, 0771) != -1;
}

long
java_io_File_renameTo(Hjava_io_File *this, Hjava_io_File *dest) 
{
    Classjava_io_File *thisptr = unhand(this);
    Classjava_io_File *destptr;
    char path[MAXPATHLEN];
    char dpath[MAXPATHLEN];

    if ((thisptr->path == 0) || (dest == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(thisptr->path, path, sizeof (path));
    if (! checkForWriteAccess(path, thisptr->path)) {
	return 0;
    }

    destptr = unhand(dest);
    javaString2CString(destptr->path, dpath, sizeof (dpath));
    if (! checkForWriteAccess(dpath, destptr->path)) {
	return 0;
    }

    return rename(path, dpath) != -1;
}

HArrayOfString *
java_io_File_list(Hjava_io_File *this) 
{
    Classjava_io_File *thisptr = unhand(this);
    char path[MAXPATHLEN];
    DIR *dir;
    struct dirent *ptr;
    HString **list;
    int len, maxlen;
    HArrayOfString *arr;

    if (thisptr->path == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    /* open the directory */
    javaString2CString(thisptr->path, path, sizeof (path));

    if (! checkForReadAccess(path, thisptr->path)) {
	return 0;
    }

    if ((dir = opendir(path)) == NULL) {
	return NULL;
    }

    /* allocate a list of string handles */
    len = 0;
    maxlen = 4;
    list = (HString **)malloc(maxlen * sizeof(HString *));
    if (list == NULL) {
	SignalError(0, JAVAPKG "OutOfMemoryException", path);
	return NULL;
    }

    /* scan the directory */
    while ((ptr = readdir(dir)) != NULL) {
	if (strcmp(ptr->d_name, ".") && strcmp(ptr->d_name, "..")) {
	    if (len == maxlen) {
		HString **newlist = (HString **)malloc((maxlen <<= 1) * sizeof(HString *));
		if (newlist == NULL) {
		    free(list);
		    SignalError(0, JAVAPKG "OutOfMemoryException", path);
		    return NULL;
		}
		memcpy(newlist, list, len * sizeof(HString *));
		free(list);
		list = newlist;
	    }

	    /* add it to the list */
	    list[len++] = makeJavaString(ptr->d_name, strlen(ptr->d_name));
	}
    }

    /* allocate the array */
    arr = (HArrayOfString *)ArrayAlloc(T_CLASS, len);
    if (arr == NULL) {
	free(list);
	SignalError(0, JAVAPKG "OutOfMemoryException", path);
	return NULL;
    }
    memcpy(unhand(arr), list, len * sizeof(HString *));

    /* free the list */
    free(list);
    return arr;
}
