/*
 * @(#)ResourceStrings.java	1.8 95/01/31 Arthur van Hoff
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.tools.javac;

import java.tools.java.*;
import java.util.Hashtable;
import java.io.File;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.BufferedInputStream;

/**
 * Resource Strings
 *
 * This class reads a file containing numbered
 * resource strings. The strings can later be
 * accessed by their resource number.
 */

public final
class ResourceStrings {
    /**
     * The name of the strings file
     */
    static final String stringsFileName = "java/tools/javac/strings.rsrc";

    /**
     * The path where to look for the resource strings when they are needed
     */
    private String path[];
    
    /**
     * A hashtable of resource strings.
     */
    private Hashtable hash;

    /**
     * Construct given a file name. The resource strings in the file
     * are numbered sequentially, one string per line. The count can
     * be reset by adding a line starting with a '*' followed by a
     * number. Empty lines and lines starting with '#' are ignored.
     */
    public ResourceStrings(String path[]) {
	this.path = path;
    }

    /**
     * Get a string given a resource number
     */
    public String getString(int nr) {
	if (hash == null) {
	    for (int i = 0 ; i < path.length ; i++) {
		File f = new File(path[i], stringsFileName);
		if (f.canRead()) {
		    FileInputStream file = new FileInputStream(f.getPath());
		    DataInputStream in = new DataInputStream(new BufferedInputStream(file));
		    int n = 0;
		    String str;
		    hash = new Hashtable(501);
		    
		    while ((str = in.readLine()) != null) {
			if (str.length() > 0) {
			    switch (str.charAt(0)) {
			      case '*':
				n = (Integer.valueOf(str.substring(1))).intValue();
				break;
			      case '#':
				break;
			      default:
				hash.put(new Integer(n++), str);
			    }
			}
		    }
		    file.close();
		    return (String)hash.get(new Integer(nr));
		}
	    }
	    return "can't find: " + stringsFileName;
	}
	return (String)hash.get(new Integer(nr));
    }
}
