/*
 * @(#)BitXorExpression.java	1.7 95/01/31 Arthur van Hoff
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.tools.tree;

import java.tools.java.*;
import java.tools.asm.Assembler;

public
class BitXorExpression extends BinaryBitExpression {
    /**
     * constructor
     */
    public BitXorExpression(int where, Expression left, Expression right) {
	super(BITXOR, where, left, right);
    }

    /**
     * Evaluate
     */
    Expression eval(boolean a, boolean b) {
	return new BooleanExpression(where, a ^ b);
    }
    Expression eval(int a, int b) {
	return new IntExpression(where, a ^ b);
    }
    Expression eval(long a, long b) {
	return new LongExpression(where, a ^ b);
    }

    /**
     * Simplify
     */
    Expression simplify() {
	if (left.equals(true)) {
	    return new NotExpression(where, right);
	}
	if (right.equals(true)) {
	    return new NotExpression(where, left);
	}
	if (left.equals(false) || left.equals(0)) {
	    return right;
	}
	if (right.equals(false) || right.equals(0)) {
	    return left;
	}
	return this;
    }

    /**
     * Code
     */
    void codeOperation(Environment env, Context ctx, Assembler asm) {
	asm.add(where, opc_ixor + type.getTypeCodeOffset());
    }
}
