/*
 * @(#)BooleanExpression.java	1.8 95/01/31 Arthur van Hoff
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.tools.tree;

import java.tools.java.*;
import java.tools.asm.Assembler;
import java.tools.asm.Label;
import java.io.PrintStream;

public
class BooleanExpression extends ConstantExpression {
    boolean value;
    
    /**
     * Constructor
     */
    public BooleanExpression(int where, boolean value) {
	super(BOOLEANVAL, where, Type.tBoolean);
	this.value = value;
    }

    /**
     * Get the value
     */
    public Object getValue() {
	return new Integer(value ? 1 : 0);
    }

    /**
     * Check if the expression is equal to a value
     */
    public boolean equals(boolean b) {
	return value == b;
    }

    /**
     * Code
     */
    void codeBranch(Environment env, Context ctx, Assembler asm, Label lbl, boolean whenTrue) {
	if (value == whenTrue) {
	    asm.add(where, opc_goto, lbl);
	}
    }
    public void codeValue(Environment env, Context ctx, Assembler asm) {
	asm.add(where, opc_ldc2, new Integer(value ? 1 : 0));
    }

    /**
     * Print
     */
    public void print(PrintStream out) {
	out.print(value ? "true" : "false");
    }
}
