/*
 * @(#)CastExpression.java	1.10 95/01/31 Arthur van Hoff
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.tools.tree;

import java.tools.java.*;
import java.tools.asm.Assembler;
import java.tools.asm.Label;
import java.io.PrintStream;

public
class CastExpression extends BinaryExpression {
    /**
     * constructor
     */
    public CastExpression(int where, Expression left, Expression right) {
	super(CAST, where, left.type, left, right);
    }

    /**
     * Check the expression
     */
    public long checkValue(Environment env, Context ctx, long vset) {
	type = left.toType(env, ctx);
	vset = right.checkValue(env, ctx, vset);

	if (type.isType(TC_ERROR) || right.type.isType(TC_ERROR)) {
	    // An error was already reported
	    return vset;
	}

	if (type.equals(right.type)) {
	    // The types are already the same
	    return vset;
	}

	try {
	    if (env.explicitCast(right.type, type)) {
		right = new ConvertExpression(where, type, right);
		return vset;
	    }
	} catch (ClassNotFound e) {
	    env.error(where, ERR_CLASS_NOT_FOUND, e.name, opNames[op]);
	}

	// The cast is not allowed
	env.error(where, ERR_INVALID_CAST, right.type, type);
	return vset;
    }

    /**
     * Inline
     */
    public Expression inline(Environment env, Context ctx) {
	return right.inline(env, ctx);
    }
    public Expression inlineValue(Environment env, Context ctx) {
	return right.inlineValue(env, ctx);
    }

    /**
     * Print
     */
    public void print(PrintStream out) {
	out.print("(" + opNames[op] + " ");
	if (type.isType(TC_ERROR)) {
	    left.print(out);
	} else {
	    out.print(type);
	}
	out.print(" ");
	right.print(out);
	out.print(")");
    }
}
