/*
 * @(#)EqualExpression.java	1.9 95/01/31 Arthur van Hoff
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.tools.tree;

import java.tools.java.*;
import java.tools.asm.Assembler;
import java.tools.asm.Label;

public
class EqualExpression extends BinaryEqualityExpression {
    /**
     * constructor
     */
    public EqualExpression(int where, Expression left, Expression right) {
	super(EQ, where, left, right);
    }

    /**
     * Evaluate
     */
    Expression eval(int a, int b) {
	return new BooleanExpression(where, a == b);
    }
    Expression eval(long a, long b) {
	return new BooleanExpression(where, a == b);
    }
    Expression eval(float a, float b) {
	return new BooleanExpression(where, a == b);
    }
    Expression eval(double a, double b) {
	return new BooleanExpression(where, a == b);
    }
    Expression eval(boolean a, boolean b) {
	return new BooleanExpression(where, a == b);
    }

    /**
     * Simplify
     */
    Expression simplify() {
	if (left.isConstant() && !right.isConstant()) {
	    return new EqualExpression(where, right, left);
	}
	return this;
    }

    /**
     * Code
     */
    void codeBranch(Environment env, Context ctx, Assembler asm, Label lbl, boolean whenTrue) {
	left.codeValue(env, ctx, asm);
	switch (left.type.getTypeCode()) {
	  case TC_BOOLEAN:
	  case TC_INT:
	    if (!right.equals(0)) {
		right.codeValue(env, ctx, asm);
		asm.add(where, whenTrue ? opc_if_icmpeq : opc_if_icmpne, lbl);
		return;
	    }
	    break;
	  case TC_LONG:
	    right.codeValue(env, ctx, asm);
	    asm.add(where, opc_lcmp);
	    break;
	  case TC_FLOAT:
	    right.codeValue(env, ctx, asm);
	    asm.add(where, opc_fcmpl);
	    break;
	  case TC_DOUBLE:
	    right.codeValue(env, ctx, asm);
	    asm.add(where, opc_dcmpl);
	    break;
	  case TC_ARRAY:
	  case TC_CLASS:
	    if (!right.equals(0)) {
		right.codeValue(env, ctx, asm);
		asm.add(where, whenTrue ? opc_if_acmpeq : opc_if_acmpne, lbl);
		return;
	    }
	    break;
	    
	  default:
	    throw new InternalError("Unexpected Type");
	}
	asm.add(where, whenTrue ? opc_ifeq : opc_ifne, lbl);
    }
}
