/*
 * @(#)Node.java	1.16 95/02/11 Arthur van Hoff
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.tools.tree;

import java.tools.java.*;
import java.io.PrintStream;

public
class Node implements Constants {
    int op;
    int where;

    /**
     * Constructor
     */
    Node(int op, int where) {
	this.op = op;
	this.where = where;
    }

    /**
     * Get the operator
     */
    public int getOp() {
	return op;
    }

    /**
     * Get where 
     */
    public int getWhere() {
	return where;
    }

    /**
     * Implicit conversions
     */
    public Expression convert(Environment env, Context ctx, Type t, Expression e) {
	if (e.type.isType(TC_ERROR) || t.isType(TC_ERROR)) {
	    // An error was already reported
	    return e;
	}

	if (e.type.equals(t)) {
	    // The types are already the same
	    return e;
	}

	try {
	    if (env.implicitCast(e.type, t)) {
		if (env.warnings() && !e.fitsType(env, t)) {
		    env.error(e.where, WARN_LOSE_PRECISION, e.type, t);
		}
		return new ConvertExpression(where, t, e);
	    }

	    if (env.explicitCast(e.type, t)) {
		env.error(where, ERR_EXPLICIT_CAST_NEEDED, opNames[op], e.type, t);
		return new ConvertExpression(where, t, e);
	    }
	} catch (ClassNotFound ee) {
	    env.error(where, ERR_CLASS_NOT_FOUND, ee.name, opNames[op]);
	}

	// The cast is not allowed
	env.error(where, ERR_INCOMPATIBLE_TYPE, opNames[op], e.type, t);
	return new ConvertExpression(where, Type.tError, e);
    }

    /**
     * Print
     */
    public void print(PrintStream out) {
	throw new InternalError("print");
    }
}
