/*
 * @(#)NotExpression.java	1.10 95/01/31 Arthur van Hoff
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.tools.tree;

import java.tools.java.*;
import java.tools.asm.Assembler;
import java.tools.asm.Label;

public
class NotExpression extends UnaryExpression {
    /**
     * Constructor
     */
    public NotExpression(int where, Expression right) {
	super(NOT, where, Type.tBoolean, right);
    }
 
    /**
     * Select the type of the expression
     */
    void selectType(Environment env, Context ctx, int tm) {
	right = convert(env, ctx, Type.tBoolean, right);
    }

    /**
     * Check an and condition
     */
    public void checkCondition(Environment env, Context ctx, ConditionVars cvars) {
	right.checkCondition(env, ctx, cvars);
	right = convert(env, ctx, Type.tBoolean, right);
	// swap true and false
	cvars.vsTrue ^= cvars.vsFalse;
	cvars.vsFalse ^= cvars.vsTrue;
	cvars.vsTrue ^= cvars.vsFalse;
    }

    /**
     * Evaluate
     */
    Expression eval(boolean a) {
	return new BooleanExpression(where, !a);
    }

    /**
     * Simplify
     */
    Expression simplify() {
	// Check if the expression can be optimized
	switch (right.op) {
	  case NOT:
	    return ((NotExpression)right).right;

	  case EQ:
	  case NE:
	  case LT:
	  case LE:
	  case GT:
	  case GE:
	    break;

	  default:
	    return this;
	}

	// Can't negate real comparisons
	BinaryExpression bin = (BinaryExpression)right;
	if (bin.left.type.inMask(TM_REAL)) {
	    return this;
	}
	
	// Negate comparison
	switch (right.op) {
	  case EQ:
	    return new NotEqualExpression(where, bin.left, bin.right);
	  case NE:
	    return new EqualExpression(where, bin.left, bin.right);
	  case LT:
	    return new GreaterOrEqualExpression(where, bin.left, bin.right);
	  case LE:
	    return new GreaterExpression(where, bin.left, bin.right);
	  case GT:
	    return new LessOrEqualExpression(where, bin.left, bin.right);
	  case GE:
	    return new LessExpression(where, bin.left, bin.right);
	}
	return this;
    }

    /**
     * Code
     */
    void codeBranch(Environment env, Context ctx, Assembler asm, Label lbl, boolean whenTrue) {
	right.codeBranch(env, ctx, asm, lbl, !whenTrue);
    }
}
