/*
 * @(#)SynchronizeStatement.java	1.9 95/01/31 Arthur van Hoff
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.tools.tree;

import java.tools.java.*;
import java.tools.asm.Assembler;
import java.tools.asm.Label;
import java.tools.asm.TryData;
import java.tools.asm.CatchData;
import java.io.PrintStream;

public
class SynchronizeStatement extends Statement {
    Expression expr;
    Statement body;

    /**
     * Constructor
     */
    public SynchronizeStatement(int where, Expression expr, Statement body) {
	super(SYNCHRONIZE, where);
	this.expr = expr;
	this.body = body;
    }

    /**
     * Check statement
     */
    long check(Environment env, Context ctx, long vset) {
	CheckContext newctx = new CheckContext(ctx, this);
	vset = reach(env, vset);
	vset = expr.checkValue(env, newctx, vset);
	expr = convert(env, newctx, Type.tClass(idJavaLangObject), expr);
	vset = body.check(env, newctx, vset);
	return vset & newctx.vsBreak;
    }

    /**
     * Inline
     */
    public Statement inline(Environment env, Context ctx) {
	if (body != null) {
	    body = body.inline(env, ctx);
	}
	if (body == null) {
	    return new ExpressionStatement(where, expr).inline(env, ctx);
	}
	expr = expr.inlineValue(env, ctx);
	return this;
    }

    /**
     * Create a copy of the statement for method inlining
     */
    public Statement copyInline(Context ctx, boolean valNeeded) {
	throw new InternalError("copyInline");
    }

    /**
     * Code
     */
    public void code(Environment env, Context ctx, Assembler asm) {
	expr.codeValue(env, ctx, asm);
	
	ctx = new Context(ctx, null);
	LocalField f1 = new LocalField(where, ctx.field.getClassDefinition(), 0, Type.tObject, null);
	LocalField f2 = new LocalField(where, ctx.field.getClassDefinition(), 0, Type.tInt, null);
	Integer num1 = new Integer(ctx.declare(env, f1));
	Integer num2 = new Integer(ctx.declare(env, f2));

	Label endLabel = new Label();

	TryData td = new TryData();
	td.add(null);

	// lock the object
	asm.add(where, opc_astore, num1);
	asm.add(where, opc_aload, num1);
	asm.add(where, opc_monitorenter);

	// Main body
	CodeContext bodyctx = new CodeContext(ctx, this);
	asm.add(where, opc_try, td);
	body.code(env, bodyctx, asm);
	asm.add(bodyctx.breakLabel);
	asm.add(td.getEndLabel());

	// Cleanup afer body
	asm.add(where, opc_aload, num1);
	asm.add(where, opc_monitorexit);
	asm.add(where, opc_goto, endLabel);

	// Catch code
	CatchData cd = td.getCatch(0);
	asm.add(cd.getLabel());
	asm.add(where, opc_aload, num1);
	asm.add(where, opc_monitorexit);
	asm.add(where, opc_athrow);

	// Final body
	asm.add(bodyctx.contLabel);
	asm.add(where, opc_astore, num2);
	asm.add(where, opc_aload, num1);
	asm.add(where, opc_monitorexit);
	asm.add(where, opc_ret, num2);

	asm.add(endLabel);
    }

    /**
     * Print
     */
    public void print(PrintStream out, int indent) {
	super.print(out, indent);
	out.print("synchronize ");
	expr.print(out);
	out.print(" ");
	body.print(out, indent);
    }
}
