/*
 * @(#)TryStatement.java	1.11 95/04/02 Arthur van Hoff
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.tools.tree;

import java.tools.java.*;
import java.tools.asm.Assembler;
import java.tools.asm.Label;
import java.tools.asm.TryData;
import java.tools.asm.CatchData;
import java.io.PrintStream;

public
class TryStatement extends Statement {
    Statement body;
    Statement args[];

    /**
     * Constructor
     */
    public TryStatement(int where, Statement body, Statement args[]) {
	super(TRY, where);
	this.body = body;
	this.args = args;
    }

    /**
     * Check statement
     */
    long check(Environment env, Context ctx, long vset) {
	vset = reach(env, vset);
	long vs = body.check(env, new CheckContext(ctx, this), vset);
	
	for (int i = 0 ; i < args.length ; i++) {
	    vs &= args[i].check(env, new CheckContext(ctx, this), vset);
	}
	return vs;
    }

    /**
     * Inline
     */
    public Statement inline(Environment env, Context ctx) {
	if (body != null) {
	    body = body.inline(env, new Context(ctx, this));
	}
	if (body == null) {
	    return null;
	}
	for (int i = 0 ; i < args.length ; i++) {
	    if (args[i] != null) {
		args[i] = args[i].inline(env, new Context(ctx, this));
	    }
	}
	return (args.length == 0) ? eliminate(env, body) : this;
    }

    /**
     * Create a copy of the statement for method inlining
     */
    public Statement copyInline(Context ctx, boolean valNeeded) {
	throw new InternalError("copyInline");
    }

    /**
     * Code
     */
    public void code(Environment env, Context ctx, Assembler asm) {
	CodeContext newctx = new CodeContext(ctx, this);
	
	TryData td = new TryData();
	for (int i = 0 ; i < args.length ; i++) {
	    Type t = ((CatchStatement)args[i]).field.getType();
	    if (t.isType(TC_CLASS)) {
		td.add(env.getClassDeclaration(t));
	    } else {
		td.add(t);
	    }
	}
	asm.add(where, opc_try, td);
	if (body != null) {
	    body.code(env, newctx, asm);
	}

	asm.add(td.getEndLabel());
	asm.add(where, opc_goto, newctx.breakLabel);
	
	for (int i = 0 ; i < args.length ; i++) {
	    CatchData cd = td.getCatch(i);
	    asm.add(cd.getLabel());
	    args[i].code(env, newctx, asm);
	    asm.add(where, opc_goto, newctx.breakLabel);
	}
	
	asm.add(newctx.breakLabel);
    }
    
    /**
     * Print
     */
    public void print(PrintStream out, int indent) {
	super.print(out, indent);
	out.print("try ");
	if (body != null) {
	    body.print(out, indent);
	} else {
	    out.print("<empty>");
	}
	for (int i = 0 ; i < args.length ; i++) {
	    out.print(" ");
	    args[i].print(out, indent);
	}
    }
}
