/*
 * @(#)linker.c	1.15 95/03/13  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*-
 *      Link foreign methods.  This file contains the
 *	machine independent dynamic linking routines.
 *	See "BUILD_PLATFORM"/java/lang/linker_md.c to see
 *	the implementation of this shared dynamic linking
 *	interface.
 *
 *	NOTE - source in this file is POSIX.1 compliant, host
 *	       specific code lives in the platform specific
 *	       code tree.
 */

#include <oobj.h>
#include <interpreter.h>
#include <code.h>
#include <tree.h>
#include <javaString.h>

#include <sys_api.h>
#include <path.h>

#include "installpath.h"

/* This is a hack to make sure that the stubs are linked properly */
/* There has to be some reference to a stub otherwise none of the */
/* stubs are included in the link */
extern void java_util_Linker_load_stub();
static void *bogus_reference = (void *)&java_util_Linker_load_stub;

#include <java_util_Linker.h>

/* defined in file_acl.c */
extern long currentClassHasClassLoader();

/* 
 * Initialize the linker. This is needed to add the libraries
 * directory to the LD_LIBRARY_PATH.
 */

HString *
java_util_Linker_initialize(struct Hjava_util_Linker *Hlink) 
{
    char *ldpathbuf;

    if (currentClassHasClassLoader()) {
	SignalError(0, JAVAPKG "IOException","permission error");
	return 0;
    }

    ldpathbuf = sysInitializeLinker();
    sysSaveLDPath(ldpathbuf);
    return makeJavaString(ldpathbuf + 16, strlen(ldpathbuf + 16));
}

long
java_util_Linker_getSeparator(struct Hjava_util_Linker *Hlink)
{
    return(sysGetSeparator());
}


/*
 * Load a shared library, this may throw an UnsatisfiedLinkException
 * if the shared library can't be found
 */

void
java_util_Linker_load(struct Hjava_util_Linker *Hlinker, HString *Hfilename) {
    char fname[128];

    if (Hfilename == 0) {
	SignalError(0, JAVAPKG "NullPointerException", "filename");
	return;
    }

    if (currentClassHasClassLoader()) {
	SignalError(0, JAVAPKG "IOException","permission error");
	return;
    }

    javaString2CString(Hfilename, fname, sizeof fname);
    if (!sysAddDLSegment(fname)) {
	SignalError(0, JAVAPKG "UnsatisfiedLinkException", fname);
    }
}


/*
 * build a complete ld library string name with path, filename, and ext's
 */

HString *
java_util_Linker_buildLibName(struct Hjava_util_Linker *Hlinker,
			     HString *Hpath,
			     HString *Hfilename)
{
    char fname[128];
    char pname[1024];
    char holder[1280];

    if ((Hpath == 0) || (Hfilename == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(Hpath, pname, sizeof(pname));    
    javaString2CString(Hfilename, fname, sizeof(fname));
    sysBuildLibName(holder, pname, fname);
    return(makeJavaString(holder, strlen(holder)));
}
