/*
 * @(#)math.c	1.15 95/02/22  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <oobj.h>
#include <interpreter.h>
#include <javaString.h>
#include <typecodes.h>

#include <java_lang_Math.h>
#include <java_lang_Number.h>
#include <java_lang_Integer.h>
#include <java_lang_Long.h>
#include <java_lang_Float.h>
#include <java_lang_Double.h>

#include <math.h>
#include <path.h>

Hjava_lang_String *
java_lang_Float_toString(Hjava_lang_Float *hf, float f) 
{
    char buf[32];
    sprintf(buf,"%g",f);
    return makeJavaString(buf,strlen(buf));
}

Hjava_lang_Integer *
java_lang_Integer_valueOf(Hjava_lang_Integer *f, struct Hjava_lang_String *s) 
{
    char buf[32];
    int result;

    if (s == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(s, buf, sizeof(buf));
    result = atoi(buf);

    return (Hjava_lang_Integer *)
         execute_java_constructor(0, JAVAPKG "Integer", 0, "(I)", result);
}

Hjava_lang_Long *
java_lang_Long_valueOf(Hjava_lang_Long *f, struct Hjava_lang_String *s) 
{
    char buf[64];
    int64_t result;

    if (s == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(s, buf, sizeof(buf));
    result = atoll(buf);

    return (Hjava_lang_Long *)
         execute_java_constructor(0, JAVAPKG "Long", 0, "(J)", result);
}

Hjava_lang_Float *
java_lang_Float_valueOf(Hjava_lang_Float *f, struct Hjava_lang_String *s) 
{
    char buf[64], *p;
    float result;

    if (s == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(s, buf, sizeof(buf));
    result = (float)strtod(buf, &p);

    if (p == buf) {
	SignalError(0, JAVAPKG "NumberFormatException", buf);
	return 0;
    }

    return (Hjava_lang_Float *)
         execute_java_constructor(0, JAVAPKG "Float", 0, "(F)", result);
}

Hjava_lang_Double *
java_lang_Double_valueOf(Hjava_lang_Double *f, struct Hjava_lang_String *s) 
{
    char buf[64], *p;
    double result;

    if (s == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(s, buf, sizeof(buf));
    result = strtod(buf, &p);

    if (p == buf) {
	SignalError(0, JAVAPKG "NumberFormatException", buf);
	return 0;
    }

    return (Hjava_lang_Double *)
        execute_java_constructor(0, JAVAPKG "Double", 0, "(D)", result);
}

double 
java_lang_Math_cos(Hjava_lang_Math *fh, double f) 
{
    return cos(f);
}

double 
java_lang_Math_sin(Hjava_lang_Math *fh, double f) 
{
    return sin(f);
}

double 
java_lang_Math_tan(Hjava_lang_Math *fh, double f) 
{
    return tan(f);
}

double 
java_lang_Math_asin(Hjava_lang_Math *fh, double f) 
{
    return asin(f);
}

double 
java_lang_Math_acos(Hjava_lang_Math *fh, double f) 
{
    return acos(f);
}

double 
java_lang_Math_atan(Hjava_lang_Math *fh, double f) 
{
    return atan(f);
}

double 
java_lang_Math_exp(Hjava_lang_Math *fh, double f) 
{
    return exp(f);
}

double 
java_lang_Math_log(Hjava_lang_Math *fh, double f) 
{
    return log(f);
}

double 
java_lang_Math_sqrt(Hjava_lang_Math *fh, double f) 
{
    return sqrt(f);
}

double 
java_lang_Math_ceil(Hjava_lang_Math *fh, double f) 
{
    return ceil(f);
}

double 
java_lang_Math_floor(Hjava_lang_Math *fh, double f) 
{
    return floor(f);
}

double 
java_lang_Math_rint(Hjava_lang_Math *fh, double f) 
{
    return rint(f);
}

double 
java_lang_Math_atan2(Hjava_lang_Math *f, double f1, double f2) 
{
    return atan2(f1,f2);
}

double
java_lang_Math_annuity(Hjava_lang_Math *f, double f1, double f2) {
    return annuity(f1,f2);
}

double 
java_lang_Math_pow(Hjava_lang_Math *f, double f1, double f2) 
{
    return pow(f1,f2);
}

void 
java_lang_Math_srandom(Hjava_lang_Math *f, long seed) 
{
    void srand48(long);
    srand48(seed);
}

double 
java_lang_Math_random(Hjava_lang_Math *f) 
{
    double drand48(void);
    return drand48();
}
