/*
 * @(#)object.c	1.24 95/03/17  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*-
 *      Implementation of class Object
 *
 *      former threadruntime.c, Sun Sep 22 12:09:39 1991
 */

#include <stdio.h>
#include <signal.h>

#include <tree.h>
#include <decode.h>
#include <javaString.h>
#include <signature.h>

#include <monitor.h>
#include <threads.h>
#include <exceptions.h>
#include <path.h>

#include <java_lang_Object.h>

long
java_lang_Object_hashCode(Handle *this)
{
    return (int)this;
}

void
java_lang_Object_wait(register Handle*p, long millis)
{
    register ClassClass *clb;

    if (p == 0)
	return;
    clb = obj_classblock(p);

    if (millis) {
	monitorWait(obj_monitor(p), millis);
    } else {
	monitorWait(obj_monitor(p), TIMEOUT_INFINITY);
    }
    return;
}

void
java_lang_Object_notify(register Handle* p)
{

    if (p == 0)
	return;
    monitorNotify(obj_monitor(p));
}

void
java_lang_Object_notifyAll(register Handle*p)
{
    if (p == 0)
	return;
    monitorNotifyAll(obj_monitor(p));
}

void
java_lang_Object_enterMonitor(HObject *Hthis)
{
    monitorEnter(obj_monitor(Hthis));
}

void
java_lang_Object_exitMonitor(HObject *Hthis)
{
    monitorExit(obj_monitor(Hthis));
}

void
java_lang_Object_copy(Handle*dst, Handle*src)
{
    if (dst == 0 || src == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if (obj_flags(src) != obj_flags(dst)) {
	SignalError(0, JAVAPKG "ClassCastException", 0);
	return;
    }
    if (obj_flags(src) == T_NORMAL_OBJECT) {
	ClassClass *cb = obj_classblock(dst);
	if (obj_methodtable(src) != obj_methodtable(dst)) {
	    SignalError(0, JAVAPKG "ClassCastException", 0);
	} else {
	    int size = cbInstanceSize(cb);
	    memcpy((char *) unhand(dst), (char *) unhand(src), size);
	}
    } else {
	if (obj_length(src) != obj_length(dst)) {
	    SignalError(0, JAVAPKG "ClassCastException", 0);
	} else {
	    memcpy((char *)unhand(dst), (char *)unhand(src), 
		   sizearray(obj_flags(src), obj_length(src)));
	}
    }
}


HObject *
java_lang_Object_clone(HObject *this)
{
    if (obj_flags(this) == T_NORMAL_OBJECT) {
	ClassClass *cb = obj_classblock(this);
	HObject *obj = ObjAlloc(cb, 0);

	if (obj == 0) {
	    char buf[128];
	    SignalError(0, JAVAPKG "OutOfMemoryException", 
			classname2string(classname(cb), buf, sizeof(buf)));
	    return 0;
	}
	/* Initialize the new object from the prototype */
	memcpy((char *)unhand(obj), 
	       (char *)unhand(cbPrototype(cb)), cbInstanceSize(cb));
    
	/* Copy the content of the current object into the new object */
	execute_java_dynamic_method(0, obj, "copy", "(L" JAVAPKG "Object;)V", 
				   this);
	return obj;
    } else {
	int type = obj_flags(this);
	int size = obj_length(this);
	HObject *obj = ArrayAlloc(type, size);
	if (obj == 0) {
	    SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	    return 0;
	}
	memcpy((char*)unhand(obj), (char*)unhand(this), sizearray(type, size));
	return obj;
    }
}

static char *
class2string(ClassClass * cb, char *buf, char *limit)
{
    if (cb == 0)
	buf = addstr("NULL-classblock", buf, limit, 0);
    else {
	extern ClassClass *classobject;
	if (cbSuperclass(cb) && unhand(cbSuperclass(cb)) != classobject) {
	    buf = class2string(unhand(cbSuperclass(cb)), buf, limit);
	    buf = addstr(".", buf, limit, 0);
	}
	buf = addstr(classname(cb), buf, limit, 0);
    }
    return buf;
}

char *
obj2string(Handle*h, char *buf, char *limit)
{
    if (h == 0)
	return "NULL";

    if (obj_flags(h) == T_NORMAL_OBJECT) {
	ClassObject *o = unhand(h);
	ClassClass *clb = obj_classblock(h);
	bool_t needcomma = FALSE;
	buf = class2string(clb, buf, limit);
	buf = addstr("[", buf, limit, 0);
	for ( ; cbSuperclass(clb); clb = unhand(cbSuperclass(clb))) {
	    long n, *addr;
	    struct fieldblock *fb;
	    for (n = clb->fields_count, fb = cbFields(clb); --n >= 0; fb++) {
		if (IsPrivate(fb->access) && !PrintPrivate) 
		    continue;
		if (fb->access & ACC_STATIC) {
		    char type = fb->signature[0];
		    addr = (type == SIGNATURE_DOUBLE || 
			    type == SIGNATURE_LONG)
			         ?  twoword_static_address(fb)
				 :  normal_static_address(fb);
		} else 
		    addr = (long *) ((char *) o + fb->u.offset);
		if (needcomma)
		    buf = addstr(", ", buf, limit, 0);
		needcomma = TRUE;
		buf = addstr(fieldname(fb), buf, limit, 0);
		buf = addstr("=", buf, limit, 0);
		switch (fieldsig(fb)[0]) {

		case SIGNATURE_ARRAY:
		case SIGNATURE_CLASS:  {
		    Handle*hlcl = *(Handle**)addr;
		    if (hlcl == 0)
			buf = addstr("NULL", buf, limit, 0);
		    else {
			ClassObject *lcl = unhand(hlcl);
			ClassClass *vcl;

			switch (obj_flags(hlcl)) {

			case T_NORMAL_OBJECT:
			    vcl = obj_classblock(hlcl);
			    buf = class2string(vcl, buf, limit);
			    break;
			case T_CHAR:
			    buf = addstr(unicode2rd(((ArrayOfChar *)lcl)->body,
						    obj_length(hlcl)),
					 buf, limit, 0);
			    break;
			default:
			    buf = addstr(arrayinfo[obj_flags(hlcl)].name,
					 buf, limit, 0);
			    buf = addstr("[", buf, limit, 0);
			    buf = adddec(obj_length(hlcl), buf, limit);
			    buf = addstr("]", buf, limit, 0);
			}
		    }
		    break;
		}
		    
		case SIGNATURE_CHAR: case SIGNATURE_ENUM:
		case SIGNATURE_INT: case SIGNATURE_SHORT:
		    buf = adddec(*addr, buf, limit);
		    break;
		case SIGNATURE_FLOAT: {
		    char        fbuf[20];
		    extern void gconvert(double, long, long, char *);
		    gconvert(*(float *) addr, 8, 0, fbuf);
		    buf = addstr(fbuf, buf, limit, 0);
		    break;
		}
		
		default:
		    buf = addstr("?", buf, limit, 0);
		    buf = addhex(*addr, buf, limit);
		    break;
		} /* of switch */
	    }  /* of for loop  */
	}
	if (buf >= limit - 1)
	    buf = addstr("...]", buf - 4, limit, 0);
	else
	    buf = addstr("]", buf, limit, 0);
	return buf;
    } else {
	/* We have an array, or object. */
	buf = addstr(arrayinfo[obj_flags(h)].name, buf, limit, ']');
	buf = adddec(obj_length(h), buf, limit);
	buf = addstr("]", buf, limit, 0);
	return buf;
    }
}
	

HString *
java_lang_Object_toString(HObject *p)
{
    char        pbuf[256];
    return makeJavaString(pbuf, obj2string(p, pbuf, pbuf + sizeof(pbuf) - 1) - pbuf);
}


HClass *
java_lang_Object_getClass(HObject *p)
{
    if (p == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    } else if (obj_flags(p) == T_NORMAL_OBJECT) {
	return cbHandle(obj_classblock(p));
    } else {
	char buffer[256], *bp = buffer;
	ClassClass *fromClass;
	*bp++ = SIGNATURE_ARRAY;
	switch (obj_flags(p)) {
	    case T_INT:      *bp++ = SIGNATURE_INT; break;
	    case T_LONG:     *bp++ = SIGNATURE_LONG; break;
	    case T_FLOAT:    *bp++ = SIGNATURE_FLOAT; break;
	    case T_DOUBLE:   *bp++ = SIGNATURE_DOUBLE; break;
	    case T_BOOLEAN:  *bp++ = SIGNATURE_BOOLEAN; break;
	    case T_BYTE:     *bp++ = SIGNATURE_BYTE; break;
	    case T_CHAR:     *bp++ = SIGNATURE_CHAR; break;
	    case T_SHORT:    *bp++ = SIGNATURE_SHORT; break;
	    case T_ARRAY:	/* should go away */
	    case T_CLASS: {
		int length = obj_length(p);
		ArrayOfObject *array = (ArrayOfObject *)unhand(p);
		fromClass = (ClassClass *)(array->body[length]);
		if (fromClass->name[0] == SIGNATURE_ARRAY) {
		    strcpy(bp, fromClass->name);
		    bp += strlen(bp);
		} else {
		    *bp++ = SIGNATURE_CLASS;
		    strcpy(bp, fromClass->name);
		    bp += strlen(bp);
		    *bp++ = SIGNATURE_ENDCLASS;
		    break;
		}
	    }
	}
	*bp = '\0';
	return cbHandle(FindClassFromClass(0, buffer, FALSE, fromClass));
    }
}

