/*
 * @(#)objectscope.c	1.11 95/05/01 James Gosling
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*-
 *	Routines for examining other objects
 *	objectscope.c, Thu Sep 22 15:28:14 1994
 */

#include "oobj.h"
#include "tree.h"
#include "interpreter.h"
#include "javaString.h"
#include "typecodes.h"
#include "signature.h"

#include "java_lang_System.h"

/* defined in file_acl.c */
extern long classLoadersInStack();

HObject *
java_util_ObjectScope_getFields(HObject * this, HObject * o)
{
    ClassClass *cb;
    int nslots = 0;
    int i;
    HObject *ret;
    struct fieldblock **slots;

    if (classLoadersInStack()) {
	SignalError(0, JAVAPKG "IllegalAccessException","permission error");
	return 0;
    }

    if (o == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
    cb = obj_classblock(o);
    makeslottable(cb);
    nslots = cbSlotTableSize(cb);
    slots = cbSlotTable(cb);
    ret = ArrayAlloc(T_CLASS, nslots);
    if (ret == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return 0;
    }

    for (i = 0; i < nslots; i++)
	if (slots[i]) {
	    char *s = fieldname(slots[i]);
	    int len = 0;
	    OBJECT os;
	    while (s[len] != 0)
		len++;
	    os = (OBJECT) makeJavaString(s, len);
	    obj_setslot(ret, i, os);
	}
    return ret;
}

HObject *
java_util_ObjectScope_getMethods(HObject * this, HObject * o)
{
    ClassClass *cb;
    int nslots = 0;
    int i;
    HObject *ret;
    struct methodtable *mt;

    if (classLoadersInStack()) {
	SignalError(0, JAVAPKG "IllegalAccessException","permission error");
	return 0;
    }

    if (o == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
    cb = obj_classblock(o);
    mt = cbMethodTable(cb);
    nslots = cbMethodTableSize(cb);
    ret = ArrayAlloc(T_CLASS, nslots);
    if (ret == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return 0;
    }
    for (i = 0; i < nslots; i++) {
	struct methodblock *mb = mt_slot(mt, i);
	char *s = mb ? fieldname(&mb->fb) : "<empty>";
	OBJECT os = (OBJECT) makeJavaString(s, strlen(s));
	obj_setslot(ret, i, os);
    }
    return ret;
}

#define obj_getoffset(o, off) (*(OBJECT *)((char *)unhand(o)+(off)))

HObject *
java_util_ObjectScope_getObject(HObject * this, HObject * o, long slot)
{
    ClassClass *cb;
    struct fieldblock *fb, **slots;
    int nslots;

    if (classLoadersInStack()) {
	SignalError(0, JAVAPKG "IllegalAccessException","permission error");
	return 0;
    }

    if (o == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
    cb = obj_classblock(o);
    makeslottable(cb);
    nslots = cbSlotTableSize(cb);
    slots = cbSlotTable(cb);
    if (slot >= nslots || slot < 0 || (fb = slots[slot]) == 0
	    || fieldsig(fb)[0] != SIGNATURE_CLASS) {
	SignalError(0, "IllegalArgumentException", 0);
	return 0;
    }
    return (fb->access & ACC_STATIC
	    ? *(HObject **) normal_static_address(fb)
	    : (HObject *) obj_getoffset(o, fb->u.offset));
}

HObject *
java_util_ObjectScope_getValues(HObject * this, HObject * o)
{
    extern char *obj2string(Handle * h, char *buf, char *limit);
    ClassClass *cb;
    int nslots = 0;
    int i;
    HObject *ret;
    struct fieldblock **slots;

    if (classLoadersInStack()) {
	SignalError(0, JAVAPKG "IllegalAccessException","permission error");
	return 0;
    }

    if (o == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
    cb = obj_classblock(o);
    makeslottable(cb);
    nslots = cbSlotTableSize(cb);
    slots = cbSlotTable(cb);
    ret = ArrayAlloc(T_CLASS, nslots);
    if (ret == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return 0;
    }
    for (i = 0; i < nslots; i++) {
	char pbuf[256];
	struct fieldblock *fb = slots[i];
	OBJECT *v;
	char *s = pbuf;
	OBJECT os;
	if (fb == 0)
	    s = "<empty>";
	else {
	    int sig = fieldsig(fb)[0];
	    v = (fb->access & ACC_STATIC
		 ? (sig == SIGNATURE_DOUBLE || sig == SIGNATURE_LONG
		    ? twoword_static_address(fb)
		    : normal_static_address(fb))
		 : &obj_getoffset(o, fb->u.offset));
	    switch (sig) {
	      case SIGNATURE_ARRAY:
		s = *v ? arrayinfo[obj_flags((HObject *) *v)].name : "null";
		break;
	      case SIGNATURE_CHAR:
		sprintf(pbuf, *v >= ' ' && *v < 0177 ? "'%c'" : "%#X", *v);
		break;
	      case SIGNATURE_CLASS:
		s = *v ? "=>" : "null";
		break;
	      case SIGNATURE_FLOAT:
		sprintf(pbuf, "%g", *(float *) v);
		break;
	      case SIGNATURE_DOUBLE:
		sprintf(pbuf, "%G", GET_DOUBLE(v));
		break;
	      default:
		s = "bogus type signature";
		break;
	      case SIGNATURE_INT:
	      case SIGNATURE_BYTE:
	      case SIGNATURE_SHORT:
		sprintf(pbuf, "%d", *v);
		break;
	      case SIGNATURE_LONG:
		sprintf(pbuf, "long(%X,%X)", v[0], v[1]);
		break;
	      case SIGNATURE_VOID:
		s = "void";
		break;
	      case SIGNATURE_BOOLEAN:
		s = *v ? "true" : "false";
		break;
	    }
	}
	os = (OBJECT) makeJavaString(s, strlen(s));
	obj_setslot(ret, i, os);
    }
    return ret;
}
