/*
 * @(#)system.c	1.27 95/05/10  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Modification History:
 *
 * 22 Aug 1994	eugene
 *
 *	inserted a "machine independent" porting layer for machine
 *	specific non-posix lib and system calls.
 *	removed the open boot prom reading and writing code.  should
 *	use getenv and putenv for environment variable moshing.  no
 *	other reason to keep prom code around (we run hosted now!!).
 *
 */

#include <stdlib.h>
#include <fcntl.h>
#include <standardlib.h>

#include <oobj.h>
#include <tree.h>
#include <interpreter.h>
#include <javaString.h>
#include <typecodes.h>
#include <sys_api.h>

#include <java_lang_System.h>
#include <java_io_FileInputStream.h>

long 
java_lang_System_nowMillis(Hjava_lang_System *this)
{
    return(sysGetMilliTicks());
}

long 
java_lang_System_currentTime(Hjava_lang_System *this)
{
    return sysTime(0);
}


int64_t
java_lang_System_currentTimeMillis(Hjava_lang_System *this)
{
    return sysTimeMillis();
}

long
java_lang_System_freeMemory(Hjava_lang_System *this)
{
    extern int FreeMemory();
    return FreeMemory();
}

long 
java_lang_System_totalMemory(Hjava_lang_System *this)
{
    extern int TotalMemory();
    return TotalMemory();
}

void
java_lang_System_gc(Hjava_lang_System *this) {
    extern void gc(int, unsigned int);
    gc(0, 1);
}

void
java_lang_System_runFinalization(Hjava_lang_System *this) {
    extern void runFinalization();
    runFinalization();
}

void
java_lang_System_boolean_arraycopy(Hjava_lang_System *this, HArrayOfInt *srch, long src_pos,
			      HArrayOfInt *desth, long dest_pos, long length) 
{
    ArrayOfInt *srcptr, *destptr;

    if ((srch == 0) || (desth == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    if ((length < 0) || (src_pos < 0) || (dest_pos < 0) ||
	(length + src_pos > obj_length(srch)) ||
	(length + dest_pos > obj_length(desth))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }

    srcptr = unhand(srch);
    destptr = unhand(desth);
    memmove(&destptr->body[dest_pos], &srcptr->body[src_pos],
	    length * sizeof(destptr->body[0]));
}

void
java_lang_System_byte_arraycopy(Hjava_lang_System *this, HArrayOfByte *srch, long src_pos,
			      HArrayOfByte *desth, long dest_pos, long length) 
{
    ArrayOfByte *srcptr, *destptr;

    if ((srch == 0) || (desth == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    if ((length < 0) || (src_pos < 0) || (dest_pos < 0) ||
	(length + src_pos > obj_length(srch)) ||
	(length + dest_pos > obj_length(desth))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }

    srcptr = unhand(srch);
    destptr = unhand(desth);
    memmove(&destptr->body[dest_pos], &srcptr->body[src_pos],
	    length * sizeof(destptr->body[0]));
}

void
java_lang_System_short_arraycopy(Hjava_lang_System *this, HArrayOfShort *srch, long src_pos,
			      HArrayOfShort *desth, long dest_pos, long length) 
{
    ArrayOfShort *srcptr, *destptr;

    if ((srch == 0) || (desth == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    if ((length < 0) || (src_pos < 0) || (dest_pos < 0) ||
	(length + src_pos > obj_length(srch)) ||
	(length + dest_pos > obj_length(desth))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }

    srcptr = unhand(srch);
    destptr = unhand(desth);
    memmove(&destptr->body[dest_pos], &srcptr->body[src_pos],
	    length * sizeof(destptr->body[0]));
}

void
java_lang_System_char_arraycopy(Hjava_lang_System *this, HArrayOfChar *srch, long src_pos,
			      HArrayOfChar *desth, long dest_pos, long length) {
    ArrayOfChar *srcptr, *destptr;

    if ((srch == 0) || (desth == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    if ((length < 0) || (src_pos < 0) || (dest_pos < 0) ||
	(length + src_pos > obj_length(srch)) ||
	(length + dest_pos > obj_length(desth))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }

    srcptr = unhand(srch);
    destptr = unhand(desth);
    memmove(&destptr->body[dest_pos], &srcptr->body[src_pos],
	    length * sizeof(destptr->body[0]));
}

void
java_lang_System_int_arraycopy(Hjava_lang_System *this, HArrayOfInt *srch, long src_pos,
			     HArrayOfInt *desth, long dest_pos, long length) 
{
    ArrayOfInt *srcptr, *destptr;

    if ((srch == 0) || (desth == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    if ((length < 0) || (src_pos < 0) || (dest_pos < 0) ||
	(length + src_pos > obj_length(srch)) ||
	(length + dest_pos > obj_length(desth))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }

    srcptr = unhand(srch);
    destptr = unhand(desth);
    memmove(&destptr->body[dest_pos], &srcptr->body[src_pos],
	    length * sizeof(destptr->body[0]));
}

void
java_lang_System_long_arraycopy(Hjava_lang_System *this, HArrayOfLong *srch, long src_pos,
			     HArrayOfLong *desth, long dest_pos, long length)
{
    ArrayOfLong *srcptr, *destptr;

    if ((srch == 0) || (desth == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    if ((length < 0) || (src_pos < 0) || (dest_pos < 0) ||
	(length + src_pos > obj_length(srch)) ||
	(length + dest_pos > obj_length(desth))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }

    srcptr = unhand(srch);
    destptr = unhand(desth);
    memmove(&destptr->body[dest_pos], &srcptr->body[src_pos],
	    length * sizeof(destptr->body[0]));
}



void
java_lang_System_float_arraycopy(Hjava_lang_System *this, HArrayOfFloat *srch, long src_pos,
			       HArrayOfFloat *desth, long dest_pos, long length) {
    ArrayOfFloat *srcptr, *destptr;

    if ((srch == 0) || (desth == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    if ((length < 0) || (src_pos < 0) || (dest_pos < 0) ||
	(length + src_pos > obj_length(srch)) ||
	(length + dest_pos > obj_length(desth))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }

    srcptr = unhand(srch);
    destptr = unhand(desth);
    memmove(&destptr->body[dest_pos], &srcptr->body[src_pos],
	    length * sizeof(destptr->body[0]));
}

void
java_lang_System_double_arraycopy(Hjava_lang_System *this, HArrayOfDouble *srch, long src_pos,
			       HArrayOfDouble *desth, long dest_pos, long length)
{
    ArrayOfDouble *srcptr, *destptr;

    if ((srch == 0) || (desth == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    if ((length < 0) || (src_pos < 0) || (dest_pos < 0) ||
	(length + src_pos > obj_length(srch)) ||
	(length + dest_pos > obj_length(desth))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }

    srcptr = unhand(srch);
    destptr = unhand(desth);
    memmove(&destptr->body[dest_pos], &srcptr->body[src_pos],
	    length * sizeof(destptr->body[0]));
}

void
java_lang_System_object_arraycopy(Hjava_lang_System *this, HArrayOfObject *srch, long src_pos,
			        HArrayOfObject *desth, long dest_pos, long length) 
{
    ArrayOfObject *srcptr, *destptr;
    ClassClass *srcc, *destc;
    int srclen, destlen, i;

    if ((srch == 0) || (desth == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    srclen = obj_length(srch);
    destlen = obj_length(desth);

    if ((length < 0) || (src_pos < 0) || (dest_pos < 0) ||
	(length + src_pos > srclen) || (length + dest_pos > destlen)) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }

    srcptr = unhand(srch);
    destptr = unhand(desth);
    srcc = (ClassClass *)srcptr->body[srclen];
    destc = (ClassClass *)destptr->body[destlen];

    if ((srcc != destc) && (destc != classobject)) {
	/* if the array types are not identical we need to check each
 	 * non-null element. Note that you can't do the check first and
	 * then use memmove later. This would create an opportunity
	 * for another thread to modify the array.
	 */
	for (i = 0 ; i < length ; i++) {
	    HObject *elem = srcptr->body[src_pos + i];
	    if (elem && !is_instance_of(elem, destc, 0)) {
		SignalError(0, JAVAPKG "IncompatibleTypeException", 0);
		return;
	    }
	    destptr->body[dest_pos + i] = elem;
	}
    } else {
	memmove(&destptr->body[dest_pos], &srcptr->body[src_pos],
		length * sizeof(destptr->body[0]));
    }
}

void
java_lang_System_exit(Hjava_lang_System *this, long code)
{
    exit(code);
}

long
java_lang_System_exec0(Hjava_lang_System *this, HString *cmd, long DoRead) {
    extern long classLoadersInStack();
    int fd;
    char *s;
    FILE *stream;

    if (cmd == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
    if (classLoadersInStack()) {
	SignalError(0, JAVAPKG "IOException","exec permission error");
	return 0;
    }
    s = allocCString(cmd);

    /* printf("java_lang_System_exec:  about to popen '%s'\n", s); */
    stream = popen(s, DoRead > 0 ? "r" : "w");
    free(s);
    if (stream == NULL) {
	SignalError(0, JAVAPKG "FileNotFoundException", s);
        return 0;
    }

    /* FIXME: How do I know when to pclose() this??? */
    /* FIXME: Also, what should we do about stderr from the command? */
    /* FIXME: Shouldn't use popen: should invoke an _md routine that
       on unix does the fork & exec and returns pipes. */

    if (DoRead < 0) {
	fclose(stream);
	return 0;
    }
    fd = fileno(stream);
    /* Create, and return, a FileInputStream for fd. */
    return fd;
}
    
HString *
java_lang_System_getenv(Hjava_lang_System *this, HString *var) {
    char buf[256];
    char *val;

    if (var == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    javaString2CString(var, buf, sizeof(buf));
    val = getenv(buf);
    return val ? makeJavaString(val, strlen(val)) : 0;
}

HString *
java_lang_System_getCWD(Hjava_lang_System *this) {
    char buf[256];

    getcwd(buf, sizeof(buf));
    return makeJavaString(buf, strlen(buf));
}

#include "OSName_md.h"

HString *
java_lang_System_getOSName(Hjava_lang_System *this) {
    static char s[] = OSName_MD;
    return makeJavaString(s, sizeof s - 1);
}


void
java_lang_System_traceInstructions(Hjava_lang_System *this, long on) {
    extern void TraceInstructions(int);
    (void) TraceInstructions(on);
}

void
java_lang_System_traceMethodCalls(Hjava_lang_System *this, long on) {
    extern void TraceMethodCalls(int);
    (void) TraceMethodCalls(on);
}

void
java_lang_System_profile(Hjava_lang_System *this, long on) {
    extern void javamon(int);
    (void) javamon(on);
}

void
java_lang_System_dumpProfile(Hjava_lang_System *this) {
    extern void java_mon_dump();
    (void) java_mon_dump();
}

void 
java_lang_System_dumpMon(Hjava_lang_System *this, HObject *obj) {
    extern void java_mon_dump();
    (void) java_mon_dump();
}
