/*
 * @(#)thread.c	1.29 95/02/22  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*-
 *      Stuff for dealing with threads.
 *      originally in threadruntime.c, Sun Sep 22 12:09:39 1991
 */

#include <stdio.h>
#include <signal.h>

#include <tree.h>
#include <threads.h>

#include <monitor.h>
#include <threads.h>
#include <exceptions.h>
#include <sys_api.h>

#include <java_lang_Object.h>
#include <java_lang_Thread.h>

static void    /* first routine called in the new thread */
ThreadRT0(register Hjava_lang_Thread *p)
{
    ExecEnv ee;

#ifndef trace
    if (trace) {
	fprintf(stderr, "Started thread %X\n", p);
    }
#endif

    InitializeExecEnv(&ee, (Handle *) p);
    threadInit(p, (stackp_t) &ee);
    execute_java_dynamic_method(&ee, (void *)p, "run", "()V");
    if (exceptionOccurred(&ee)) {
	exceptionDescribe(&ee);
    }

    /* Falling off bottom of thread... */
    threadExit();
}

void
java_lang_Thread_start(register Hjava_lang_Thread *p) {
    struct Classjava_lang_Thread *tid;

    monitorEnter(obj_monitor(p));

    tid = (struct Classjava_lang_Thread *) unhand(p);

    /* It is illegal to call Start on an active thread */
    if (tid->PrivateInfo) {
	SignalError(0, JAVAPKG "IllegalStateException", 0);
    } else {
	int flag = (tid->daemon != 0) ? THR_SYSTEM : THR_USER;

	if (threadCreate(p, flag, ProcStackSize, (void *(*)(void *)) ThreadRT0)) {
	    SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	} else {
	    /* Set a default priority: see comments on {s,g}etPriority() */
	    threadSetPriority(p, tid->priority);
	    threadResume(p);
	}
    }

    monitorExit(obj_monitor(p));
}

void
java_lang_Thread_stop(Hjava_lang_Thread *p, struct Hjava_lang_Object *exc) {
    struct Classjava_lang_Thread *tid;

    if (exc == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    tid = (struct Classjava_lang_Thread *) unhand(p);
    if (!tid->PrivateInfo) { 
	SignalError(0, JAVAPKG "IllegalStateException", 0);
	return;
    }
    if (!threadPostException(p, exc)) {
	/* It is illegal to call Stop() on a dead thread */
	if (!tid->PrivateInfo) { 
	    SignalError(0, JAVAPKG "IllegalStateException", 0);
	    return;
	}
	threadKill(p);
    }
}

void
java_lang_Thread_postException(Hjava_lang_Thread *p, struct Hjava_lang_Object *exc) {
    struct Classjava_lang_Thread *tid;

    if (exc == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    tid = (struct Classjava_lang_Thread *) unhand(p);
    if (!tid->PrivateInfo) { 
	SignalError(0, JAVAPKG "IllegalStateException", 0);
	return;
    }
    threadPostException(p, exc);
}

/*
 * Note that we don't go to the native thread to determine whether the Java
 * thread is alive.  The status of the native thread doesn't matter, to the
 * first approximation at least, if the Java thread object isn't set up
 * properly.  However, we could also double-check the liveness of the native
 * thread.
 */
long
java_lang_Thread_isAlive(Hjava_lang_Thread *p)
{
    struct Classjava_lang_Thread *tid;
    tid = (struct Classjava_lang_Thread *) unhand(p);
    return ((long)tid->PrivateInfo);
}

void
java_lang_Thread_suspend(Hjava_lang_Thread *p) {
    struct Classjava_lang_Thread *tid;

    tid = (struct Classjava_lang_Thread *) unhand(p);
    if (tid->PrivateInfo) {
	(void) threadSuspend(p);
    } else {
	SignalError(0, JAVAPKG "IllegalStateException", 0);
    }
}

void
java_lang_Thread_resume(Hjava_lang_Thread *p)
{
    struct Classjava_lang_Thread *tid;

    tid = (struct Classjava_lang_Thread *) unhand(p);
    if (tid->PrivateInfo) {
	(void) threadResume(p);
    } else {
	SignalError(0, JAVAPKG "IllegalStateException", 0);
    }
}

/* We want to rely on priorities as set/got via the threadX functions.
 * However, it is legal to call setPriority() and getPriority() after
 * 'new' and before start() is called, in which case the thread's TID has
 * been allocated but its thread not yet created.  So we also maintain
 * tid->priority to bridge the gap.
 *
 * The priority should already have been check to fall within a
 * valid range by the software above.  All we do is pass it on to
 * the system level.  The JAVA level also set the priority field.
 */
void
java_lang_Thread_setThreadPriority(register Hjava_lang_Thread *p, long prio) {
    struct Classjava_lang_Thread *tid;

    tid = (struct Classjava_lang_Thread *) unhand(p);
    if (tid->PrivateInfo) {
	threadSetPriority(p, prio);
    } 
}

long
java_lang_Thread_getPriority(register Hjava_lang_Thread *p)
{
    long prio;
    struct Classjava_lang_Thread *tid;
    tid = (struct Classjava_lang_Thread *) unhand(p);
    if (tid->PrivateInfo) {
	/* When it's been set, rely on the thread's actual priority */
        threadGetPriority(p, (int *) &prio);
    } else {
	prio = tid->priority;
    }
    return prio;
}

void
java_lang_Thread_yield(Hjava_lang_Thread *p)
{
    threadYield();
}

void
java_lang_Thread_sleep(Hjava_lang_Thread *p, long millis)
{
    if (millis < 5) {
	threadYield();
	return;
    }

    threadSleep(millis);
    return;
}

Hjava_lang_Thread *
java_lang_Thread_currentThread(Hjava_lang_Thread *p)
{
    return threadSelf();
}

/* This doesn't count methods implemented in C. */
long
java_lang_Thread_countStackFrames(Hjava_lang_Thread *hp)
{
    if (hp == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return -1;
    } else {
	Classjava_lang_Thread *p = unhand(hp);
	ExecEnv *this_ee = (ExecEnv *) p->eetop;
	JavaFrame *frame = this_ee->current_frame;
	int count = 0;
	for (count = 0; frame != 0; frame = frame->prev, count++);
	return count;
    }
}

void
java_lang_Thread_StartSingleStepping(Hjava_lang_Thread *hp, ExecEnv *ee) 
{
    Classjava_lang_Thread *p = unhand(hp);
    p->single_step = 1;
}

void
java_lang_Thread_StopSingleStepping(Hjava_lang_Thread *hp, ExecEnv *ee) 
{
    Classjava_lang_Thread *p = unhand(hp);
    p->single_step = 0;
}

long
java_lang_Thread_activeCount(struct Hjava_lang_Thread *hThread)
{
    return (long)threadCount();
}

long
java_lang_Thread_enumerate(struct Hjava_lang_Thread *hThread, HArrayOfObject *htarray)
{
    TID *tarray;
    int maxlen;

    if (htarray == NULL) {
	SignalError(0, JAVAPKG "NullPointerException", "You must supply a valid array foe me to fill.");
	return 0;
    }
    maxlen = obj_length(htarray);
    tarray = (TID*) unhand(htarray)->body;
    return (long) threadEnumerate(tarray, maxlen);
}

