/*
 * @(#)classruntime.c	1.36 95/04/06  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*-
 *      Runtime routines to support classes.
 */

#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <stdlib.h>

#include <oobj.h>
#include <tree.h>
#include <interpreter.h>
#include <bool.h>
#include <timeval.h>
#include <threads.h>
#include <monitor.h>
#include <exceptions.h>
#include <javaString.h>
#include <path.h>

#include <sys_api.h>

#include <java_lang_Exception.h>
#include <java_lang_Thread.h>

bool_t 
invokeCompiledMethod(Handle *o, struct methodblock *mb, 
		     int args_size, ExecEnv *ee);

#ifdef DEBUG
void
VerifyClasses()
{
    static      lastn;
    int         i;
    assert(nbinclasses >= lastn);
    lastn = nbinclasses;
    assert(nbinclasses <= sizebinclasses);
    assert(binclasses != 0 || nbinclasses == 0);
    for (i = 0; i < nbinclasses; i++) {
	ClassClass *clb = binclasses[i];
	assert(clb);
	assert(((int) clb & 3) == 0);
	assert(clb->fields_count < 1000);
	assert(clb->methods_count < 1000);
	assert(cbInstanceSize(clb) && cbInstanceSize(clb) < 4000);
	assert(classname(clb)[0]);
    }
}
#endif

/* These are all flavors of method invocation depending on the type
   of method (defined by its access flags) being invoked.  There
   are 4 possibilities: java method, synchronized java method,
   native (C) method, synchronized native method.  Each methodblock
   has a pointer to the appropriate one for its access, and each
   one of these procedures optimizes its particular case.

   They don't all use all they parameters, which is unfortunate. */


bool_t
invokeJavaMethod(Handle *o, struct methodblock *mb, int args_size, ExecEnv *ee)
{
    int         nlocals = mb->nlocals; /* maximum number of locals */
    int         maxstack = mb->maxstack; /* maximum stack usage */
    JavaFrame   *old_frame = ee->current_frame; /* previous frame */
    stack_item *optop = old_frame->optop; /* Points to just before args */
    JavaStack   *javastack = old_frame->javastack; /* Current stack pointer */
    JavaFrame   *frame = (JavaFrame *)(optop + nlocals); /* tentative  */

    if (trace) {
	char where_am_i[100];
	pc2string(mb->code, mb, where_am_i, where_am_i + sizeof where_am_i);
	fprintf(stdout, "Entering %s\n", where_am_i);
	fflush(stdout);
    }
    if (frame->ostack + mb->maxstack > javastack->end_data) {
	stack_item *copied_optop;
	if (javastack->stack_so_far >= JavaStackSize) {
	    SignalError(ee, JAVAPKG "StackOverflowException", 0);
	    return FALSE;
	}
	javastack = javastack->next 
	        ? javastack->next : CreateNewJavaStack(ee, javastack);
	frame = (JavaFrame *)(javastack->data + nlocals);
	for (copied_optop = javastack->data; --args_size >= 0 ; ) 
	    copied_optop[args_size] = optop[args_size];
	optop = copied_optop;
    }
    frame->vars = optop;
    frame->javastack = javastack;
    frame->prev = old_frame;
    frame->optop = frame->ostack;
    frame->current_method = mb; 
    frame->constant_pool = cbConstantPool(fieldclass(&mb->fb));
    frame->returnpc = mb->code;
    frame->monitor = 0;
    ee->current_frame = frame;
    if (java_monitor)
	  frame->mon_starttime = now();
    return TRUE;
}


bool_t
invokeSynchronizedJavaMethod(Handle *o, struct methodblock *mb, int args_size,
			    ExecEnv *ee)
{
    int         nlocals = mb->nlocals; /* maximum number of locals */
    int         maxstack = mb->maxstack; /* maximum stack usage */
    JavaFrame   *old_frame = ee->current_frame; /* previous frame */
    stack_item *optop = old_frame->optop; /* Points to just before args */
    JavaStack   *javastack = old_frame->javastack; /* Current stack pointer */
    JavaFrame   *frame = (JavaFrame *)(optop + nlocals); /* tentative  */

    if (trace) {
	char where_am_i[100];
	pc2string(mb->code, mb, where_am_i, where_am_i + sizeof where_am_i);
	fprintf(stdout, "Entering %s\n", where_am_i);
	fflush(stdout);
    }
    if (frame->ostack + mb->maxstack > javastack->end_data) {
	stack_item *copied_optop;
	if (javastack->stack_so_far >= JavaStackSize) {
	    SignalError(ee, JAVAPKG "StackOverflowException", 0);
	    return FALSE;
	}
	javastack = javastack->next 
	             ? javastack->next : CreateNewJavaStack(ee, javastack);
	frame = (JavaFrame *)(javastack->data + nlocals);
	for (copied_optop = javastack->data; --args_size >= 0 ; ) 
	    copied_optop[args_size] = optop[args_size];
	optop = copied_optop;
    }
    frame->javastack = javastack;
    frame->prev = old_frame;
    frame->vars = optop;
    frame->optop = frame->ostack;
    frame->current_method = mb; 
    frame->constant_pool = cbConstantPool(fieldclass(&mb->fb));
    frame->returnpc = mb->code;
    frame->monitor = o;
    ee->current_frame = frame;
    monitorEnter(obj_monitor(o));
    if (java_monitor)
	  frame->mon_starttime = now();
    return TRUE;
}

bool_t
invokeNativeMethod(Handle *o, struct methodblock *mb, int args_size, 
		   ExecEnv *ee)
{
    JavaFrame *old_frame = ee->current_frame;
    stack_item *optop = old_frame->optop;
    if (java_monitor) {
	long start_time, end_time;
	start_time = now();
	optop = (*(stack_item * (*) ()) mb->code) (optop, ee);
	end_time = now();
	java_mon(old_frame->current_method, mb, end_time - start_time);
	/* java_mon(fieldclass(&mb->fb), mb, ee, 0, end_time - start_time); */
    } else {
	optop = (*(stack_item * (*) ()) mb->code) (optop, ee);
    }
    old_frame->optop = optop;
    return !(exceptionOccurred(ee));
}

bool_t
invokeSynchronizedNativeMethod(Handle *o, struct methodblock *mb, int args_size,
			       ExecEnv *ee)
{
    JavaFrame *old_frame = ee->current_frame;
    stack_item *optop = old_frame->optop;
    monitorEnter(obj_monitor(o));
    if (java_monitor) {
	long start_time, end_time;
	start_time = now();
	optop = (*(stack_item * (*) ()) mb->code) (optop, ee);
	end_time = now();
	java_mon(old_frame->current_method, mb, end_time - start_time);
	/* java_mon(fieldclass(&mb->fb), mb, ee, 0, end_time - start_time); */
    } else {
	optop = (*(stack_item * (*) ()) mb->code) (optop, ee);
    }
    old_frame->optop = optop;
    monitorExit(obj_monitor(o));
    return !(exceptionOccurred(ee));
}

bool_t
invokeLazyNativeMethod(Handle *o, struct methodblock *mb, int args_size, 
		       ExecEnv *ee)
{     
    if ((mb->code != 0) || dynoLink(mb)) {
	mb->invoker = (mb->fb.access & ACC_SYNCHRONIZED) 
	             ?  invokeSynchronizedNativeMethod : invokeNativeMethod;

	return (mb->invoker)(o, mb, args_size, ee);
    } else {
	SignalError(ee, JAVAPKG "UnsatisfiedLinkException", fieldname(&mb->fb));
	return FALSE;
    }
}

bool_t
invokeAbstractMethod(Handle *o, struct methodblock *mb, int args_size, 
		     ExecEnv *ee)
{     
    SignalError(ee, JAVAPKG "AbstractMethodException", 
		fieldname(&mb->fb));
    return FALSE;
}



bool_t
dynoLink(struct methodblock *mb)
{
    int    i;
    char sym[256];
    char *symp, *ptr;
    ClassClass *cb = fieldclass(&mb->fb);
 
    symp = sym;
#ifndef SOLARIS2
    *symp++ = '_';
#endif
 
    /* Copy the classname over, but convert any slashes to underbars */
    for (ptr = classname(cb); ; ptr++) {
        char c = *ptr;
        if (c == '\0')
            break;
        else {
            if (c == '/') c = '_';
            *symp++ = c;
        }
    }
 
    /* Copy over an underscore as a separator */
    *symp++ = '_';
 
    /* Copy over the methodname, up until the SIGNATURE_FUNC */
    for (ptr = fieldname(&mb->fb); ; ptr++) {
        char c = *ptr;
        if (c == '\0' || c == SIGNATURE_FUNC)
            break;
        else {
            *symp++ = c;
        }     
    }
 
    /* And then add "_stub" */
    strcpy(symp, "_stub");
    mb->code = (unsigned char *)sysDynamicLink(sym);
    return (mb->code != 0);
}



/*  I put this into a separate function so that other parts of the
    interpreter can use it without knowing how an object is actually
    created. - csw
*/
HObject *
newobject(ClassClass *cb, unsigned char *pc, struct execenv *ee)
{
    HObject *ret;
    ClassObject *new_obj;

    if ((ret = ObjAlloc(cb, 0)) == 0) {
	char buf[128];
	ee->current_frame->lastpc = pc;
	SignalError(ee, JAVAPKG "OutOfMemoryException", classname2string(classname(cb), buf, sizeof(buf)));
	return 0;
    }
    new_obj = unhand(ret);
    memcpy((char *) new_obj, (char *) unhand(cbPrototype(cb)), 
	   cbInstanceSize(cb));
    return ret;
}


void
RunStaticInitializers(ClassClass *cb)
{
    struct methodblock *mb;
    int size;
    char *s;
    struct methodblock *StaticInitializer = 0;

    mb = cbMethods(cb);
    for (size = 0; size < (int) cb->methods_count; size++, mb++) {
	assert(mb->fb.u.offset < cbMethodTableSize(cb));
	s = fieldname(&mb->fb);
	if (s[0] == '<' && strcmp(s, "<clinit>") == 0 
	        && strcmp(fieldsig(&mb->fb), "()V") == 0)
	    StaticInitializer = mb;
    }
    if (StaticInitializer && !InhibitExecute) {
	struct execenv *ee = EE();
#ifndef TRIMMED
	if (verbose)
	    fprintf(stderr, "[Running static initializer for %s]\n", 
		    classname(cb));
#endif
	do_execute_java_method(ee, cb, 0, 0, StaticInitializer, TRUE);
	if (ee && exceptionOccurred(ee)) {
	    exceptionDescribe(ee);
	}
    }
}

void
InitializeInvoker(ClassClass *cb)
{
    struct methodblock *mb;
    int size;

    mb = cbMethods(cb);
    for (size = 0; size < (int) cb->methods_count; size++, mb++) {
	unsigned int access = mb->fb.access;
	assert(mb->fb.u.offset < cbMethodTableSize(cb));
	if (access & ACC_MACHINE_COMPILED) { 
	    mb->invoker = invokeCompiledMethod;
	} else if ((access & (ACC_ABSTRACT | ACC_NATIVE)) == 0) {
	    /* Normal java method */
	    mb->invoker = (access & ACC_SYNCHRONIZED) 
		? invokeSynchronizedJavaMethod
		: invokeJavaMethod;
	} else if (access & ACC_ABSTRACT) {
	    mb->invoker = invokeAbstractMethod;
	} else { 
	    mb->invoker = invokeLazyNativeMethod;
	}
    }
}

void
SignalError(struct execenv * ee, char *ename, char *DetailMessage)
{
    ClassClass          *cb;
    ClassObject         *new_obj;
    Handle              *ret;
    JavaFrame            *frame;
    char where[100];
    int n;

    if (ee == 0)
        ee = EE();

    /*
     * Find the exception class (or the default).  If we can't find it
     * throw a preallocated object
     */
    if ((cb = FindClassFromClass(ee, ename, TRUE, NULL)) == 0 &&
	(cb = FindClassFromClass(ee, JAVAPKG "UnknownException", TRUE, NULL)) == 0) {
#ifndef TRIMMED
	pc2string(ee->current_frame->lastpc, 
		  ee->current_frame->current_method, 
		  where, where + sizeof where);
	fprintf(stderr, "Class missing for error: %s at %s\n", ename, where);
#endif
	exceptionThrow(ee, exceptionInternalObject(IEXC_NoClassDefinitionFound));
	return;
    }

    /*
     * Instantiate the object.  If we can't then throw a preallocated
     * object.
     */
    if ((ret = ObjAlloc(cb, 0)) == 0) {
	exceptionThrow(ee, exceptionInternalObject(IEXC_OutOfMemory));
	return;
    }
    new_obj = unhand(ret);
    memcpy((char *) new_obj, (char *) unhand(cbPrototype(cb)), 
	   cbInstanceSize(cb));

    /* Backtrace through the interpreter frames. */
    frame = ee ? ee->current_frame : 0;
    if (frame == 0) {
#ifndef TRIMMED
	fprintf(stderr, "Exception: %s (can't backtrace because of a missing "
		"context)\n", ename);
#endif
	return;	
    }
    if (trace) {
	printf("%6X %8X\tERROR %s\n", threadSelf(), frame->lastpc, ename);
	fflush(stdout);
    }
    assert(cbInstanceSize(cb) >= 7 * sizeof(void *));
    if (DetailMessage)
	uobj_setslot(new_obj, java_lang_Exception_TRACE_BACK_SIZE,
		     (OBJECT) makeJavaString(DetailMessage,
					 strlen(DetailMessage)));

    for (n = 0;
	   (frame != 0) && (n < java_lang_Exception_TRACE_BACK_SIZE);
	   frame = frame->prev) {
	if (frame->current_method) {
	    uobj_setslot(new_obj, n, (OBJECT)frame->lastpc);
	    n++;
	}
    }
    /* This must be the last thing, since makeJavaString() may clear the
     * signal Error slot. 
     */
    exceptionThrow(ee, ret);
}


char       *
addhex(long n, char *buf, char *limit)
{
    int         i;
    for (i = 32; (i -= 4) >= 0 && buf < limit;)
	*buf++ = "0123456789ABCDEF"[(n >> i) & 0xF];
    return buf;
}

char       *
adddec(long n, char *buf, char *limit)
{
    if (n < 0) {
	n = -n;
	if (buf < limit)
	    *buf++ = '-';
    }
    if (n >= 10)
	buf = adddec(n / 10, buf, limit);
    if (buf < limit)
	*buf++ = (n % 10) + '0';
    return buf;
}


int
pc2lineno(struct methodblock *mb, unsigned int pc_offset)
{
    int length = mb->line_number_table_length;

    if (length > 0) {
	struct lineno *ln = &mb->line_number_table[length];
	int i;
	for (i = mb->line_number_table_length; --i >= 0; )
	    if (pc_offset >= (--ln)->pc)
	      return ln->line_number;
    }
    return -1;
}

static struct methodblock *pc2method(unsigned char *pc) {
    struct methodblock *mb;
    ClassClass *cb;
    int i, j;

    for (i = nbinclasses; --i >= 0; ) {
	cb = binclasses[i];
	for (mb = cbMethods(cb), j = cb->methods_count; --j >= 0; mb++) {
	    if (!(mb->fb.access & ACC_NATIVE)
		  && pc >= (unsigned char *) mb->code 
		  && pc < ((unsigned char *) mb->code + mb->code_length)) {
		return mb;
	    }
	}
    }
    return 0;
}

char *
pc2string(unsigned char *pc, struct methodblock *mb, char *buf, char *limit)
{
    int         i, fbslot;

    if (buf >= limit)
	return buf;
    if (mb == 0) {
        mb = pc2method(pc);
    }
    if (mb != 0) {
	char cbuf[128];
	ClassClass *cb = fieldclass(&mb->fb);
	buf = addstr(classname2string(classname(cb), cbuf, sizeof(cbuf)), buf, limit, 0);
	buf = addstr(".", buf, limit, 0);
	buf = addstr(fieldname(&mb->fb), buf, limit, '(');
	if (cb != 0) {
	    char        *fn;
	    int         lineno;
	    if (cb->source_name)
	        fn = strrchr(classsrcname(cb), '/');
	    else
	        fn = 0;
	    buf = addstr("(", buf, limit, 0);
	    buf = addstr(fn ? fn + 1 : classsrcname(cb), buf, limit, 0);
	    lineno = pc2lineno(mb, pc - mb->code);
	    if (lineno >= 0) {
		buf = addstr(":", buf, limit, 0);
		buf = adddec(lineno, buf, limit);
	    }
	    buf = addstr(")", buf, limit, 0);
	}
	*buf = 0;
    } else {
	buf[0] = '\0';
    }
    return buf;
}

/*
   Make an Java unicode string out of a series of bytes that are
   already encoded as unicode.
*/
HArrayOfChar *
MakeUniString(unicode  *str, long len)
{
    ArrayOfChar *new_str;
    Handle *ret;
    unicode *tmp;

    assert (str != 0);
    for (tmp = str; *tmp; tmp++)
	;
    if (len > tmp-str)
	len = (int) (tmp-str);
    ret = ArrayAlloc(T_CHAR, len);
    if (ret == 0)
	return 0;
    new_str = (ArrayOfChar *) unhand(ret);
    memmove((char *) new_str->body, (char *) str, len * sizeof(unicode));
    return (HArrayOfChar *)ret;
}


/*
    Make an Java unicode string from a non-unicode C-style string.
*/
HArrayOfChar *
MakeString(char *str, long len)
{
    ArrayOfChar *new_str;
    HArrayOfChar  *ret;

    assert(str != 0);
    ret = (HArrayOfChar *) ArrayAlloc(T_CHAR, len);
    if (ret == 0)
	return 0;
    new_str = unhand(ret);
    str2unicode(str, new_str->body, len);
    return ret;
}

/*
    Make an Java array of bytes
*/
HArrayOfByte *
MakeByteString(char *str, long len)
{
    ArrayOfByte *new_str;
    Handle *ret;

    ret = ArrayAlloc(T_BYTE, len);
    if (ret == 0)
	return 0;
    new_str = (ArrayOfByte *) unhand(ret);
    if (str)
	memmove((char *) new_str->body, (char *)str, len);
    else
	memset((char *) new_str->body, 0, len);

    return (HArrayOfByte *)ret;
}


ExecEnv *
EE()
{
    ExecEnv *ee;
    if (threadSelf() == 0)
	ee = DefaultExecEnv;
    else
	ee = (ExecEnv *) THREAD(threadSelf())->eetop;
    assert(ee != 0);
    return ee;
}
