/*
 * @(#)counter.c	1.9 95/01/31  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Differential counter implementation
 *
 * Differential counters are just single value counters that are
 * implemented by maintaining two different counters.  One is used
 * for incrementing, and the other for decrementing.  This removes
 * the need to lock the counter completely whenever two threads
 * want to manipulate it.
 *
 * Only nonlocking counters are needed in shared code.  Anything
 * else is below the HPI.
 */

#include <counter.h>

/*
 * Initialize the counter.
 */
counter_t *
counterInit(void *mem, int initial_value)
{
    counter_t *c;

    c = (counter_t *) mem;
    c->inc = initial_value;
    c->dec = 0;
    return c;
}

/*
 * Return the value of the counter.  No need to lock.
 */
int
counterValue(counter_t *c)
{
    return (c->inc - c->dec);
}


/*
 *  Asynchronous Interface
 */

/*
 * NOTE: there is no locking here.  The assumption is that users
 * of this interface will, by convention, serialize their access.
 */

/*
 * Increment the counter.
 */
void
counterIncrement(counter_t *c, unsigned int val)
{
    c->inc += val;
}

/*
 * Decrement the counter.
 */
void
counterDecrement(counter_t *c, unsigned int val)
{
    c->dec += val;
}
