/*
 * @(#)exceptions.c	1.20 95/04/03 Peter King, Bruce Martin
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * The Java runtime exception handling mechanism.
 */


/*
 * Header files.
 */

#include <exceptions.h>

#include <java_lang_Exception.h>
#include <java_lang_NoClassDefFoundException.h>
#include <java_lang_OutOfMemoryException.h>
#include <java_lang_Object.h>


/*
 * Static data.
 */

/* Internal exception objects */
static Classjava_lang_NoClassDefFoundException noClassDefFound;
static Classjava_lang_OutOfMemoryException outOfMemory;

/* The classes the internal exception objects belong to */
static char *internalExceptionClasses[IEXC_END] = {
    "",						/* IEXC_NoException */
    JAVAPKG "NoClassDefFoundException",
    JAVAPKG "OutOfMemoryException"
};

/* An array of handles to the internal exception objects */
static Handle internalExceptions[IEXC_END] = {
    { NULL, NULL },
    { (ClassObject *) &noClassDefFound, NULL },
    { (ClassObject *) &outOfMemory, NULL }
};

/*
 * Internal routines.
 */

/*
 * exceptionInitInternalObjects() -- Initialize the exception subsystem's
 *	internal objects.
 */
void
exceptionInit(void) {
    internal_exception_t	ie;
    ClassClass			*cb;
    ClassObject			*object;
    Handle			*handle;

    /* Initialize the internal objects */
    for (ie = IEXC_NONE + 1; ie < IEXC_END; ie++) {

	/* Look up class */
	if (cb = FindClassFromClass(NULL, internalExceptionClasses[ie], TRUE, NULL)) {
	    
	    handle = &internalExceptions[ie];
	    handle->methods = cbMethodTable(cb);
	    
	    /* Initialize the object */
	    object = unhand(handle);
	    memcpy((char *) object, (char *) unhand(cbPrototype(cb)),
		   cbInstanceSize(cb));
	}
    }
}

/*
 * exceptionInternalObject() -- Return an internal, preallocated
 *	exception object.  These are shared by all threads, so they
 *	should only be used in a last ditch effort.
 */
exception_t	    
exceptionInternalObject(internal_exception_t exc)
{
    if (exc <= IEXC_NONE || exc >= IEXC_END ||
	internalExceptions[exc].methods == NULL) {
	return NULL;
    }
    return &internalExceptions[exc];
}

/*
 * exceptionDescribe() -- Print out a description of a given exception
 *	object.
 */
void
exceptionDescribe(ExecEnv *ee)
{
    exception_t exc = ee->exception.exc;
    extern void DumpMonitors();

    if (is_instance_of(exc, FindClass(ee, JAVAPKG "ThreadDeath", FALSE), ee)) {
	/* Don't print anything if we are being killed. */
    } else {
	prints("Exception ");
	if (threadSelf()) {
	    prints("in thread ");
	    prints(Object2CString((Handle *)getThreadName()));
	}

	if (is_instance_of(exc, 
			   FindClass(ee, JAVAPKG "Exception", FALSE), ee)) {
	    prints(" ");
	    /* Object is an exception. */
	    execute_java_dynamic_method(ee, exc, "printStackTrace", "()V");
	} else {
	    char buf[128];
	    ClassClass *class = unhand(java_lang_Object_getClass(exc));
	    
	    printf(".  Uncaught exception of type %s\n", 
		   classname2string(classname(class), buf, sizeof(buf)));
	}
#ifndef TRIMMED
	fflush(stdout);
#endif
    }

}




