/*
 * @(#)verify_code.c	1.11 95/01/31 Arthur van Hoff
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <stdio.h>
#include <stdarg.h>
#include <oobj.h>
#include <tree.h>
#include <signature.h>
#include <opcodes.h>
#include <verify.h>

/*
 * Compute argument size
 */
static int compute_arg_size(char *sig) 
{
    int n = 1;
    while (*sig++ != SIGNATURE_FUNC);
    while (1) {
	switch (*sig++) {
	  case SIGNATURE_ENDFUNC:
	    return n;

	  case SIGNATURE_CLASS:
	    while ((*sig != SIGNATURE_ENDCLASS) && (*sig != '\0')) sig++;
	    sig++;
	    n += 1;
	    break;

	  case SIGNATURE_ARRAY:
	    while (*sig == SIGNATURE_ARRAY) sig++;
	    if (*sig++ == SIGNATURE_CLASS) {
		while (*sig++ != SIGNATURE_ENDCLASS);
	    }
	    n += 1;
	    break;

	  case SIGNATURE_BOOLEAN:
	  case SIGNATURE_BYTE:
	  case SIGNATURE_USHORT:
	  case SIGNATURE_CHAR:
	  case SIGNATURE_INT:
	  case SIGNATURE_FLOAT:
	    n += 1;
	    break;

	  case SIGNATURE_LONG:
	  case SIGNATURE_DOUBLE:
	    n += 2;
	    break;
	}
    }
}

#define F_OPCODE	0x1
#define F_LABEL		0x2

/**
 * check bytecodes
 */
static int check_code(int nconst, uchar **cpool, int sig, uchar *code, int codelen, int maxvar) 
{
    int pc, pc2, i, n, m;
    unsigned short *info;

    info = (unsigned short *)malloc(sizeof(unsigned short) * (codelen + 1));
    if (info == NULL) {
	verify_error("can't allocated code info");
	free(info);
	return 0;
    }
    memset(info, 0, sizeof(unsigned short) * (codelen + 1));

    for (pc = 0 ; pc < codelen ; ) {
	info[pc] |= F_OPCODE;

	switch (code[pc]) {
	  case opc_aconst_null:
	  case opc_iconst_m1:
	  case opc_iconst_0:
	  case opc_iconst_1:
	  case opc_iconst_2:
	  case opc_iconst_3:
	  case opc_iconst_4:
	  case opc_iconst_5:
	  case opc_lconst_0:
	  case opc_lconst_1:
	  case opc_fconst_0:
	  case opc_fconst_1:
	  case opc_fconst_2:
	  case opc_dconst_0:
	  case opc_dconst_1:
	  case opc_arraylength:
	  case opc_iaload:
	  case opc_laload:
	  case opc_faload:
	  case opc_daload:
	  case opc_aaload:
	  case opc_saload:
	  case opc_siaload:
	  case opc_iastore:
	  case opc_lastore:
	  case opc_fastore:
	  case opc_dastore:
	  case opc_aastore:
	  case opc_sastore:
	  case opc_siastore:
	  case opc_nop:
	  case opc_pop:
	  case opc_pop2:
	  case opc_dup:
	  case opc_dup2:
	  case opc_dup_x1:
	  case opc_dup2_x1:
	  case opc_dup_x2:
	  case opc_dup2_x2:
	  case opc_swap:
	  case opc_iadd:
	  case opc_ladd:
	  case opc_fadd:
	  case opc_dadd:
	  case opc_isub:
	  case opc_lsub:
	  case opc_fsub:
	  case opc_dsub:
	  case opc_imul:
	  case opc_lmul:
	  case opc_fmul:
	  case opc_dmul:
	  case opc_idiv:
	  case opc_ldiv:
	  case opc_fdiv:
	  case opc_ddiv:
	  case opc_imod:
	  case opc_lmod:
	  case opc_fmod:
	  case opc_dmod:
	  case opc_ineg:
	  case opc_lneg:
	  case opc_fneg:
	  case opc_dneg:
	  case opc_ishl:
	  case opc_ishr:
	  case opc_iushr:
	  case opc_lshl:
	  case opc_lshr:
	  case opc_lushr:
	  case opc_iand:
	  case opc_land:
	  case opc_ior:
	  case opc_lor:
	  case opc_ixor:
	  case opc_lxor:
	  case opc_i2l:
	  case opc_i2f:
	  case opc_i2d:
	  case opc_l2i:
	  case opc_l2f:
	  case opc_l2d:
	  case opc_f2i:
	  case opc_f2l:
	  case opc_f2d:
	  case opc_d2i:
	  case opc_d2l:
	  case opc_d2f:
	  case opc_int2byte:
	  case opc_int2char:
	  case opc_int2short:
	  case opc_lcmp:
	  case opc_fcmpl:
	  case opc_fcmpg:
	  case opc_dcmpl:
	  case opc_dcmpg:
	  case opc_ireturn:
	  case opc_lreturn:
	  case opc_freturn:
	  case opc_dreturn:
	  case opc_areturn:
	  case opc_return:
	  case opc_athrow:
	  case opc_monitorenter:
	  case opc_monitorexit:
	  case opc_verifystack:
	    pc += 1;
	    break;

	  case opc_bipush:
	  case opc_ret:
	    pc += 2;
	    break;

	  case opc_sipush:
	    pc += 3;
	    break;

	  case opc_ldc1:
	    n = get_ubyte(code + pc + 1);
	    if ((n < 2) || (n >= nconst)) {
		verify_error("invalid constant: op=%s, const=%d", opnames[code[pc]], n);
		return 0;
	    }
	    switch (*cpool[n]) {
	      case CONSTANT_String:
	      case CONSTANT_Integer:
	      case CONSTANT_Float:
		break;
	      default:
		verify_error("invalid constant: op=%s, const=%d", opnames[code[pc]], n);
		return 0;
	    }
	    pc += 2;
	    break;

	  case opc_ldc2:
	    n = get_ushort(code + pc + 1);
	    if ((n < 2) || (n >= nconst)) {
		verify_error("invalid constant: op=%s, const=%d", opnames[code[pc]], n);
		return 0;
	    }
	    switch (*cpool[n]) {
	      case CONSTANT_String:
	      case CONSTANT_Integer:
	      case CONSTANT_Float:
		break;
	      default:
		verify_error("invalid constant: op=%s, const=%d", opnames[code[pc]], n);
		return 0;
	    }
	    pc += 3;
	    break;

	  case opc_ldc2w:
	    n = get_ushort(code + pc + 1);
	    if ((n < 2) || (n >= nconst)) {
		verify_error("invalid constant: op=%s, const=%d", opnames[code[pc]], n);
		return 0;
	    }
	    switch (*cpool[n]) {
	      case CONSTANT_Double:
	      case CONSTANT_Long:
		break;
	      default:
		verify_error("invalid constant: op=%s, const=%d", opnames[code[pc]], n);
		return 0;
	    }
	    pc += 3;
	    break;

	  case opc_iload:
	  case opc_fload:
	  case opc_aload:
	  case opc_istore:
	  case opc_fstore:
	  case opc_astore:
	    n = get_ubyte(code + pc + 1);
	    if (n >= maxvar) {
		verify_error("invalid local var: op=%s, var=%d", opnames[code[pc]], n);
		return 0;
	    }
	    pc += 2;
	    break;

	  case opc_lload:
	  case opc_dload:
	  case opc_lstore:
	  case opc_dstore:
	    n = get_ubyte(code + pc + 1);
	    if (n + 1 >= maxvar) {
		verify_error("invalid local var: op=%s, var=%d", opnames[code[pc]], n);
		return 0;
	    }
	    pc += 2;
	    break;

	  case opc_iload_0:
	  case opc_fload_0:
	  case opc_aload_0:
	  case opc_istore_0:
	  case opc_fstore_0:
	  case opc_astore_0:
	    if (0 >= maxvar) {
		verify_error("invalid local var: op=%s, var=%d", opnames[code[pc]], 0);
		return 0;
	    }
	    pc += 1;
	    break;

	  case opc_lload_0:
	  case opc_dload_0:
	  case opc_lstore_0:
	  case opc_dstore_0:
	    if (0 + 1 >= maxvar) {
		verify_error("invalid local var: op=%s, var=%d", opnames[code[pc]], 0);
		return 0;
	    }
	    pc += 1;
	    break;


	  case opc_iload_1:
	  case opc_fload_1:
	  case opc_aload_1:
	  case opc_istore_1:
	  case opc_fstore_1:
	  case opc_astore_1:
	    if (1 >= maxvar) {
		verify_error("invalid local var: op=%s, var=%d", opnames[code[pc]], 1);
		return 0;
	    }
	    pc += 1;
	    break;

	  case opc_lload_1:
	  case opc_dload_1:
	  case opc_lstore_1:
	  case opc_dstore_1:
	    if (1 + 1 >= maxvar) {
		verify_error("invalid local var: op=%s, var=%d", opnames[code[pc]], 1);
		return 0;
	    }
	    pc += 1;
	    break;

	  case opc_iload_2:
	  case opc_fload_2:
	  case opc_aload_2:
	  case opc_istore_2:
	  case opc_fstore_2:
	  case opc_astore_2:
	    if (2 >= maxvar) {
		verify_error("invalid local var: op=%s, var=%d", opnames[code[pc]], 2);
		return 0;
	    }
	    pc += 1;
	    break;

	  case opc_lload_2:
	  case opc_dload_2:
	  case opc_lstore_2:
	  case opc_dstore_2:
	    if (2 + 1 >= maxvar) {
		verify_error("invalid local var: op=%s, var=%d", opnames[code[pc]], 2);
		return 0;
	    }
	    pc += 1;
	    break;

	  case opc_iload_3:
	  case opc_fload_3:
	  case opc_aload_3:
	  case opc_istore_3:
	  case opc_fstore_3:
	  case opc_astore_3:
	    if (3 >= maxvar) {
		verify_error("invalid local var: op=%s, var=%d", opnames[code[pc]], 3);
		return 0;
	    }
	    pc += 1;
	    break;

	  case opc_lload_3:
	  case opc_dload_3:
	  case opc_lstore_3:
	  case opc_dstore_3:
	    if (3 + 1 >= maxvar) {
		verify_error("invalid local var: op=%s, var=%d", opnames[code[pc]], 3);
		return 0;
	    }
	    pc += 1;
	    break;

	  case opc_iinc:
	    n = get_ubyte(code + pc + 1);
	    if (n >= maxvar) {
		verify_error("invalid local var: op=%s, var=%d", opnames[code[pc]], n);
		return 0;
	    }
	    pc += 3;
	    break;

	  case opc_newarray:
	    switch (get_ubyte(code + pc + 1)) {
	      case T_ARRAY:
	      case T_BOOLEAN:
	      case T_CHAR:
	      case T_FLOAT:
	      case T_DOUBLE:
	      case T_BYTE:
	      case T_SHORT:
	      case T_INT:
	      case T_LONG:
		pc += 2;
		break;

	      default:
		verify_error("invalid type: op=%s, type=%d", opnames[code[pc]], get_ubyte(code + pc + 1));
		return 0;
	    }
	    break;

	  case opc_anewarray:
	    n = get_ushort(code + pc + 1);
	    if ((n >= nconst) || 
		(n == 0) || 
		((n != 1) && (*cpool[n] != CONSTANT_Class))) {
		verify_error("invalid class constant: op=%s, const=%d", 
			     opnames[code[pc]], n);
		return 0;
	    }
	    pc += 3;
	    break;

	  case opc_ifeq:
	  case opc_ifne:
	  case opc_iflt:
	  case opc_ifle:
	  case opc_ifgt:
	  case opc_ifge:
	  case opc_if_icmpeq:
	  case opc_if_icmpne:
	  case opc_if_icmplt:
	  case opc_if_icmple:
	  case opc_if_icmpgt:
	  case opc_if_icmpge:
	  case opc_if_acmpeq:
	  case opc_if_acmpne:
	  case opc_goto:
	  case opc_jsr:
	    n = pc + get_short(code + pc + 1);
	    if ((n < 0) || (n > codelen)) {
		verify_error("invalid branch: op=%s, off=%d", opnames[code[pc]], n - pc);
		return 0;
	    }
	    info[n] |= F_LABEL;
	    pc += 3;
	    break;

	  case opc_putfield:
	  case opc_putstatic:
	  case opc_getfield:
	  case opc_getstatic:
	    n = get_ushort(code + pc + 1);
	    if ((n >= nconst) || (n == 0) || (*cpool[n] != CONSTANT_Fieldref)) {
		verify_error("invalid field constant: op=%s, const=%d", opnames[code[pc]], n);
		return 0;
	    }
	    pc += 3;
	    break;

	  case opc_invokevirtual:
	  case opc_invokenonvirtual:
	  case opc_invokestatic:
	    n = get_ushort(code + pc + 1);
	    if ((n >= nconst) || (n == 0) || (*cpool[n] != CONSTANT_Methodref)) {
		verify_error("invalid method constant: op=%s, const=%d", opnames[code[pc]], n);
		return 0;
	    }
	    pc += 3;
	    break;

	  case opc_invokeinterface:
	    n = get_ushort(code + pc + 1);
	    if ((n >= nconst) || (n == 0) || (*cpool[n] != CONSTANT_InterfaceMethodref)) {
		verify_error("invalid interface constant: op=%s, const=%d", opnames[code[pc]], n);
		return 0;
	    }
	    /* REMIND: fix
	    if (compute_arg_size((char *)cpool[n] + 1) != get_ubyte(code, pc + 3)) {
		verify_error("invalid interface argsize: op=%s, sig=%s, got=%d, expected=%d", 
			     opnames[code[pc]], cpool[n] + 1, get_ubyte(code , pc + 3),
			     compute_arg_size((char *)cpool[n] + 1));
		return 0;
	    }
	    */
	    pc += 5;
	    break;

	  case opc_new:
	    n = get_ushort(code + pc + 1);
	    if ((n >= nconst) || (n == 0) || ((n != 1) && (*cpool[n] != CONSTANT_Class))) {
		verify_error("invalid class constant: op=%s, const=%d", opnames[code[pc]], n);
		return 0;
	    }
	    pc += 3;
	    break;

	  case opc_checkcast:
	  case opc_instanceof:
	    n = get_ushort(code + pc + 1);
	    if ((n >= nconst) || (n == 0) || ((n != 1) && (*cpool[n] != CONSTANT_Class))) {
		verify_error("invalid class/interface constant: op=%s, const=%d", opnames[code[pc]], n);
		return 0;
	    }
	    pc += 3;
	    break;

	  case opc_tableswitch:
	    for (pc2 = pc + 1 ; (pc2 % 4) != 0 ; pc2++);
	    n = (get_uint(code + pc2 + 8) - get_uint(code + pc2 + 4)) + 1;
	    if (n <= 0) {
		verify_error("invalid table length: op=%s, len=%d", opnames[code[pc]], n);
		return 0;
	    }

	    /* check default branch */
	    m = pc + get_uint(code + pc2);
	    if ((m < 0) || (m > codelen)) {
		verify_error("invalid default branch: op=%s, off=%d", opnames[code[pc]], m - pc);
		return 0;
	    }
	    info[m] |= F_LABEL;

	    /* check each branch */
	    for (pc2 += 12, i = 0 ; i < n ; pc2 += 4, i++) {
		m = pc + get_uint(code + pc2);
		if ((m < 0) || (m > codelen)) {
		    verify_error("invalid branch: op=%s, n=%d, off=%d", opnames[code[pc]], i, m - pc);
		    return 0;
		}
		info[m] |= F_LABEL;
	    }
	    pc = pc2;
	    break;

	  case opc_lookupswitch:
	    for (pc2 = pc + 1 ; (pc2 % 4) != 0 ; pc2++);
	    n = get_uint(code + pc2 + 4);
	    if (n <= 0) {
		verify_error("invalid table length: op=%s, len=%d", opnames[code[pc]], n);
		return 0;
	    }

	    /* check each branch */
	    for (pc2 += 8, i = 0 ; i < n ; pc2 += 8, i++) {
		m = pc + get_uint(code + pc2 + 4);
		if ((m < 0) || (m > codelen)) {
		    verify_error("invalid branch: op=%s, n=%d, off=%d", opnames[code[pc]], i, m - pc);
		    return 0;
		}
		info[m] |= F_LABEL;
	    }
	    pc = pc2;
	    break;

	  default:
	    verify_error("invalid byte code: op=%d", code[pc]);
	    return 0;
	}
    }

    /* check for branch into the middle of instructions */
    for (pc = 0 ; pc <= codelen ; pc++) {
	if (((info[pc] & F_LABEL) != 0) && ((info[pc] & F_OPCODE) == 0)) {
	    verify_error("branch to the middle of an instruction: destpc=%d", pc);
	    return 0;
	}
    }

    /* REMIND: CHECK EXCEPTION TABLE OFFSET */
    return 1;
}

/*
 * verify a code attribute
 */
int verify_code(int nconst, uchar **cpool, int name, int sig, uchar *data, int length)
{
    uchar *ptr = data, *code;
    int maxstack, maxvar, codelen;
    int nexcept, i, natt, attlen, attname;

    /* verify stack size */
    maxstack = read_ubyte(&ptr);
    if ((maxstack < MINSTACK) || (maxstack >= MAXSTACK)) {
	verify_error("bad stack size: size=%d", maxstack);
	return 0;
    }

    /* verify local vars */
    maxvar = read_ubyte(&ptr);
    if ((maxvar < MINLOCALVAR) || (maxvar >= MAXLOCALVAR)) {
	verify_error("bad number of local variables: vars=%d", maxvar);
	return 0;
    }

    /* verify code length */
    codelen = read_ushort(&ptr);
    if ((codelen < MINCODELEN) || (codelen >= MAXCODELEN)) {
	verify_error("bad code length: len=%d", codelen);
	return 0;
    }

    /* verify code */
    code = ptr;
    ptr += codelen;
    if (!check_code(nconst, cpool, sig, code, codelen, maxvar)) {
	verify_error("bad code");
	return 0;
    }

    /* verify exceptions */
    nexcept = read_ushort(&ptr);
    for (i = 0 ; i < nexcept ; i++) {
	int pc0 = read_ushort(&ptr);
	int pc1 = read_ushort(&ptr);
	int handler = read_ushort(&ptr);
	int type = read_ushort(&ptr);
    }

    /* verify attributes */
    natt = read_ushort(&ptr);
    for (i = 0 ; i < natt ; i++) {
	attname = read_ushort(&ptr);
	attlen = read_uint(&ptr);
	ptr += attlen;
    }

    /* verify length */
    if (ptr - data != length) {
	verify_error("trailing code attribute bytes: remain=%d", length - (ptr - data));
	return 0;
    }
    return 1;
}
