/*
 * @(#)awt_Button.c	1.39 95/05/12 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"

static void
Button_callback(Widget w,
		XtPointer client_data,
		XmPushButtonCallbackStruct *call_data)
{
    JAVA_UPCALL((EE(),
		(void *)client_data,
		"selected", "(Lawt/Component;I)V",
		(void *)client_data,
		-1));
}

void
awt_WServer_buttonCreate(Hawt_WServer *ws,
			 Hawt_Button *this,
			 Hjava_lang_String *label,
			 Hawt_Image *normalImage,
			 Hawt_Image *pressedImage,
			 Hawt_Image *disabledImage,
			 Hawt_Window *parent)
{
    Classawt_Button	*thisPtr;
    struct ButtonData	*bdata;
    struct WindowData	*wdata;
    struct ImageData	*idata = 0;
    char		*clabel;
    Arg			args[20];
    int			argc;
    Pixel		bg;
    Dimension		w, h, marginH, marginW, borderW;

    if (this==0 || (label==0 && normalImage==0) || parent == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    thisPtr = unhand(this);
    wdata = PDATA(WindowData,parent);
    bdata = (struct ButtonData *)malloc(sizeof(struct ButtonData));
    bg = awt_getColor(ws, unhand(parent)->background);

    if (bdata == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }

    SET_PDATA(this, bdata);
    argc=0;
    XtSetArg(args[argc], XmNx, 0); argc++;
    XtSetArg(args[argc], XmNy, 0); argc++;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    XtSetArg(args[argc], XmNhighlightOnEnter, False); argc++;
    XtSetArg(args[argc], XmNshowAsDefault, 0); argc++;
    XtSetArg(args[argc], XmNdefaultButtonShadowThickness, 0); argc++;
    XtSetArg(args[argc], XmNfillOnArm, True); argc++;
    XtSetArg(args[argc], XmNtraversalOn, False); argc++;
    XtSetArg(args[argc], XmNrecomputeSize, False); argc++;
    if (label != 0) {
	clabel = makeCString(label);
    } else {
	clabel = "";
    }
    if (normalImage != 0) {
	idata = PDATA(ImageData, normalImage);
	if (idata==0 || idata->xpixmap==0) {
	    SignalError(0,JAVAPKG "NullPointerException","no pixmap");
	    return;
	}
	XtSetArg(args[argc], XmNlabelType, XmPIXMAP); argc++;
	XtSetArg(args[argc], XmNmarginTop, 0); argc++;
	XtSetArg(args[argc], XmNmarginBottom, 0); argc++;
	XtSetArg(args[argc], XmNmarginLeft, 0); argc++;
	XtSetArg(args[argc], XmNmarginRight, 0); argc++;
	XtSetArg(args[argc], XmNmarginHeight, 0); argc++;
	XtSetArg(args[argc], XmNmarginWidth, 0); argc++;
	XtSetArg(args[argc], XmNlabelPixmap, idata->xpixmap); argc++;
	XtSetArg(args[argc], XmNshadowThickness, 0); argc++;
    }
    if (pressedImage != 0) {
	idata = PDATA(ImageData, pressedImage);
	if (idata==0 || idata->xpixmap==0) {
	    SignalError(0,JAVAPKG "NullPointerException",0);
	    return;
	}
	XtSetArg(args[argc], XmNarmPixmap, idata->xpixmap); argc++;
    }
    if (disabledImage != 0) {
	idata = PDATA(ImageData, disabledImage);
	if (idata==0 || idata->xpixmap==0) {
	    SignalError(0,JAVAPKG "NullPointerException",0);
	    return;
	}
	XtSetArg(args[argc], XmNlabelInsensitivePixmap, idata->xpixmap); argc++;
    }
    bdata->button = XmCreatePushButton(wdata->w,
				       clabel,
				       args,
				       argc);
    XtAddCallback(bdata->button,
		  XmNactivateCallback,
		  Button_callback,
		  (XtPointer)this);

    XtSetMappedWhenManaged(bdata->button, False);
    XtManageChild(bdata->button);

    awt_util_getSizes(bdata->button,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));

    thisPtr->x = 0;
    thisPtr->y = 0;
}

void 
awt_WServer_buttonMoveTo(Hawt_WServer *ws,
			 Hawt_Button *this,
			 long x,
			 long y)
{
    struct ButtonData *bdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    bdata = PDATA(ButtonData,this);

    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    awt_util_move(bdata->button, x, y);
}

void 
awt_WServer_buttonReshape(Hawt_WServer *ws,
			  Hawt_Button *this,
			  long x,
			  long y,
			  long w,
			  long h)
{
    struct ButtonData *bdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    bdata = PDATA(ButtonData,this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_reshape(bdata->button, x, y, w, h);
}


void 
awt_WServer_buttonDispose(Hawt_WServer *ws, Hawt_Button *this)
{
    struct ButtonData *bdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    bdata = PDATA(ButtonData,this);

    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_hide(bdata->button);
    XtDestroyWidget(bdata->button);
    free((void *)bdata);
    SET_PDATA(this, 0);
}

void 
awt_WServer_buttonShow(Hawt_WServer *ws, Hawt_Button *this)
{
    struct ButtonData *bdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    bdata = PDATA(ButtonData,this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_show(bdata->button);
}

void 
awt_WServer_buttonHide(Hawt_WServer *ws, Hawt_Button *this)
{
    struct ButtonData *bdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    bdata = PDATA(ButtonData,this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_hide(bdata->button);
}

void 
awt_WServer_buttonEnable(Hawt_WServer *ws, Hawt_Button *this)
{
    struct ButtonData *bdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    bdata = PDATA(ButtonData,this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    XtSetSensitive(bdata->button, True);
}

void 
awt_WServer_buttonDisable(Hawt_WServer *ws, Hawt_Button *this)
{
    struct ButtonData *bdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    bdata = PDATA(ButtonData,this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    XtSetSensitive(bdata->button, False);    
}

