/*
 * @(#)awt_FileDialog.c	1.25 95/05/12 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"
#include <sys/param.h>
#include <string.h>
#include <stdlib.h>

static int fileAnswered = 0;

static void
FileDialog_OK(
    Widget w,
    void *client_data,
    XmFileSelectionBoxCallbackStruct *call_data
)
{
    Hawt_FileDialog		*this = (Hawt_FileDialog *)client_data;
    struct FileDialogData	*fdata = PDATA(FileDialogData,this);

    XtUnmanageChild(fdata->dialog);
    if (fdata->file) {
	XtFree(fdata->file);
    }
    XmStringGetLtoR(call_data->value,
		    XmSTRING_DEFAULT_CHARSET,
		    &fdata->file);
    fileAnswered = 1;
}

static void
FileDialog_CANCEL(
    Widget w,
    void *client_data,
    XmFileSelectionBoxCallbackStruct *call_data
)
{
    Hawt_FileDialog		*this = (Hawt_FileDialog *)client_data;
    struct FileDialogData	*fdata = PDATA(FileDialogData,this);

    XtUnmanageChild(fdata->dialog);
    fdata->file = 0;
    fileAnswered = 1;
}
void
changeBackground(Widget w, void *bg)
{
    XtVaSetValues(w,
		  XmNbackground, (Pixel)bg,
		  NULL);
}

void
awt_WServer_fileDialogCreate(Hawt_WServer *ws,
			     Hawt_FileDialog *this,
			     Hjava_lang_String *title,
			     Hawt_Frame	*parent)
{
    struct FileDialogData	*fdata;
    struct WindowData		*wdata;
    int				argc;
    Arg				args[10];
    char			*ctitle;
    Widget			child;
    XmString			xim;
    Pixel			bg;

    if (parent == 0 || this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    bg = awt_getColor(ws, unhand(parent)->background);
    wdata = PDATA(WindowData,parent);
    fdata = (struct FileDialogData *)malloc(sizeof(struct FileDialogData));
    SET_PDATA(this, fdata);

    if (fdata == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }

    fdata->file = 0;

    if (title != 0) {
	ctitle = makeCString(title);
    } else {
	ctitle = "File Chooser";
    }


    argc=0;
    XtSetArg(args[argc], XmNmustMatch, False); argc++;
    XtSetArg(args[argc], XmNautoUnmanage, False); argc++;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    xim=XmStringCreateLtoR(ctitle,XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[argc], XmNdialogTitle, xim); argc++;

    fdata->dialog = XmCreateFileSelectionDialog(wdata->shell,
						ctitle,
						args,
						argc);
    awt_util_mapChildren(fdata->dialog, changeBackground, (void *)bg);
    XmStringFree(xim);
    child = XmFileSelectionBoxGetChild(fdata->dialog, XmDIALOG_HELP_BUTTON);
    if (child != 0) {
	XtUnmanageChild(child);
    }
    child = XmFileSelectionBoxGetChild(fdata->dialog, XmDIALOG_WORK_AREA);
    if (child != 0) {
	XtUnmanageChild(child);
    }

    XtAddCallback(fdata->dialog,
		  XmNokCallback,
		  (XtCallbackProc)FileDialog_OK,
		  (XtPointer)this);
    XtAddCallback(fdata->dialog,
		  XmNcancelCallback,
		  (XtCallbackProc)FileDialog_CANCEL,
		  (XtPointer)this);
}


void
awt_WServer_fileDialogDispose(Hawt_WServer *ws, Hawt_FileDialog *this)
{
    struct FileDialogData *fdata = PDATA(FileDialogData,this);

    XtDestroyWidget(fdata->dialog);
    free((void *)fdata);
}

static int
WaitForFileAnswered(void *data)
{
    return fileAnswered != 0;
}

Hjava_lang_String *
awt_WServer_fileDialogChooseFile(Hawt_WServer *ws,
				 Hawt_FileDialog *this,
				 Hjava_lang_String *initValue)
{
    Hjava_lang_String		*rval;
    struct FileDialogData	*fdata;
    int		argc;
    Arg		args[10];
    XmString	dir_mask = 0;
    char	*cvalue = 0;
    XmString	xim = 0;
    char	resolved[MAXPATHLEN];
    char	path[MAXPATHLEN];
    char	*dirmark;
    char	*file;
    struct WServerData	*wsdata = PDATA(WServerData,ws);

    fdata = PDATA(FileDialogData,this);

    if (initValue != 0) {
	cvalue = makeCString(initValue);
	file = strdup(cvalue);
	dirmark = strrchr(cvalue, '/');
	if (dirmark) {
	    if (*(dirmark+1)) {
		*(dirmark+1) = 0;
	    }
	    if (realpath(cvalue, resolved)) {
		sprintf(path,"%s/*", resolved);
		xim=XmStringCreateLtoR(path,XmSTRING_DEFAULT_CHARSET);
		XtVaSetValues(fdata->dialog,
			      XmNdirMask, xim,
			      NULL);
		dir_mask = xim;
	    }
	}
    }

    if (dir_mask == 0) {
	/* make the file selection box appear. The DoSearch
	   function is called each time so that new files are
	   incorporated into the list of available files.*/
	argc=0;
	XtSetArg(args[argc], XmNdirMask, &dir_mask); argc++;
	XtGetValues(fdata->dialog, args, argc);
    }

    XmFileSelectionDoSearch(fdata->dialog, dir_mask);

    if (xim) {
	XmStringFree(xim);
    }

    XtVaSetValues(fdata->dialog,
		  XmNdialogStyle, XmDIALOG_SYSTEM_MODAL,
		  NULL);
    XtManageChild(fdata->dialog);
    fileAnswered = 0;
    awt_WServer_modalWait(WaitForFileAnswered, 0);

    if (fdata->file) {
	return makeJavaString(fdata->file, strlen(fdata->file));
    } else {
	return 0;
    }
}

