/*
 * @(#)awt_Font.c	1.29 95/05/17 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"
#include <string.h>


/*
 * Hardwired list of mappings for generic font names "Helvetica",
 * "TimesRoman", "Courier", "Dialog", and "DialogInput".
 */
static char 	*defaultfontname = "fixed";
static char 	*defaultfoundry = "misc";
static char	*anyfoundry = "*";
static char	*anystyle = "*-*";
static char	*isolatin1 = "iso8859-1";

static int
FontName(Hjava_lang_String *name,
	 char **foundry, char **facename, char **encoding)
{
    char *cname;

    if (name == 0) {
	return 0;
    }
    cname = makeCString(name);

    if (strcmp(cname,"Helvetica")==0) {
	*foundry = "linotype";
	*facename = "helvetica";
	*encoding = isolatin1;
    } else if (strcmp(cname,"TimesRoman") == 0) {
	*foundry = "adobe";
	*facename = "times";
	*encoding = isolatin1;
    } else if (strcmp(cname,"Courier") == 0) {
	*foundry = "adobe";
	*facename = "courier";
	*encoding = isolatin1;
    } else if (strcmp(cname,"Dialog") == 0) {
	*foundry = "b&h";
	*facename = "lucida";
	*encoding = isolatin1;
    } else if (strcmp(cname,"DialogInput") == 0) {
	*foundry = "b&h";
	*facename = "lucidatypewriter";
	*encoding = isolatin1;
    } else if (strcmp(cname,"ZapfDingbats") == 0) {
	*foundry = "itc";
	*facename = "zapfdingbats";
	*encoding = "*-*";
    } else {
#ifdef DEBUG
	fprintf(stderr, "Unknown font: %s\n", cname);
#endif
	*foundry = defaultfoundry;
	*facename = defaultfontname;
	*encoding = isolatin1;
    }

    return 1;
}

    
static char *
Style(long s)
{
    switch (s) {
      case awt_Font_ITALIC:
	return "medium-i";
      case awt_Font_BOLD:
	return "bold-r";
      case awt_Font_BOLD+awt_Font_ITALIC:
	return "bold-i";
      case awt_Font_UNDERLINE:
	return "underline";
      case awt_Font_PLAIN:
      default:
	return "medium-r";
    }
}
	

struct FontData *
awt_GetFontData(Hawt_WServer *ws, Hawt_Font *font, char **errmsg)
{
    Classawt_Font	*f;
    Display		*display;
    struct FontData	*fdata;
    char		fontSpec[1024];
    long		height;
    long		oheight;
    int			above = 0; /* tries above height */
    int			below = 0; /* tries below height */
    char		*foundry;
    char		*name;
    char		*encoding;
    char		*style;
    XFontStruct		*xfont;

    if (font == 0 || ws == 0) {
	if (errmsg) {
	    *errmsg = JAVAPKG "NullPointerException";
	}
	return (struct FontData *) NULL;
    }
    display = DSPU(ws);
    f = unhand(font);

    fdata = PDATA(FontData,font);
    if (fdata != NULL && fdata->xfont != NULL) {
	return fdata;
    }

    if (!FontName(f->family, &foundry, &name, &encoding)) {
	if (errmsg) {
	    *errmsg = JAVAPKG "NullPointerException";
	}
	return (struct FontData *) NULL;
    }
    style = Style(f->style);
    oheight = height = f->size;

    while (1) {
	sprintf(fontSpec,"-%s-%s-%s-*-*-%d-*-*-*-*-*-%s",
		foundry,
		name,
		style,
		height,
		encoding);

	/*fprintf(stderr,"LoadFont: %s\n", fontSpec);*/
	xfont = XLoadQueryFont(display, fontSpec);
	/* XXX: sometimes XLoadQueryFont returns a bogus font structure */
	/* with negative ascent. */
	if (xfont == (Font) NULL || xfont->ascent < 0) {
	    if (xfont) {
		XFreeFont(display, xfont);
	    }
	    if (foundry != anyfoundry) {
		/* Try any other foundry before messing with the sizes */
		foundry = anyfoundry;
		continue;
	    }
	    /* We couldn't find the font. We'll try to find an */
	    /* alternate by searching for heights above and below our */
	    /* preferred height. We try for 4 heights above and below. */
	    /* If we still can't find a font we repeat the algorithm */
	    /* using misc-fixed as the font. If we then fail, then we */
	    /* give up and signal an error. */
	    if (above == below) {
		above++;
		height = oheight + above;
	    } else {
		below++;
		if (below > 4) {
		    if (name != defaultfontname || style != anystyle) {
			name = defaultfontname;
			foundry = defaultfoundry;
			height = oheight;
			style = anystyle;
			encoding = isolatin1;
			above = below = 0;
			continue;
		    } else {
			if (errmsg) {
			    *errmsg = JAVAPKG "FileNotFoundException";
			}
			return (struct FontData *) NULL;
		    }
		}
		height = oheight - below;
	    }
	    continue;
	} else {
	    fdata = (struct FontData *)malloc(sizeof(struct FontData));
	    fdata->xfont = xfont;
	    f->wServer = ws;
	    SET_PDATA(font, fdata);

	    if (fdata == NULL && errmsg) {
		*errmsg = JAVAPKG "OutOfMemoryException";
	    }
	    return fdata;
	}
    }
}

void
awt_WServer_loadFontMetrics(Hawt_WServer *ws,
			    Hawt_Window *window,
			    Hawt_WSFontMetrics *fontmetrics)
{
    Classawt_WSFontMetrics	*fm;
    Classawt_Font		*f;
    struct FontData		*fdata;
    long			*widths;
    int				ccount;
    int				i;
    char			*err;

    if (fontmetrics == NULL) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    fm = unhand(fontmetrics);
    if (fm == NULL || fm->font == NULL) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    fdata = awt_GetFontData(ws, fm->font, &err);
    if (fdata == NULL) {
	SignalError(0, err, 0);
	return;
    }
    f = unhand(fm->font);

    fm->ascent = f->ascent = fdata->xfont->ascent;
    fm->descent = f->descent = fdata->xfont->descent;
    fm->leading = 1;
    fm->height = fm->ascent + fm->descent + fm->leading;
    f->actualHeight = fm->height;
    fm->maxAscent = fdata->xfont->max_bounds.ascent;
    fm->maxDescent = fdata->xfont->max_bounds.descent;
    fm->maxHeight = fm->maxAscent + fm->maxDescent + fm->leading;
    fm->maxAdvance = fdata->xfont->max_bounds.width;

    fm->widths = (HArrayOfInt *)ArrayAlloc(T_INT, 256);
    if (fm->widths == NULL) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }
    f->widths = fm->widths;
    widths = unhand(fm->widths)->body;
    memset(widths, 0, 256 * sizeof(long));
    widths += fdata->xfont->min_char_or_byte2;
    ccount = fdata->xfont->max_char_or_byte2 - fdata->xfont->min_char_or_byte2;
    if (fdata->xfont->per_char) {
	for (i = 0; i <= ccount; i++) {
	    *widths++ = (long) fdata->xfont->per_char[i].width;
	}
    } else {
	for (i = 0; i <= ccount; i++) {
	    *widths++ = (long) fdata->xfont->max_bounds.width;
	}
    }
}

#ifdef foo
void
awt_WServer_fontCreate(Hawt_WServer *ws,
		       Hawt_Font *this,
		       Hjava_lang_String *_name,
		       long style,
		       long height)
{
    char		*name;
    struct WServerData	*wsdata = PDATA(WServerData,ws);

    if (_name == NULL) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    name = makeCString(_name);
    CreateFont(wsdata->display, this, name, style, height);
}
#endif


void
awt_WServer_fontDispose(Hawt_WServer *ws, Hawt_Font *this)
{
    struct FontData	*fdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    fdata = PDATA(FontData,this);
    if (fdata != 0) {
	XFreeFont(DSPU(ws), fdata->xfont);
	free((void*)fdata);
    }
    SET_PDATA(this,0);
}

long
awt_WServer_fontStringWidth(Hawt_WServer *ws,
			    Hawt_WSFontMetrics *this,
			    Hjava_lang_String *_s)
{
    HArrayOfChar	*data;
    unicode		*s;
    long		w;
    int			ch;
    int			cnt;
    long		*widths;
    int			widlen;
    Classawt_WSFontMetrics	*fm;

    if (this == 0 || _s == 0 || (data = unhand(_s)->value) == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
    s = unhand(data)->body + unhand(_s)->offset;
    cnt = javaStringLength(_s);
    fm = unhand(this);
    if (fm->widths) {
	w = 0;
	widths = unhand(fm->widths)->body;
	widlen = obj_length(fm->widths);
	while (--cnt >= 0) {
	    ch = *s++;
	    if (ch < widlen) {
		w += widths[ch];
	    } else {
		w += fm->maxAdvance;
	    }
	}
    } else {
	w = fm->maxAdvance * cnt;
    }

    return w;
}

long
awt_WServer_fontCharsWidth(Hawt_WServer *ws,
			   Hawt_WSFontMetrics *this,
			   HArrayOfChar *str,
			   long off,
			   long len)
{
    long		w;
    unicode		*pstr;
    int			ch;
    long		*widths;
    int			widlen;
    Classawt_WSFontMetrics	*fm;

    if (this == 0 || str == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
    fm = unhand(this);
    if ((len < 0) || (off < 0) || (len + off > obj_length(str))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }
    if (fm->widths) {
	w = 0;
	widths = unhand(fm->widths)->body;
	widlen = obj_length(fm->widths);
	for (pstr = unhand(str)->body + off ; len ; len--) {
	    ch = *pstr++;
	    if (ch < widlen) {
		w += widths[ch];
	    } else {
		w += fm->maxAdvance;
	    }
	}
    } else {
	w = fm->maxAdvance * len;
    }

    return w;
}

long
awt_WServer_fontBytesWidth(Hawt_WServer *ws,
			   Hawt_WSFontMetrics *this,
			   HArrayOfByte *str,
			   long off,
			   long len)
{
    long		w;
    unsigned char	*pstr;
    int			ch;
    long		*widths;
    int			widlen;
    Classawt_WSFontMetrics	*fm;

    if (this == 0 || str == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
    fm = unhand(this);
    if ((len < 0) || (off < 0) || (len + off > obj_length(str))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }
    if (fm->widths) {
	w = 0;
	widths = unhand(fm->widths)->body;
	widlen = obj_length(fm->widths);
	for (pstr = (unsigned char *)unhand(str)->body + off ; len ; len--) {
	    ch = *pstr++;
	    if (ch < widlen) {
		w += widths[ch];
	    } else {
		w += fm->maxAdvance;
	    }
	}
    } else {
	w = fm->maxAdvance * len;
    }

    return w;
}
