/*
 * @(#)awt_Frame.c	1.51 95/05/12 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"
#include <X11/Shell.h>
#include <Xm/VendorS.h>
#include <Xm/DialogS.h>

extern char *makeCString(Hjava_lang_String *);

static void
Frame_event_handler(Widget w, XtPointer data, XEvent *event, Boolean *cont)
{
    Hawt_Frame *this = (Hawt_Frame *)data;
    Classawt_Frame *thisPtr = unhand(this);
    struct FrameData	*wdata = PDATA(FrameData,this);
    int woff, hoff;
    Dimension	mheight;

    /*fprintf(stderr, "window event %d\n", event->xany.type);*/
    switch (event->xany.type) {
      case ConfigureNotify:
	thisPtr->x = event->xconfigure.x;
	thisPtr->y = event->xconfigure.y;
/*
	fprintf(stderr,"thisPtr: w=%d,h=%d, xconfigure: w=%d, h=%d\n",
		thisPtr->width, thisPtr->height,
		event->xconfigure.width,
		event->xconfigure.height);
*/
	if (wdata->menuBar != NULL) {
	    woff = 0;
	    XtVaGetValues(wdata->menuBar, XmNheight, &mheight, NULL);
	    hoff = mheight;
	} else {
	    woff = 0;
	    hoff = 0;
	}
	if ((thisPtr->width + woff) != event->xconfigure.width ||
	    (thisPtr->height + hoff) != event->xconfigure.height) {
	    thisPtr->width = event->xconfigure.width - woff;
	    thisPtr->height = event->xconfigure.height - hoff;

	    JAVA_UPCALL((EE(), (void *)this,"resize","()V"));
	}
	    
	break;
    }
}

static void
Frame_quit(Widget w,
	   XtPointer client_data,
	   XtPointer call_data)
{
    JAVA_UPCALL((EE(), (void *)client_data,"handleQuit","()V"));
}
	   
static void
setDeleteCallback(Hawt_Frame *this, struct FrameData *wdata)
{
    Atom xa_WM_DELETE_WINDOW;
    Atom xa_WM_PROTOCOLS;

    XtVaSetValues(wdata->winData.shell,
		  XmNdeleteResponse, XmDO_NOTHING,
		  NULL);
    xa_WM_DELETE_WINDOW = XmInternAtom(XtDisplay(wdata->winData.shell),
				       "WM_DELETE_WINDOW", False);
    xa_WM_PROTOCOLS = XmInternAtom(XtDisplay(wdata->winData.shell),
				   "WM_PROTOCOLS", False);

    XmAddProtocolCallback(wdata->winData.shell,
			  xa_WM_PROTOCOLS,
			  xa_WM_DELETE_WINDOW,
			  Frame_quit, (XtPointer)this);
}

void
awt_WServer_frameCreate(Hawt_WServer *ws,
			Hawt_Frame *this,
			long hasTitleBar,
			long isModal,
			Hawt_Frame *parentFrame,
			long width,
			long height,
			Hawt_Color *bg)
{
    struct WServerData	*wsdata = PDATA(WServerData, ws);
    Dimension		w,h;
    Arg			args[20];
    int			argc;
    struct FrameData	*wdata;
    struct FrameData	*parentData;
    Pixel		wbg;
    Classawt_Frame	*thisPtr;

    if (this == 0 || bg == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    thisPtr = unhand(this);
    wbg = awt_getColor(ws, bg);

    wdata = (struct FrameData *)malloc(sizeof(struct FrameData));
    SET_PDATA(this, wdata);

    if (wdata == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }

    wdata->isModal = isModal;
    wdata->winData.hscroll = 0;
    wdata->winData.vscroll = 0;
    wdata->winData.sw = 0;

    /* create a top-level shell */
    argc = 0;
    if (parentFrame != 0) {
	parentData = PDATA(FrameData,parentFrame);
	XtSetArg(args[argc], XmNtransientFor, parentData->winData.shell);argc++;
    }
    XtSetArg(args[argc], XmNsaveUnder, False); argc++;
    XtSetArg(args[argc], XmNwidth, (XtArgVal)width + 10); argc++;
    XtSetArg(args[argc], XmNheight, (XtArgVal)height + 18); argc++;
    XtSetArg(args[argc], XmNallowShellResize, TRUE); argc++;
    XtSetArg(args[argc], XmNmarginWidth, 0); argc++;
    XtSetArg(args[argc], XmNmarginHeight, 0); argc++;

    if (parentFrame != 0) {
	wdata->winData.shell = XtCreatePopupShell("dialog",
						  xmDialogShellWidgetClass,
						  parentData->winData.shell,
						  args,
						  argc);
    } else {
	wdata->winData.shell = XtAppCreateShell("AWTapp",
						"XApplication",
						vendorShellWidgetClass,
						DSPU(ws),
						args,
						argc);
	setDeleteCallback(this, wdata);
    }

    thisPtr->x = 0;
    thisPtr->y = 0;
    thisPtr->width = width;
    thisPtr->height = height;

    XtAddEventHandler(wdata->winData.shell, StructureNotifyMask,
		      FALSE, Frame_event_handler, this);
    argc = 0;
    XtSetArg(args[argc], XmNbackground, wbg); argc++;
    XtSetArg(args[argc], XmNwidth, (XtArgVal)width); argc++;
    XtSetArg(args[argc], XmNheight, (XtArgVal)height); argc++;
    XtSetArg(args[argc], XmNmainWindowMarginHeight, 0); argc++;
    XtSetArg(args[argc], XmNmainWindowMarginWidth, 0); argc++;
    XtSetArg(args[argc], XmNmarginWidth, 0); argc++;
    XtSetArg(args[argc], XmNmarginHeight, 0); argc++;
    XtSetArg(args[argc], XmNspacing, 0); argc++;
    wdata->mainW = XmCreateMainWindow(wdata->winData.shell,
				      "main",
				      args,
				      argc);
    argc = 0;
    XtSetArg(args[argc], XmNwidth, (Dimension)width); argc++;
    XtSetArg(args[argc], XmNheight, (Dimension)height); argc++;
    XtSetArg(args[argc], XmNmarginHeight, 0); argc++;
    XtSetArg(args[argc], XmNmarginWidth, 0); argc++;
    XtSetArg(args[argc], XmNbackground, wbg); argc++;
    XtSetArg(args[argc], XmNresizePolicy, XmRESIZE_NONE); argc++;
    wdata->winData.w = XmCreateBulletinBoard(wdata->mainW,
					     "mainW",
					     args,
					     argc);
    awt_util_getSizes(wdata->winData.w,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));

    wdata->menuBar = NULL;
}

void
awt_WServer_frameSetIconImage(Hawt_WServer *ws,
			      Hawt_Frame *this,
			      Hawt_Image *image)
{
    struct WServerData	*wsdata = PDATA(WServerData, ws);
    struct FrameData	*wdata;
    struct ImageData	*idata;
    unsigned int width, height, border_width, depth;
    Window		win;
    Window		oldwin;
    Window		root;
    int x,y;

    if (this == 0 || image == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(FrameData,this);
    idata = PDATA(ImageData,image);
    if (wdata == 0 || idata == 0 || idata->xpixmap == 0 ||
	wdata->winData.shell==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaGetValues(wdata->winData.shell,
		  XmNiconWindow, &win,
		  NULL);
    if (!win) {
	if (!XGetGeometry(wsdata->display,
			  idata->xpixmap,
			  &root,
			  &x,
			  &y,
			  &width,
			  &height,
			  &border_width,
			  &depth) ||
	    ! (win = XCreateSimpleWindow(wsdata->display,
					 root,
					 0, 0, width, height,
					 (unsigned)0,
					 CopyFromParent,
					 CopyFromParent))) {
	    XtVaSetValues(wdata->winData.shell,
			  XmNiconPixmap, idata->xpixmap,
			  NULL);
	    return;
	}
    }
    XtVaSetValues(wdata->winData.shell,
		  XmNiconWindow, win,
		  NULL);
    XSetWindowBackgroundPixmap(wsdata->display, win, idata->xpixmap);
    XClearWindow(wsdata->display, win);
}

static int
WaitForUnmap(void *data)
{
    return XtIsManaged((Widget)data) == False;
}

void
awt_WServer_frameShow(Hawt_WServer *ws, Hawt_Frame *this)
{
    struct WServerData	*wsdata = PDATA(WServerData,ws);
    struct FrameData	*wdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(FrameData,this);
    if (wdata == 0 || wdata->winData.w==0 || wdata->winData.shell==0 ||
	wdata->mainW == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    XtRealizeWidget(wdata->winData.shell);
    XtManageChild(wdata->winData.w);
    XtManageChild(wdata->mainW);
    XRaiseWindow(wsdata->display, XtWindow(wdata->winData.shell));
    if (wdata->isModal) {
	awt_WServer_modalWait(WaitForUnmap, (void *)(wdata->mainW));
    }
}

void
awt_WServer_frameHide(Hawt_WServer *ws, Hawt_Frame *this)
{
    struct FrameData	*wdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(FrameData,this);
    if (wdata == 0 || wdata->winData.w==0 || wdata->winData.shell==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtUnmanageChild(wdata->winData.w);
    XtUnmanageChild(wdata->mainW);
    XtUnrealizeWidget(wdata->winData.shell);
}


void
awt_WServer_frameSetTitle(Hawt_WServer *ws,
			  Hawt_Frame *this,
			  Hjava_lang_String *title)
{
    char *ctitle;
    struct FrameData	*wdata;

    if (this == 0 || title == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(FrameData,this);
    if (wdata == 0 || wdata->winData.shell == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    ctitle = makeCString(title);
    XtVaSetValues(wdata->winData.shell,
		  XmNtitle, ctitle,
		  XmNiconName, ctitle,
		  XmNname, ctitle,
		  NULL);
}

void
awt_WServer_frameDispose(Hawt_WServer *ws, Hawt_Frame *this)
{
    struct FrameData	*wdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(FrameData,this);
    if (wdata == 0 || wdata->mainW==0 || wdata->winData.shell==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtDestroyWidget(wdata->mainW);
    XtDestroyWidget(wdata->winData.shell);
    free((void *)wdata);
    SET_PDATA(this,0);
}

void
awt_WServer_frameSetMinSize(Hawt_WServer *ws,
			    Hawt_Frame *this,
			    long width, long height)
{
    struct FrameData	*wdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(FrameData,this);
    if (wdata == 0 || wdata->winData.w==0 || wdata->winData.shell==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaSetValues(wdata->winData.shell,
		  XtNminWidth, (XtArgVal)width - 10,
		  XtNminHeight, (XtArgVal)height - 30,
		  NULL);
    XtVaSetValues(wdata->winData.w,
		  XtNwidth, (XtArgVal)width - 10,
		  XtNheight, (XtArgVal)height - 30,
		  NULL);
}

void
awt_WServer_frameReshape(Hawt_WServer *ws,
			 Hawt_Frame *this,
			 long x, long y,
			 long w, long h)
{
    Classawt_Frame	*thisp;
    struct FrameData	*wdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(FrameData,this);
    if (wdata == 0 || wdata->winData.w==0 || wdata->winData.shell==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    thisp = unhand(this);
    thisp->x = x;
    thisp->y = y;
    thisp->width = w;
    thisp->height = h;

    XtUnrealizeWidget(wdata->winData.shell);
    XtUnmanageChild(wdata->winData.w);
    XtVaSetValues(wdata->winData.shell,
		  XtNx, (XtArgVal) x,
		  XtNy, (XtArgVal) y,
		  XtNwidth, (XtArgVal) w,
 		  XtNheight, (XtArgVal) h,
		  NULL);

    XtVaSetValues(wdata->winData.w,
		  XtNx, (XtArgVal) x,
		  XtNy, (XtArgVal) y,
		  XtNwidth, (XtArgVal) w,
		  XtNheight, (XtArgVal) h,
		  NULL);
    XtManageChild(wdata->winData.w);
    XtRealizeWidget(wdata->winData.shell);
}

void
awt_WServer_frameSetDefaultFont(Hawt_WServer *ws,
				Hawt_Frame *this,
				Hawt_Font *font)
{
    struct FrameData	*wdata;
    struct FontData	*fdata;
    XmFontList		fontlist;
    char		*err;

    if (this == 0 || font == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    fdata = awt_GetFontData(ws, font, &err);
    if (fdata == 0) {
	SignalError(0, err, 0);
	return;
    }

    wdata = PDATA(FrameData,this);
    if (wdata == 0 || wdata->winData.w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    fontlist = XmFontListCreate(fdata->xfont, "labelFont");
    if (fontlist != NULL) {
	XtVaSetValues(wdata->winData.w,
		      XmNtextFontList, fontlist,
		      XmNlabelFontList, fontlist,
		      XmNbuttonFontList, fontlist,
		      NULL);
	XmFontListFree(fontlist);
    }
}

long
awt_WServer_frameHasStatusBar(Hawt_WServer *ws, Hawt_Frame *this)
{
  return 0;
}

void
awt_WServer_frameShowStatusBar(Hawt_WServer *ws, Hawt_Frame *this, long show)
{
}

long
awt_WServer_frameShowingStatusBar(Hawt_WServer *ws, Hawt_Frame *this)
{
  return 0;
}

void
awt_WServer_frameSetStatusMessage(Hawt_WServer *ws,
				  Hawt_Frame *this,
				  Hjava_lang_String *title)
{
}
