/*
 * @(#)awt_Graphics.c	1.27 95/05/14 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"
#include <X11/keysym.h>

#define INIT_GC(disp, gdata)\
if (gdata==0 || gdata->gc == 0 && !awt_init_gc(disp,gdata)) {\
    return;\
}

#define INIT_GC2(disp, gdata, rval)\
if (gdata==0 || gdata->gc == 0 && !awt_init_gc(disp,gdata)) {\
    return rval;\
}

#define OX(this)		(unhand(this)->originX)
#define OTY(this)		(unhand(this)->originY)
#define SX(this)		(unhand(this)->scaleX)
#define STY(this)		(unhand(this)->scaleY)
#define TX(this, x)		(((int)((x) * SX(this))) + OX(this))
#define TY(this, y)		(((int)((y) * STY(this))) + OTY(this))
#define TW(this, w)		((int)((w) * SX(this)))
#define TH(this, h)		((int)((h) * STY(this)))


int
awt_init_gc(Display *display, struct GraphicsData *gdata)
{
    if (gdata->drawable == 0) {
	gdata->drawable = XtWindow(gdata->win);
	if (gdata->drawable == 0) {
	    return 0;
	}
    }

    if ((gdata->gc = XCreateGC(display, gdata->drawable, 0, 0)) != 0) {
	XSetGraphicsExposures(display, gdata->gc, True);
	return 1;
    }

    return 0;
}

void
awt_WServer_graphicsCreate(Hawt_WServer *ws,
			   Hawt_Graphics *this,
			   Hawt_Window *w)
{
    struct GraphicsData	*gdata;
    struct WindowData *wdata;

    if (this==0 || w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    gdata = (struct GraphicsData *)malloc(sizeof(struct GraphicsData));
    SET_PDATA(this, gdata);

    if (gdata == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }
    wdata=PDATA(WindowData,w);
    if (wdata == 0 || wdata->w == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    gdata->gc = 0;
    gdata->win = wdata->w;
    gdata->drawable = 0;
    gdata->clipset = False;
}


void
awt_WServer_imageGraphicsCreate(Hawt_WServer *ws,
				Hawt_Graphics *this,
				Hawt_Image *im)
{
    struct GraphicsData	*gdata;
    struct ImageData	*idata;

    if (this==0 || im==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    gdata = (struct GraphicsData *)malloc(sizeof(struct GraphicsData));
    SET_PDATA(this, gdata);

    if (gdata == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }
    idata=PDATA(ImageData,im);
    if (idata == 0 || idata->xpixmap == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    gdata->gc = 0;
    gdata->win = 0;
    gdata->drawable = idata->xpixmap;
    gdata->clipset = False;
}


void
awt_WServer_graphicsSetFont(Hawt_WServer *ws,
			  Hawt_Graphics *this,
			  Hawt_Font *font)
{
    struct FontData	*fdata;
    struct GraphicsData	*gdata;
    struct WServerData *wsdata = PDATA(WServerData,ws);
    char		*err;

    if (this == 0 || font == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);
    fdata = awt_GetFontData(ws, font, &err);
    if (fdata == 0) {
	SignalError(0, err, 0);
	return;
    }

    INIT_GC(wsdata->display, gdata);
    XSetFont(wsdata->display, gdata->gc, fdata->xfont->fid);
}

void
awt_WServer_graphicsSetForeground(Hawt_WServer *ws,
				Hawt_Graphics *this,
				Hawt_Color *c)
{
    struct GraphicsData		*gdata;

    if (this == 0 || c == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);

    INIT_GC(DSPU(ws), gdata);
    XSetForeground(DSPU(ws), gdata->gc, awt_getColor(ws, c));
}

void
awt_WServer_graphicsSetBackground(Hawt_WServer *ws,
				Hawt_Graphics *this,
				Hawt_Color *c)
{
    struct GraphicsData		*gdata;

    if (this == 0 || c == 0 || unhand(c) == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata =PDATA(GraphicsData,this);

    INIT_GC(DSPU(ws), gdata);
    XSetBackground(DSPU(ws), gdata->gc, awt_getColor(ws, c));
}


void
awt_WServer_graphicsDispose(Hawt_WServer *ws,
			  Hawt_Graphics *this)
{
    struct GraphicsData		*gdata;
    struct WServerData		*wsdata = PDATA(WServerData,ws);

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);

    if (gdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if (gdata->gc) {
	XFreeGC(wsdata->display, gdata->gc);
    }
    free(gdata);
    SET_PDATA(this,0);
}


void
awt_WServer_graphicsDrawString(Hawt_WServer *ws,
			     Hawt_Graphics *this,
			     Hjava_lang_String *text,
			     long x,
			     long y)
{
    ushort			*data;
    int				length;
    Classjava_lang_String	*tptr;
    struct GraphicsData		*gdata;

    if (this == 0 || text==0 || (tptr = unhand(text)) == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);
    INIT_GC(DSPU(ws), gdata);
    data = unhand(tptr->value)->body + tptr->offset;
    length = javaStringLength(text);
    XDrawString16(DSPU(ws),
		  gdata->drawable,
		  gdata->gc,
		  TX(this,x),
		  TY(this,y),
		  data,
		  length);
}

long
awt_WServer_graphicsDrawStringWidth(Hawt_WServer *ws,
				    Hawt_Graphics *this,
				    Hjava_lang_String *text,
				    long x,
				    long y)
{
    ushort			*data;
    int				length;
    Classjava_lang_String	*tptr;
    struct GraphicsData		*gdata;
    struct FontData		*fdata;
    char			*err;

    if (this == 0 || text==0 || (tptr = unhand(text)) == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    fdata = awt_GetFontData(ws, unhand(this)->font, &err);
    if (fdata == 0) {
	SignalError(0, err, 0);
	return -1;
    }

    gdata = PDATA(GraphicsData,this);
    INIT_GC2(DSPU(ws), gdata,-1);
    data = unhand(tptr->value)->body + tptr->offset;
    length = javaStringLength(text);
    XDrawString16(DSPU(ws),
		  gdata->drawable,
		  gdata->gc,
		  TX(this,x),
		  TY(this,y),
		  data,
		  length);
    return XTextWidth16(fdata->xfont, data, length);
}

void
awt_WServer_graphicsDrawChars(Hawt_WServer *ws,
			      Hawt_Graphics *this,
			      HArrayOfChar *text,
			      long offset,
			      long length,
			      long x,
			      long y)
{
    ushort			*data;
    struct GraphicsData		*gdata;

    if (this == 0 || text == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    if (offset < 0 || length < 0 || offset + length > obj_length(text)) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }
    gdata = PDATA(GraphicsData,this);
    INIT_GC(DSPU(ws), gdata);
    data = unhand(text)->body + offset;
    XDrawString16(DSPU(ws),
		  gdata->drawable,
		  gdata->gc,
		  TX(this,x),
		  TY(this,y),
		  data,
		  length);
}

long
awt_WServer_graphicsDrawCharsWidth(Hawt_WServer *ws,
				   Hawt_Graphics *this,
				   HArrayOfChar *text,
				   long offset,
				   long length,
				   long x,
				   long y)
{
    ushort			*data;
    struct GraphicsData		*gdata;
    struct FontData		*fdata;
    char			*err;

    if (this == 0 || text == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    fdata = awt_GetFontData(ws, unhand(this)->font, &err);
    if (fdata == 0) {
	SignalError(0, err, 0);
	return;
    }

    if (offset < 0 || length < 0 || offset + length > obj_length(text)) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }
    gdata = PDATA(GraphicsData,this);
    INIT_GC2(DSPU(ws), gdata,-1);
    data = unhand(text)->body + offset;
    XDrawString16(DSPU(ws),
		  gdata->drawable,
		  gdata->gc,
		  TX(this,x),
		  TY(this,y),
		  data,
		  length);
    return XTextWidth16(fdata->xfont, data, length);
}

void
awt_WServer_graphicsDrawBytes(Hawt_WServer *ws,
			      Hawt_Graphics *this,
			      HArrayOfByte *text,
			      long offset,
			      long length,
			      long x,
			      long y)
{
    char			*data;
    struct GraphicsData		*gdata;

    if (this == 0 || text == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    if (offset < 0 || length < 0 || offset + length > obj_length(text)) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }
    gdata = PDATA(GraphicsData,this);
    INIT_GC(DSPU(ws), gdata);
    data = unhand(text)->body + offset;
    XDrawString(DSPU(ws),
		gdata->drawable,
		gdata->gc,
		TX(this,x),
		TY(this,y),
		data,
		length);
}

long
awt_WServer_graphicsDrawBytesWidth(Hawt_WServer *ws,
				   Hawt_Graphics *this,
				   HArrayOfByte *text,
				   long offset,
				   long length,
				   long x,
				   long y)
{
    char			*data;
    struct GraphicsData		*gdata;
    struct FontData		*fdata;
    char			*err;

    if (this == 0 || text == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    fdata = awt_GetFontData(ws, unhand(this)->font, &err);
    if (fdata == 0) {
	SignalError(0, err, 0);
	return;
    }

    if (offset < 0 || length < 0 || offset + length > obj_length(text)) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    }
    gdata = PDATA(GraphicsData,this);
    INIT_GC2(DSPU(ws), gdata,-1);
    data = unhand(text)->body + offset;
    XDrawString(DSPU(ws),
		gdata->drawable,
		gdata->gc,
		TX(this,x),
		TY(this,y),
		data,
		length);
    return XTextWidth(fdata->xfont, data, length);
}


void
awt_WServer_graphicsDrawLine(Hawt_WServer *ws,
			   Hawt_Graphics *this,
			   long x1, long y1,
			   long x2, long y2)
{
    struct GraphicsData	*gdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);
    INIT_GC(DSPU(ws), gdata);
    XDrawLine(DSPU(ws),
	      gdata->drawable,
	      gdata->gc,
	      TX(this,x1), TY(this,y1),
	      TX(this,x2), TY(this,y2));
}

void
awt_WServer_graphicsClearRect(Hawt_WServer *ws,
			    Hawt_Graphics *this,
			    long x,
			    long y,
			    long w,
			    long h)
{
    struct GraphicsData	*gdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);
    INIT_GC(DSPU(ws), gdata);
    XClearArea(DSPU(ws),
	       gdata->drawable,
	       TX(this,x), TY(this,y), TW(this,w), TH(this,h), False);
}

void
awt_WServer_graphicsFillRect(Hawt_WServer *ws,
			   Hawt_Graphics *this,
			   long x,
			   long y,
			   long w,
			   long h)
{
    struct GraphicsData	*gdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata =PDATA(GraphicsData,this);
    INIT_GC(DSPU(ws), gdata);
    XFillRectangle(DSPU(ws),
		   gdata->drawable,
		   gdata->gc,
		   TX(this,x), TY(this,y), TW(this,w), TH(this,h));
}

void
awt_WServer_graphicsDrawRect(Hawt_WServer *ws,
			   Hawt_Graphics *this,
			   long x,
			   long y,
			   long w,
			   long h)
{
    struct GraphicsData	*gdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);
    INIT_GC(DSPU(ws),gdata);
    XDrawRectangle(DSPU(ws),
		   gdata->drawable,
		   gdata->gc,
		   TX(this,x), TY(this,y), TW(this,w), TH(this,h));
}

void
awt_WServer_graphicsSetLineWidth(Hawt_WServer *ws,
			       Hawt_Graphics *this,
			       long n)
{
    struct WServerData *wsdata = PDATA(WServerData,ws);
    struct GraphicsData	*gdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);

    INIT_GC(wsdata->display,gdata);
    XSetLineAttributes(wsdata->display,
		       gdata->gc, n,
		       LineSolid,
		       CapButt,
		       JoinMiter);
}

void
awt_WServer_graphicsPaintMode(Hawt_WServer *ws, Hawt_Graphics *this)
{
    struct WServerData *wsdata = PDATA(WServerData,ws);
    struct GraphicsData	*gdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);

    INIT_GC(wsdata->display, gdata);
    XSetFunction(wsdata->display, gdata->gc, GXcopy);
}

void
awt_WServer_graphicsInvertMode(Hawt_WServer *ws, Hawt_Graphics *this)
{
    struct WServerData *wsdata = PDATA(WServerData,ws);
    struct GraphicsData	*gdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);

    INIT_GC(wsdata->display, gdata);
    XSetFunction(wsdata->display, gdata->gc, GXinvert);
}

void
awt_WServer_graphicsFlipColorsMode(Hawt_WServer *ws,
				 Hawt_Graphics *this,
				 Hawt_Color *c1,
				 Hawt_Color *c2)
{
    struct WServerData *wsdata = PDATA(WServerData,ws);
    Pixel p1;
    Pixel p2;
    struct GraphicsData	*gdata;

    if (this == 0 || c1 == 0 || c2 == 0 ) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    p1 = awt_getColor(ws, c1);
    p2 = awt_getColor(ws, c2);
    gdata = PDATA(GraphicsData,this);

    INIT_GC(wsdata->display, gdata);
    XSetFunction(wsdata->display, gdata->gc, GXxor);
    XSetForeground(wsdata->display, gdata->gc, p1 ^ p2);
}


void
awt_WServer_graphicsXorMode(Hawt_WServer *ws, Hawt_Graphics *this)
{
    struct WServerData *wsdata = PDATA(WServerData,ws);
    struct GraphicsData	*gdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);

    INIT_GC(wsdata->display, gdata);
    XSetFunction(wsdata->display, gdata->gc, GXxor);
}

void
awt_WServer_graphicsCopyInvertedMode(Hawt_WServer *ws, Hawt_Graphics *this)
{
    struct WServerData *wsdata = PDATA(WServerData,ws);
    struct GraphicsData	*gdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);

    INIT_GC(wsdata->display, gdata);
    XSetFunction(wsdata->display, gdata->gc, GXcopyInverted);
}

void
awt_WServer_graphicsClipRect(Hawt_WServer *ws,
			     Hawt_Graphics *this,
			     long x, long y,
			     long w, long h)
{
    struct WServerData	*wsdata = PDATA(WServerData,ws);
    struct GraphicsData	*gdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);

    INIT_GC(wsdata->display, gdata);

    gdata->cliprect.x = (Position)TX(this,x);
    gdata->cliprect.y = (Position)TY(this,y);
    gdata->cliprect.width = (Dimension)TW(this,w);
    gdata->cliprect.height = (Dimension)TH(this,h);
    gdata->clipset = True;

    XSetClipRectangles(wsdata->display,
		       gdata->gc,
		       0, 0,
		       &gdata->cliprect, 1,
		       YXBanded);
}


void
awt_WServer_graphicsClearClip(Hawt_WServer *ws,
			      Hawt_Graphics *this)
{
    struct WServerData *wsdata = PDATA(WServerData,ws);
    struct GraphicsData	*gdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    gdata = PDATA(GraphicsData,this);

    INIT_GC(wsdata->display, gdata);
    gdata->clipset = False;
    XSetClipMask(wsdata->display, gdata->gc, None);
}


void
awt_WServer_graphicsCopyArea(Hawt_WServer *ws,
			     Hawt_Graphics *this,
			     long x,
			     long y,
			     long width,
			     long height,
			     long dx,
			     long dy)
{
   struct GraphicsData	*gdata;
   struct WServerData	*wsdata = PDATA(WServerData,ws);
   Window	w;

   if (this == 0) {
       SignalError(0, JAVAPKG "NullPointerException", 0);
       return;
   }

   gdata = PDATA(GraphicsData,this);
   INIT_GC(wsdata->display, gdata);

   XCopyArea(wsdata->display, gdata->drawable, gdata->drawable, gdata->gc,
	     TX(this,x), TY(this,y), TW(this,width), TH(this,height),
	     TX(this,dx), TY(this, dy));
}


void
awt_WServer_graphicsSetOrigin(Hawt_WServer *ws,
			      Hawt_Graphics *this,
			      long x,
			      long y)
{
}

void
awt_WServer_graphicsSetScaling(Hawt_WServer *ws,
			       Hawt_Graphics *this,
			       float sx,
			       float sy)
{
}


void
awt_WServer_offscreenImageCreate(Hawt_WServer *ws,
				 Hawt_Image *this,
				 long w,
				 long h)
{
    Classawt_Image	*I;
    struct ImageData	*idata;
    Hawt_Window		*Wh;
    struct WServerData	*wsdata = PDATA(WServerData,ws);
    struct WindowData	*wdata;
    Window		win;

    if (this == 0 || wsdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    I = unhand(this);
    Wh = (Hawt_Window *) I->win;
    if (Wh == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(WindowData,Wh);
    idata = (struct ImageData *)malloc(sizeof(struct ImageData));
    SET_PDATA(this, idata);

    I->width = w;
    I->height = h;
    win = XtWindow(wdata->w);
    if (win==0) {
	win = wsdata->root;
    }
    idata->xpixmap =  XCreatePixmap(wsdata->display, win,
				    w, h, awtImage->Depth);
    idata->xmask = 0;
}


void
awt_WServer_imageCreate(Hawt_WServer *ws,
			Hawt_Image *this,
			Hawt_DIBitmap *dibh)
{
    int w, h;
    u_long *ddb;
    u_char *r, *g, *b, *p;
    XImage *xim;
    Classawt_Image	*I;
    struct ImageData	*idata;
    Hawt_Window		*Wh;
    struct WServerData	*wsdata = PDATA(WServerData,ws);
    struct WindowData	*wdata;
    Classawt_DIBitmap	*dib;
    Window		win;
    int			i, j;
    int			bpp;

    if (dibh == 0 || this == 0 || wsdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    I = unhand(this);
    Wh = (Hawt_Window *) I->win;
    if (Wh == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(WindowData,Wh);
    dib = unhand(dibh);

    if (dib->width==0 || dib->height==0 || dib->raster==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    w = dib->width;
    h = dib->height;
    assert((awtImage->awtBitsPerPixel & 7) == 0);
    ddb = (u_long *) malloc(awtImage->awtBitsPerPixel / 8 * w * h);

    if (ddb == (u_long *) 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }
    r = (u_char *) unhand(dib->red);
    g = (u_char *) unhand(dib->green);
    b = (u_char *) unhand(dib->blue);
    p = (u_char *) unhand(dib->raster);

    awtImage->Ditherfrom8(r, g, b, p, 0, ddb, 0, w, h, dib->trans_index);

    bpp = awtImage->wsImageFormat.bits_per_pixel;
    if (awtImage->awtBitsPerPixel == bpp) {
	xim = XCreateImage(wsdata->display, wsdata->visual,
			   awtImage->Depth, ZPixmap, 0, (u_char *) ddb,
			   w, h, bpp, 0);
    } else {
	u_long *ddb2;
	u_char *p;
	int slp = awtImage->wsImageFormat.scanline_pad;

	ddb2 = (u_long *) malloc((bpp * w + slp - 1) / 8 * h);
	if (ddb2 == (u_long *) 0) {
	    free(ddb);
	    SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	    return;
	}

	xim = XCreateImage(wsdata->display, wsdata->visual,
			   awtImage->Depth, ZPixmap, 0, (u_char *) ddb2,
			   w, h, slp, 0);

	assert(awtImage->awtBitsPerPixel == 8);
	p = (u_char *) ddb;
	for (j = 0; j < h; j++) {
	    for (i = 0; i < w; i++) {
		XPutPixel(xim, i, j, *p++);
	    }
	}
	free(ddb);
	ddb = ddb2;
    }

    idata = (struct ImageData *)malloc(sizeof(struct ImageData));
    SET_PDATA(this, idata);

    I->width = w;
    I->height = h;
    win = XtWindow(wdata->w);
    if (win==0) {
	win = wsdata->root;
    }
    idata->xpixmap =  XCreatePixmap(wsdata->display, win,
				    w, h, awtImage->Depth);

    XPutImage(wsdata->display, idata->xpixmap,
	      wsdata->imagegc, xim, 0, 0, 0, 0, w, h);
    free(ddb);
    XFree(xim);

    if (dib->trans_index >= dib->num_colors) {
	idata->xmask = None;
    } else {
	/* bpsl could be made (w+31)/32 for performance. */
	int bpsl = (w + 7) / 8;
	u_char *ddm = (u_char *) malloc(h * bpsl);
	u_char *b = ddm;
	u_char *p = (u_char *) unhand(dib->raster);
	u_char t = dib->trans_index;
	int any = 0;


	if (ddm == (u_char *) 0) {
	    SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	    return;
	}

	/*
	 * Scan through the pixels building a 1 bit deep mask where the
	 * 1's are where the pixels are NOT equal to the transparent
	 * index.  We check to see if 'any' pixels are transparent, if
	 * NOT, we just chuck the raster since it would be full of 0xff's
	 * anyway.  This could probably be made a lot faster using 32 bit
	 * chunks instead of 8.
	 */

	for (j = 0; j < h; j++) {
	    u_char out = 0;
	    u_char mask = 0x80;
	    for (i = 0; i < w; i++) {
		if (*p++ == t) {
		    any = 1;
		} else {
		    out |= mask;
		}
		mask >>= 1;
		if (mask == 0) {
		    *b++ = out;
		    mask = 0x80;
		    out = 0;
		}
	    }
	    if (mask != 0x80)	/* got a partially built mask in out. */
		*b++ = out;
	}

	if (any) {
	    xim = XCreateImage(wsdata->display, wsdata->visual, 1,
			       XYBitmap, 0, ddm, w, h, 8, bpsl);
	    xim->bitmap_bit_order = MSBFirst;
	    xim->bitmap_unit = 8;
	    idata->xmask = XCreatePixmap(wsdata->display,
					 wsdata->root,
					 w, h, 1);
	    XPutImage(wsdata->display,
		      idata->xmask,
		      wsdata->maskgc, xim, 0, 0, 0, 0, w, h);
	    XFree(xim);
	} else {
	    idata->xmask = None;
	}
	free(ddm);
    }
    XFlush(wsdata->display);
}

void
awt_WServer_scaledImageCreate(Hawt_WServer *ws,
			      Hawt_Image *this,
			      Hawt_DIBitmap *dibh,
			      long srcx, long srcy, long srcw, long srch,
			      long dstw, long dsth)
{
    u_long *ddb;
    u_char *dtb, *r, *g, *b, *p;
    XImage *xim;
    Classawt_Image	*I;
    struct ImageData	*idata;
    Hawt_Window		*Wh;
    struct WServerData	*wsdata = PDATA(WServerData,ws);
    struct WindowData	*wdata;
    Classawt_DIBitmap	*dib;
    Window		win;
    int			i, j;
    int			bpp;

    if (dibh == 0 || this == 0 || wsdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    I = unhand(this);
    Wh = (Hawt_Window *) I->win;
    if (Wh == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(WindowData,Wh);
    dib = unhand(dibh);

    if (srcx < 0 || srcy < 0 || srcw <= 0 || srch <= 0
	|| srcx+srcw > dib->width || srcy+srch > dib->height
	|| dstw <= 0 || dsth <= 0 || dib->raster == 0)
    {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    assert((awtImage->awtBitsPerPixel & 7) == 0);
    ddb = (u_long *) malloc(awtImage->awtBitsPerPixel / 8 * dstw * dsth);
    if (ddb == (u_long *) 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }
    dtb = (u_char *) malloc(dstw * dsth);
    if (dtb == (u_char *) 0) {
	free(ddb);
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }

    r = (u_char *) unhand(dib->red);
    g = (u_char *) unhand(dib->green);
    b = (u_char *) unhand(dib->blue);
    p = (u_char *) unhand(dib->raster);
    p += srcy * dib->width + srcx;

    awtImage->DitherScalefrom8(r, g, b,
			       p, dib->width - srcw, srcw, srch,
			       ddb, dtb, 0, dstw, dsth,
			       dib->trans_index);

    bpp = awtImage->wsImageFormat.bits_per_pixel;
    if (awtImage->awtBitsPerPixel == bpp) {
	xim = XCreateImage(wsdata->display, wsdata->visual,
			   awtImage->Depth, ZPixmap, 0, (u_char *) ddb,
			   dstw, dsth, bpp, 0);
    } else {
	u_long *ddb2;
	u_char *p;
	int slp = awtImage->wsImageFormat.scanline_pad;

	ddb2 = (u_long *) malloc((bpp * dstw + slp - 1) / 8 * dsth);
	if (ddb2 == (u_long *) 0) {
	    free(ddb);
	    free(dtb);
	    SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	    return;
	}

	xim = XCreateImage(wsdata->display, wsdata->visual,
			   awtImage->Depth, ZPixmap, 0, (u_char *) ddb2,
			   dstw, dsth, slp, 0);

	assert(awtImage->awtBitsPerPixel == 8);
	p = (u_char *) ddb;
	for (j = 0; j < dsth; j++) {
	    for (i = 0; i < dstw; i++) {
		XPutPixel(xim, i, j, *p++);
	    }
	}
	free(ddb);
	ddb = ddb2;
    }

    idata = (struct ImageData *)malloc(sizeof(struct ImageData));
    SET_PDATA(this, idata);

    I->width = dstw;
    I->height = dsth;
    win = XtWindow(wdata->w);
    if (win==0) {
	win = RootWindow(wsdata->display, DefaultScreen(wsdata->display));
    }
    idata->xpixmap =  XCreatePixmap(wsdata->display, win,
				    dstw, dsth, awtImage->Depth);

    XPutImage(wsdata->display, idata->xpixmap,
	      wsdata->imagegc, xim, 0, 0, 0, 0, dstw, dsth);
    free(ddb);
    XFree(xim);

    if (dib->trans_index >= dib->num_colors) {
	idata->xmask = None;
    } else {
	/* bpsl could be made (dstw+31)/32 for performance. */
	int bpsl = (dstw + 7) / 8;
	u_char *ddm = (u_char *) malloc(dsth * bpsl);
	u_char *b = ddm;
	u_char *p = dtb;
	int any = 0;


	if (ddm == (u_char *) 0) {
	    free(dtb);
	    SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	    return;
	}

	/*
	 * Scan through the pixels building a 1 bit deep mask where the
	 * 1's are where the pixels are NOT at least half transparent.
	 * We check to see if 'any' pixels are transparent, if
	 * NOT, we just chuck the raster since it would be full of 0xff's
	 * anyway.  This could probably be made a lot faster using 32 bit
	 * chunks instead of 8, and storing the transparency accumulations
	 * more efficiently than one byte per pixel.
	 */

	for (j = 0; j < dsth; j++) {
	    u_char out = 0;
	    u_char mask = 0x80;
	    for (i = 0; i < dstw; i++) {
		if (*p++) {
		    any = 1;
		} else {
		    out |= mask;
		}
		mask >>= 1;
		if (mask == 0) {
		    *b++ = out;
		    mask = 0x80;
		    out = 0;
		}
	    }
	    if (mask != 0x80)	/* got a partially built mask in out. */
		*b++ = out;
	}

	if (any) {
	    xim = XCreateImage(wsdata->display, wsdata->visual, 1,
			       XYBitmap, 0, ddm, dstw, dsth, 8, bpsl);
	    xim->bitmap_bit_order = MSBFirst;
	    xim->bitmap_unit = 8;
	    idata->xmask = XCreatePixmap(wsdata->display,
					 wsdata->root,
					 dstw, dsth, 1);
	    XPutImage(wsdata->display,
		      idata->xmask,
		      wsdata->maskgc, xim, 0, 0, 0, 0, dstw, dsth);
	    XFree(xim);
	} else {
	    idata->xmask = None;
	}
	free(ddm);
    }
    free(dtb);
    XFlush(wsdata->display);
}

#define RBITS	3
#define GBITS	3
#define BBITS	2
#define BYTEMASK(n)	((1 << (n)) - 1)
#define RBYTEMASK	BYTEMASK(RBITS)
#define GBYTEMASK	BYTEMASK(GBITS)
#define BBYTEMASK	BYTEMASK(BBITS)
#define RBYTEOFF	(8 - RBITS)
#define GBYTEOFF	(RBYTEOFF - GBITS)
#define BBYTEOFF	(GBYTEOFF - BBITS)
#define RPIXOFF		(awtImage->rOff)
#define GPIXOFF		(awtImage->gOff)
#define BPIXOFF		(awtImage->bOff)

void
awt_WServer_bitmapRetrieve(Hawt_WServer *ws,
			   Hawt_Image *this,
			   Hawt_DIBitmap *dibh)
{
    int w, h;
    int i, j;
    u_char *r, *g, *b, *p;
    XImage *xim;
    Classawt_Image	*I;
    struct ImageData	*idata;
    Hawt_Window		*Wh;
    struct WServerData	*wsdata = PDATA(WServerData,ws);
    struct WindowData	*wdata;
    Classawt_DIBitmap	*dib;

    if (dibh == 0 || this == 0 || wsdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    I = unhand(this);
    idata = PDATA(ImageData,this);
    Wh = (Hawt_Window *) I->win;
    if (Wh == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(WindowData,Wh);
    dib = unhand(dibh);

    w = dib->width;
    h = dib->height;
    if (w != I->width || h != I->height || dib->num_colors != 256) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
    }
    if (w == 0 || h == 0 || dib->raster == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    r = (u_char *) unhand(dib->red);
    g = (u_char *) unhand(dib->green);
    b = (u_char *) unhand(dib->blue);
    p = (u_char *) unhand(dib->raster);

    if (awtImage->Depth == 24) {
	/* 8 x 8 x 4 color cube is easy to calculate... */
	xim = XGetImage(wsdata->display, idata->xpixmap,
			0, 0, w, h, ~0, ZPixmap);
	for (j = 0; j < h; j++) {
	    for (i = 0; i < w; i++) {
		int pixel = XGetPixel(xim, i, j);
		int r = ((pixel >> RPIXOFF) & 0xff) >> (8 - RBITS);
		int g = ((pixel >> GPIXOFF) & 0xff) >> (8 - GBITS);
		int b = ((pixel >> BPIXOFF) & 0xff) >> (8 - BBITS);
		*p++ = (r << RBYTEOFF) | (g << GBYTEOFF) | (b << BBYTEOFF);
	    }
	}
	for (i = 0; i < 256; i++) {
	    r[i] = ((i >> RBYTEOFF) & RBYTEMASK) << (8 - RBITS);
	    g[i] = ((i >> GBYTEOFF) & GBYTEMASK) << (8 - GBITS);
	    b[i] = ((i >> BBYTEOFF) & BBYTEMASK) << (8 - BBITS);
	}
    } else {
	/* XGetSubImage takes care of expanding to 8 bit if necessary. */
	/* But, that is not a supported feature.  There are better ways */
	/* of handling this... */
	xim = XCreateImage(wsdata->display, wsdata->visual,
			   8, ZPixmap, 0, p, w, h, 8, w);
	XGetSubImage(wsdata->display, idata->xpixmap,
		     0, 0, w, h, ~0, ZPixmap, xim, 0, 0);
	XFree(xim);
	awt_retrieve_colors(r, g, b);
    }

    if (idata->xmask == None) {
	dib->trans_index = dib->num_colors;
    } else {
	u_char pixels_used[256];

	memset(pixels_used, 0, 256);
	for (i = w * h; --i >= 0; ) {
	    pixels_used[*p++] = 1;
	}
	for (i = 0; i < 256; i++) {
	    if (!pixels_used[i])
		break;
	}
	if (i < 256)
	    dib->trans_index = i;
	else
	    dib->trans_index = awtImage->ColorMatch(255, 255, 255);

	xim = XGetImage(wsdata->display, idata->xmask, 0, 0, w, h, 1, ZPixmap);
	p = (u_char *) unhand(dib->raster);
	for (j = 0; j < h; j++) {
	    for (i = 0; i < w; i++) {
		if (!XGetPixel(xim, i, j))
		    *p = dib->trans_index;
		p++;
	    }
	}
	XDestroyImage(xim);
    }
    XFlush(wsdata->display);
}

void
awt_WServer_imageDispose(Hawt_WServer *ws,
			 Hawt_Image *this)
{
    Classawt_Image	*img;
    struct ImageData	*idata;
    struct WServerData	*wsdata = PDATA(WServerData,ws);
    Display		*dpy;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    img = unhand(this);
    idata = PDATA(ImageData, this);
    if (idata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if (idata->xmask != 0) {
	XFreePixmap(wsdata->display, idata->xmask);
	idata->xmask = 0;
    }
    if (idata->xpixmap != 0) {
	XFreePixmap(wsdata->display, idata->xpixmap);
	idata->xpixmap = 0;
    }
    free((char *) idata);
    SET_PDATA(this,0);
}


void
awt_WServer_graphicsDrawImage(Hawt_WServer *ws,
			      Hawt_Graphics *this,
			      Hawt_Image *Ih,
			      long x, long y)
{
    Classawt_Image	*I;
    struct		GraphicsData *gdata;
    struct		WServerData *wsdata;
    struct		ImageData *idata;
    Display		*dpy;
    long		tx;
    long		ty;
    long		wx;
    long		wy;
    long		ix;
    long		iy;
    long		iw;
    long		ih;

    if (this == 0 || Ih == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    wsdata = PDATA(WServerData,ws);
    gdata = PDATA(GraphicsData,this);
    INIT_GC(wsdata->display,gdata);
    I = unhand(Ih);
    idata = PDATA(ImageData,Ih);
    if (idata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", "pixmap was destroyed");
	return;
    } else {
	wx = tx = TX(this,x);
	wy = ty = TY(this,y);
	ix = 0;
	iy = 0;
	iw = I->width;
	ih = I->height;
	if (gdata->clipset) {
	    if (wx < gdata->cliprect.x) {
		if (wx + iw <= gdata->cliprect.x)
		    return;
		ix = (gdata->cliprect.x - wx);
		iw -= ix;
		wx = gdata->cliprect.x;
	    }
	    if (wy < gdata->cliprect.y) {
		if (wy + ih <= gdata->cliprect.y)
		    return;
		iy = (gdata->cliprect.y - wy);
		ih -= iy;
		wy = gdata->cliprect.y;
	    }
	    if (wx + iw > gdata->cliprect.x + (int) gdata->cliprect.width) {
		if (wx >= gdata->cliprect.x + (int) gdata->cliprect.width)
		    return;
		iw = gdata->cliprect.x + gdata->cliprect.width - wx;
	    }
	    if (wy + ih > gdata->cliprect.y + (int) gdata->cliprect.height) {
		if (wy >= gdata->cliprect.y + (int) gdata->cliprect.height)
		    return;
		ih = gdata->cliprect.y + gdata->cliprect.height - wy;
	    }
	}
	XSetClipMask(wsdata->display, gdata->gc, idata->xmask);
	XSetClipOrigin(wsdata->display, gdata->gc, tx, ty);
	if (gdata->drawable == 0) {
	    printf("DRAWABLE is NULL\n");
	    return;
	}
	XCopyArea(wsdata->display,
		  idata->xpixmap, gdata->drawable, gdata->gc,
		  ix, iy, iw, ih, wx, wy);
	if (gdata->clipset) {
	    XSetClipRectangles(wsdata->display,
			       gdata->gc,
			       0, 0,
			       &gdata->cliprect, 1,
			       YXBanded);
	} else {
	    XSetClipMask(wsdata->display, gdata->gc, None);
	}
    }
}
