/*
 * @(#)awt_Label.c	1.29 95/05/12 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"

void
awt_WServer_labelCreate(Hawt_WServer *ws,
			Hawt_Label *this,
			Hjava_lang_String *label,
			Hawt_Window *parent,
			Hawt_Font *font)
{
    char		*fontTag = XmFONTLIST_DEFAULT_TAG;
    Classawt_Label	*thisPtr;
    struct ButtonData	*bdata;
    struct WindowData	*wdata;
    Pixel		bg;
    Dimension		w, h, marginH, marginW, borderW;
    char		*clabel;
    XmFontList		fontlist;
    struct FontData	*fdata;

    if (label == 0 || parent == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    thisPtr = unhand(this);
    wdata = PDATA(WindowData,parent);
    bg = awt_getColor(ws, unhand(parent)->background);
    clabel = makeCString(label);
    bdata = (struct ButtonData *)malloc(sizeof(struct ButtonData));
    if (bdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    SET_PDATA(this, bdata);
    if (font != 0 && (fdata = awt_GetFontData(ws, font, 0)) != 0) {
	fontlist = XmFontListCreate(fdata->xfont, "charset1");
	bdata->button = XtVaCreateManagedWidget(clabel,
						xmLabelWidgetClass, wdata->w,
						XmNbackground, bg,
						XmNx, 0,
						XmNy, 0,
						XmNrecomputeSize,False,
						XmNalignment, XmALIGNMENT_BEGINNING,
						XmNfontList, fontlist,
						NULL);
	XmFontListFree(fontlist);
    } else {
	bdata->button = XtVaCreateManagedWidget(clabel,
						xmLabelWidgetClass, wdata->w,
						XmNx, 0,
						XmNy, 0,
						XmNbackground, bg,
						XmNrecomputeSize, False,
						XmNalignment, XmALIGNMENT_BEGINNING,
						NULL);
    }
    awt_util_getSizes(bdata->button,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
    if (font != 0) {
	thisPtr->height = fdata->xfont->ascent + fdata->xfont->descent;
    } else {
	thisPtr->height = 12;
    }
    thisPtr->x = 0;
    thisPtr->y = 0;
}

void 
awt_WServer_labelSetText(Hawt_WServer *ws,
			 Hawt_Label *this,
			 Hjava_lang_String *label)
{
    struct WServerData *wsdata =PDATA(WServerData,ws);
    char		*clabel;
    char		*clabelEnd;
    struct ButtonData *bdata;
    XmString		xim;
    Classawt_Label	*thisPtr = unhand(this);

    if (label == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    clabel = allocCString(label);
    /* scan for any \n's and terminate the string at that point */
    clabelEnd = strchr(clabel,'\n');
    if (clabelEnd != 0) {
	*clabelEnd = 0;
    }
    bdata = PDATA(ButtonData,this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    xim = XmStringCreateLtoR(clabel, "labelFont");
    XtVaSetValues(bdata->button,
		  XmNlabelString,
		  xim,
		  NULL);
    free(clabel);
    XmStringFree(xim);
    XFlush(wsdata->display);
}

void 
awt_WServer_labelSetColor(Hawt_WServer *ws,
			  Hawt_Label *this,
			  Hawt_Color *c)
{
    struct ButtonData *bdata;
    Pixel		color;

    if (c == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    bdata = PDATA(ButtonData,this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    color = awt_getColor(ws, c);
    XtVaSetValues(bdata->button,
		  XmNforeground, color,
		  NULL);
}

void 
awt_WServer_labelSetFont(Hawt_WServer *ws,
			 Hawt_Label *this,
			 Hawt_Font *f)
{
    struct ButtonData *bdata;
    struct FontData *fdata;
    XmFontList	fontlist;
    char	*err;

    if (f == 0 || this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    fdata = awt_GetFontData(ws, f, &err);
    if (fdata == 0) {
	SignalError(0, err, 0);
	return;
    }
    bdata = PDATA(ButtonData,this);
    if (fdata==0 || fdata->xfont==0 || bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    fontlist = XmFontListCreate(fdata->xfont, "labelFont");
    if (fontlist != NULL) {
	XtVaSetValues(bdata->button,
		      XmNfontList, fontlist,
		      NULL);
	XmFontListFree(fontlist);
    } else {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
}


void 
awt_WServer_labelMoveTo(Hawt_WServer *ws,
			Hawt_Label *this,
			long x,
			long y)
{
    struct ButtonData *bdata;

    bdata = PDATA(ButtonData,this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_move(bdata->button, x, y);
}

void 
awt_WServer_labelShow(Hawt_WServer *ws,
		      Hawt_Label *this)
{
    struct ButtonData *bdata;

    bdata = PDATA(ButtonData,this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_show(bdata->button);
}

void 
awt_WServer_labelHide(Hawt_WServer *ws,
		      Hawt_Label *this)
{
    struct ButtonData *bdata;

    bdata = PDATA(ButtonData,this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_hide(bdata->button);
}


void 
awt_WServer_labelReshape(Hawt_WServer *ws,
			 Hawt_Label *this,
			 long x,
			 long y,
			 long w,
			 long h)
{
    struct ButtonData *bdata;
    Classawt_Label	*thisPtr;

    bdata = PDATA(ButtonData,this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_reshape(bdata->button, x, y, w, h);
}

void 
awt_WServer_labelDispose(Hawt_WServer *ws, Hawt_Label *this)
{
    struct ButtonData *bdata;

    bdata = PDATA(ButtonData,this);

    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtDestroyWidget(bdata->button);
    free((void *)bdata);
    SET_PDATA(this, 0);
}

void
awt_WServer_labelDimensions(Hawt_WServer *ws,
			    Hawt_Label *this)
{
    struct ButtonData *bdata;
    Classawt_Label 	*thisPtr;
    long junk;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    bdata = PDATA(ButtonData, this);
    thisPtr = unhand(this);
    if (bdata == 0 || bdata->button==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_getSizes(bdata->button,
		      &(thisPtr->width),
		      &junk,
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
}
