/*
 * @(#)awt_List.c	1.36 95/02/08 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"

static void
Slist_callback(Widget w, XtPointer client_data, XtPointer call_data)
{
    struct ListData *sdata = (struct ListData *)client_data;
    XmListCallbackStruct *cbs = (XmListCallbackStruct *)call_data;

    switch (cbs->reason) {
      case XmCR_DEFAULT_ACTION:
	JAVA_UPCALL((EE(),
		    (void *)sdata->choiceData.handler,
		    "doubleClick","(Lawt/Component;I)V",
		    sdata->choiceData.target,
		    (cbs->item_position - 1)));
	break;
      case XmCR_SINGLE_SELECT:
	JAVA_UPCALL((EE(),
		    (void *)sdata->choiceData.handler,
		    "selected","(Lawt/Component;I)V",
		    sdata->choiceData.target,
		    (cbs->item_position - 1)));
	break;
      case XmCR_MULTIPLE_SELECT:
	JAVA_UPCALL((EE(),
		    (void *)sdata->choiceData.handler,
		    "selected","(Lawt/Component;I)V",
		    sdata->choiceData.target,
		    -1));
	break;
      default:
	break;
    }
}

void
awt_WServer_listCreate(Hawt_WServer *ws,
		       Hawt_List *this,
		       Hawt_Window *parent,
		       Hawt_ChoiceHandler *handler,
		       long visibleLines,
		       long multipleSelections,
		       long resizable)
{
    int			argc;
    Arg			args[40];
    Classawt_List	*thisPtr;
    struct WindowData	*wdata;
    struct ListData	*sdata;
    Pixel		bg;
    Dimension		w,h;

    if (parent == 0 || handler == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    thisPtr = unhand(this);
    wdata = PDATA(WindowData,parent);
    sdata = (struct ListData *)malloc(sizeof(struct ListData));
    SET_PDATA(this, sdata);
    if (sdata == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }
    XtVaGetValues(wdata->w,
		  XmNbackground, &bg,
		  NULL);
    sdata->choiceData.target = (Hawt_Component *)this;
    if (handler) {
	sdata->choiceData.handler = handler;
    } else {
	sdata->choiceData.handler = (Hawt_ChoiceHandler *)this;
    }
    thisPtr->x = thisPtr->y = 0;
    argc = 0;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    if (resizable) {
	XtSetArg(args[argc], XmNlistSizePolicy, XmRESIZE_IF_POSSIBLE); argc++;
    } else {
	XtSetArg(args[argc], XmNlistSizePolicy, XmCONSTANT); argc++;
    }
    XtSetArg(args[argc], XmNx, 0); argc++;
    XtSetArg(args[argc], XmNy, 0); argc++;
    XtSetArg(args[argc], XmNvisibleItemCount, visibleLines); argc++;
    XtSetArg(args[argc], XmNmarginTop, 0); argc++;
    XtSetArg(args[argc], XmNmarginBottom, 0); argc++;
    XtSetArg(args[argc], XmNmarginLeft, 0); argc++;
    XtSetArg(args[argc], XmNmarginRight, 0); argc++;
    XtSetArg(args[argc], XmNmarginHeight, 0); argc++;
    XtSetArg(args[argc], XmNmarginWidth, 0); argc++;
    XtSetArg(args[argc], XmNlistMarginHeight, 0); argc++;
    XtSetArg(args[argc], XmNlistMarginWidth, 0); argc++;
    XtSetArg(args[argc], XmNtraversalOn, False); argc++;
    XtSetArg(args[argc], XmNscrolledWindowMarginWidth, 0); argc++;
    XtSetArg(args[argc], XmNscrolledWindowMarginHeight, 0); argc++;
    if (multipleSelections) {
	XtSetArg(args[argc], XmNselectionPolicy, XmMULTIPLE_SELECT); argc++;
    } else {
	XtSetArg(args[argc], XmNselectionPolicy, XmSINGLE_SELECT); argc++;
    }
    sdata->slist = XmCreateScrolledList(wdata->w,
					"slist",
					args,
					argc);
/*
    XtVaSetValues(XtParent(sdata->slist),
		  XmNmarginTop, 0,
		  XmNmarginBottom, 0,
		  XmNmarginLeft, 0,
		  XmNmarginRight, 0,
		  XmNmarginHeight, 0,
		  XmNmarginWidth, 0,
		  XmNscrolledWindowMarginWidth, 0,
		  XmNscrolledWindowMarginHeight, 0,
		  NULL);
*/
    XtSetMappedWhenManaged(XtParent(sdata->slist), False);

    awt_util_getSizes(XtParent(sdata->slist),
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
    thisPtr->marginWidth = thisPtr->marginHeight = 0;

    XtAddCallback(sdata->slist,
		  XmNdefaultActionCallback,
		  Slist_callback,
		  (XtPointer)sdata);
    if (multipleSelections) {
	XtAddCallback(sdata->slist,
		      XmNmultipleSelectionCallback,
		      Slist_callback,
		      (XtPointer)sdata);
    } else {
	XtAddCallback(sdata->slist,
		      XmNsingleSelectionCallback,
		      Slist_callback,
		      (XtPointer)sdata);
    }
    XtManageChild(sdata->slist);
    XtManageChild(XtParent(sdata->slist));
}


long
awt_WServer_listIsSelected(Hawt_WServer *ws,
				    Hawt_List *this,
				    long pos)
{
    struct ListData *sdata;

    sdata = PDATA(ListData,this);
    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    pos++;
    if (XmListPosSelected(sdata->slist, pos) == True) {
	return 1;
    } else {
	return 0;
    }
}

void
awt_WServer_listAddItem(Hawt_WServer *ws,
				 Hawt_List *this,
				 Hjava_lang_String *item)
{
    Classawt_List	*thisPtr;
    XmString		im;
    Dimension		w,h;
    struct ListData	*sdata;
    int			nLines;
    int			nItems;

    if (item == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    sdata = PDATA(ListData,this);
    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    thisPtr = unhand(this);
    im = XmStringCreateLocalized(makeCString(item));
    XmListAddItemUnselected(sdata->slist, im, 0);
    XmStringFree(im);
    awt_util_getSizes(XtParent(sdata->slist),
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
    thisPtr->marginWidth = thisPtr->marginHeight = 0;
}
    
void
awt_WServer_listMakeVisible(Hawt_WServer *ws,
			    Hawt_List *this,
			    long pos)
{
    int top, visible;
    struct ListData	*sdata = PDATA(ListData,this);;

    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaGetValues(sdata->slist,
		  XmNtopItemPosition, &top,
		  XmNvisibleItemCount, &visible,
		  NULL);
    pos++;
    if (pos < top) {
	XmListSetPos(sdata->slist, pos);
    } else if (pos > (top + visible)) {
	XmListSetBottomPos(sdata->slist, pos);
    }
}


void
awt_WServer_listSetNVisible(Hawt_WServer *ws,
				     Hawt_List *this,
				     long nLines)
{
    struct ListData *sdata = PDATA(ListData,this);

    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaSetValues(sdata->slist,
		  XmNvisibleItemCount, nLines,
		  NULL);
}

void
awt_WServer_listSelect(Hawt_WServer *ws,
				Hawt_List *this,
				long pos)
{
    struct ListData *sdata = PDATA(ListData,this);

    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    pos++;
    XmListSelectPos(sdata->slist, pos);
}

void
awt_WServer_listDeselect(Hawt_WServer *ws,
				Hawt_List *this,
				long pos)
{
    struct ListData *sdata = PDATA(ListData,this);

    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    pos++;
    XmListDeselectPos(sdata->slist, pos);
}

void
awt_WServer_listDelItems(Hawt_WServer *ws,
				  Hawt_List *this,
				  long start,
				  long end)
{
    struct ListData *sdata = PDATA(ListData,this);

    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    start++;
    end++;
    if (start == end) {
	XmListDeletePos(sdata->slist, start);
    } else {
	XmListDeleteItemsPos(sdata->slist, end - start + 1, start);
    }
}	
    

void
awt_WServer_listMoveTo(Hawt_WServer *ws,
				Hawt_List *this,
				long x,
				long y)
{
    struct ListData *sdata = PDATA(ListData,this);
    Widget parent;

    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_move(XtParent(sdata->slist), x, y);
}


void
awt_WServer_listReshape(Hawt_WServer *ws,
				 Hawt_List *this,
				 long x,
				 long y,
				 long w,
				 long h)
{
    struct ListData *sdata = PDATA(ListData,this);
    Widget parent;
    Classawt_List *thisPtr;

    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    parent = XtParent(sdata->slist);
    XtUnmanageChild(parent);
    XtVaSetValues(parent,
		  XmNx, (Position)x,
		  XmNy, (Position)y,
		  XmNwidth, (Dimension)w,
		  XmNheight, (Dimension)h,
		  NULL);
    XtManageChild(parent);
    thisPtr = unhand(this);
    awt_util_getSizes(parent,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
    thisPtr->marginWidth = thisPtr->marginHeight = 0;
}

void
awt_WServer_listDispose(Hawt_WServer *ws,
				 Hawt_List *this)
{
    struct ListData *sdata = PDATA(ListData,this);

    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_hide(XtParent(sdata->slist));
    XtDestroyWidget(XtParent(sdata->slist));
    free((void *)sdata);
    SET_PDATA(this,0);
}

void 
awt_WServer_listShow(Hawt_WServer *ws, Hawt_List *this)
{
    struct ListData *sdata = PDATA(ListData,this);

    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    awt_util_show(XtParent(sdata->slist));
}

void 
awt_WServer_listHide(Hawt_WServer *ws, Hawt_List *this)
{
    struct ListData *sdata = PDATA(ListData,this);

    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_hide(XtParent(sdata->slist));
}

void
awt_WServer_listDimensions(Hawt_WServer *ws,
			   Hawt_List *this)
{
    struct ListData *sdata;
    Classawt_List 	*thisPtr;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    sdata = PDATA(ListData, this);
    thisPtr = unhand(this);
    if (sdata == 0 || sdata->slist==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_getSizes(XtParent(sdata->slist),
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
    thisPtr->marginWidth = thisPtr->marginHeight = 0;
}
