/*
 * @(#)awt_Menu.c	1.16 95/04/10 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"
#include <Xm/Separator.h>

void
awt_WServer_menuCreate(Hawt_WServer *ws,
		       Hawt_Menu *this,
		       Hjava_lang_String *title,
		       Hawt_MenuBar *menuBar,
		       long canTearOff)
{
    int			argc;
    Arg			args[10];
    char		*ctitle;
    struct MenuData	*mdata;
    struct MenuBarData	*mbdata;
    struct FontData	*fdata;
    Pixel		bg;
    Pixel		fg;
    Classawt_Menu	*menuPtr;
    Hawt_Font		*font;
    XmFontList		fontlist;
    Widget		tearOff;

    if (title == 0 || menuBar == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    mbdata = PDATA(MenuBarData,menuBar);
    ctitle = makeCString(title);
    mdata = (struct MenuData *)malloc(sizeof(struct MenuData));
    SET_PDATA(this, mdata);

    argc = 0;
    XtSetArg(args[argc], XmNbackground, &bg); argc++;
    XtGetValues(mbdata->menuBar, args, argc);

    argc = 0;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    if (canTearOff) {
	XtSetArg(args[argc], XmNtearOffModel, XmTEAR_OFF_ENABLED); argc++;
    }
    mdata->itemData.w = XmCreatePulldownMenu(mbdata->menuBar,
					     ctitle,
					     args,
					     argc);
    fg = awtImage->ColorMatch(0,0,0);
    if (canTearOff) {
	tearOff = XmGetTearOffControl(mdata->itemData.w);
    } else {
	tearOff = mdata->itemData.w;
    }
    XtVaSetValues(tearOff,
		  XmNbackground, bg,
		  XmNforeground, fg,
		  XmNhighlightColor, fg,
		  NULL);
    argc=0;
    XtSetArg(args[argc], XmNsubMenuId, mdata->itemData.w); argc++;
    XtSetArg(
	args[argc],
	XmNlabelString,
	XmStringCreateLtoR(ctitle,
			   XmSTRING_DEFAULT_CHARSET)); argc++;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    menuPtr = unhand(this);
    font = menuPtr->defaultFont;
    if (font != 0 && (fdata = awt_GetFontData(ws, font, 0)) != 0) {
	fontlist = XmFontListCreate(fdata->xfont, "labelFont");
	XtSetArg(args[argc], XmNfontList, fontlist); argc++;
    }
    mdata->cascade = XmCreateCascadeButton(mbdata->menuBar,
					   ctitle,
					   args,
					   argc);
    if (!strcmp(ctitle,"Help")) {
	XtVaSetValues(mbdata->menuBar,
		      XmNmenuHelpWidget, mdata->cascade,
		      NULL);
    }
		      
    if (font != 0) {
	XmFontListFree(fontlist);
    }
    XtManageChild(mdata->cascade);
}

void
awt_WServer_menuShow(Hawt_WServer *ws, Hawt_Menu *this)
{
    struct MenuData *mdata = PDATA(MenuData,this);

    XtManageChild(mdata->cascade);
}

void
awt_WServer_menuHide(Hawt_WServer *ws, Hawt_Menu *this)
{
    struct MenuData *mdata = PDATA(MenuData,this);

    XtUnmanageChild(mdata->cascade);
}


void
awt_WServer_menuDispose(Hawt_WServer *ws, Hawt_Menu *this)
{
    struct MenuData *mdata = PDATA(MenuData,this);

    XtDestroyWidget(mdata->itemData.w);
    XtDestroyWidget(mdata->cascade);
    free((void *)mdata);
}


void 
awt_WServer_menuAddSeparator(Hawt_WServer *ws,
			     Hawt_Menu *this)
{
    Widget		separator;
    int			argc;
    Arg			args[10];
    struct MenuData	*menuData;
    Pixel		bg;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    menuData = PDATA(MenuData,this);

    argc = 0;
    XtSetArg(args[argc], XmNbackground, &bg); argc++;
    XtGetValues(menuData->itemData.w, args, argc);

    argc = 0;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    separator =  XmCreateSeparator(menuData->itemData.w,
				   "",
				   args,
				   argc);
    XtManageChild(separator);
}
