/*
 * @(#)awt_MenuItem.c	1.19 95/04/10 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"

extern char *makeCString(Hjava_lang_String *);

static void
MenuItem_selected(Widget w, XtPointer client_data, XmAnyCallbackStruct *s)
{
    JAVA_UPCALL((EE(), (void *)client_data, "selected", "()V"));
}

void
awt_WServer_menuItemCreate(Hawt_WServer *ws,
			   Hawt_MenuItem *this,
			   Hjava_lang_String *label,
			   Hawt_Menu *menu,
			   long isToggle)
{
    int			argc;
    Arg			args[10];
    Widget		item;
    char		*clabel;
    struct MenuData	*menuData;
    struct MenuItemData *mdata;
    struct FontData	*fdata;
    Pixel		bg;
    Classawt_Menu	*menuPtr;
    Hawt_Font		*font;
    XmFontList		fontlist;

    if (label == 0 || menu == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    menuData = PDATA(MenuData,menu);
    clabel = makeCString(label);
    mdata = (struct MenuItemData *)malloc(sizeof(struct MenuItemData));
    SET_PDATA(this, mdata);

    argc = 0;
    XtSetArg(args[argc], XmNbackground, &bg); argc++;
    XtGetValues(menuData->itemData.w, args, argc);

    argc = 0;
    XtSetArg(
	args[argc],
	XmNlabelString,
	XmStringCreateLtoR(clabel,
			   XmSTRING_DEFAULT_CHARSET));  argc++;

    XtSetArg(args[argc], XmNbackground, bg); argc++;
    menuPtr = unhand(menu);
    font = menuPtr->defaultFont;
    if (font != 0 && (fdata = awt_GetFontData(ws, font, 0)) != 0) {
	fontlist = XmFontListCreate(fdata->xfont, "labelFont");
	XtSetArg(args[argc], XmNfontList, fontlist); argc++;
    }
    if (isToggle) {
	XtSetArg(args[argc], XmNset, False); argc++;
	XtSetArg(args[argc], XmNvisibleWhenOff, True); argc++;
	mdata->w = XmCreateToggleButton(menuData->itemData.w,
					clabel,
					args,
					argc);
    } else {
	mdata->w = XmCreatePushButton(menuData->itemData.w,
				      clabel,
				      args,
				      argc);
    }
    XtManageChild(mdata->w);
    if (font != 0) {
	XmFontListFree(fontlist);
    }
    XtAddCallback(mdata->w,
		  ((isToggle) ? XmNvalueChangedCallback : XmNactivateCallback),
		  (XtCallbackProc)MenuItem_selected,
		  (XtPointer)this);
    XtSetSensitive(mdata->w, True);
}

void
awt_WServer_menuItemEnable(Hawt_WServer *ws, Hawt_MenuItem *this)
{
    struct MenuItemData *mdata = PDATA(MenuItemData,this);
    XtSetSensitive(mdata->w, True);
}

void
awt_WServer_menuItemDisable(Hawt_WServer *ws, Hawt_MenuItem *this)
{
    struct MenuItemData *mdata = PDATA(MenuItemData,this);
    XtSetSensitive(mdata->w, False);
}

void
awt_WServer_menuItemDispose(Hawt_WServer *ws, Hawt_MenuItem *this)
{
    struct MenuItemData *mdata = PDATA(MenuItemData,this);

    XtDestroyWidget(mdata->w);
    free((void *)mdata);
}

void
awt_WServer_menuItemSetMark(Hawt_WServer *ws,
			    Hawt_MenuItem *this,
			    long state)
{
    struct MenuItemData *mdata = PDATA(MenuItemData,this);

    XtVaSetValues(mdata->w,
		  XmNset, (Boolean)state,
		  NULL);
}

long
awt_WServer_menuItemGetMark(Hawt_WServer *ws, Hawt_MenuItem *this)
{
    struct MenuItemData *mdata = PDATA(MenuItemData,this);
    Boolean		state;

    XtVaGetValues(mdata->w,
		  XmNset, &state,
		  NULL);

    return (long)state;
}
