/*
 * @(#)awt_MessageDialog.c	1.6 95/05/12 Sami Shaio
 *
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"
#include <sys/param.h>
#include <string.h>
#include <stdlib.h>
#include <Xm/MessageB.h>

static int dialogButtonChosen = 0;

static void
MessageDialog_ok(Widget w,
		 XtPointer client_data,
		 XtPointer call_data)
{
    dialogButtonChosen = 1;

    JAVA_UPCALL((EE(),
		(HObject *)client_data,
		"okCallback",
		"(Lawt/Dialog;)V",
		(void *)client_data));
}


static void
MessageDialog_cancel(Widget w,
		     XtPointer client_data,
		     XtPointer call_data)
{
    dialogButtonChosen = 2;

    JAVA_UPCALL((EE(),
		(HObject *)client_data,
		"cancelCallback",
		"(Lawt/Dialog;)V",
		client_data));
}


static void
MessageDialog_help(Widget w,
		   XtPointer client_data,
		   XtPointer call_data)
{
    JAVA_UPCALL((EE(),
		(HObject *)client_data,
		"helpCallback",
		"(Lawt/Dialog;)V",
		client_data));
}

void
awt_WServer_messageDialogCreate(Hawt_WServer *ws,
				Hawt_MessageDialog *this,
				Hawt_Frame *parent,
				Hjava_lang_String *title,
				Hjava_lang_String *message,
				long dialogType,
				long nButtons,
				/*boolean*/ long isModal,
				Hjava_lang_String *okLabel,
				Hjava_lang_String *cancelLabel,
				Hjava_lang_String *helpLabel)
{
    Arg				args[20];
    Cardinal			argc;
    struct MessageDialogData	*mdata = 0;
    struct WindowData		*fdata;
    XmString			mOk = 0;
    XmString			mCancel = 0;
    XmString			mHelp = 0;
    XmString			mMessage = 0;
    XmString			mTitle = 0;
    Pixel			wbg;

    if (parent == 0 || this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    fdata = PDATA(WindowData,parent);

    if (!fdata) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wbg = awt_getColor(ws, unhand(parent)->background);
    mdata = (struct MessageDialogData *)malloc(sizeof(struct MessageDialogData));
    SET_PDATA(this, mdata);
    mdata->isModal = isModal;
    if (!mdata) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }
    argc = 0;
    XtSetArg(args[argc], XmNbackground, wbg); argc++;
    if (title != 0) {
	mTitle = XmStringCreateLocalized(makeCString(title));
	XtSetArg(args[argc], XmNdialogTitle, mTitle); argc++;
    }
    if (okLabel != 0) {
	mOk = XmStringCreateLocalized(makeCString(okLabel));
	XtSetArg(args[argc], XmNokLabelString, mOk); argc++;
    }
    if (nButtons > 1 && cancelLabel != 0) {
	mCancel = XmStringCreateLocalized(makeCString(cancelLabel));
	XtSetArg(args[argc], XmNcancelLabelString, mCancel); argc++;
    }
    if (nButtons > 2 && helpLabel != 0) {
	mHelp = XmStringCreateLocalized(makeCString(helpLabel));
	XtSetArg(args[argc], XmNhelpLabelString, mHelp); argc++;
    }
    if (message != 0) {
	mMessage = XmStringCreateLocalized(makeCString(message));
	XtSetArg(args[argc], XmNmessageString, mMessage); argc++;
    }
    if (isModal) {
	XtSetArg(args[argc], XmNdialogStyle, XmDIALOG_SYSTEM_MODAL); argc++;
    }
    switch (dialogType) {
      case awt_MessageDialog_ERROR_TYPE:
	mdata->dialog = XmCreateErrorDialog(fdata->shell,
					    "Error",
					    args,
					    argc);
	break;
      case awt_MessageDialog_QUESTION_TYPE:
	mdata->dialog = XmCreateQuestionDialog(fdata->shell,
					       "Question",
					       args,
					       argc);
	break;
      case awt_MessageDialog_INFO_TYPE:
      default:
	mdata->dialog = XmCreateInformationDialog(fdata->shell,
						  "Info",
						  args,
						  argc);
	break;
    }
    switch (nButtons) {
      case 1:
	XtUnmanageChild(XmMessageBoxGetChild(mdata->dialog,
					     XmDIALOG_CANCEL_BUTTON));
	XtUnmanageChild(XmMessageBoxGetChild(mdata->dialog,
					     XmDIALOG_HELP_BUTTON));
	break;
      case 2:
	XtUnmanageChild(XmMessageBoxGetChild(mdata->dialog,
					     XmDIALOG_HELP_BUTTON));
	break;
      default:
	break;
    }

    if (mOk) {
	XmStringFree(mOk);
    }
    if (mCancel) {
	XmStringFree(mCancel);
    }
    if (mHelp) {
	XmStringFree(mHelp);
    }
    if (mMessage) {
	XmStringFree(mMessage);
    }
    if (mTitle) {
	XmStringFree(mTitle);
    }
    XtAddCallback(mdata->dialog,
		  XmNokCallback,
		  MessageDialog_ok,
		  (XtPointer)this);
    XtAddCallback(mdata->dialog,
		  XmNcancelCallback,
		  MessageDialog_cancel,
		  (XtPointer)this);
    XtAddCallback(mdata->dialog,
		  XmNhelpCallback,
		  MessageDialog_help,
		  (XtPointer)this);
}


void
awt_WServer_messageDialogSetMessage(Hawt_WServer *ws,
				    Hawt_MessageDialog *this,
				    Hjava_lang_String *message)
{
    Arg				args[20];
    Cardinal			argc;
    XmString			mMessage;
    struct MessageDialogData	*mdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    mdata = PDATA(MessageDialogData,this);
    argc = 0;
    if (message != 0) {
	mMessage = XmStringCreateLocalized(makeCString(message));
	XtSetArg(args[argc], XmNmessageString, mMessage); argc++;
    }
    XtSetValues(mdata->dialog, args, argc);
}

static int
WaitForDialogButton(void *data)
{
    return dialogButtonChosen != 0;
}

long
awt_WServer_mesageDialogShow(Hawt_WServer *ws,
			     Hawt_MessageDialog *this)
{
    struct MessageDialogData	*mdata;
    struct WServerData		*wsdata = PDATA(WServerData,ws);
 
    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    dialogButtonChosen = 0;
    mdata = PDATA(MessageDialogData,this);
    XtManageChild(mdata->dialog);
    if (mdata->isModal) {
	awt_WServer_modalWait(WaitForDialogButton, 0);

	return dialogButtonChosen;
    }

    return -1;
}

void
awt_WServer_mesageDialogHide(Hawt_WServer *ws,
			     Hawt_MessageDialog *this)
{
    struct MessageDialogData	*mdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    mdata = PDATA(MessageDialogData,this);
    XtUnmanageChild(mdata->dialog);
}

void
awt_WServer_messageDialogDispose(Hawt_WServer *ws,
				 Hawt_MessageDialog *this)
{
    struct MessageDialogData	*mdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    mdata = PDATA(MessageDialogData,this);
    XtDestroyWidget(mdata->dialog);
    free(mdata);
    SET_PDATA(this,0);
}

