/*
 * @(#)awt_OptionMenu.c	1.14 95/02/03 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_p.h"
#include <Xm/Separator.h>

static void
OptionMenu_callback(Widget menu_item,
		    Hawt_OptionMenu *this,
		    XmAnyCallbackStruct *cbs)
{
    long		  index;
    struct OptionMenuData *odata = PDATA(OptionMenuData,this);
    Classawt_OptionMenu	 *thisPtr = unhand(this);
    struct DPos		*dp;

    XtVaGetValues(menu_item,
		  XmNuserData, &dp,
		  NULL);

    thisPtr->selectedIndex = (int)dp->data;
    JAVA_UPCALL((EE(), (void *)this, "selected", "(I)V", (int)dp->data));
}

void
awt_WServer_optionMenuCreate(Hawt_WServer *ws,
			     Hawt_OptionMenu *this,
			     Hawt_Window *parent,
			     Hjava_lang_String *label)
{
    XmString			xstr = 0;
    struct OptionMenuData	*odata;
    struct WindowData		*wdata;
    Arg				args[30];
    int				argc;
    char			*clabel;
    Pixel			bg;
    Classawt_OptionMenu		*thisPtr;

    if (parent == 0 || this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    odata = (struct OptionMenuData *)malloc(sizeof(struct OptionMenuData));
    SET_PDATA(this, odata);

    if (!odata) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }
    wdata = PDATA(WindowData,parent);
    thisPtr = unhand(this);
    XtVaGetValues(wdata->w,
		  XmNbackground, &bg,
		  NULL);

    argc = 0;
    XtSetArg(args[argc], XmNx, 0); argc++;
    XtSetArg(args[argc], XmNy, 0); argc++;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    XtSetArg(args[argc], XmNrecomputeSize, False); argc++;
    XtSetArg(args[argc], XmNtraversalOn, False); argc++;
    odata->menu = XmCreatePulldownMenu(wdata->w,
				       "pulldown",
				       args,
				       argc);

    argc = 0;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    XtSetArg(args[argc], XmNsubMenuId, odata->menu); argc++;
    XtSetArg(args[argc], XmNrecomputeSize, False); argc++;
    if (label != 0) {
	clabel = makeCString(label);
	xstr = XmStringCreateSimple(clabel);
	XtSetArg(args[argc], XmNlabelString, xstr); argc++;
    } else {
	clabel = "";
    }
    XtSetArg(args[argc], XmNtraversalOn, False); argc++;
    odata->w = XmCreateOptionMenu(wdata->w,
				  clabel,
				  args,
				  argc);
    if (xstr != 0) {
	XmStringFree(xstr);
    }

    XtSetMappedWhenManaged(odata->w, False);
    XtManageChild(odata->w);

    awt_util_getSizes(odata->w,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
}

void
awt_WServer_optionMenuDispose(Hawt_WServer *ws,
			      Hawt_OptionMenu *this)
{
    struct OptionMenuData *odata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    odata = PDATA(OptionMenuData, this);
    if (odata == 0 || odata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_hide(odata->w);
    XtDestroyWidget(odata->w);
    free((void *)odata);
    SET_PDATA(this,0);
}

void
awt_WServer_optionMenuMoveTo(Hawt_WServer *ws,
			     Hawt_OptionMenu *this,
			     long x,
			     long y)
{
    struct OptionMenuData *odata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    odata = PDATA(OptionMenuData, this);

    if (odata == 0 || odata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_move(odata->w, x, y);
}

void
awt_WServer_optionMenuShow(Hawt_WServer *ws,
			   Hawt_OptionMenu *this)
{
    struct OptionMenuData *odata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    odata = PDATA(OptionMenuData, this);
    if (odata == 0 || odata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_show(odata->w);
}


void
awt_WServer_optionMenuHide(Hawt_WServer *ws,
			   Hawt_OptionMenu *this)
{
    struct OptionMenuData *odata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    odata = PDATA(OptionMenuData, this);
    if (odata == 0 || odata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_hide(odata->w);
}

void
awt_WServer_optionMenuAddItem(Hawt_WServer *ws,
			      Hawt_OptionMenu *this,
			      Hjava_lang_String *item,
			      long index)
{
    Classawt_OptionMenu	  *thisPtr = unhand(this);
    char		  *citem;
    struct OptionMenuData *odata;
    Widget		  bw;
    Arg			  args[10];
    int			  argc;
    Pixel		  bg;
    struct DPos		  *dp;

    if (this == 0 || item == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    citem = makeCString(item);
    odata = PDATA(OptionMenuData, this);
    if (odata == 0 || odata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaGetValues(odata->w,
		  XmNbackground, &bg,
		  NULL);

    dp = (struct DPos *)malloc(sizeof(struct DPos));
    dp->x = -1;
    dp->data = (void *)index;
    argc = 0;
    XtSetArg(args[argc], XmNuserData, dp); argc++;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    XtSetArg(args[argc], XmNtraversalOn, False); argc++;
    bw = XmCreatePushButton(odata->menu,
			    citem,
			    args,
			    argc);
    odata->items[index] = bw;
    XtManageChild(bw);
    XtAddCallback(bw, XmNactivateCallback, OptionMenu_callback, this);
    thisPtr->width = thisPtr->height = 0;
    awt_util_getSizes(odata->w,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
}


void
awt_WServer_optionMenuSelect(Hawt_WServer *ws,
			      Hawt_OptionMenu *this,
			      long index)
{
    Classawt_OptionMenu	  *thisPtr = unhand(this);
    char		  *citem;
    struct OptionMenuData *odata;
    Widget		  bw;
    Arg			  args[10];
    int			  argc;
    Pixel		  bg;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    odata = PDATA(OptionMenuData, this);
    if (odata == 0 || odata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaSetValues(odata->w,
		  XmNmenuHistory, odata->items[index],
		  NULL);
    awt_util_getSizes(odata->w,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
}


void
awt_WServer_optionMenuAddSeparator(Hawt_WServer *ws,
				   Hawt_OptionMenu *this,
				   long sIndex)
{
    char		  *citem;
    struct OptionMenuData *odata;
    Widget		  bw;
    int			argc;
    Arg			args[10];
    Pixel		bg;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    odata = PDATA(OptionMenuData, this);
    if (odata == 0 || odata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaGetValues(odata->w,
		  XmNbackground, &bg,
		  NULL);
    argc = 0;
    XtSetArg(args[argc], XmNseparatorType, XmSINGLE_LINE); argc++;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    bw = XmCreateSeparator(odata->menu,
			   "",
			   args,
			   argc);
    XtManageChild(bw);
}



void
awt_WServer_optionMenuReshape(Hawt_WServer *ws,
			      Hawt_OptionMenu *this,
			      long x,
			      long y,
			      long w,
			      long h)
{
    struct OptionMenuData *odata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    odata = PDATA(OptionMenuData, this);

    if (odata == 0 || odata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_reshape(odata->w, x, y, w, h);
}


void
awt_WServer_optionMenuDimensions(Hawt_WServer *ws,
				 Hawt_OptionMenu *this)
{
    struct OptionMenuData *odata;
    Classawt_OptionMenu   *thisPtr;
    long junk;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    thisPtr = unhand(this);
    odata = PDATA(OptionMenuData, this);
    if (odata == 0 || odata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_getSizes(odata->w,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
}


