/*
 * @(#)awt_Scrollbar.c	1.31 95/05/12 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"


static void
Scrollbar_lineUp(Widget	w,
		 XtPointer	client_data,
		 XtPointer	call_data)
{
    JAVA_UPCALL((EE(),
		(void *)unhand((struct Hawt_Scrollbar *)client_data)->scrollTarget,
		"lineUp","()V"));
}

static void
Scrollbar_lineDown(Widget	w,
		   XtPointer	client_data,
		   XtPointer	call_data)
{
    JAVA_UPCALL((EE(),
		(void *)unhand((struct Hawt_Scrollbar *)client_data)->scrollTarget,
		"lineDown","()V"));
}

static void
Scrollbar_pageUp(Widget	w,
		 XtPointer	client_data,
		 XtPointer	call_data)
{
    JAVA_UPCALL((EE(),
		(void *)unhand((struct Hawt_Scrollbar *)client_data)->scrollTarget,
		"pageUp","()V"));
}

static void
Scrollbar_pageDown(Widget	w,
		   XtPointer	client_data,
		   XtPointer	call_data)
{
    JAVA_UPCALL((EE(), 
		(void *)unhand((struct Hawt_Scrollbar *)client_data)->scrollTarget,
		"pageDown","()V"));
}

static void
Scrollbar_dragAbsolute(Widget	w,
		       XtPointer	client_data,
		       XtPointer	call_data)
{
    int new_value = ((XmScrollBarCallbackStruct *)call_data)->value;

    JAVA_UPCALL((EE(), 
		(void *)unhand((struct Hawt_Scrollbar *)client_data)->scrollTarget,
		"dragAbsolute","(I)V",new_value));
}


void
awt_WServer_scrollbarCreate(Hawt_WServer	*ws,
			    Hawt_Scrollbar	*this,
			    Hawt_Window		*parent,
			    long		orientation,
			    long		manageScrollbar)
{
    Classawt_Window		*wptr;
    int				argc;
    Arg				args[10];
    struct WindowData		*wdata;
    struct ScrollbarData	*sdata;
    Dimension			d;
    Pixel			bg;

    if (this == 0 || parent == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    wptr = unhand(parent);
    wdata = PDATA(WindowData,parent);
    bg = awt_getColor(ws, unhand(parent)->background);

    sdata = (struct ScrollbarData *)malloc(sizeof(struct ScrollbarData));
    SET_PDATA(this, sdata);

    if (sdata == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }

    switch (orientation) {
      case awt_Scrollbar_HORIZONTAL:
	argc=0;
	XtSetArg(args[argc], XmNwidth, &d); argc++;
	XtGetValues(wdata->w, args, argc);

	argc=0;
	XtSetArg(args[argc], XmNorientation, XmHORIZONTAL); argc++;
	break;
      case awt_Scrollbar_VERTICAL:
	argc=0;
	XtSetArg(args[argc], XmNheight, &d); argc++;
	XtGetValues(wdata->w, args, argc);

	argc=0;
	XtSetArg(args[argc], XmNorientation, XmVERTICAL); argc++;
	break;
      default:
	free(sdata);
	SignalError(0, JAVAPKG "IllegalArgumentException",
		    "bad scrollbar orientation");
	return;
    }
    XtSetArg(args[argc], XmNx, 0); argc++;
    XtSetArg(args[argc], XmNy, 0); argc++;
    XtSetArg(args[argc], XmNmaximum, (int)d); argc++;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    if (manageScrollbar) {
	sdata->scrollbar = XmCreateScrollBar(wdata->sw,
					     "scrollbar",
					     args,
					     argc);
	if (orientation == 0) {
	    wdata->hscroll = sdata->scrollbar;
	} else {
	    wdata->vscroll = sdata->scrollbar;
	}
    } else {
	sdata->scrollbar = XmCreateScrollBar(wdata->w,
					     "scrollbar",
					     args,
					     argc);
	XtSetMappedWhenManaged(sdata->scrollbar, False);
    }

    XtManageChild(sdata->scrollbar);
    XtAddCallback(sdata->scrollbar,
		  XmNdecrementCallback,
		  Scrollbar_lineUp,
		  (XtPointer)this);
    XtAddCallback(sdata->scrollbar,
		  XmNincrementCallback,
		  Scrollbar_lineDown,
		  (XtPointer)this);
    XtAddCallback(sdata->scrollbar,
		  XmNpageDecrementCallback,
		  Scrollbar_pageUp,
		  (XtPointer)this);
    XtAddCallback(sdata->scrollbar,
		  XmNpageIncrementCallback,
		  Scrollbar_pageDown,
		  (XtPointer)this);
    XtAddCallback(sdata->scrollbar,
		  XmNdragCallback,
		  Scrollbar_dragAbsolute,
		  (XtPointer)this);
}

void
awt_WServer_scrollbarMoveTo(Hawt_WServer *ws,
			    Hawt_Scrollbar *this,
			    long x, long y)
{
    struct ScrollbarData *sdata = PDATA(ScrollbarData,this);
    if (sdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_move(sdata->scrollbar, x, y);
}

void
awt_WServer_scrollbarReshape(Hawt_WServer *ws,
			     Hawt_Scrollbar *this,
			     long x, long y,
			     long w, long h)
{
    struct ScrollbarData *sdata = PDATA(ScrollbarData,this);

    if (sdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_reshape(sdata->scrollbar, x, y, w, h);
}


void
awt_WServer_scrollbarShow(Hawt_WServer *ws,
			  Hawt_Scrollbar *this)
{
    struct ScrollbarData *sdata = PDATA(ScrollbarData,this);

    if (sdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    awt_util_show(sdata->scrollbar);
}


void
awt_WServer_scrollbarHide(Hawt_WServer *ws,
			  Hawt_Scrollbar *this)
{
    struct ScrollbarData *sdata = PDATA(ScrollbarData,this);

    if (sdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    awt_util_hide(sdata->scrollbar);
}


long
awt_WServer_scrollbarMinimum(Hawt_WServer *ws, Hawt_Scrollbar *this)
{
    struct ScrollbarData *sdata = PDATA(ScrollbarData,this);
    long rval;

    if (sdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    XtVaGetValues(sdata->scrollbar,
		  XmNminimum, &rval,
		  NULL);
    return rval;
}

long
awt_WServer_scrollbarMaximum(Hawt_WServer *ws,
			     Hawt_Scrollbar *this)
{
    struct ScrollbarData *sdata = PDATA(ScrollbarData,this);
    long rval;

    if (sdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    XtVaGetValues(sdata->scrollbar,
		  XmNmaximum, &rval,
		  NULL);
    return rval;
}

void
awt_WServer_scrollbarSetValues(Hawt_WServer *ws,
			       Hawt_Scrollbar *this,
			       long newValue,
			       long visible,
			       long minimum,
			       long maximum)
{
    struct ScrollbarData *sdata = PDATA(ScrollbarData,this);
    long	realValue;
    
    if (sdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    if (visible > maximum) {
	visible = maximum;
    }
    realValue = newValue;
    if (newValue > (maximum - visible)) {
	newValue = maximum - visible;
    }
    /* pass in visible for sliderSize since Motif will calculate the */
    /* slider's size for us. */
    XtVaSetValues(sdata->scrollbar,
		  XmNminimum, minimum,
		  XmNmaximum, maximum,
		  XmNvalue, newValue,
		  XmNpageIncrement, visible,
		  XmNsliderSize, visible,
		  NULL);
}

void
awt_WServer_scrollbarSetValue(Hawt_WServer *ws,
			      Hawt_Scrollbar *this,
			      long newValue)
{
    struct ScrollbarData *sdata = PDATA(ScrollbarData,this);

    if (sdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaSetValues(sdata->scrollbar,
		  XmNvalue, newValue,
		  NULL);
}

long
awt_WServer_scrollbarValue(Hawt_WServer *ws,
			   Hawt_Scrollbar *this)
{
    struct ScrollbarData *sdata = PDATA(ScrollbarData,this);
    long rval;

    if (sdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaGetValues(sdata->scrollbar,
		  XmNvalue, &rval,
		  NULL);
    return rval;
}

void
awt_WServer_scrollbarDispose(Hawt_WServer *ws,
			     Hawt_Scrollbar *this)
{
    struct ScrollbarData *sdata = PDATA(ScrollbarData,this);

    if (sdata == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtDestroyWidget(sdata->scrollbar);
    free((void *)sdata);
    SET_PDATA(this,0);
}

