/*
 * @(#)awt_TextArea.c	1.15 95/05/12 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"

void awt_WServer_textAreaCreate(Hawt_WServer *ws,
				Hawt_TextArea *this,
				Hawt_Window *parent,
				Hawt_Font *font,
				long columns,
				long rows)
{
    XmFontList		fontlist;
    struct FontData	*fdata;
    struct TextAreaData	*tdata;
    Arg			args[30];
    int			argc;
    Pixel		bg;
    struct WindowData	*wdata;
    Classawt_TextArea	*thisPtr;
    char		*err;

    if (parent == 0 || this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    thisPtr = unhand(this);
    tdata = (struct TextAreaData *)malloc(sizeof(struct TextAreaData));
    SET_PDATA(this, tdata);

    if (!tdata) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }
    wdata = PDATA(WindowData,parent);
    XtVaGetValues(wdata->w,
		  XmNbackground, &bg,
		  NULL);

    argc = 0;
    if (font != 0) {
	fdata = awt_GetFontData(ws, font, &err);
	if (fdata == 0) {
	    SignalError(0, err, 0);
	    free(tdata);
	    return;
	}
	fontlist = XmFontListCreate(fdata->xfont, "charset1");
	XtSetArg(args[argc], XmNfontList, fontlist); argc++;
    }
    XtSetArg(args[argc], XmNx, 0); argc++;
    XtSetArg(args[argc], XmNy, 0); argc++;
    XtSetArg(args[argc], XmNcolumns, columns); argc++;
    XtSetArg(args[argc], XmNrows, rows); argc++;
    XtSetArg(args[argc], XmNeditMode, XmMULTI_LINE_EDIT); argc++;
    XtSetArg(args[argc], XmNwordWrap, True); argc++;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    XtSetArg(args[argc], XmNmarginHeight, 0); argc++;
    XtSetArg(args[argc], XmNmarginWidth, 0); argc++;
    XtSetArg(args[argc], XmNtraversalOn, False); argc++;
    tdata->text = XmCreateScrolledText(wdata->w,
				       "textA",
				       args,
				       argc);
    XtSetMappedWhenManaged(XtParent(tdata->text), False);
    XtManageChild(tdata->text);

    XtVaSetValues(XtParent(tdata->text),
		  XmNspacing, 0,
		  XmNmarginWidth, 0,
		  XmNmarginHeight, 0,
		  XmNscrolledWindowMarginWidth, 0,
		  XmNscrolledWindowMarginHeight, 0,
		  NULL);

    XtManageChild(XtParent(tdata->text));
    awt_util_getSizes(XtParent(tdata->text),
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
    thisPtr->marginWidth = thisPtr->marginHeight = 0;
}

void awt_WServer_textAreaSetEditable(Hawt_WServer *ws,
				     Hawt_TextArea *this,
				     /*boolean*/ long e)
{
    struct TextAreaData	*tdata;

    if (this==0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaSetValues(tdata->text,
		  XmNeditable, (Boolean)e,
		  NULL);
}

void 
awt_WServer_textAreaSetColor(Hawt_WServer *ws,
			     Hawt_TextArea *this,
			     Hawt_Color *c)
{
    struct TextAreaData *bdata = PDATA(TextAreaData,this);
    Pixel		color;

    if (bdata == 0 || bdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if (c==0) {
	SignalError(0,JAVAPKG "NullPointerException","null color");
	return;
    }
    color = awt_getColor(ws, c);
    XtVaSetValues(bdata->text,
		  XmNforeground, color,
		  NULL);
}

void 
awt_WServer_textAreaSetBackColor(Hawt_WServer *ws,
				 Hawt_TextArea *this,
				 Hawt_Color *c)
{
    struct TextAreaData *bdata = PDATA(TextAreaData,this);
    Pixel		color;

    if (bdata == 0 || bdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if (c==0) {
	SignalError(0,JAVAPKG "NullPointerException","null color");
	return;
    }
    color = awt_getColor(ws, c);
    XtVaSetValues(bdata->text,
		  XmNbackground, color,
		  NULL);
}



long
awt_WServer_textAreaCursorPos(Hawt_WServer *ws,
			      Hawt_TextArea *this)
{
    struct TextAreaData	*tdata;
    XmTextPosition	pos;

    if (this==0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    pos = XmTextGetCursorPosition(tdata->text);

    return pos;
}

void
awt_WServer_textAreaSetCursorPos(Hawt_WServer *ws,
				 Hawt_TextArea *this,
				 long pos)
{
    struct TextAreaData	*tdata;

    if (this==0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XmTextSetCursorPosition(tdata->text, (XmTextPosition)pos);
}

long
awt_WServer_textAreaEndPos(Hawt_WServer *ws,
			   Hawt_TextArea *this)
{
    struct TextAreaData	*tdata;
    XmTextPosition	pos;

    if (this==0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    pos = XmTextGetLastPosition(tdata->text);

    return pos;
}

void
awt_WServer_textAreaSetText(Hawt_WServer *ws,
			    Hawt_TextArea *this,
			    Hjava_lang_String *txt)
{
    struct TextAreaData	*tdata;
    XmTextPosition	pos;
    char		*cTxt;

    if (this==0 || txt == 0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    cTxt = makeCString(txt);
    XtVaSetValues(tdata->text,
		  XmNvalue, cTxt,
		  NULL);
}

Hjava_lang_String *
awt_WServer_textAreaGetText(Hawt_WServer *ws,
			    Hawt_TextArea *this)
{
    struct TextAreaData	*tdata;
    XmTextPosition	pos;
    char		*cTxt;
    Hjava_lang_String	*rval;

    if (this==0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    cTxt = XmTextGetString(tdata->text);
    rval = makeJavaString(cTxt, strlen(cTxt));
    XtFree(cTxt);

    return rval;
}

void
awt_WServer_textAreaInsertText(Hawt_WServer *ws,
			       Hawt_TextArea *this,
			       Hjava_lang_String *txt,
			       long pos)
{
    struct TextAreaData	*tdata;
    char		*cTxt;

    if (this==0 || txt == 0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    cTxt = makeCString(txt);
    XmTextInsert(tdata->text, (XmTextPosition)pos, cTxt);
}

void
awt_WServer_textAreaReplaceText(Hawt_WServer *ws,
				Hawt_TextArea *this,
				Hjava_lang_String *txt,
				long start,
				long end)
{
    struct TextAreaData	*tdata;
    char		*cTxt;

    if (this==0 || txt == 0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    cTxt = makeCString(txt);
    XmTextReplace(tdata->text,
		  (XmTextPosition)start,
		  (XmTextPosition)end,
		  cTxt);
}


void
awt_WServer_textAreaDispose(Hawt_WServer *ws,
			    Hawt_TextArea *this)
{
    struct TextAreaData	*tdata;

    if (this==0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_hide(XtParent(tdata->text));
    XtDestroyWidget(XtParent(tdata->text));
    free((void *)tdata);
    SET_PDATA(this,0);
}

void
awt_WServer_textAreaMoveTo(Hawt_WServer *ws,
			   Hawt_TextArea *this,
			   long x,
			   long y)
{
    struct TextAreaData	*tdata;

    if (this==0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_move(XtParent(tdata->text), x, y);
}

void
awt_WServer_textAreaReshape(Hawt_WServer *ws,
			    Hawt_TextArea *this,
			    long x,
			    long y,
			    long w,
			    long h)
{
    Classawt_TextArea	*thisPtr;
    struct TextAreaData	*tdata;
    Widget parent;

    if (this==0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    parent = XtParent(tdata->text);
    XtUnmanageChild(parent);
    XtVaSetValues(parent,
		  XmNx, (Position)x,
		  XmNy, (Position)y,
		  XmNwidth, (Dimension)w,
		  XmNheight, (Dimension)h,
		  NULL);
    XtManageChild(parent);
    thisPtr = unhand(this);
    awt_util_getSizes(XtParent(tdata->text),
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
    thisPtr->marginWidth = thisPtr->marginHeight = 0;
}

void
awt_WServer_textAreaShow(Hawt_WServer *ws,
			 Hawt_TextArea *this)
{
    struct TextAreaData	*tdata;

    if (this==0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_show(XtParent(tdata->text));
}


void
awt_WServer_textAreaHide(Hawt_WServer *ws,
			 Hawt_TextArea *this)
{
    struct TextAreaData	*tdata;

    if (this==0) {
	SignalError(0,JAVAPKG "NullPointerException",0);
	return;
    }
    tdata = PDATA(TextAreaData,this);
    if (tdata == 0 || tdata->text==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_hide(XtParent(tdata->text));
}


