/*
 * @(#)awt_TextField.c	1.44 95/05/12 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"

#define ECHO_BUFFER_LEN 1024

static void
TextField_changed(Widget w, XtPointer client_data, XmAnyCallbackStruct *s)
{
    JAVA_UPCALL((EE(), (void *)client_data,"selected","()V"));
}

static void
TextField_focusIn(Widget w, XtPointer client_data, XmAnyCallbackStruct *s)
{
    XmProcessTraversal(XtParent(w), XmTRAVERSE_CURRENT);
}

static void
TextField_focusOut(Widget w, XtPointer client_data, XmAnyCallbackStruct *s)
{
}

void
awt_WServer_textFieldCreate(Hawt_WServer *ws,
			    Hawt_TextField *this,
			    Hjava_lang_String *initValue,
			    Hawt_Window *parent,
			    long editable)
{
    Classawt_TextField	*thisPtr = unhand(this);
    struct WindowData *wdata;
    struct TextFieldData *bdata;
    char *cinitValue;
    Arg args[40];
    int argc;
    Pixel	bg;
    Dimension	w,h;
    struct DPos *dp;

    if (parent==0) {
	SignalError(0,JAVAPKG "NullPointerException","null parent value");
	return;
    }
    if (initValue != 0) {
	cinitValue = makeCString(initValue);
    } else {
	cinitValue = "";
    }
    bg = awt_getColor(ws, unhand(parent)->background);
    wdata = PDATA(WindowData,parent);
    bdata = (struct TextFieldData *)malloc(sizeof(struct TextFieldData));
    SET_PDATA(this, bdata);

    if (bdata == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }

    argc = 0;
    XtSetArg(args[argc], XmNx, 0); argc++;
    XtSetArg(args[argc], XmNy, 0); argc++;
    if (editable) {
	XtSetArg(args[argc], XmNeditable, True); argc++;
    } else {
	XtSetArg(args[argc], XmNeditable, False); argc++;
	XtSetArg(args[argc], XmNcursorPositionVisible, False); argc++;
    }

    XtSetArg(args[argc], XmNvalue, cinitValue); argc++;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    XtSetArg(args[argc], XmNhighlightColor, bg); argc++;
    XtSetArg(args[argc], XmNrecomputeSize, False); argc++;
    XtSetArg(args[argc], XmNtraversalOn, False); argc++;
    bdata->w = XmCreateTextField(wdata->w, "TextField", args, argc);
    XtSetMappedWhenManaged(bdata->w, False);
    XtManageChild(bdata->w);

    thisPtr->x = 0;
    thisPtr->y = 0;

    awt_util_getSizes(bdata->w,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));

    XtAddCallback(bdata->w,
		  XmNactivateCallback,
		  (XtCallbackProc)TextField_changed,
		  (XtPointer)this);
    XtAddCallback(bdata->w,
		  XmNmotionVerifyCallback,
		  (XtCallbackProc)TextField_focusIn,
		  (XtPointer)this);
    XtAddCallback(bdata->w,
		  XmNlosingFocusCallback,
		  (XtCallbackProc)TextField_focusOut,
		  (XtPointer)this);
}

Hjava_lang_String *
awt_WServer_textFieldGetText(Hawt_WServer *ws,
			     Hawt_TextField *this)
{
    struct TextFieldData *bdata = PDATA(TextFieldData,this);
    Classawt_TextField	*thisPtr = unhand(this);
    char *val;
    struct DPos	*dp;

    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if (thisPtr->echoSet) {
	XtVaGetValues(bdata->w,  XmNuserData, &dp,  NULL);
	val = (char *)(dp->data);
    } else {
	XtVaGetValues(bdata->w,  XmNvalue, &val,  NULL);
    }

    return makeJavaString(val, strlen(val));
}


void 
awt_WServer_textFieldSetColor(Hawt_WServer *ws,
			      Hawt_TextField *this,
			      Hawt_Color *c)
{
    struct TextFieldData *bdata = PDATA(TextFieldData,this);
    Pixel		color;

    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if (c==0) {
	SignalError(0,JAVAPKG "NullPointerException","null color");
	return;
    }
    color = awt_getColor(ws, c);
    XtVaSetValues(bdata->w,
		  XmNforeground, color,
		  NULL);
}

void 
awt_WServer_textFieldSetBackColor(Hawt_WServer *ws,
				  Hawt_TextField *this,
				  Hawt_Color *c)
{
    struct TextFieldData *bdata = PDATA(TextFieldData,this);
    Pixel	color;

    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if (c==0) {
	SignalError(0,JAVAPKG "NullPointerException","null color");
	return;
    }
    color = awt_getColor(ws, c);
    XtVaSetValues(bdata->w,
		  XmNbackground, color,
		  NULL);
}

void 
awt_WServer_textFieldSetText(Hawt_WServer *ws,
			     Hawt_TextField *this,
			     Hjava_lang_String *l)
{
    struct TextFieldData	*bdata = PDATA(TextFieldData,this);
    Classawt_TextField		*thisPtr = unhand(this);
    char			*cl;
    struct DPos			*dp;

    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if (l==0) {
	cl = "";
    } else {
	cl = makeCString(l);
    }

    XtVaSetValues(bdata->w,
		  XmNvalue, cl,
		  NULL);
}


void
awt_WServer_textFieldSetEditable(Hawt_WServer *ws,
				 Hawt_TextField *this,
				 long editable)
{
    struct TextFieldData *bdata = PDATA(TextFieldData,this);

    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaSetValues(bdata->w,
		  XmNeditable, (editable ? True : False),
		  XmNcursorPositionVisible, (editable ? True : False),
		  NULL);
}

void 
awt_WServer_textFieldMoveTo(Hawt_WServer *ws,
			    Hawt_TextField *this,
			    long x,
			    long y)
{
    struct TextFieldData *bdata = PDATA(TextFieldData,this);

    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_move(bdata->w, x, y);
}

void 
awt_WServer_textFieldReshape(Hawt_WServer *ws,
			     Hawt_TextField *this,
			     long x,
			     long y,
			     long w,
			     long h)
{
    struct TextFieldData *bdata = PDATA(TextFieldData,this);

    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_reshape(bdata->w, x, y, w, h);
}

void
awt_WServer_textFieldDispose(Hawt_WServer *ws,
			     Hawt_TextField *this)
{
    struct TextFieldData *bdata = PDATA(TextFieldData,this);
    Classawt_TextField	*thisPtr = unhand(this);
    char *val;
    struct DPos *dp;

    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if (thisPtr->echoSet) {
	XtVaGetValues(bdata->w, XmNuserData, &dp, NULL);
	if (dp) {
	    if (dp->data) {
		free(dp->data);
	    }
	    free(dp);
	}
    }
    awt_util_hide(bdata->w);    
    XtDestroyWidget(bdata->w);
    free((void *)bdata);
    SET_PDATA(this, 0);
}

void
awt_WServer_textFieldShow(Hawt_WServer *ws,
			     Hawt_TextField *this)
{
    struct TextFieldData *bdata = PDATA(TextFieldData,this);

    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_show(bdata->w);
}


void
awt_WServer_textFieldHide(Hawt_WServer *ws,
			  Hawt_TextField *this)
{
    struct TextFieldData *bdata = PDATA(TextFieldData,this);

    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_hide(bdata->w);
}


void 
awt_WServer_textFieldSetFont(Hawt_WServer *ws,
			     Hawt_TextField *this,
			     Hawt_Font *f)
{
    struct TextFieldData *bdata;
    struct FontData *fdata;
    XmFontList	fontlist;
    char	*err;

    if (f == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    fdata = awt_GetFontData(ws, f, &err);
    if (fdata == 0) {
	SignalError(0, err, 0);
	return;
    }
    bdata = PDATA(TextFieldData,this);
    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    fontlist = XmFontListCreate(fdata->xfont, "labelFont");
    if (fontlist != NULL) {
	XtVaSetValues(bdata->w,
		      XmNfontList, fontlist,
		      NULL);
	XmFontListFree(fontlist);
    } else {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
}

static void
echoChar(Widget text_w, XtPointer c, XmTextVerifyCallbackStruct *cbs)
{
    int len;
    char *val;
    struct DPos *dp;

    XtVaGetValues(text_w,
		  XmNuserData, &dp,
		  NULL);
    val = (char *)(dp->data);
    len = strlen(val);
    if (cbs->text->ptr == NULL) {
	if (cbs->text->length == 0 && cbs->startPos == 0) {
	    val[0] = 0;
	    return;
	} else if (cbs->startPos == (len - 1)) {
	    /* handle deletion */
	    cbs->endPos = strlen(val);
	    val[cbs->startPos] = 0;
	    return;
	} else {
	    /* disable deletes anywhere but at the end */
	    cbs->doit = False;
	    return;
	}
    }
    if (cbs->startPos != len) {
	/* disable "paste" or inserts into the middle */
	cbs->doit = False;
	return;
    }
    /* append the value typed in */
    if ((cbs->endPos + cbs->text->length) > ECHO_BUFFER_LEN) {
	val = realloc(val, cbs->endPos + cbs->text->length + 10);
    }
    strncat(val, cbs->text->ptr, cbs->text->length);
    val[cbs->endPos + cbs->text->length] = 0;

    /* modify the output to be the echo character */
    for (len = 0; len < cbs->text->length; len++) {
	cbs->text->ptr[len] = (char)c;
    }
}

void 
awt_WServer_textFieldSetEchoCharacter(Hawt_WServer *ws,
				      Hawt_TextField *this,
				      unicode c)
{
    char *val;
    char *cval;
    struct TextFieldData *bdata;
    Classawt_TextField *thisPtr;
    struct DPos *dp;
    int i, len;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    thisPtr = unhand(this);
    bdata = PDATA(TextFieldData,this);
    if (bdata == 0 || bdata->w==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    XtVaGetValues(bdata->w,
		  XmNvalue, &cval,
		  NULL);

    if (strlen(cval) > ECHO_BUFFER_LEN) {
	val = (char *)malloc(strlen(cval) + 1);
    } else {
	val = (char *)malloc(ECHO_BUFFER_LEN);
    }
    if (cval) {
	strcpy(val, cval);
    } else {
	*val = 0;
    }
    if (val == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryException", 0);
	return;
    }
    dp = (struct DPos *)malloc(sizeof(struct DPos));
    dp->x = -1;
    dp->data = (void *)val;
    len = strlen(cval);
    for (i = 0; i < len; i++) {
	cval[i] = (char)(c);
    }
    XtVaSetValues(bdata->w,
		  XmNvalue, cval,
		  NULL);
    XtAddCallback(bdata->w, XmNmodifyVerifyCallback, echoChar, (XtPointer)c);
    XtVaSetValues(bdata->w,
		  XmNuserData, (XtPointer)dp,
		  NULL);
}

