/*
 * @(#)awt_Toggle.c	1.22 95/05/12 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"

static void
Toggle_callback(Widget w,
		XtPointer client_data,
		XmAnyCallbackStruct *call_data)
{
    JAVA_UPCALL((EE(), (void *)client_data, "handleStateChanged","()V"));
}

void
awt_WServer_toggleCreate(Hawt_WServer *ws,
			 Hawt_Toggle *this,
			 Hjava_lang_String *label,
			 Hawt_Window *parent,
			 Hawt_RadioGroup *group,
			 long initialState)
{
    Classawt_Toggle	*thisPtr = unhand(this);
    struct ToggleData	*bdata;
    struct WindowData	*wdata;
    Pixel		bg;
    Arg			args[10];
    int			argc;
    Dimension		w,h;

    if (parent == 0) {
	SignalError(0,JAVAPKG "NullPointerException","null parent");
	return;
    }
    bg = awt_getColor(ws, unhand(parent)->background);
    wdata = PDATA(WindowData,parent);
    bdata = (struct ToggleData *)malloc(sizeof(struct ToggleData));
    SET_PDATA(this, bdata);
    argc=0;
    XtSetArg(args[argc], XmNx, 0); argc++;
    XtSetArg(args[argc], XmNy, 0); argc++;
    XtSetArg(args[argc], XmNbackground, bg); argc++;
    XtSetArg(args[argc], XmNset, (Boolean)initialState); argc++;
    XtSetArg(args[argc], XmNvisibleWhenOff, True); argc++;
    XtSetArg(args[argc], XmNtraversalOn, False); argc++;
    XtSetArg(args[argc], XmNspacing, 0); argc++;
    XtSetArg(args[argc], XmNrecomputeSize, False); argc++;
    if (group != 0) {
	XtSetArg(args[argc], XmNindicatorType, XmONE_OF_MANY); argc++;
    }

    bdata->toggle = XmCreateToggleButton(wdata->w,
					 makeCString(label),
					 args,
					 argc);
    XtAddCallback(bdata->toggle,
		  XmNvalueChangedCallback,
		  Toggle_callback,
		  (XtPointer)this);

    XtSetMappedWhenManaged(bdata->toggle, False);
    XtManageChild(bdata->toggle);


    thisPtr->x = 0;
    thisPtr->y = 0;
    awt_util_getSizes(bdata->toggle,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
}

void 
awt_WServer_toggleSetState(Hawt_WServer *ws,
			   Hawt_Toggle *this,
			   long state)
{
    struct ToggleData *bdata = PDATA(ToggleData,this);

    if (bdata == 0 || bdata->toggle==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaSetValues(bdata->toggle,
		  XmNset, (Boolean)state,
		  NULL);
}

long
awt_WServer_toggleGetState(Hawt_WServer *ws,
			   Hawt_Toggle *this)
{
    struct ToggleData *bdata = PDATA(ToggleData,this);
    Boolean state;

    if (bdata == 0 || bdata->toggle==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaGetValues(bdata->toggle,
		  XmNset, &state,
		  NULL);

    return state == False ? 0 : 1;
}


void 
awt_WServer_toggleMoveTo(Hawt_WServer *ws,
			 Hawt_Toggle *this,
			 long x,
			 long y)
{
    struct ToggleData *bdata = PDATA(ToggleData,this);

    if (bdata == 0 || bdata->toggle==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_move(bdata->toggle, x, y);
}

void 
awt_WServer_toggleDispose(Hawt_WServer *ws,
			  Hawt_Toggle *this)
{
    struct ToggleData	*bdata = PDATA(ToggleData,this);

    if (bdata == 0 || bdata->toggle==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_hide(bdata->toggle);
    XtDestroyWidget(bdata->toggle);
    free((void *)bdata);
    SET_PDATA(this, 0);
}

void 
awt_WServer_toggleReshape(Hawt_WServer *ws,
			  Hawt_Toggle *this,
			  long x,
			  long y,
			  long w,
			  long h)
{
    struct ToggleData *bdata = PDATA(ToggleData,this);

    if (bdata == 0 || bdata->toggle==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_reshape(bdata->toggle, x, y, w, h);
}

void 
awt_WServer_toggleShow(Hawt_WServer *ws, Hawt_Toggle *this)
{
    struct ToggleData *bdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    bdata = PDATA(ToggleData,this);
    if (bdata == 0 || bdata->toggle==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_show(bdata->toggle);
}

void 
awt_WServer_toggleHide(Hawt_WServer *ws, Hawt_Toggle *this)
{
    struct ToggleData *bdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    bdata = PDATA(ToggleData,this);

    if (bdata == 0 || bdata->toggle==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    awt_util_hide(bdata->toggle);
}
