/*
 * @(#)awt_WServer.c	1.38 95/03/09 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"
#include "iomgr.h"
#include <sys/filio.h>

int inModalWait = 0;

/*
 * Time to wait when there are no pending events.
 */
static int yieldTimeout = 50;

/*
 * only used by X error handlers which don't let you associate any
 * data with them so we keep a global pointer so we know what wserver
 * to unlock.
 */
Hawt_WServer *the_wserver = (Hawt_WServer *)0;


/*
 * error handlers
 */

static int
xerror_handler(Display *disp, XErrorEvent *err)
{
    char msg[128];
    char buf[128];
    XGetErrorText(disp, err->error_code, msg, sizeof(msg));
    fprintf(stderr, "Xerror %s\n", msg);
    sprintf(buf, "%d", err->request_code);
    XGetErrorDatabaseText(disp, "XRequest", buf, "Unknown", msg, sizeof(msg));
    fprintf(stderr, "Major opcode %d (%s)\n", err->request_code, msg);
    if (err->request_code > 128) {
	fprintf(stderr, "Minor opcode %d\n", err->minor_code);
    }
    if (the_wserver) {
	/*SignalError(lockedee->lastpc, lockedee, "fp/ade/gui/GUIException", msg);*/
    }
}

static int
xioerror_handler(Display *disp)
{
    if (the_wserver) {
	/*SignalError(lockedee->lastpc, lockedee, "fp/ade/gui/GUIException", "I/O error");*/
    }
}

Boolean scrollBugWorkAround;

void
awt_WServer_pInit(Hawt_WServer *this)
{
    int	argc = 0;
    Arg args[20];
    char *argv[1];
    struct WServerData *wdata;
    Display *dpy;

    wdata = (struct WServerData *)malloc(sizeof(struct WServerData));
    SET_PDATA(this, wdata);

    XtToolkitInitialize();
    argv[0] = 0;
    wdata->appContext = XtCreateApplicationContext();
    dpy = wdata->display = XtOpenDisplay(wdata->appContext,
					 NULL,
					 "WServer app",
					 "XApplication",
					 NULL,
					 0,
					 &argc,
					 argv);

    if (!dpy) {
	SignalError(0, JAVAPKG "FileNotFoundException", "X11 server");
	return;
    }

    scrollBugWorkAround = 
	(strcmp(XServerVendor(dpy), "Sun Microsystems, Inc.") == 0
	 && XVendorRelease(dpy) == 3400);

    nonblock_io(wdata->display->fd, IO_DONTBLOCK);
    XSetErrorHandler(xerror_handler);
    XSetIOErrorHandler(xioerror_handler);

    (void)awt_allocate_colors(wdata->display);
    wdata->defaultBg = awtImage->ColorMatch(200, 200, 200);
    wdata->screen = DefaultScreen(wdata->display);
    wdata->visual =  DefaultVisual(wdata->display, wdata->screen);
    wdata->root = RootWindow(wdata->display, wdata->screen);
    wdata->imagegc = XCreateGC(wdata->display, wdata->root, 0, NULL);
    wdata->blackpixel = BlackPixel(wdata->display, wdata->screen);
    wdata->whitepixel = WhitePixel(wdata->display, wdata->screen);
    wdata->defaultFg = wdata->blackpixel;
    argc = 0;
    the_wserver = this;
    {
	Pixmap one_bit;
	XGCValues xgcv;
	xgcv.background = 0;
	xgcv.foreground = 1;
	one_bit = XCreatePixmap(wdata->display, wdata->root, 1, 1, 1);
	wdata->maskgc = XCreateGC(wdata->display, one_bit,
				  GCForeground | GCBackground, &xgcv);
	XFreePixmap(wdata->display, one_bit);
    }
}

void
awt_WServer_loop(struct WServerData *wsdata, struct timeval *poll, int isModal)
{
    XEvent	event;
    fd_set	rdset;
    int		count;
    XtInputMask	iMask;

    /*
     * While there are is no input on the X connection, we block
     * outside of the monitor allowing other threads access.
     */
    monitorEnter(obj_monitor(the_wserver));
    if (!isModal) {
	while (inModalWait) {
	    monitorWait(obj_monitor(the_wserver), TIMEOUT_INFINITY);
	}
    }
    while ((iMask = XtAppPending(wsdata->appContext)) == 0) {
	monitorExit(obj_monitor(the_wserver));
	FD_ZERO(&rdset);
	FD_SET(wsdata->display->fd, &rdset);
	select(wsdata->display->fd+1, &rdset, 0,  0, poll);
	monitorEnter(obj_monitor(the_wserver));
	if (!isModal) {
	    while (inModalWait) {
		monitorWait(obj_monitor(the_wserver), TIMEOUT_INFINITY);
	    }
	}
    }

    /*
     * Need to check here in case data on the connection was read
     * before we acquired the monitor.
     */
    if ((iMask = XtAppPending(wsdata->appContext)) > 0) {
	XtAppProcessEvent(wsdata->appContext, iMask);
	/* XXX: this flush could be a real drag on a slow */
	/* connection */
	XFlush(wsdata->display);
    }

    monitorExit(obj_monitor(the_wserver));
}

void
awt_WServer_eventLoop(Hawt_WServer *this)
{
    struct WServerData	*wsdata = PDATA(WServerData,this);

    struct timeval poll;
    poll.tv_sec = 0;
    poll.tv_usec = yieldTimeout * 1000;

    assert(this == the_wserver);
    /* notify the thread that started the server that initialization
       is done. */
    monitorEnter(obj_monitor(the_wserver));
    monitorNotify(obj_monitor(the_wserver));
    monitorExit(obj_monitor(the_wserver));

    for (;;) {
	awt_WServer_loop(wsdata, &poll, 0);
    }
}

void
awt_WServer_sync(Hawt_WServer *this)
{
    struct WServerData	*wsdata = PDATA(WServerData,this);

    XFlush(wsdata->display);
}


void
awt_WServer_modalWait(int (*terminateFn)(void *data), void *data)
{
    struct timeval poll;
    struct WServerData 	*wsdata = PDATA(WServerData,the_wserver);

    monitorEnter(obj_monitor(the_wserver));
    inModalWait = 1;
    monitorExit(obj_monitor(the_wserver));
    poll.tv_sec = 0;
    poll.tv_usec = 50 * 1000;
	
    while ((*terminateFn)(data) == 0) {
	awt_WServer_loop(wsdata, &poll, 1);
    }
    monitorEnter(obj_monitor(the_wserver));
    inModalWait = 0;
    monitorNotify(obj_monitor(the_wserver));
    monitorExit(obj_monitor(the_wserver));
}
