/*
 * @(#)awt_Window.c	1.102 95/05/12 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"
#include <X11/keysym.h>
#include <X11/IntrinsicP.h>
#include "CClassHeaders/awt_Event.h"

extern void awt_setWidgetGravity(Widget w, int gravity);
Boolean scrollBugWorkAround;

Pixel
awt_pixel_by_name(
    Display *dpy,
    char *colorname,
    char *default_color
)
{
    int		scr = DefaultScreen(dpy);
    Colormap	cmap = DefaultColormap(dpy,scr);
    XColor	color, ignore;

    if (XAllocNamedColor(dpy, cmap, colorname, &color, &ignore)) {
	return color.pixel;
    } else if (XAllocNamedColor(dpy, cmap, default_color, &color, &ignore)) {
	return color.pixel;
    } else {
	printf("Warning: Couldn't allocate color %s\n", colorname);
	return BlackPixel(dpy, scr);
    }
}

static void
eatAllExposures(Display *dpy,
		Window windowid,
		XRectangle *rect)
{
    XEvent ev;
    int x, y, x2, y2;

    x = rect->x;
    y = rect->y;
    x2 = x + rect->width;
    y2 = y + rect->height;

#ifndef MIN
#define MIN(a,b) (int)((int)(a) < (int)(b) ? (a) : (b))
#define MAX(a,b) (int)((int)(a) > (int)(b) ? (a) : (b))
#endif

    XSync(dpy, False);
    while (XCheckTypedWindowEvent(dpy, windowid, Expose, &ev)) {
	if (x == -1) {
	    x = ev.xexpose.x;
	    y = ev.xexpose.y;
	    x2 = ev.xexpose.x + ev.xexpose.width;
	    y2 = ev.xexpose.y + ev.xexpose.height;
	} else {
	    x = MIN(x, ev.xexpose.x);
	    y = MIN(y, ev.xexpose.y);
	    x2 = MAX(x2, ev.xexpose.x + ev.xexpose.width);
	    y2 = MAX(y2, ev.xexpose.y + ev.xexpose.height);
	}
    }
    rect->x = x;
    rect->y = y;
    rect->width = x2 - x;
    rect->height = y2 - y;
}

static void
callJavaExpose(void *javaObject, XRectangle *rect)
{
    JAVA_UPCALL((EE(), javaObject,
		"handleExpose","(IIII)V",
		rect->x, rect->y, rect->width, rect->height));
}

static void
HandleExposeEvent(Widget w,
		  struct WindowData *wdata,
		  void *javaObject,
		  XEvent *event)
{
    Hawt_Window	*this = (Hawt_Window *)javaObject;
    Display *dpy = XtDisplay(wdata->w);

    switch (event->type) {
    case Expose:
    case GraphicsExpose:
	{
	    XRectangle	rect;

	    rect.x = event->xexpose.x;
	    rect.y = event->xexpose.y;
	    rect.width = event->xexpose.width;
	    rect.height = event->xexpose.height;

/*
	printf("HANDLING EXPOSE of <%d,%d,%d,%d>\n",
	       rect.x,
	       rect.y,
	       rect.width,
	       rect.height);
*/

	    eatAllExposures(dpy, XtWindow(w), &rect);
	    callJavaExpose(javaObject, &rect);
	}
	break;

    default:
	printf("Got event %d in HandleExposeEvent!\n", event->type);
    }
}

static void
Window_resize(Widget wd, XtPointer client_data, XtPointer call_data)
{
    Hawt_Window	*this = (Hawt_Window *)client_data;
    Classawt_Window	*thisPtr = unhand(this);
    struct WindowData *wdata = PDATA(WindowData,this);
  
    awt_util_getSizes(wdata->w,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
    JAVA_UPCALL((EE(), (void *)client_data, "handleResize","()V"));
}

static void
Window_event_handler(Widget w, XtPointer client_data, XEvent *event, Boolean *cont)
{
    int 	modifiers = 0;
    Classawt_Window	*this = unhand((Hawt_Window *)client_data);
    struct WindowData	*wdata = (struct WindowData *)this->pData;
    extern int inModalWait;

    if (inModalWait) {
	return;
    }
    switch (event->type) {
      case GraphicsExpose:
      case Expose:
	HandleExposeEvent(w, wdata, client_data, event);
	break;
      case FocusIn:
	    JAVA_UPCALL((EE(),(void *)client_data, "gotFocus","()V"));
	    break;
      case FocusOut:
	    JAVA_UPCALL((EE(), (void *)client_data, "lostFocus","()V"));
	    break;
      case ButtonPress:
	XmProcessTraversal(wdata->w, XmTRAVERSE_CURRENT);
	/*XtSetKeyboardFocus(wdata->w, wdata->w);*/
	if (event->xbutton.state & ShiftMask) {
	    modifiers += awt_Event_SHIFT_DOWN;
	}
	if (event->xbutton.state & ControlMask) {
	    modifiers += awt_Event_CTRL_DOWN;
	}
	JAVA_UPCALL((EE(),
		    (void *)client_data,
		    "handleMouseDown","(III)V",
		    event->xkey.x,
		    event->xkey.y,
		    modifiers));
	break;
      case ButtonRelease:
	if (event->xbutton.state & ShiftMask) {
	    modifiers += awt_Event_SHIFT_DOWN;
	}
	if (event->xbutton.state & ControlMask) {
	    modifiers += awt_Event_CTRL_DOWN;
	}
	JAVA_UPCALL((EE(),
		    (void *)client_data,
		    "handleMouseUp","(III)V",
		    event->xkey.x,
		    event->xkey.y,
		    modifiers));
	break;
      case MotionNotify:
	    if (event->xmotion.state & ShiftMask) {
		modifiers += awt_Event_SHIFT_DOWN;
	    }
	    if (event->xmotion.state & ControlMask) {
		modifiers += awt_Event_CTRL_DOWN;
	    }
	    if (event->xmotion.state & (Button1Mask | Button2Mask | Button3Mask)) {
		JAVA_UPCALL((EE(),
			    (void *)client_data,
			    "handleMouseDrag","(III)V",
			    event->xkey.x,
			    event->xkey.y,
			    modifiers));
	    } else {
		JAVA_UPCALL((EE(),
			    (void *)client_data,
			    "handleMouseMoved","(III)V",
			    event->xkey.x,
			    event->xkey.y,
			    modifiers));
	    }
	    break;
      case KeyPress:
	{
	    char buf[64];
	    int len = sizeof(buf);
	    KeySym keysym;

	    len = XLookupString(&event->xkey, buf, len - 1, &keysym, NULL);
	    buf[len - 1] = '\0';

	    if (event->xkey.state & ShiftMask) {
		modifiers += awt_Event_SHIFT_DOWN;
	    }
	    if (event->xkey.state & ControlMask) {
		modifiers += awt_Event_CTRL_DOWN;
	    }
	    if (keysym < 256) {
		if ((event->xkey.state & ControlMask) &&
		    isalpha(keysym)) {
		    JAVA_UPCALL((EE(),
				(void *)client_data,
				"handleKeyPress","(III)V",
				tolower(keysym) - 'a' + 1, 1, modifiers));
		} else {
		    JAVA_UPCALL((EE(),
				(void *)client_data,
				"handleKeyPress","(III)V",
				keysym, 1, modifiers));
		}
	    } else {
		switch (keysym) {
		  case XK_BackSpace:
		  case XK_Delete:
		    JAVA_UPCALL((EE(),
				(void *)client_data,
				"handleKeyPress","(III)V",
				'\010', 1, modifiers));
		    break;
		  case XK_Tab:
		    JAVA_UPCALL((EE(),
				(void *)client_data,
				"handleKeyPress","(III)V",
				'\t', 1, modifiers));
		    break;
		  case XK_Return:
		  case XK_Linefeed:
		    JAVA_UPCALL((EE(),
				(void *)client_data,
				"handleKeyPress","(III)V",
				'\n', 1, modifiers));
		    break;
		  case XK_F27:
		    /* Home */
		    break;
		  case XK_R13:
		    /* End */
		    break;
		  case XK_F29:
		    /* PgUp */
		    break;
		  case XK_F35:
		    /* PgDn */
		    break;
		  case XK_Up:
		    /* Up */
		    break;
		  case XK_Down:
		    /* Down */
		    break;
		  case XK_Left:
		    /* Left */
		    break;
		  case XK_Right:
		    /* Right */
		    break;
		  default:
		    break;
		}
	    }
	}
	break;
      case KeyRelease:
	break;
      case EnterNotify:
	XmProcessTraversal(w, XmTRAVERSE_CURRENT);
      case LeaveNotify:
	/*  Enter/Leave notify -- need to mask out events that */
	/* happen inside the parent window. Otherwise any native gui */
	/* element that happens to use its own window will generate */
	/* these events. */

	/* XXX - there's got to be a better way... */
	if (event->xkey.x < (this->width - 5) &&
	    event->xkey.y < (this->height - 5) &&
	    event->xkey.x > 5 &&
	    event->xkey.y > 5) {
	    return;
	}
	switch (event->type) {
	  case EnterNotify:
	    JAVA_UPCALL((EE(),
			(void *)client_data,
			"handleMouseEnter","(II)V",
			event->xkey.x,
			event->xkey.y));
	    break;
	  case LeaveNotify:
	    JAVA_UPCALL((EE(),
			(void *)client_data,
			"handleMouseLeave","(II)V",
			event->xkey.x,
			event->xkey.y));
	    break;
	}
	break;

      default:
	break;
    }
}

static void
Window_destroy(Widget w, XtPointer client_data, XtPointer call_data)
{
}


void
awt_WServer_windowEnablePointerMotionEvents(Hawt_WServer *ws,
					    Hawt_Window *this)
{
    struct WindowData	*wdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    wdata = PDATA(WindowData,this);

    if (wdata == 0 || wdata->sw==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    XtAddEventHandler(wdata->w, 
		      PointerMotionMask,
		      False,
		      Window_event_handler,
		      this);
}

void
awt_WServer_windowDisablePointerMotionEvents(Hawt_WServer *ws,
					     Hawt_Window *this)
{
    struct WindowData	*wdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    wdata = PDATA(WindowData,this);
    if (wdata == 0 || wdata->sw==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    XtRemoveEventHandler(wdata->w,
			 PointerMotionMask,
			 False,
			 Window_event_handler,
			 this);
}

void
awt_WServer_windowCreateInFrame(Hawt_WServer *ws,
				Hawt_Window *this,
				Hawt_Frame *container,
				long trackMotion,
				Hawt_Color *bg,
				long width,
				long height)

{
    awt_WServer_windowCreate(ws, this, (Hawt_Window *)container, trackMotion,
			     bg, width, height);
}


void
awt_WServer_windowCreate(Hawt_WServer *ws,
			 Hawt_Window *this,
			 Hawt_Window *container,
			 long trackMotion,
			 Hawt_Color *bg,
			 long width,
			 long height)

{
    Classawt_Window	*thisPtr;
    Arg			args[20];
    int			argc;
    struct WindowData	*wdata;
    struct WindowData   *cdata;
    Pixel		wbg;


    if (this == 0 || container == 0 || bg == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    thisPtr = unhand(this);
    cdata = PDATA(WindowData,container);
    thisPtr->width = width;
    thisPtr->height = height;
    thisPtr->x = thisPtr->y = 0;
    wbg = awt_getColor(ws, bg);
    wdata = (struct WindowData *)malloc(sizeof(struct WindowData));
    SET_PDATA(this, wdata);
    
    wdata->vscroll = 0;
    wdata->hscroll = 0;
    wdata->w = 0;
    wdata->sw = 0;
    wdata->flags = 0;

    argc = 0;
    XtSetArg(args[argc], XmNwidth, (XtArgVal)width); argc++;
    XtSetArg(args[argc], XmNheight, (XtArgVal)height); argc++;
    XtSetArg(args[argc], XmNbackground, wbg); argc++;
    XtSetArg(args[argc], XmNscrollingPolicy, XmAPPLICATION_DEFINED); argc++;
    XtSetArg(args[argc], XmNscrollBarDisplayPolicy, XmSTATIC); argc++;
    XtSetArg(args[argc], XmNspacing, 0); argc++;
    XtSetArg(args[argc], XmNmarginWidth, 0); argc++;
    XtSetArg(args[argc], XmNmarginHeight, 0); argc++;
    XtSetArg(args[argc], XmNscrolledWindowMarginWidth, 0); argc++;
    XtSetArg(args[argc], XmNscrolledWindowMarginHeight, 0); argc++;
    wdata->sw = XmCreateScrolledWindow(cdata->w, "swindow", args, argc);

    argc=0;
    XtSetArg(args[argc], XmNwidth, (XtArgVal)width); argc++;    
    XtSetArg(args[argc], XmNheight, (XtArgVal)height); argc++;
    XtSetArg(args[argc], XmNbackground, wbg); argc++;
    XtSetArg(args[argc], XmNmarginHeight, 0); argc++;
    XtSetArg(args[argc], XmNmarginWidth, 0); argc++;
    XtSetArg(args[argc], XmNancestorSensitive, True); argc++;
    XtSetArg(args[argc], XmNresizePolicy, XmRESIZE_NONE); argc++;
    wdata->w = XmCreateDrawingArea(wdata->sw,"canvas",args,argc);
    XtManageChild(wdata->w);
    if (!scrollBugWorkAround) {
	awt_setWidgetGravity(wdata->w, StaticGravity);
    }
    
    wdata->shell = cdata->shell;

    XtOverrideTranslations(wdata->w,
			   XtParseTranslationTable("<KeyDown>:DrawingAreaInput()"));

    XtAddCallback(wdata->w,
		  XmNresizeCallback,
		  Window_resize,
		  (XtPointer)this);

    XtAddCallback(wdata->w,
		  XmNdestroyCallback,
		  Window_destroy,
		  (XtPointer)this);

    XtSetSensitive(wdata->w, True);
    XtAddEventHandler(wdata->w, ExposureMask |
		      KeyPressMask | ButtonPressMask | 
		      ButtonReleaseMask | FocusChangeMask |
		      EnterWindowMask | LeaveWindowMask,
		      True, Window_event_handler, this);

    if (trackMotion) {
	XtAddEventHandler(wdata->w,
			  PointerMotionMask,
			  False,
			  Window_event_handler, this);
    }

    awt_util_getSizes(wdata->w,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
}


void
awt_WServer_windowDispose(Hawt_WServer *ws,
			  Hawt_Window *this)
{
    struct WindowData		*wdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    wdata = PDATA(WindowData,this);
    if (wdata == 0 || wdata->sw==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtDestroyWidget(wdata->sw);
    free(wdata);
    SET_PDATA(this,0);
}

void
awt_WServer_windowShow(Hawt_WServer *ws,
		       Hawt_Window *this)
{
    struct WindowData		*wdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    wdata = PDATA(WindowData,this);
    if (wdata == 0 || wdata->sw==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtManageChild(wdata->sw);
}

void
awt_WServer_windowHide(Hawt_WServer *ws,
		       Hawt_Window *this)
{
    struct WindowData		*wdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    wdata = PDATA(WindowData,this);
    if (wdata == 0 || wdata->sw==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtUnmanageChild(wdata->sw);
}


void
awt_WServer_windowMoveTo(Hawt_WServer *ws,
			 Hawt_Window *this,
			 long x,
			 long y)
{
    struct WindowData	*wdata;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    wdata = PDATA(WindowData,this);
    if (wdata == 0 || wdata->sw==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_move(wdata->sw, x, y);
}

void
awt_WServer_windowReshape(Hawt_WServer *ws,
			  Hawt_Window *this,
			  long x, long y,
			  long w, long h)
{
    Classawt_Window	*thisPtr;
    Dimension	vw = 0;
    Dimension	hw = 0;
    struct WindowData	*wdata;

    if (this == 0 || (thisPtr = unhand(this)) == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    wdata = PDATA(WindowData,this);
    if (wdata == 0 || wdata->sw==0 || wdata->w == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_reshape(wdata->sw, x, y, w, h);
    awt_util_getSizes(wdata->w,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
}

long
awt_WServer_windowWidth(Hawt_WServer *ws,
			Hawt_Window *this)
{
   Dimension		wd;
   struct WindowData	*wdata;

   if (this == 0) {
       SignalError(0, JAVAPKG "NullPointerException", 0);
       return;
   }

   wdata = PDATA(WindowData,this);
   if (wdata == 0 || wdata->sw==0) {
       SignalError(0, JAVAPKG "NullPointerException", 0);
       return;
   }
   XtVaGetValues(wdata->sw,
		 XmNwidth, &wd,
		 NULL);

   return (long)wd;
}

long
awt_WServer_windowHeight(Hawt_WServer *ws,
			 Hawt_Window *this)
{
   Dimension		wd;
   struct WindowData	*wdata;

   if (this == 0) {
       SignalError(0, JAVAPKG "NullPointerException", 0);
       return;
   }

   wdata = PDATA(WindowData,this);
    if (wdata == 0 || wdata->sw==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
   XtVaGetValues(wdata->sw,
		 XmNheight, &wd,
		 NULL);

   return (long)wd;
}


void
awt_WServer_windowSetMargin(Hawt_WServer *ws,
			    Hawt_Window *this,
			    long margin)
{
   struct WindowData	*wdata;

   if (this == 0) {
       SignalError(0, JAVAPKG "NullPointerException", 0);
       return;
   }

   wdata = PDATA(WindowData,this);
   if (wdata == 0 || wdata->sw==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
   XtVaSetValues(wdata->sw,
		 XmNshadowThickness, margin,
		 NULL);
}

static void
messWithGravity(Widget w, int gravity)
{
    extern void awt_changeAttributes(Display *dpy, Widget w,
				     unsigned long mask,
				     XSetWindowAttributes *xattr);
    XSetWindowAttributes    xattr;

    xattr.bit_gravity = gravity;
    xattr.win_gravity = gravity;

    awt_changeAttributes(XtDisplay(w), w,
			 CWBitGravity|CWWinGravity, &xattr);
}

struct MoveRecord {
    long dx;
    long dy;
};

void 
moveWidget(Widget w, void *data)
{
    struct MoveRecord *rec = (struct MoveRecord *)data;

    if (XtIsRealized(w) && XmIsRowColumn(w)) {
	    w->core.x -= rec->dx;
	    w->core.y -= rec->dy;
    }
}

/* Scroll entire contents of window by dx and dy.  Currently only
   dy is supported.  A negative dy means scroll backwards, i.e.,
   contents in window move down. */

void
awt_WServer_windowScrollWindow(Hawt_WServer *ws,
			       Hawt_Window *this,
			       long dx, long dy)
{
    struct WServerData	    *wsdata = PDATA(WServerData, ws);
    struct WindowData	    *wdata;
    Classawt_Window	    *thisPtr;
    Window		    win;
    XSetWindowAttributes    xattr;
    XWindowChanges	    xchgs;
    Window		    root;
    int			    x, y, width, height, junk;
    Display		    *dpy;
    struct	MoveRecord  mrec;

    mrec.dx = dx;
    mrec.dy = dy;
    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    thisPtr = unhand(this);
    wdata = PDATA(WindowData,this);
    if (wdata == 0 || wdata->sw==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    dpy = XtDisplay(wdata->w);
    win = XtWindow(wdata->w);

    /* REMIND: consider getting rid of this! */
    XGetGeometry(wsdata->display,
		 win,
		 &root,
		 &x,
		 &y,
		 &width,
		 &height,
		 &junk,
		 &junk);

    if (!scrollBugWorkAround && (wdata->flags & W_GRAVITY_INITIALIZED) == 0) {
	wdata->flags |= W_GRAVITY_INITIALIZED;
	awt_setWidgetGravity(wdata->w, StaticGravity);
    }

    /* we need to actually update the coordinates for manager widgets, */
    /* otherwise the parent won't pass down events to them properly */
    /* after scrolling... */
    awt_util_mapChildren(wdata->w, moveWidget, &mrec);

    if (dy < 0) {
	/* scrolling backward */

	if (scrollBugWorkAround) {
	    messWithGravity(wdata->w, SouthGravity);
	}

	xchgs.x = x;
	xchgs.y = y + dy;
	xchgs.width = width;
	xchgs.height = height - dy;
	XConfigureWindow(wsdata->display,
			 win,
			 CWX|CWY|CWWidth|CWHeight,
			 &xchgs);

	if (scrollBugWorkAround) {
	    messWithGravity(wdata->w, NorthWestGravity);
	}

	xchgs.x = x;
	xchgs.y = y;
	XConfigureWindow(wsdata->display,
			 win,
			 CWX|CWY,
			 &xchgs);

	xchgs.width = width;
	xchgs.height = height;
	XConfigureWindow(wsdata->display,
			 win,
			 CWWidth|CWHeight,
			 &xchgs);
    } else {
	/* forward scrolling */

	/* make window a little taller */
	xchgs.width = width;
	xchgs.height = height + dy;
	XConfigureWindow(wsdata->display,
			 win,
			 CWWidth|CWHeight,
			 &xchgs);

	/* move window by amount we're scrolling */
	xchgs.x = x;
	xchgs.y = y - dy;
	XConfigureWindow(wsdata->display,
			 win,
			 CWX|CWY,
			 &xchgs);

	if (scrollBugWorkAround) {
	    messWithGravity(wdata->w, SouthGravity);
	}

	/* resize to original size */
	xchgs.x = x;
	xchgs.y = y;
	xchgs.width = width;
	xchgs.height = height;
	XConfigureWindow(wsdata->display,
			 win,
			 CWX|CWY|CWWidth|CWHeight,
			 &xchgs);
	if (scrollBugWorkAround) {
	    messWithGravity(wdata->w, NorthWestGravity);
	}

    }
    /* Because of the weird way we're scrolling this window,
       we have to eat all the exposure events that result from
       scrolling forward, and translate them up by the amount we're
       scrolling by.

       Rather than just eating all the exposures and having the
       java code fill in what it knows is exposed, we do it this
       way.  The reason is that there might be some other exposure
       events caused by overlapping windows on top of us that we
       also need to deal with. */
    {
	XRectangle	rect;

	rect.x = -1;
	eatAllExposures(dpy, win, &rect);
	if (rect.x != -1) {	/* we got at least one expose event */
	    if (dy > 0) {
		rect.y -= dy;
	    }
	    callJavaExpose(this, &rect);
	}
    }
}

void
awt_WServer_windowDimensions(Hawt_WServer *ws,
			     Hawt_Window *this)
{
    struct WindowData *wdata;
    Classawt_Window 	*thisPtr;
    long junk;

    if (this == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    wdata = PDATA(WindowData, this);
    thisPtr = unhand(this);
    if (wdata == 0 || wdata->sw==0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    awt_util_getSizes(wdata->w,
		      &(thisPtr->width),
		      &(thisPtr->height),
		      &(thisPtr->marginWidth),
		      &(thisPtr->marginHeight));
}


