/*
 * @(#)awt_p.h	1.39 95/05/13 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Motif-specific data structures for AWT Java objects.
 *
 */
#ifndef _AWT_P_H_
#define _AWT_P_H_

#include "awt.h"

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h>
#include <Xm/CascadeB.h>
#include <Xm/DrawingA.h>
#include <Xm/FileSB.h>
#include <Xm/BulletinB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrollBar.h>
#include <Xm/ScrolledW.h>
#include <Xm/SelectioB.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleB.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/List.h>
#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include "color.h"
#include "timeval.h"
#include "monitor.h"
#include "awt_util.h"

extern Hawt_WServer *the_wserver;

/* C calling Java...
 */
extern long execute_method(ExecEnv *ee, void *obj, char *method_name, ...);
extern char *makeCString(Hjava_lang_String *);
extern Pixel awt_pixel_by_name(Display *dpy, char *color, char *defaultColor);

struct ChoiceData {
    Hawt_Component	*target;
    Hawt_ChoiceHandler	*handler;
};

struct MessageDialogData {
    Widget		dialog;
    long		isModal;
};

struct GraphicsData {
    Widget	win;
    Drawable	drawable;
    GC		gc;
    Boolean	clipset;
    XRectangle	cliprect;
};

struct ListData {
    struct ChoiceData	choiceData;
    Widget		slist;
};

struct WServerData {
    XtAppContext	appContext;
    Display		*display;
    Pixel		defaultBg;
    Pixel		defaultFg;
    Pixel		whitepixel;
    Pixel		blackpixel;
    Visual		*visual;
    GC			imagegc;
    GC			maskgc;
    Window		root;
    long		screen;
};

struct MenuItemData {
    int		index;
    Widget	w;
};

struct MenuData {
    struct MenuItemData	itemData;
    Widget		cascade;
};

struct MenuBarData {
    Widget	menuBar;
};

struct WindowData {
    Widget	w;
    Widget	sw;
    Widget	hscroll;
    Widget	vscroll;
    Widget	shell;
    int		flags;
};

#define W_GRAVITY_INITIALIZED 1

struct ColorData {
    int		initialized;
    Pixel	color;
};

struct FrameData {
    struct WindowData winData;
    long	isModal;
    Widget	mainW;
    Widget	menuBar;
};

struct FileDialogData {
    Widget	dialog;
    char	*file;
};

struct FontData {
    XFontStruct	*xfont;
};

/* XXX: fix items to be a growable list. */
struct OptionMenuData {
    Widget	w;
    Widget	menu;
    Widget	items[256];
    int		n_items;
};

struct ButtonData {
    Widget	button;
};

struct ToggleData {
    Widget	toggle;
};

struct TextFieldData {
    Widget	w;
};

struct TextAreaData {
    Widget	text;
};

struct ScrollbarData {
    Widget	scrollbar;
};

struct ImageData {
    Pixmap	xpixmap;
    Pixmap	xmask;
};

extern int awt_init_gc(Display *display, struct GraphicsData *gdata);
extern struct FontData *awt_GetFontData(Hawt_WServer *ws,
					Hawt_Font *font,
					char **msg);

#define PDATA(T, x) ((struct T *)(unhand(x)->pData))
#define SET_PDATA(x, d) (unhand(x)->pData = (int)d)
#define DSP(server) ((struct WServerData *)server->pData)->display
#define DSPU(server) ((struct WServerData *)(unhand(server)->pData))->display
#define BG(server) ((struct WServerData *)(unhand(server)->pData))->defaultBg
#define FG(server) ((struct WServerData *)(unhand(server)->pData))->defaultFg
extern struct GlobalData awt_gData;

#define JAVA_UPCALL(args)\
monitorExit(obj_monitor(the_wserver));\
execute_java_dynamic_method args;\
monitorEnter(obj_monitor(the_wserver));\
if (exceptionOccurred(EE())) {\
exceptionDescribe(EE());\
exceptionClear(EE());\
}

void	awt_WServer_modalWait(int (*terminateFn)(void *data), void *data);
Pixel	awt_getColor(Hawt_WServer *ws, Hawt_Color *self);

#endif           /* _AWT_P_H_ */
