/*
 * @(#)awt_util.c	1.22 95/02/09 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include "awt_p.h"

void awt_setWidgetGravity(Widget w, int gravity)
{
    XSetWindowAttributes    xattr;
    Window  win = XtWindow(w);

    if (win != 0) {
	xattr.bit_gravity = StaticGravity;
	xattr.win_gravity = StaticGravity;
	XChangeWindowAttributes(XtDisplay(w), win,
				CWBitGravity|CWWinGravity,
				&xattr);
    }
}

void
awt_util_getSizes(Widget w,
		  long *width,
		  long *height,
		  long *marginWidth,
		  long *marginHeight)
{
    Dimension wd, h, marginH, marginW, borderW;
    struct DPos	*p;

    if (w == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaGetValues(w,
		  XmNwidth, &wd,
		  XmNheight, &h,
		  XmNmarginHeight, &marginH,
		  XmNmarginWidth, &marginW,
		  XmNborderWidth, &borderW,
		  XmNuserData, &p,
		  NULL);

    *width = (long)wd;
    *height = (long)h;
    *marginWidth = (long)(marginW + borderW) + 4;
    *marginHeight = (long)(marginH + borderW) + 4;
    if (p == NULL) {
	p = (struct DPos *)malloc(sizeof(struct DPos));
	p->x = -1;
	p->data = 0;
	p->mapped = 0;
	XtVaSetValues(w, XmNuserData, p, NULL);
    }    
}


void
awt_util_move(Widget w, long x, long y) 
{
    struct DPos	    *p;
    XWindowChanges  xchgs;
    Window	    win;

    if (w == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
/*
    if ((win = XtWindow(w)) != 0) {
	XSetWindowAttributes	xattr;

	xattr.bit_gravity = StaticGravity;
	xattr.win_gravity = StaticGravity;
	XChangeWindowAttributes(XtDisplay(w),
				XtWindow(w),
				CWBitGravity|CWWinGravity,
				&xattr);
    }
*/
    XtMoveWidget(w, x, y);
    XtVaGetValues(w, XmNuserData, &p, NULL);

    if (p == NULL) {
	p = (struct DPos *)malloc(sizeof(struct DPos));
	p->x = -1;
	p->data = 0;
	XtVaSetValues(w, XmNuserData, p, NULL);
    }
    p->x = x;
    p->y = y;
}

void
awt_util_reshape(Widget w, long x, long y, long wd, long h) 
{
    if (w == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtConfigureWidget(w,
		      (Position)x,
		      (Position)y,
		      (Dimension)wd,
		      (Dimension)h,
		      0);
}

void
awt_util_hide(Widget w) 
{
    struct DPos *dp;

    if (w == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaGetValues(w, XmNuserData, &dp, NULL);
    if (dp) {
	dp->mapped = 0;
    }
    XtSetMappedWhenManaged(w, False);
}

void
awt_util_show(Widget w) 
{
    extern Boolean  scrollBugWorkAround;
    struct DPos *dp;

    if (w == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    XtVaGetValues(w, XmNuserData, &dp, NULL);
    if (dp) {
	dp->mapped = 1;
    }
    XtSetMappedWhenManaged(w, True);
    if (!scrollBugWorkAround) {
	awt_setWidgetGravity(w, StaticGravity);
    }
}

void
awt_util_mapChildren(Widget w, void (*func)(Widget,void *), void *data) {
    WidgetList			wlist;
    Cardinal			wlen = 0;
    int				i;

    if (!XtIsComposite(w)) {
	return;
    }

    XtVaGetValues(w,
		  XmNchildren, &wlist,
		  XmNnumChildren, &wlen,
		  NULL);
    if (wlen > 0) {
	for (i=0; i < wlen; i++) {
	    (*func)(wlist[i], data);
	    awt_util_mapChildren(wlist[i], func, data);
	}
    }
}

void
awt_changeAttributes(Display *dpy, Widget w, unsigned long mask,
		     XSetWindowAttributes *xattr)
{
    int			i;
    WidgetList		wlist;
    Cardinal		wlen = 0;

    if (XtWindow(w) && XtIsRealized(w)) {
	XChangeWindowAttributes(dpy,
				XtWindow(w),
				mask,
				xattr);
    } else {
	return;
    }
    XtVaGetValues(w,
		  XmNchildren, &wlist,
		  XmNnumChildren, &wlen,
		  NULL);
    for (i = 0; i < wlen; i++) {
	if (XtWindow(wlist[i]) && XtIsRealized(wlist[i])) {
	    XChangeWindowAttributes(dpy,
				    XtWindow(wlist[i]),
				    mask,
				    xattr);
	}
    }
}
