/*
 * @(#)context_md.h	1.10 95/01/31  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Solaris-dependent API
 */

#ifndef _SOLARIS_CONTEXT_MD_H_
#define _SOLARIS_CONTEXT_MD_H_

#include <errno.h>
#include <ucontext.h>
#include <sys/regset.h>

/*
 * Routines that call getcontext have strange control flow graphs,
 * since a call to a routine that calls setcontext will eventually
 * return at the getcontext site, not the original call site.  This
 * utterly wrecks control flow analysis.
 */
#pragma unknown_control_flow(getcontext)

/*
 * A solaris2 context.  Everything else is on the stack.
 */
typedef struct {
    unsigned int unix_errno;
    ucontext_t ucontext;
} context_t;

#define yieldContext(contextp, func, preemption) \
    if (! getcontext(&((contextp)->ucontext))) { \
	(contextp)->unix_errno = errno; \
	func(preemption); \
    }

#define switchContext(contextp) { \
    errno = (contextp)->unix_errno; \
    ((contextp)->ucontext).uc_mcontext.gregs[11] = 1; \
    setcontext(&((contextp)->ucontext)); \
}

/* return from trap into preempting thread */
#define returnFromTrap(c) \
    switchContext(c)

#endif /* !_SOLARIS_CONTEXT_MD_H_ */
