/*
 * @(#)file_acl.h	1.17 95/05/10 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#ifndef _FILE_ACL_H_
#define _FILE_ACL_H_

/* Uncomment to disable file access lists */
#define DO_FILE_ACL

#ifdef DO_FILE_ACL
#include "oobj.h"
#include "interpreter.h"
#include "javaString.h"
#include <java_io_AccessErrorHandler.h>
#include <java_io_File.h>

/*
 * Return 0 if none of the classes currently on the stack has a class
 * loader context. Otherwise, return 1.
 *
 * This function is useful to restrict capabilities to classes that
 * were loaded by class loaders.
 */
long classLoadersInStack();

/*
 * Return 1 if the class currently on the stack has a class
 * loader context. Otherwise, return 0.
 *
 * This function is useful to restrict capabilities to classes that
 * were loaded by class loaders.
 */
long currentClassHasClassLoader();

/*
 * Initialize the read and write acls. Must be called in order for any
 * file io calls to work.
 */
void initReadACL(Hjava_lang_String *path);
void initWriteACL(Hjava_lang_String *path);
void initExecACL(Hjava_lang_String *path);

/*
 * Initialize the access error handler
 */
void setAccessErrorHandler(Hjava_io_AccessErrorHandler *h);

/*
 * Get the value of the read acl
 */
char *getReadACL();

/*
 * Get the value of the write acl
 */
char *getWriteACL();

/* 
 * Check to see that the given filename is available for read. If so,
 * return 1 otherwise return 0
 */
long checkForReadAccess(char *name, Hjava_lang_String *hfile);

/* 
 * Check to see that the given filename is available for write. If so,
 * return 1 otherwise return 0
 */
long checkForWriteAccess(char *name, Hjava_lang_String *hfile);

#define DEFAULT_READ_PATH "/tmp/hotjava/:$HOTJAVA_HOME:~/public_html/:~/.mailcap:/etc/mailcap:/usr/etc/mailcap:/usr/local/etc/mailcap"
#define DEFAULT_WRITE_PATH ""

#else
#define initReadACL(path)
#define initWriteACL(path)
#define initExecACL(path)
#define checkForReadAccess(name,hfile) 1
#define checkForWriteAccess(name,hfile) 1
#endif				/* DO_FILE_ACL */


#endif           /* _FILE_ACL_H_ */
