/*
 * @(#)monitor_md.h	1.17 95/01/31  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Monitor interface	10/25/94
 *
 * Private data structures and interfaces used in the monitor code.
 * This file is used to share declarations and such between the different
 * files implementing monitors.  It does not contain exported API.
 */

#ifndef	_GREEN_MONITOR_MD_H_
#define	_GREEN_MONITOR_MD_H_

#include <threads_md.h>

/*
 * Type definitions.
 */

/* The system-level monitor data structure */
typedef struct _sys_monitor {
    /* Cache support */
    struct _sys_monitor	*pendingq;	/* Queue of interrupt info */

    short		entry_count;	/* Recursion depth */
    short		flags;		/* Various flags */

#define SYS_MON_STICKY_NOTIFICATION	0x1
#define SYS_MON_PENDING_NOTIFICATION	0x2
#define SYS_MON_INVERTED_PRIORITY	0x4
#define SYS_MON_IN_USE			0x8

    int			oldPriority;	/* For priority inversion */

    /* Thread currently executing in this monitor */
    sys_thread_t	*monitor_owner;

    /* Queue of threads trying to enter monitor */
    sys_thread_t	*monitor_waitq;

    /* Queue of threads suspended while in monitor */
    sys_thread_t	*suspend_waitq;

    /* Condition Variable, otherwise know as the queue of threads
       currently executing "wait()" inside a monitor */
    sys_thread_t	*condvar_waitq;
} sys_mon_t;

typedef enum {
	ASYNC_REGISTER,
	ASYNC_UNREGISTER
} async_action_t;

/*
 * Macros
 */

#define SYS_MID_NULL ((sys_mon_t *) 0)

/*
 * Symbolic constant to be used when waiting indefinitly on a condition
 * variable
 */
#define SYS_TIMEOUT_INFINITY -1

typedef enum {
	SYS_ASYNC_MON_ALARM = 1,
 	SYS_ASYNC_MON_IO,
 	SYS_ASYNC_MON_EVENT,
 	SYS_ASYNC_MON_MAX
} async_mon_key_t;

#define SYS_ASYNC_MON_INPUT SYS_ASYNC_MON_IO
#define SYS_ASYNC_MON_OUTPUT SYS_ASYNC_MON_IO

/*
 * Support for the async part of async GC
 */
extern sys_mon_t *PendingNotifyQ;
#define SYS_INTERRUPTS_PENDING() (PendingNotifyQ)

void monitorApplyInversion(sys_mon_t *, sys_thread_t *);
void monitorUndoInversion(sys_mon_t *, sys_thread_t *);
sys_mon_t *asyncMon(async_mon_key_t);

#endif	/* _GREEN_MONITOR_MD_H_ */
