/*
 * @(#)schedule.h	1.6 95/01/31  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Scheduling functions that other interfaces will need.  Generic to
 * Green threads package.
 */

#ifndef _GREEN_SCHEDULE_H_
#define _GREEN_SCHEDULE_H_

#include <queue.h>
#include <context.h>
#include <monitor_md.h> 

/*
 * The scheduling lock.  This is used to arbitrate access by the
 * asynchronous condition variable notification interface.
 * See: async_event_notify()
 */
extern int _scheduling_lock;

#define SCHED_LOCKED()	    (_scheduling_lock)

#define SCHED_LOCK() {				\
    _scheduling_lock++;				\
}

/*
 * Unlock the scheduling lock, and process any interrupts we may
 * have received.
 */
extern void _sched_unlock();
extern sys_mon_t *PendingNotifyQ;

#define SCHED_UNLOCK() {			\
    _scheduling_lock--;				\
    assert(_scheduling_lock >= 0);		\
    if (!_scheduling_lock && PendingNotifyQ)	\
	_sched_unlock();			\
}

typedef enum {SYNCHRONOUS, ASYNCHRONOUS} preemption_t;

/*
 * Switch to a new thread.
 */
void reschedule(preemption_t);

#define YIELD(preemption) { \
    queueInsert(&runnable_queue, sysThreadSelf()); \
    yieldContext(CONTEXT(sysThreadSelf()), reschedule, preemption); \
}

#endif /* !_GREEN_SCHEDULE_H_ */
