/*
 * @(#)javah.c	1.29 95/05/10 James Gosling
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*-
 *      produce C header files from an java object file
 *      produce C stub files from an java object file
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <ctype.h>
#include "tree.h"
#include "oobj.h"
#include "interpreter.h"
#include "signature.h"

void makeslottable(ClassClass *);

long stubmode = 0;
long stubtable[2];

static FILE *outfp = 0;
static char *out = 0;
static char *dir = ".";
static char *tempfn = 0;
static char *tempdir = "/tmp";
static char *currentclassname;

/*
    We will need the class name in two forms -
	1) in a form suitable for a file name.  To get this we replace
	   the slashes between package names with F_DELIM.
	2) in the form of a C-symbol.  To get this we replace the
	   slashes between package names with C_DELIM.

    NOTE - for now we use the same deliminter for both C symbols and
	   for filenames.  It is '_' which does not guarrantee no
	   conflicts in either case but at least is consistent.
*/

#define DIR_DELIM   '/'
#define C_DELIM     '_'
#define F_DELIM     '_'


static char *
strsub(char *src, char *dst, char sub)
{
    char c;
    char *d = dst;

    while (c = *src++)
	if (c == DIR_DELIM)
	    *d++ = sub;
	else
	    *d++ = c;
    *d = '\0';
    return dst;
}

/* Open a temporary file */
static FILE *open_output() 
{
    if (outfp != 0) {
	return outfp;
    }
    tempfn = tempnam(tempdir, "javah");
    if ((outfp = fopen(tempfn, "w")) == 0) {
	perror(tempfn);
    }
    /* fprintf(stderr, "writing to %s\n", tempfn); */

    fprintf(outfp, "/* DO NOT EDIT THIS FILE - it is machine generated */\n");
    if (stubmode) {
	fprintf(outfp, "#include <StubPreamble.h>\n");
    } else {
	fprintf(outfp, "#include <native.h>\n");
    }
    return outfp;
}

/* Close the temporary file and rename it
 * to the final output file if the content
 * has changed. */
static int close_output() 
{
    char fname[300];

    if (outfp == 0) {
	return;
    }

    fclose(outfp);
    outfp = 0;

    if (out) {
	strncpy(fname, out, sizeof(fname));
    } else {
	char tempfname[300];
	strsub(currentclassname, tempfname, F_DELIM);
	sprintf(fname, "%s/%s%s", dir, tempfname, stubmode ? ".c" : ".h");
    }

    /* fprintf(stderr, "rename %s to %s\n", tempfn, fname); */
    return renameIfDiff(tempfn, fname, dir);
}

static char *
SprintReverseType(s, fname, out)
    register char *s, *fname, *out;
{
    char buf[1000];
    register char *tn = "?";
    if (fname == 0)
	fname = "";
    switch (*s++) {
    default:
	switch (s[-1]) {
	case SIGNATURE_ANY:
	    tn = "void *";
	    break;
	case SIGNATURE_CHAR:
	    tn = "unicode";
	    break;
	case SIGNATURE_BYTE:
	    tn = "char";
	    break;
	case SIGNATURE_ENUM:
	    tn = "enum";
	    break;
	case SIGNATURE_FLOAT:
	    tn = "float";
	    break;
	case SIGNATURE_DOUBLE:
	    tn = "double";
	    break;
	case SIGNATURE_BOOLEAN:
	    tn = "/*boolean*/ long";
	    break;
	case SIGNATURE_INT:
	    tn = "long";
	    break;
	case SIGNATURE_LONG:
	    tn = "int64_t";
	    break;
	case SIGNATURE_SHORT:
	    tn = "short";
	    break;
	case SIGNATURE_CLASS:
	    tn = buf;
	    {
		register char *sp = "struct H";
		while (*sp)
		    *tn++ = *sp++;
	    }
	    for (; *s && *s != SIGNATURE_ENDCLASS;) {
		char c = *s++;

		if (c == DIR_DELIM)
		    *tn++ = C_DELIM;
		else
		    *tn++ = c;
	    }
	    *tn++ = ' ';
	    *tn++ = '*';
	    *tn = 0;
	    if (*s == SIGNATURE_ENDCLASS)
		s++;
	    tn = buf;
	    break;
	case SIGNATURE_VOID:
	    tn = "void";
	    break;
	}
	sprintf(out, (*fname
	      ? (tn[strlen(tn) - 1] == '*'
		 ? "%s%s"
		 : "%s %s")
	      : "%s"),
	tn, fname);
	break;
    case 0:
	strcpy(out, fname);
	s--;
	break;
    case SIGNATURE_ARRAY:
	switch (*s++) {
	  case SIGNATURE_BYTE:
	    sprintf(out, *fname ? "HArrayOfByte *%s" : "HArrayOfByte *", fname);
	    break;
	  case SIGNATURE_CHAR:
	    sprintf(out, *fname ? "HArrayOfChar *%s" : "HArrayOfChar *", fname);
	    break;
	  case SIGNATURE_SHORT:
	    sprintf(out, *fname ? 
		    "HArrayOfShort *%s" : "HArrayOfShort *", fname);
	    break;
	  case SIGNATURE_INT:
	    sprintf(out, *fname ? "HArrayOfInt *%s" : "HArrayOfInt *", fname);
	    break;
	  case SIGNATURE_BOOLEAN:
	    sprintf(out, *fname ? "/*boolean*/ HArrayOfInt *%s" :
	        "HArrayOfInt *", fname);
	    break;
	  case SIGNATURE_LONG:
	    sprintf(out, *fname ? "HArrayOfLong *%s" : "HArrayOfLong *", fname);
	    break;
	  case SIGNATURE_FLOAT:
	    sprintf(out, 
		    *fname ? "HArrayOfFloat *%s" : "HArrayOfFloat *", fname);
	    break;
	  case SIGNATURE_DOUBLE:
	    sprintf(out, 
		    *fname ? "HArrayOfDouble *%s" : "HArrayOfDouble *", fname);
	    break;
	  case SIGNATURE_CLASS:
	    if (!strncmp(s,"java/lang/String;", sizeof("java/lang/String;")-1)) {
	        s += sizeof("java/lang/String;") - 1;
		sprintf(out, *fname ? "HArrayOfString *%s" : "HArrayOfString *",
			fname);
	    } else {
		while (*s++ != SIGNATURE_ENDCLASS);
		sprintf(out, *fname ? "HArrayOfObject *%s" : "HArrayOfObject *",
			fname);
	    }
	    break;
	  case SIGNATURE_ARRAY:
	    while (*++s == SIGNATURE_ARRAY);
	    if (*s++ == SIGNATURE_CLASS) 
		while (*s++ != SIGNATURE_ENDCLASS);
	    sprintf(out, 
		    *fname ? "HArrayOfArray *%s" : "HArrayOfArray *", fname);
	    break;
	}
	break;
    case SIGNATURE_FUNC:
	{
	    int constructor = strcmp(fname, "<init>") == 0;
	    char cname[300];

	    strsub(currentclassname, cname, C_DELIM);
	    sprintf(buf, "%s_%s(",
		    cname,
		    constructor ? "Initializor" : fname);
	    fname = buf + strlen(buf);
	    sprintf(fname, "struct H%s *,", cname);
	    fname += strlen(fname);
	    while (*s != SIGNATURE_ENDFUNC && *s) {
		s = SprintReverseType(s, (char *) 0, fname);
		fname += strlen(fname);
		*fname++ = ',';
	    }
	    if (fname[-1] == ',') fname--;
	    *fname++ = SIGNATURE_ENDFUNC;
	    *fname++ = 0;
	    s = SprintReverseType(constructor ? "" : *s ? s + 1 : s, buf, out);
	}
	break;
    }
    return s;
}

static char *
PrintType(FILE *f, char *s, char *fname)
{
    char buf[1000];
    char fnb[100];
    register char *src, *dst;
    dst = fnb;
    for (src = fname; *src && dst - fnb < sizeof fnb - 1 &&
        *src != SIGNATURE_FUNC;) {
	char c = *src++;
	if (c == DIR_DELIM)
	    *dst++ = C_DELIM;
	else
	    *dst++ = c;
    }
    *dst = 0;
    s = SprintReverseType(s, fnb, buf);
    fwrite(buf, strlen(buf), 1, f);
    return s;
}


/*
 * Returns 1 if diff, 0 if same, -1 if error
 */
static int
renameIfDiff(char *tempfn, char *newfn, char *newdir)
{
    FILE *f, *nf;
    char buf[1024];
    long c;

    if ((f = fopen(tempfn, "r")) == 0) {
	perror(tempfn);
	return -1;
    } else if (nf = fopen(newfn, "r")) {
	while ((c = getc(f)) == getc(nf))
	    if (c == -1) {
		unlink(tempfn);
		fclose(f);
		fclose(nf);
		if (verbose)
		    fprintf(stderr, "  (no change)\n");
		return 0;
	    }
    }

    rewind(f);
    if (nf)
	fclose(nf);
    if ((nf = fopen(newfn, "w")) == 0) {
	if (mkdir(newdir, 0777) != 0) {
	    perror(newdir);
	    return -1;
	}
	nf = fopen(newfn, "w");
    }
    if (nf == 0) {
	perror(newfn);
	return -1;
    }
    while ((c = fread(buf, sizeof(char), sizeof(buf), f)) > 0) {
	fwrite(buf, sizeof(char), c, nf);
    }
    fclose(nf);
    unlink(tempfn);
    if (verbose)
	fprintf(stderr, "  %s => %s\n", tempfn, newfn);
    return 1;
}

static int
DumpClassHeader(ClassClass * cb)
{
    FILE *f;
    register struct fieldblock *fb;
    register struct methodblock *mb;
    register nslots;
    int offset = 0;
    char hfname[300];   /* header file name */
    char hcname[300];   /* C symbol name.  */

    currentclassname = classname(cb);
    strsub(currentclassname, hfname, F_DELIM);
    strsub(currentclassname, hcname, C_DELIM);

    if ((f = open_output()) == 0) {
	return -1;
    }

    if (verbose)
	fprintf(stderr, "=> %s\n", tempfn);
    fprintf(f, "/* Header for class %s */\n\n", hfname);
    fprintf(f, "#ifndef _Included_%s\n#define _Included_%s\n",
	    hcname, hcname);
    makeslottable(cb);
    if (strcmp(currentclassname, "java/lang/Object") != 0
	&& strcmp(currentclassname, "java/lang/Class")) {
	int NumFieldsWritten = 0;
	fprintf(f, "typedef struct Class%s {\n",
		hcname);
	for (nslots = 0; (unsigned)nslots < cbSlotTableSize(cb); nslots++) {
	    fb = (cbSlotTable(cb))[nslots];
	    if ((fb->access & ACC_STATIC) == 0) {
		char *fs = fieldsig(fb);
		NumFieldsWritten++;
		if (fb->u.offset != offset)
		    fprintf(f, "    char _PAD%d_[%d];\n",
			    fb->u.offset, fb->u.offset - offset);
		fprintf(f, "    ");
		switch (fs[0]) {
		  case SIGNATURE_ARRAY:
		    switch (fs[1]) {
		      case SIGNATURE_BYTE:
			fprintf(f, "struct HArrayOfByte *%s;\n", fieldname(fb));
			break;
		      case SIGNATURE_CHAR:
			fprintf(f, "struct HArrayOfChar *%s;\n", fieldname(fb));
			break;
		      case SIGNATURE_BOOLEAN:
			fprintf(f, "struct HArrayOfInt *%s;\n", fieldname(fb));
			break;
		      case SIGNATURE_SHORT:
			fprintf(f, "struct HArrayOfShort *%s;\n", fieldname(fb));
			break;
		      case SIGNATURE_INT:
			fprintf(f, "struct HArrayOfInt *%s;\n", fieldname(fb));
			break;
		      case SIGNATURE_LONG:
			fprintf(f, "struct HArrayOfLong *%s;\n", fieldname(fb));
			break;
		      case SIGNATURE_FLOAT:
			fprintf(f, "struct HArrayOfFloat *%s;\n", fieldname(fb));
			break;
		      case SIGNATURE_DOUBLE:
			fprintf(f, "struct HArrayOfDouble *%s;\n", fieldname(fb));
			break;
		      case SIGNATURE_ARRAY:
			fprintf(f, "struct HArrayOfArray *%s;\n", fieldname(fb));
			break;
		      case SIGNATURE_CLASS:
			fprintf(f, "struct HArrayOfObject *%s;\n", fieldname(fb));
			break;
		      default:
			fprintf(f, "void *%s;\n", fieldname(fb));
			break;
		    }
		    break;
		  case SIGNATURE_BYTE:
		  case SIGNATURE_CHAR:
		  case SIGNATURE_SHORT:
		    fprintf(f, "long %s;\n", fieldname(fb));
		    break;
		  default:
		    PrintType(f, fs, fieldname(fb));
		    fprintf(f, ";\n");
		}
		switch (fs[0]) {
		case SIGNATURE_ANY:
		    offset = fb->u.offset + sizeof(void *);
		    break;
		case SIGNATURE_BYTE:
		    offset = fb->u.offset + sizeof(/*char*/long);
		    break;
		case SIGNATURE_CHAR:
		    offset = fb->u.offset + sizeof(/*unicode*/long);
		    break;
		case SIGNATURE_DOUBLE:
		    offset = fb->u.offset + sizeof(double);
		    break;
		case SIGNATURE_ENUM:
		    offset = fb->u.offset + sizeof(/*enum*/long);
		    break;
		case SIGNATURE_FLOAT:
		    offset = fb->u.offset + sizeof(float);
		    break;
		case SIGNATURE_INT:
		    offset = fb->u.offset + sizeof(long);
		    break;
		case SIGNATURE_LONG:
		    offset = fb->u.offset + sizeof(int64_t);
		    break;
		case SIGNATURE_SHORT:
		    offset = fb->u.offset + sizeof(/*short*/long);
		    break;
		case SIGNATURE_VOID:
		    offset = fb->u.offset + sizeof(/*void*/long);
		    break;
		case SIGNATURE_BOOLEAN:
		    offset = fb->u.offset + sizeof(long);
		    break;
		default:
		    offset = fb->u.offset + sizeof(OBJECT);
		    break;
		}
	    } else if (fb->access & ACC_VALKNOWN)
		fprintf(f, "#define %s_%s %d\n", hcname,
			fieldname(fb), fb->u.offset);
		else
		    fprintf(f, "/* Inaccessible static: %s */\n", fieldname(fb));
	}
	if (NumFieldsWritten == 0)
	    fprintf(f, "    char PAD;\t/* ANSI C requires structures to have a least one member */\n");
	fprintf(f, "} Class%s;\n\n", hcname);
	fprintf(f, "HandleTo(%s);\n", hcname);
    }
    for (nslots = cb->methods_count, mb = cbMethods(cb); --nslots >= 0; mb++) {
	    fb = &mb->fb;
	    if (fb->access & ACC_NATIVE) {
		char *fn = fieldname(fb);
		fprintf(f, "extern ");
		PrintType(f, fieldsig(fb), fieldname(fb));
		fprintf(f, ";\n");
	    } 
    }
    fprintf(f, "#endif\n");

    return out ? 0 : close_output();
}


/*
 * Just decode those sigs that are meaningful to stubs.  We don't
 * support lots of things.
 */
static char *
SprintReverseArgs(char *sig, char *out, int argn, int *arg_size_p)
{
    char *pre = "", *post = "", *type = "", *macro = "";

    int arg_size = 1;		/*  */
    switch(*sig++) {
	case SIGNATURE_CLASS:
	    while (*sig++ != SIGNATURE_ENDCLASS);
	    post = ".p";
	    type = "void *";
	    break;
	case SIGNATURE_BOOLEAN:
	case SIGNATURE_BYTE:
	case SIGNATURE_SHORT:
	case SIGNATURE_CHAR:
	case SIGNATURE_INT:
	    type = "long";
	    post = ".i";
	    break;
	case SIGNATURE_LONG:
	    type = "int64_t";
	    macro = "GET_INT64";
	    arg_size = 2;
	    break;
	case SIGNATURE_FLOAT:
	    type = "float";
	    post = ".f";
	    break;
	case SIGNATURE_DOUBLE:
	    type = "double";
	    macro = "GET_DOUBLE";
	    arg_size = 2;
	    break;
	case SIGNATURE_VOID:
	    pre = "(void *)";	/* ??? */
	    type = "void";
	    post = ".p";
	    break;
	case SIGNATURE_ARRAY:
	    for (; *sig == SIGNATURE_ARRAY ; sig++);
	    if (*sig++ == SIGNATURE_CLASS) {
		while (*sig++ != SIGNATURE_ENDCLASS);
	    }
	    post = ".p";
	    type = "void *";
	    break;
	default:
	    fprintf(stderr, "%s: illegal signature\n", currentclassname);
	    return 0;
    }
    if (arg_size_p != NULL) 
        *arg_size_p = arg_size;

    if (argn == -1) {
	sprintf(out, ",%s", type);
    } else if (arg_size == 1) {
	sprintf(out, ",(%s(_P_[%d]%s))", pre, argn, post);
    } else {
	sprintf(out, ",%s(_P_+%d)", macro, argn);
    }
    return sig;
}

static int
PrintStub(FILE *f, char *fname, char *cname, struct fieldblock *fb)
{
    char *s;
    char proto[1024], funcall[1024], *c;
    char *obj_arg;
    
    int arg;

    if (!(fb->access & ACC_NATIVE))
	return 0;

    fprintf(f, "/* SYMBOL: \"%s/%s%s\", %s_%s_stub, */\n",
	    currentclassname, fieldname(fb), fieldsig(fb), 
	    cname, fieldname(fb));
    if (fb->access & ACC_NATIVE) {
	fprintf(f, "stack_item *%s_%s_stub(stack_item *_P_,struct execenv *_EE_) {\n",
		cname, fieldname(fb));

	/* Generate function prototype args */
	s = fieldsig(fb) + 1;   /* first char is SIGNATURE_FUNC */
	c = proto;
	*c = '\0';
	while (s && (c < proto + sizeof(proto)) && *s != SIGNATURE_ENDFUNC) {
	    s = SprintReverseArgs(s, c, -1, NULL);
	    if (s == 0)
		return -1;
	    c += strlen(c);
	}
	if (c >= proto + sizeof(proto)) {
	    fprintf(stderr, "out of buffer space!\n");
	    return -1;
	}

	/* Generate function call args */
	s = fieldsig(fb) + 1;   /* first char is SIGNATURE_FUNC */
	c = funcall;
	if (fb->access & ACC_STATIC) {
	    arg = 0;		/* first argument is _P_[0] */
	    strcpy(c, "NULL");
	} else {
	    arg = 1;
	    strcpy(c, "_P_[0].p");
	}
	c += strlen(c);
	while (s && (c < funcall + sizeof(funcall))
	         && *s != SIGNATURE_ENDFUNC) {
	    int argsize;
	    s = SprintReverseArgs(s, c, arg, &argsize);
	    if (s == 0)
		return -1;
	    c += strlen(c);
	    arg += argsize;
	}
	if (c >= funcall + sizeof(funcall)) {
	    fprintf(stderr, "out of buffer space!\n");
	    return -1;
	}

	switch(*++s) {
	  case SIGNATURE_VOID:
	      fprintf(f, "\textern void %s_%s(void *%s);\n",
		      cname, fieldname(fb), proto);
	      fprintf(f, "\t(void) %s_%s(%s);\n", 
		      cname, fieldname(fb), funcall);
	      fprintf(f, "\treturn _P_;\n}\n");
	      break;


	  case SIGNATURE_FLOAT:
	      fprintf(f, "\textern float %s_%s(void *%s);\n",
		      cname, fieldname(fb), proto);
	      fprintf(f, "\t_P_[0].f = %s_%s(%s);\n", 
		      cname, fieldname(fb), funcall);
	      fprintf(f, "\treturn _P_ + 1;\n}\n");
	      break;

	  case SIGNATURE_DOUBLE:
	      fprintf(f, "\textern double %s_%s(void *%s);\n",
		      cname, fieldname(fb), proto);
	      fprintf(f, "\tSET_DOUBLE(_P_, %s_%s(%s));\n", 
		      cname, fieldname(fb), funcall);
	      fprintf(f, "\treturn _P_ + 2;\n}\n");
	      break;

	  case SIGNATURE_BOOLEAN:
	  case SIGNATURE_BYTE:
	  case SIGNATURE_SHORT:
	  case SIGNATURE_CHAR:
	  case SIGNATURE_INT:
	  case SIGNATURE_CLASS:
	  case SIGNATURE_ARRAY:
	      fprintf(f, "\textern long %s_%s(void *%s);\n",
		      cname, fieldname(fb), proto);
	      fprintf(f, "\t_P_[0].i = %s_%s(%s);\n", 
		      cname, fieldname(fb), funcall);
	      fprintf(f, "\treturn _P_ + 1;\n}\n");
	      break;
	      
	  case SIGNATURE_LONG:
	      fprintf(f, "\textern int64_t %s_%s(void *%s);\n",
		      cname, fieldname(fb), proto);
	      fprintf(f, "\tSET_INT64(_P_, %s_%s(%s));\n", 
		      cname, fieldname(fb), funcall);
	      fprintf(f, "\treturn _P_ + 2;\n}\n");
	      break;

	  default:
	      fprintf(stderr, "%s: unsupported function return type: %c\n",
		      currentclassname, *s);
	      return -1;
	  }
    } else {
	/* JavaAwareC */
	fprintf(f, "extern stack_item *%s_%s_stub(stack_item *);\n",
		cname, fieldname(fb));
    }
    return 0;
}


int
DumpClassStub(ClassClass * cb)
{
    FILE *f;
    register struct methodblock *mb;
    register nslots;
    int offset = 0;
    char fname[300];   /* stub file name */
    char cname[300];   /* C symbol name.  */

    currentclassname = classname(cb);
    strsub(currentclassname, fname, F_DELIM);
    strsub(currentclassname, cname, C_DELIM);

    if ((f = open_output()) == 0) {
	return -1;
    }
    if (verbose)
	fprintf(stderr, "=> %s\n", tempfn);

    fprintf(f, "\n/* Stubs for class %s */\n", currentclassname);
    makeslottable(cb);

    for (nslots = cb->methods_count, mb = cbMethods(cb); --nslots >= 0; mb++) {
	if (mb->code == 0) {
	    if (PrintStub(f, fname, cname, &mb->fb)) {
		fclose(f);
		unlink(tempfn);
		exit(-1);
	    }
	}
    }
    return out ? 0 : close_output();
}


static void
InitializeClassPath(int argc, char **argv)
{
    int i;
    char *c;
    const char *cp = "CLASSPATH=";

    for (i = 1; i < argc; i++) {
	if (argv[i] && strcmp(argv[i], "-classpath") == 0) {
	    if (i+1 == argc || argv[i+1] == 0) {
		fprintf(stderr,
			"Missing destination directory name after "
			"'-classpath'\n");
		exit(1);
	    }
	    c = (char *)malloc(strlen(argv[i+1]) + strlen(cp) + 1);
	    sprintf(c, "%s%s", cp, argv[i+1]);
	    putenv(c);
	}
    }
    return;
}

main(argc, argv)
    register char **argv;
{
    int retcode = 0;
    int donesomething = 0;
    int MemPoolSize = MEMPOOLSIZE;
    long T;
    SkipSourceChecks = 1;
    mainstktop = (unsigned char **) &retcode;
    progname = argv[0];
    InhibitExecute = 1;

    InitializeClassPath(argc, argv);
    if (!DoImport("java/lang/Object", 0)) {
	fprintf(stderr, "java.lang.Object not found: aborting\n");
	exit(1);
    }
    while (--argc > 0) {
	if ((++argv)[0][0] == '-') {
	    if ((strcmp(*argv, "-d") == 0) && (argc > 1)) {
		dir = *++argv;
	        argc--;
	    } else if ((strcmp(*argv, "-o") == 0) && (argc > 1)) {
		out = *++argv;
		argc--;
	    } else if ((strcmp(*argv, "-td") == 0) && (argc > 1)) {
		tempdir = *++argv;
		argc--;
	    } else if (strcmp(*argv, "-stubs") == 0) {
		stubmode++;
	    } else if (strcmp(*argv, "-v") == 0) {
		verbose++;
	    } else if (strcmp(*argv, "-classpath") == 0) {
		argc--;
		argv++;
	    } else {
		fprintf(stderr, "%s: illegal argument\n", *argv);
		close_output();
		return 1;
	    }
	} else if (strchr(*argv, '/')) {
	    fprintf(stderr, "Invalid class name: %s\n", *argv);
	    close_output();
	    return 1;
	} else {
	    ClassClass *cb;
	    char *classcopy = strdup(*argv), *p;

	    /* Convert all periods in the classname to slashes */
	    for (p = classcopy; ((p = strchr(p, '.')) != 0); *p++ = '/');

	    cb = FindClass(0, classcopy, TRUE);
	    donesomething++;
	    if (cb == 0) {
		retcode = 1;
		fprintf(stderr, "%s: no such class\n", classcopy);
		continue;
	    }
	    if (stubmode) {
		if (DumpClassStub(cb) < 0) {
		    retcode = 1;
		}
	    } else {
		if (DumpClassHeader(cb) < 0) {
		    retcode = 1;
		}
	    }
	}
    }
    if (!donesomething) {
	retcode = 1;
	fprintf(stderr, "Usage: %s [ -v ] [ -l filename ] classes...\n", progname);
    }
    if (close_output() < 0) {
	retcode = 1;
    }
    return retcode;
}

int
OpenCode(char *fn, char *sfn, char *dir, struct stat * st)
{
    extern int open(char *, int, int);
    extern int fstat(int, struct stat *);
    long codefd;
    if (fn == 0 || (codefd = open(fn, 0, 0644)) < 0
	    || fstat(codefd, st) < 0)
	return -2;
    return codefd;
}

HObject *
AllocHandle(struct methodtable *mptr, ClassObject *p)
{
    Handle *h = malloc(sizeof(Handle));

    h->methods = mptr;
    h->obj = (ClassObject *) p;
    return (HObject *) h;
}




/* Dummies */
#undef DumpThreads

void
DumpThreads()
{
}

char *
vramalloc(int n)
{
    return malloc(n);
}

void
ClassDefHook(ClassClass * cb)
{
}

long
CallInterpreted(struct methodblock *mb, void *obj, ...) {
    return 0;
}

ExecEnv *
EE() {
    static struct execenv lee;
    return &lee;
}

/* ResolveClassStringConstant is called by the classloader to resolve the
 * constant pool entry, before assigning a value to a final [i.e. constant]
 * String.
 *
 * For javah, we don't care.
 */

bool_t
ResolveClassStringConstant(ClassClass *cb, unsigned index, struct execenv *ee)
{
    return TRUE;
}
