/*
 * @(#)javai.c	1.53 95/04/18 James Gosling
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*-
 *      interpret an java object file
 */

#include <stdio.h>
#include <threads.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <tree.h>
#include <oobj.h>
#include <interpreter.h>
#include <javaString.h>
#include <log.h>
#include "installpath.h"
#include <bool.h>
#ifndef NATIVE
#include <iomgr.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <unistd.h>
#else NATIVE
#include <monitor.h>
#endif NATIVE

#ifdef MONITOR
#include <mon.h>
#endif

#ifndef NATIVE
#include <java_lang_Thread.h>
#endif NATIVE

extern Hjava_lang_Thread *InitializeClassThread(ExecEnv *ee, char **);

extern void WaitToDie();


static void usage(void);

#ifdef MONITOR
static void MonitorTeardown() {
    monitor( 0, 0, 0, 0, 0 );
}

static void MonitorSetup() {
    int atexit(void (*)() );
    extern int etext();
    extern int _start();
    int (* low)() = &_start;
    int (* high)() = & etext;
    WORD * stuff;
    int textwords = ((int)high - (int)low) / 4;
    int nbytes = sizeof(struct hdr) + 10*sizeof( struct cnt )
				    + textwords*sizeof(WORD);
    stuff = (WORD*)malloc( nbytes );
    monitor( low, high, stuff, nbytes/sizeof(WORD), 10 );
    atexit( MonitorTeardown );
}
#endif

/*
 * have to put the hidden copy of the command line argument list where GC
 * will find it
 */

HArrayOfString *
commandLineArguments(int argc, char **argv, struct execenv *ee)
{
    HArrayOfString *ret = (HArrayOfString *) ArrayAlloc(T_CLASS, argc);
    if (ret == 0) {
OutOfMemory:
	SignalError(ee, JAVAPKG "OutOfMemoryException", 0);
	return 0;
    }
    /* Give the array a "type" */
    unhand((HArrayOfObject *)ret)->body[argc] = 
                        (Handle *)FindClass(ee, JAVAPKG "String", TRUE);
    while (--argc >= 0) {
	char *s = argv[argc];
	if ((unhand(ret)->body[argc] = makeJavaString(s, strlen(s))) == 0)
	    goto OutOfMemory;
    }
    return (HArrayOfString *) ret;
}

long
atomi(register char *p)
{
    long v = 0;
    long mult = 1;
    long c;
    while (c = *p++)
	switch (c) {
	case 'M':
	case 'm':
	    mult = 1024 * 1024;
	    break;
	case 'K':
	case 'k':
	    mult = 1024;
	    break;
	default:
	    if (c < '0' || c > '9')
		return -1;
	    v = v * 10 + c - '0';
	    break;
	}
    return v * mult;
}

void
#ifndef NATIVE
makeTTYsane()
{
    nonblock_io(0, IO_BLOCK);
    nonblock_io(1, IO_BLOCK);
    nonblock_io(2, IO_BLOCK);
}
#endif NATIVE

extern void InitializeAsyncMonitors();

int
start_java(argc, argv)
    register char **argv;
{
    int retcode;
    char *sourcefile = 0;
    char **args = argv;
    char **dargv = argv;
    int MemPoolSize = MEMPOOLSIZE;
    long T;
    Hjava_lang_Thread *self;
    int on_exit(void (*)(), void *);
    struct methodblock  *mb = 0;
    extern int noasyncgc;
    extern int tracegc;
    ExecEnv ee;
    char *errmsg;

    intrInit();
#ifndef NATIVE
    atexit(makeTTYsane);
#endif NATIVE
    /* Initialize the async monitors */
    InitializeAsyncMonitors();
    /* Create the monitor cache */
    monitorCacheInit();
    /* Initialize the monitor registry */
    monitorRegistryInit();
#ifdef DEBUG
/*    on_exit(PrintStats, 0); */
#endif
    {   /* assume -cs if executed as "java"
	   rather than "java" [DOES THIS MAKE SENSE ANY MORE??? */
	char *p = &argv[0][strlen(argv[0]) - 1];
	if (p[0] == 'g' && p[-1] == '_')
	    p -= 2;
	if (p[0] != 'k')
	    SkipSourceChecks = 1;
    }
    mainstktop = (unsigned char **) &retcode;
    progname = argv[0];
    verifyclasses = VERIFY_REMOTE; /* we've decided this is the default */
    while (--argc > 0)
	if ((++argv)[0][0] == '-' && sourcefile == 0)
#ifndef trace
	    if (strcmp(*argv, "-t") == 0)
		trace++;
	    else
#endif
#ifdef MONITOR
	    if (strcmp(*argv, "-m") == 0) {
    		MonitorSetup();
	    }
	    else
#endif	    
	    if (strcmp(*argv, "-v") == 0 || strcmp(*argv, "-verbose") == 0)
		verbose++;
	    else if (strcmp(*argv, "-noasyncgc") == 0)
		noasyncgc++;
	    else if (strcmp(*argv, "-verbosegc") == 0)
		verbosegc++;
	    else if (strcmp(*argv, "-verify") == 0)
		verifyclasses = VERIFY_ALL;
            else if (strcmp(*argv, "-verifyremote") == 0)
		verifyclasses = VERIFY_REMOTE;
            else if (strcmp(*argv, "-noverify") == 0)
		verifyclasses = VERIFY_NONE;
	    else if (strcmp(*argv, "-tracegc") == 0)
		tracegc++;
	    else if (strcmp(*argv, "-prof") == 0) {
		extern void java_mon_dump();
		extern void javamon(int i);
		atexit(java_mon_dump);
		javamon(1);
	    } else if (strcmp(*argv, "-cs") == 0 || 
		     strcmp(*argv, "-checksource") == 0)
		SkipSourceChecks = 0;
	    else if (strncmp(*argv, "-ss", 3) == 0 &&
		     (T = atomi(&argv[0][3])) > 1000)
		ProcStackSize = T;
	    else if (strncmp(*argv, "-oss", 3) == 0 &&
		     (T = atomi(&argv[0][3])) > 1000)
		JavaStackSize = T;
	    else if (strncmp(*argv, "-ms", 3) == 0 &&
		     (T = atomi(&argv[0][3])) > 1000)
		MemPoolSize = T;
#ifdef LOGGING
	    else if (strncmp(*argv, "-l", 2) == 0 &&
		     (T = atomi(&argv[0][2])))
		logging_level = T;
#endif
	    else if (strncmp(*argv, "-classpath", 10) == 0) {
		if (argc > 1) {
		    char *buf = (char *)malloc(strlen(argv[1]) + 32);
		    sprintf(buf, "CLASSPATH=%s", argv[1]);
		    putenv(buf);
		    argc--; argv++;
		} else {
		    fprintf(stderr,
			    "-classpath requires class path specification\n");
		    usage();
		    return 1;
		}
	    } else if (strcmp(*argv, "-h") || strcmp(*argv, "-help") ||
		       strncmp(*argv, "?", 1) || strncmp(*argv, "-?", 2)) {
		usage();
		return 0;
	    } else {
		fprintf(stderr, "%s: illegal argument\n", *argv);
		usage();
		return 1;
	    }
	else if (sourcefile)
	    *dargv++ = *argv;
	else
	    sourcefile = *argv;
    *dargv = 0;
    InitializeMems();
    InitializeAlloc(MemPoolSize);
    InitializeInterpreter();	/* Call before InitializeClassThread() */
    InitializeExecEnv(&ee, 0);

    self = InitializeClassThread(&ee, &errmsg);
    InstallCompiledStrings();
    if (self == 0) {
	fprintf(stderr, "Unable to initialize threads: %s\n", errmsg);
	exit(1);
    }
    setThreadName(self,MakeString("main",4));

    if (sourcefile == 0) {
	retcode = 1;
	usage();
	return 1;
    } if (strchr(sourcefile, '/')) {
	fprintf(stderr, "Invalid class name: %s\n", sourcefile);
	retcode = 1;
	usage();
	return 1;
    } else {
	ClassClass *cb;
	int i, c;
	char classname[200];

	/* Allow the use of '.' in class names (convert to /) */
	for (i = 0; i < sizeof classname - 1 && (c = sourcefile[i]) != 0; i++) {
	    classname[i] = c;
	    if (c == '.') sourcefile[i] = '/';
	    if (c == '/') classname[i] = '.';
	}
	classname[i] = 0;

	cb = FindClass(&ee, sourcefile, TRUE);
	if (cb == 0) {
	    fprintf(stderr, "Can't find class %s\n", classname);
	    return 1;
	}
	
	mb = cbMethods(cb);
	for (i = cb->methods_count; --i >= 0; mb++) {
	    if ((strcmp(fieldname(&mb->fb), "main") == 0) &&
		(strncmp(fieldsig(&mb->fb), "([Ljava/lang/String;)", 20) == 0)){
		
		if (fieldsig(&mb->fb)[21] == SIGNATURE_VOID)
		    break;
		/* If we matched all the way to the return type, and its 
		 * return type is not SIGNATURE_VOID, then we know there is
		 * no method match because the compiler would complain about
		 * duplicate method declaration. */
		fprintf(stderr, "In class %s: main must return void\n",
			classname);
		return 1;
	    }
	}
	if (i < 0) {
	    fprintf(stderr, "In class %s: void main(String argv[]) is not defined\n",
		    classname);
	    return 1;
	}
	
	if ((mb->fb.access & (ACC_STATIC|ACC_PUBLIC))
	    != (ACC_STATIC|ACC_PUBLIC)) {
	    fprintf(stderr, "In class %s: main must be public and static\n", classname);
	    return 1;
	}
	do_execute_java_method(&ee, cb, 0, 0, mb, TRUE,
		       (Handle *)commandLineArguments(dargv - args, args, &ee));
	if (exceptionOccurred(&ee))
	    exceptionDescribe(&ee);
    }

    /*
     * The primordial thread is not allowed to exit until all user
     * threads have exited.  Note that the primordial thread's
     * resources (e.g. its Java stack) have not been reclaimed at this
     * point, even though it's got no chance of running again.  The
     * thread is still on the active thread queue and will be scanned
     * on GC, so has to be left intact.  This should be revisited.
     */
    WaitToDie();

    return 0;
}


int
main(int argc, char **argv)
{
    return start_java(argc, argv);
}

void
ClassDefHook(ClassClass *cb)
{
}

static void
Execute(char **command, char *alternate)
{
    long pid, wpid;
    long lapcnt;
    int status = -1;
    if (verbose) {
	printf("[Executing");
	for (lapcnt = 0; command[lapcnt]; lapcnt++)
	    printf(" %s", command[lapcnt]);
	printf("]\n");
    }
    lapcnt = 0;
    while ((pid = fork()) < 0) {
	if (lapcnt == 0)
	    write(2, "[ Running out of system memory, waiting...", 42);
	lapcnt++;
	sleep(5);
    }
    if (pid) {
	if (lapcnt)
	    write(2, " got it ]\n", 10);
	while ((wpid = wait(&status)) == -1 || wpid != pid);
	if (status != 0) {
	    fprintf(stderr, "%s: failed (%X)\n", command[0], status);
	    exit(1);
	}
    } else {
	lapcnt = 0;
	while (1) {
	    execvp(command[0], command);
	    if (alternate)
		execvp(alternate, command);
	    if (errno != ENOMEM) {
		perror(command[0]);
		exit(1);
	    }
	    if (lapcnt == 0)
		write(2, "Waiting for system memory...\n", 29);
	    sleep(20);
	}
    }
    if (verbose)
	printf("[Finished %s]\n", command[0]);
}

/* OpenCode for the interpreter.

   Tries to open a code file (fn).  That is, a .class file.  This
   returns -1 if the specified file can be opened but cannot be
   stat'd.  REMIND: When can that happen?  It returns -2 if the
   file cannot be opened.  Otherwise, it returns a valid file
   descriptor. If SkipSourceChecks is true, then we return at that
   point.

   Otherwise, we try to recompile the source.  If fn == 0, then we
   use sfn as the source file name to use.  sfn is specified by
   LoadFile by using the source hint associated with the import
   statement, or by the source file name as extracted from the
   previously recompiled version of this class.  That only happens
   if LoadFile already successfully loaded the class, and then
   noticed that the source is more recent than the .class file it
   opened.

   If sfn is 0 then a source file name is constructed by stripping
   the ".class" off of fn and replacing it with .java.  This often
   doesn't work, because .class files can be put in difference
   directories from their source files, but it's a good guess
   sometimes.  Then javac is forked to recompile the class, and
   the resulting .class file is opened.

   In summary, OpenCode for the interpreter returns a file
   descriptor for a compiler .class file.  If it can't open the
   .class file, it forks the compiler to compile some source so
   that the .class file will be there.  If after all that there is
   still no .class file, it gives up and returns -2.

   If st == 0, then doesn't even bother trying to open the file.
   Instead, it just does the source file thing. */

int
OpenCode(char *fn, char *sfn, char *dir, struct stat * st)
{
    long fd = -1;
    char srcname[300];
    char sccsname[300];
    long len;
    long mtime;
    struct stat srcst, sccsst;
    char *src, *dst;

    if (st != 0 && fn != 0 && (fd = open(fn, 0, 0644)) >= 0
	    && fstat(fd, st) < 0) {
	close(fd);
	fd = -1;
    }
    if (SkipSourceChecks)
	return fd < 0 ? -2 : fd;
    dst = srcname;
    *dst++ = '-';
    *dst++ = 'G';
    if (sfn == 0) {
	assert(fn);
	for (src = fn; *dst++ = *src++;);
	if (dst - srcname < 3 + JAVAOBJEXTLEN
		|| strcmp(dst - JAVAOBJEXTLEN - 2,
			  "." JAVAOBJEXT) != 0)
	    return fd < 0 ? -2 : fd;
	dst -= JAVAOBJEXTLEN + 2;
	strcpy(dst, "." JAVASRCEXT);
    } else {
	for (src = sfn; *dst++ = *src++;);
    }
    while (dst > srcname + 2 && dst[-1] != '/')
	dst--;
    strncpy(sccsname, srcname + 2, dst - srcname);
    strncpy(sccsname + (dst - srcname - 2), "SCCS/s.", 7);
    strcpy(sccsname + (dst - srcname) + 5, dst);
    mtime = ((fd < 0 && fn) || st == 0) ? 0 : st->st_mtime;
    if (stat(srcname + 2, &srcst) < 0)
	srcst.st_mtime = 0;
    if (stat(sccsname, &sccsst) < 0)
	sccsst.st_mtime = 0;
    if (srcst.st_mtime > mtime
	    || sccsst.st_mtime > mtime) {
	char *com[30];
	char **comp = com;
	if (fd >= 0) {
	    close(fd);
	    fd = -1;
	}
	if (sccsst.st_mtime > srcst.st_mtime) {
	    com[0] = "/usr/sccs/get";
	    com[1] = sccsname;
	    com[2] = srcname;
	    com[3] = 0;
	    Execute(com, 0);
	}
#ifdef DEBUG
	*comp++ = "javac_g";
#else
	*comp++ = "javac";
#endif
	if (verbose)
	    *comp++ = "-verbose";
	if (dir) {
	    *comp++ = "-d";
	    *comp++ = dir;
	}
	*comp++ = srcname+2;
	*comp = 0;
	{
	    char    path[256];
	    char    *alt = 0;
	    char    *ep;


	    if ((ep = getenv("HOTJAVA_HOME")) == 0)
		alt = 0;
	    else {
		sprintf(path, "%s/bin/", ep);
#ifdef DEBUG
		strcat(path, "javac_g");
#else
		strcat(path, "javac");
#endif
		alt = path;
	    }
	    Execute(com, alt);
	}
	if (fn) {
	    fd = open(fn, 0, 0644);
	    if (fd >= 0 && st)
		if (fstat(fd, st) < 0) {
		    close(fd);
		    fd = -1;
		}
	}
    }
    return fd < 0 ? -2 : fd;
}

static void usage()
{
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "%s [-options] class\n", progname);
    fprintf(stderr, "\n");
    fprintf(stderr, "where options include:\n");
    fprintf(stderr, "    -help             print out this message\n");
#ifndef trace
    fprintf(stderr, "    -t                turn on tracing\n");
#endif
#ifdef MONITOR
    fprintf(stderr, "    -m                turn on monitoring\n");
#endif
    fprintf(stderr, "    -v -verbose       turn on verbose mode\n");
    fprintf(stderr, "    -noasyncgc        don't allow asynchronous gc's\n");
    fprintf(stderr, "    -verbosegc        print a message when GCs occur\n");
    fprintf(stderr, "    -cs -checksource  check if source is newer when loading classes\n");
    fprintf(stderr, "    -ss<number>       set the C stack size of a process\n");
    fprintf(stderr, "    -oss<number>      set the JAVA stack size of a process\n");
    fprintf(stderr, "    -ms<number>       set the memory pool size\n");
    fprintf(stderr, "    -l<number>        set the logging level\n");
    fprintf(stderr, "    -classpath <directories separated by colons>\n");
    fprintf(stderr, "                      list directories in which to look for classes\n");
    fprintf(stderr, "    -prof             output profiling data to ./java.prof\n");
    fprintf(stderr, "    -verify           verify all classes when read in\n");
    fprintf(stderr, "    -verifyremote     verify classes read in over "
                                              "the network [default]\n");
    fprintf(stderr, "    -noverify         do not verify any class\n");
}
