/*
 * @(#)CompSupportAs.s	1.3 95/04/27 Frank Yellin
 *
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */


    	.file "CompSupportAs.s"

	.section ".text",#alloc,#execinstr


/* Multiplication and Divison */

    	.global CompSupport_imul
    	.global CompSupport_idiv
    	.global CompSupport_imod
    	.global CompSupport_lmul
    	.global CompSupport_ldiv
    	.global CompSupport_lmod
    	.global CompSupport_fmod
    	.global CompSupport_dmod
    	.global CompSupport_lshl
    	.global CompSupport_lshr
    	.global CompSupport_lushr

CompSupport_imul:           
    	mov 	%o7, %g1
    	call	.mul
    	mov 	%g1, %o7
    	.type   CompSupport_imul, #function
    	.size	CompSupport_imul, (. - CompSupport_imul)

CompSupport_idiv:   
    	mov 	%o7, %g1
    	call	.div
    	mov 	%g1, %o7
    	.type   CompSupport_idiv, #function
    	.size	CompSupport_idiv, (. - CompSupport_idiv)

CompSupport_imod:           
    	mov 	%o7, %g1
    	call	.rem
    	mov 	%g1, %o7
    	.type   CompSupport_imod, #function
    	.size	CompSupport_imod, (. - CompSupport_imod)

CompSupport_lmul:           
    	mov 	%o7, %g1
    	call	__mul64
    	mov 	%g1, %o7
    	.type   CompSupport_lmul, #function
    	.size	CompSupport_lmul, (. - CompSupport_lmul)

CompSupport_ldiv:   
    	mov 	%o7, %g1
    	call	__div64
    	mov 	%g1, %o7
    	.type   CompSupport_ldiv, #function
    	.size	CompSupport_ldiv, (. - CompSupport_ldiv)

CompSupport_lmod:           
    	mov 	%o7, %g1
    	call	__rem64
    	mov 	%g1, %o7
    	.type   CompSupport_lmod, #function
    	.size	CompSupport_lmod, (. - CompSupport_lmod)

CompSupport_fmod:   
    	! remainder, two single floats
    	ld  	[%fp+68], %f0	! dividend
    	ld  	[%fp+72], %f2	! divisor 
    	fdivs	%f0, %f2, %f4	! quotient
    	fstoi   %f4, %f4	! floor(quotient)
    	fitos   %f4, %f4
    	fmuls	%f2, %f4, %f4	! floor(quotient) * divisor
    	retl
    	fsubs	%f0, %f4, %f0
    	.type   CompSupport_fmod, #function
    	.size	CompSupport_fmod, (. - CompSupport_fmod)

CompSupport_dmod:   
    	! remainder, two single floats
    	ld  	[%fp+68], %f0	! dividend
    	ld  	[%fp+72], %f1	! 
    	ld  	[%fp+76], %f2	! divisor
    	ld  	[%fp+80], %f3	! 
    	fdivd	%f0, %f2, %f4	! quotient
    	fdtoi   %f4, %f4	! floor(quotient)
    	fitod   %f4, %f4
    	fmuld	%f2, %f4, %f4	! floor(quotient) * divisor
    	retl
    	fsubd	%f0, %f4, %f0
    	.type   CompSupport_dmod, #function
    	.size	CompSupport_dmod, (. - CompSupport_dmod)



CompSupport_lshl:   
    	!! o0/o1 contain a long.  low six bits of %o2 are amount to shift left
    	andcc	%o2, 0x20, %g0	! is shift <31 or >= 32?
    	beq	1f		! it's small
    	 andcc	%o2, 0x1f, %o2  ! is shift zero?    	

    	!! the shift is between 32 and 63
     	sll 	%o1, %o2, %o0	! shift low word into high word
    	retl
    	mov 	0, %o1	        ! set high word to zero

    	!! the shift is between 0 and 31
1:      beq 	2f		! shift is zero
         neg	%o2, %o3	! 32 - shift in low 5 bits
     	sll 	%o0, %o2, %o0	! shift high word left in place
    	srl 	%o1, %o3, %o3	! transfer from low to high is high bits
    	sll 	%o1, %o2, %o1	! shift low word left in place
2:     	retl
    	or  	%o3, %o0, %o0	! or in the shifted part
    	.type   CompSupport_lshl, #function
    	.size	CompSupport_lshl, (. - CompSupport_lshl)


CompSupport_lshr:   
    	!! o0/o1 contain a long.  low six bits of %o2 are amount to shift right
    	andcc	%o2, 0x20, %g0	! is shift <31 or >= 32?
    	beq	1f		! it's small
    	 andcc	%o2, 0x1f, %o2  ! is shift zero?    	

    	!! the shift is between 32 and 63
     	sra 	%o0, %o2, %o1	! shift high word into low word
    	retl
    	sra 	%o0, 31, %o0	! high word keeps the sign

    	!! the shift is between 0 and 31
1:      beq 	2f		! shift is zero
         neg	%o2, %o3	! 32 - shift in low 5 bits
     	srl 	%o1, %o2, %o1	! shift low word right in place
    	sll 	%o0, %o3, %o3	! transfer from high to low is low bits
    	sra 	%o0, %o2, %o0	! shift high word right (arithmetic) in place
2:     	retl
    	or  	%o3, %o1, %o1	! or in the shifted part
    	.type   CompSupport_lshr, #function
    	.size	CompSupport_lshr, (. - CompSupport_lshr)

CompSupport_lushr:   
    	!! o0/o1 contain a long.  o2 contains the amount to shift left
    	andcc	%o2, 0x20, %g0	! is shift <31 or >= 32?
    	beq	1f		! it's small
    	 andcc	%o2, 0x1f, %o2  ! is shift zero?    	

    	!! the shift is between 32 and 63
     	srl 	%o0, %o2, %o1	! shift high word into low word
    	retl
    	mov 	0, %o0		! high word is zero

    	!! the shift is between 0 and 31
1:      beq 	2f		! shift is zero
         neg	%o2, %o3	! 32 - shift in low 5 bits
     	srl 	%o1, %o2, %o1	! shift low word right in place
    	sll 	%o0, %o3, %o3	! transfer from high to low is low bits
    	srl 	%o0, %o2, %o0	! shift high word right (logically) in place
2:     	retl
    	or  	%o3, %o1, %o1	! or in the shifted part
    	.type   CompSupport_lushr, #function
    	.size	CompSupport_lushr, (. - CompSupport_lushr)

	.section ".text",#alloc,#execinstr

/* Conversion of longs to floats and floats to longs */    

    	.global CompSupport_d2l
    	.global CompSupport_f2l
    	.global CompSupport_l2d
    	.global CompSupport_l2f

CompSupport_f2l:   
    	! convert float to long.  Use library routine
	ld	[%sp+68], %o0	! first word of double
        mov	%o7, %g1	! save the return address (bashed by call)
	call	__ftoll,1	! Tail call.  Result in %o0/%o1
        mov     %g1, %o7	! restore return addess
    	.type   CompSupport_f2l, #function
    	.size	CompSupport_f2l, (. - CompSupport_f2l)

CompSupport_d2l:   
    	! convert double to long
	ld	[%sp+68], %o0	! first word of double
        mov	%o7, %g1	! save the return address (bashed by call)
    	ld	[%sp+72], %o1	! second word of double
	call	__dtoll,2	! Tail call.  Result = %o0/%o1
        mov     %g1, %o7	! restore return addess
    	.type   CompSupport_d2l, #function
    	.size	CompSupport_d2l, (. - CompSupport_d2l)


CompSupport_l2f:   
    	! convert long to float
    	save	%sp, -104 ,%sp
    	std	%i0, [%sp + 96]
    	tst 	%i1		! what is the sign of the low word?
    	ldd 	[%sp + 96], %f2	! f2 = high word, f3 = low word
    	sethi   %hi(FLongMultiplier), %o0
    	fitos	%f2, %f4	! f4 = (float)high
       	ld 	[%o0 + %lo(FLongMultiplier)], %f6 ! f6 = (float)(1 << 32)
    	fitos	%f3, %f2	! f2 = (float)low
    	fmuls	%f4, %f6, %f4	! f4 = (float)(high * (1<<32))
    	bneg,a	1f		! if (low < 0) then . . . .
    	fadds	%f2, %f6, %f2	!     coerce (float)low to unsigned
1:	fadds   %f2, %f4, %f0
    	ret
	restore
    	.type   CompSupport_l2f, #function
    	.size	CompSupport_l2f, (. - CompSupport_l2f)

CompSupport_l2d:   
    	! convert long to double
    	save	%sp, -104 ,%sp
    	std	%i0, [%sp + 96]
    	tst 	%i1		! what is the sign of the low word?
    	ldd 	[%sp+96], %f2   ! f2 = high word, f3 = low word
    	sethi   %hi(DLongMultiplier), %o0
    	fitod	%f2, %f4	! f4 = (double)high
       	ldd 	[%o0 + %lo(DLongMultiplier)], %f6 ! f6 = (double)(1 << 32)
    	fitod	%f3, %f2	! f2 = (double)low
    	fmuld	%f4, %f6, %f4	! f4 = (double)(high * 1<<32)
    	bneg,a	1f		! if (low < 0) then . . .
    	  faddd	%f2, %f6, %f2	! f2 += (double) (1 << 32) (to make it unsigned)
1:      faddd   %f2, %f4, %f0	! add low and high parts.
    	ret
	restore	
    	.type   CompSupport_l2d, #function
    	.size	CompSupport_l2d, (. - CompSupport_l2d)

	.pushsection	".rodata",#alloc
        .align 8
DLongMultiplier:        .double 0r4294967296.0
FLongMultiplier:        .single 0r4294967296.0
    	.popsection



	.section ".text",#alloc,#execinstr

/* Support for invoking interfaces and methods from arrays */

    	.global CompSupport_invokevirtualobject
    	.global CompSupport_invokeinterface

CompSupport_invokevirtualobject:	    
    	!! g1 contains the supposed method table.  If the object we have is
    	!! really an array, then return classobject's method table instead.
	and	%g1, 0x1f, %g0	! are the bottom five bits zero?
    	beq 	1f
    	sethi   %hi(ObjectMethodTable), %g1
        retl
    	ld  	[%g1 + %lo(ObjectMethodTable)], %g1

1:	retl
    	nop
    	.type   CompSupport_invokevirtualobject, #function
    	.size	CompSupport_invokevirtualobject, (. - CompSupport_invokevirtualobject)


CompSupport_invokeinterface:	
    	save	%sp, -96, %sp
    	!! i0 contains an object.  (Other args are in i1 ... i5)
    	!! g1 contains an ID of a method.  

        !! call void *MethodID2CompiledMethod(Handle *o, long ID, 
        !!                                    struct methodblock **mptr)
    	!! We will get back the methodblock, and the CompiledCode field    
    	mov 	%i0, %o0	! the object
    	mov	%g1, %o1	! the methodID
        call    MethodID2CompiledMethod
    	add 	%fp, -4, %o2    ! method block pointer

    	ld  	[%fp - 4], %g1	! put the method block into %g1
    	jmp 	%o0		! Jump to the compiled function.
    	restore
    	.type   CompSupport_invokeinterface, #function
    	.size	CompSupport_invokeinterface, (. - CompSupport_invokeinterface)



/* These functions (which are all really the same function!) are used to
 * trampoline from interpreted code to compiled code.  They have multiple 
 * names because C needs to have different return value tpes.
 */
    	.global InvokeCompiledMethodVoid
     	.global InvokeCompiledMethodInt
        .global InvokeCompiledMethodFloat
    	.global InvokeCompiledMethodLong
        .global InvokeCompiledMethodDouble
    
    	.align 8
InvokeCompiledMethodVoid:   
InvokeCompiledMethodInt:    
InvokeCompiledMethodFloat:  
InvokeCompiledMethodLong:   
InvokeCompiledMethodDouble: 
    	!! InvokeCompiledMethodxxxx(mb, code, optop, argument_count)
    	!! Save enough space on the stack to copy all the arguments.
    	sll 	%o3, 2, %o4	! o4 = argument count * 4
    	add 	%o4, 96, %g1	! g1 = 96 + 4 * argument count
    	neg 	%g1, %g1
    	and 	%g1, -8, %g1	! round down to a multiple of 8
    	save	%sp, %g1, %sp
    	!! copy the arguments from optop[i] to sp[68+i].
    	!! %i3 has argument_count *4
    	subcc	%i4, 4, %i4
    	ba   	ICLoop
    	add 	%sp, (68 + 4), %i5 
ICHead:	subcc   %i4, 4, %i4
    	st  	%l0, [%i5 + %i4]
    	!! Loop invariant.  We are about to copy from [optop + %i4] to 
    	!! [sp + 72 + %i4] == [%i5 + %i4 + 4].  We still have more copying to
    	!! go if %i4 >= 0.  The cc bits on %i4 have already been set.
ICLoop:	bge 	ICHead		! continue loop
    	ld  	[%i2 + %i4],%l0 

    	!! indicate that this is the entering frame into compiled code
    	st 	%g0, [%sp + 64]

    	!! copy stuff from the stack into the out arguments.
    	ld 	[%sp + 68], %o0
    	ld 	[%sp + 72], %o1
       	ld 	[%sp + 76], %o2
       	ld 	[%sp + 80], %o3
    	mov 	%i0, %g1	! the code might want to know its methodblock
       	ld 	[%sp + 84], %o4
    	jmpl 	%i1, %o7	! jump.
       	ld 	[%sp + 88], %o5

    	! when we return, the value will either be in o0, o0/o1, f0 or f0/f1
    	mov	%o1, %i1    	
    	ret
    	restore	%o0, 0, %o0    	
    	.type   InvokeCompiledMethodVoid, #function
    	.size	InvokeCompiledMethodVoid, (. - InvokeCompiledMethodVoid)


/* These functions are used to trampoline from compiled code to interpreted
 * code.  They are stored in the CompiledCode slot of an interpreted method.
 */

	.section ".text",#alloc,#execinstr
    	.global CallbackStaticMethod
    	.global CallbackDynamicMethod
    	.global CallbackNativeMethod

    	.align 8
CallbackDynamicMethod:
    	save	%sp, -128, %sp
    	mov 	0, %l1		! is static argument
    	ba  	CallbackInterpretedMethod
    	add 	%fp, 72, %l2    ! pointer to where the first argument goes
    	.type   CallbackDynamicMethod, #function
    	.size	CallbackDynamicMethod, (. - CallbackDynamicMethod)


CallbackStaticMethod:   
    	save	%sp, -128, %sp
    	mov 	-1, %l1		! is static argument
    	ba  	CallbackInterpretedMethod
    	add 	%fp, 68, %l2    ! pointer to where the first argument goes
    	.type   CallbackStaticMethod, #function
    	.size	CallbackStaticMethod, (. - CallbackStaticMethod)


CallbackInterpretedMethod:    
    	!! call getCompiledCodeInfo(char **codePtr) to get flags and code
       	mov	%g1, %l0	! move the methodblock out of the way
    	mov	%g1, %o0
    	call	getCompiledCodeInfo
          add 	%fp, -4, %o1
    	mov 	%o0, %l3	 ! move the flags, for safekeeping
    	andcc	%o0, (1 << 0), %g0  ! is first argument on the stack?
    	bne 	0f
       	andcc	%o0, (1 << 1), %g0  ! is second argument on the stack?
    	st  	%i0, [%fp + 68]	
0:      bne 	1f
	andcc	%o0, (1 << 2), %g0 ! is third argument on the stack?
    	st 	%i1, [%fp + 72]
1:      bne 	2f
	andcc	%o0, (1 << 3), %g0 ! is fourth argument on the stack?
    	st 	%i2, [%fp + 76]
2:      bne 	3f
	andcc	%o0, (1 << 4), %g0  ! is fifth argument on the stack?
        st 	%i3, [%fp + 80]
3:      bne 	4f
    	andcc	%o0, (1 << 5), %g0  ! is sixth argument on the stack?
    	st 	%i4, [%fp + 84]
4:      bne 	5f
    	add 	%fp, -8, %l6	    ! &hibits; 
        st 	%i5, [%fp + 88]

5:      call	EE    	        ! get the current value of EE into %o0; 
    	 mov	-1, %l7		! to become value of shortfloats = true
    	mov 	%o0, %l4	! save the EE.  It's also first argument
    	mov 	%i0, %o1	! 2nd argument, "object"
    	mov	%g0, %o2	! 3rd argument, method_name (not needed)
        mov	%g0, %o3	! 4th argument, method_signature(not needed)
	mov 	%l0, %o4	! 5th argument, method block
    	mov 	%l1, %o5	! 6th argument, isStatic 
    	st 	%l2, [%sp + 92]	! 7th argument: args
    	call 	do_execute_java_method_vararg
    	std 	%l6, [%sp + 96]	! assign 8th and 9th argument
    	mov 	%o0, %l0	! save return values into locals

    	!! if there is an error reported in the ee, then unwind the stack
    	call	eeExceptionKind
          mov	%l4, %o0
       	tst	%o0		! error if value returned isn't zero
    	beq	9f 	
    	nop
    
    	call	ErrorUnwind
    	mov 	0, %o7		! make sure we don't return!

    	!! bits 8--10 of %l3 contain:	
    	!!  0 for returning a int
    	!!  1 for returning a long
    	!!  2 for returning a float
    	!!  3 for returning a double
    	!!  4 for returning a void
9:    	andcc	%l3, (0x3 << 8), %l3  !Look at bits 8 and 9.
       	beq 	.ci_int_void	! returning an int or a void

    	cmp 	%l3, (2 << 8)
    	bge,a 	.ci_floatdouble
         st 	%l0, [%fp - 4]    	    	
.ci_long:  
    	ld  	[%fp - 8], %i0	
    	ret
    	restore	%l0, %g0, %o1
.ci_int_void:			! result, if any, already in i0
    	ret
    	restore	%l0, %g0, %o0
.ci_floatdouble:    
        !! st 	%l0, [%fp - 4] in delay slot
    	bgt,a	9f		
         ldd  	[%fp - 8], %f0  
    	ld  	[%fp - 4], %f0
9:  	ret
    	restore	
    	.type   CallbackInterpretedMethod, #function
    	.size	CallbackInterpretedMethod, (. - CallbackInterpretedMethod)


CallbackNativeMethod:   
    	save	%sp, -128, %sp
    	!! call getCompiledCodeInfo(mb, char **codePtr) to get flags and code
       	mov	%g1, %l0	! move the methodblock to safe place
    	mov	%g1, %o0
    	call	getCompiledCodeInfo
          add 	%fp, -4, %o1

    	andcc	%o0, (1 << 0), %g0  ! is first argument on the stack?
    	bne 	0f
       	andcc	%o0, (1 << 1), %g0  ! is second argument on the stack?
    	st  	%i0, [%fp + 68]	
0:      bne 	1f
	andcc	%o0, (1 << 2), %g0 ! is third argument on the stack?
    	st 	%i1, [%fp + 72]
1:      bne 	2f
	andcc	%o0, (1 << 3), %g0 ! is fourth argument on the stack?
    	st 	%i2, [%fp + 76]
2:      bne 	3f
	andcc	%o0, (1 << 4), %g0  ! is fifth argument on the stack?
        st 	%i3, [%fp + 80]
3:      bne 	4f
    	andcc	%o0, (1 << 5), %g0  ! is sixth argument on the stack?
    	st 	%i4, [%fp + 84]
4:      bne 	5f
       	mov 	%o0, %l3	 ! move the flags, for safekeeping
        st 	%i5, [%fp + 88]
    
5:  	call	EE    	        ! get the current value of EE into %o0
    	 ld  	[%fp-4], %l2	! the code
    	mov 	%o0, %o1    	! move EE to argument 1
    	mov 	%o0, %l0    	! also, save the EE in %l0
    	add     %fp, 68, %o0    ! pointer to the arguments
6:     	jmpl 	%l2, %o7	! execute the native code
    	add 	%o7, (.table - 6b), %l4  !%l4 now contains .table address
    
    	!! if there is an error reported in the ee, then unwind the stack
    	mov 	%o0, %l1	! pointer to returned eetop
    	call	eeExceptionKind
          mov	%l0, %o0
    	tst	%o0
    	beq	9f 	
          srl 	%l3, (8 - 2), %l3

    	call	ErrorUnwind	
    	mov 	0, %o7		! make sure we don't return!

    	!! bits 8--10 of %l3 contain
    	!!  0 for returning a int
    	!!  1 for returning a long
    	!!  2 for returning a float
    	!!  3 for returning a double
    	!!  4 for returning a void

9: 	and 	%l3, 0x7 << 2, %l3	! look at low three bits
    	ld  	[%l4 + %l3], %l3
    	jmp 	%l4 + %l3
    	nop
.table:	.word   .val0 - .table
    	.word	.val1 - .table
    	.word	.val2 - .table
    	.word   .val3 - .table
    	.word   .val4 - .table
    	.word   .val5 - .table
    	.word   .val6 - .table
    	.word   .val7 - .table

.val0: 	ld  	[%l1-4], %i0	! return an int
.val4:  ret			! return void
       	restore	
.val1:  ld  	[%l1 - 8], %i0	! return a long
    	ld  	[%l1 - 4], %i1        
    	ret
    	restore    	
.val2:	ld  	[%l1 - 4], %f0	! return a float
    	ret
    	restore    	
.val3: 	ld  	[%l1 - 8], %f0	! return a double
    	ld  	[%l1 - 4], %f1
    	ret
    	restore
.val5:  unimp 0
.val6:  unimp 0
.val7:  unimp 0

    	.type   CallbackNativeMethod, #function
    	.size	CallbackNativeMethod, (. - CallbackNativeMethod)

    	.global ErrorUnwind
    	.global CompiledCodeError_md

CompiledCodeError_md:  
    	ba  	ErrorUnwind
  	save	%sp, -96, %sp

    	!! this function must be called by a function just past the 
    	!! compiled function.
ErrorUnwind: 
    	!! get the next frames methodblock and "object"
1:     	ldd 	[%fp + 64], %o0
    	tst 	%o0		! are we out of compiled code?
    	beq     2f		! yep, so exit 
    	nop
    	!! see if the next frame can handle the exception.  If so, this
    	!! returns the appropriate PC.  Otherwise, it returns 0, and
    	!! undoes any monitors set in that catch frame.

    	call	CompiledCodeCleanup
    	nop
    	
    	tst 	%o0		
    	beq,a	1b		! nope, nothing to look at. . . .
    	  restore
    
    	!! jump to the address indicated in the next frame
    	jmp 	%o0	
    	restore

    	!! the next frame has [%sp+64]==0.  It must be the frame that calls
    	!! into the compiled code
2:     	ret
    	restore    	

    	.type	ErrorUnwind, #function
    	.size	ErrorUnwind, (. - ErrorUnwind)





