/*
 * @(#)chunk_md.c	1.3 95/05/08
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <sys/types.h>
#include <chunk_md.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>

/*
 * Set up the mapped memory area for Solaris
 */

/* Start mapping memory space at 1G */
#define MEMSPACEBASE 0x10000000

/* The maximum memory to map: As we're mmapping the memory, it looks
 * like we should ask swapctl(2) how much swap we have, and thus what
 * the maximum we've got is.  Of course, that's way more than we would
 * want to use!  Currently, as we're only allocating stacks, we limit
 * the mapped memory pretty heavily.
 *
 * We really should be ready to map new raw chunks and use them.
 */
#define MEMSPACESIZE 100 * 1024 * 1024

static int devzero_fd;

void
InitMapping()
{
    static int init = 0;
    extern unsigned int mem_grain_size;
    extern unsigned int mems_length;
    extern char *mems_start;

    if (init) {
	return;
    }

    /*
     * Set system-specific variables used by mem allocator
     */
    mems_start = (char *) MEMSPACEBASE;
    mems_length = MEMSPACESIZE;
    if (mem_grain_size == 0) {
	mem_grain_size = sysconf(_SC_PAGESIZE);
    }

    devzero_fd = open("/dev/zero", O_RDWR);
    if (devzero_fd == -1) {
	perror("devzero");
	exit(0);
    }

    init = 1;
}

/*
 * Actually provide backing memory (use mmap(2) on Solaris).
 */
#define PROT_ALL (PROT_READ|PROT_WRITE|PROT_EXEC)

int
mapChunk(void *addr, long length)
{
    return (addr == mmap(addr, length, PROT_ALL,
			 MAP_NORESERVE|MAP_PRIVATE|MAP_FIXED,
			 devzero_fd, (off_t) addr));
}

int
unmapChunk(void *addr, long length)
{
    return (munmap(addr, length) == 0);
}
