/*
 * @(#)file_acl.c	1.28 95/05/12 Sami Shaio
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Simple file access permissions. Prevents read or write access based
 * on a read and a write pathlist. This pathlist is set outside of the
 * runtime by the user (or our hotjava script) and cannot be changed
 * or cirumvented by Java code.
 */
#include "file_acl.h"

#ifdef DO_FILE_ACL

#include <stdlib.h>
#include <sys/param.h>

static Hjava_io_AccessErrorHandler *errorHandler = 0;
static char *rACL=0;
static char **rList = (char **)0;

static char *wACL=0;
static char **wList = (char **)0;

static char **eList = (char **)0;

#define PATH_SEPARATOR ':'

/*
 * Return 0 if none of the classes currently on the stack has a class
 * loader context. Otherwise, return 1.
 *
 * This function is useful to restrict capabilities to classes that
 * were loaded by class loaders.
 */
long
classLoadersInStack()
{
    struct javaframe *frame;
    HArrayOfObject *ctx;
    int n = 0;
    HClass	*cl;

    for (frame = EE()->current_frame ; frame != 0 ; frame = frame->prev) {
	if (frame->current_method != 0) {
	    cl = (HClass *)(fieldclass(&frame->current_method->fb)->HandleToSelf);
	    if (cl != 0 && unhand(cl) && unhand(cl)->loader != 0) {
		return 1;
	    }
	} 
    }

    return 0;
}

/*
 * Return 1 if the class currently on the stack has a class
 * loader context. Otherwise, return 0.
 *
 * This function is useful to restrict capabilities to classes that
 * were loaded by class loaders.
 */
long
currentClassHasClassLoader()
{
    struct javaframe	*frame;
    HClass		*cl;

    frame = EE()->current_frame;

    /* we actually check starting up a frame because the current class */
    /* is always java.util.Linker */
    for (frame=(frame) ? frame->prev : 0 ; frame != 0 ; frame = frame->prev) {
	if (frame->current_method != 0) {
	    cl = (HClass *)(fieldclass(&frame->current_method->fb)->HandleToSelf);
	    if (cl != 0 && unhand(cl)) {
		if (unhand(cl)->loader != 0) {
		    return 1;
		} else {
		    return 0;
		}
	    }
	} 
    }

    /* we're here if scanning up the stack hasn't yielded a class */
    /* pointer that we can check. We return 1 to be paranoid in this */
    /* case since we really should have found some legal class to */
    /* check by now. */
    return 1;
}

static void
resolveEntry(char *current, char ***list, int index)
{	
    char	rpath[MAXPATHLEN];
    char	resolved[MAXPATHLEN];
    char	*var;
    char 	*next;

    switch (*current) {
      case '~':
	var = getenv("HOME");
	if (var != NULL) {
	    current++;
	    if (current && *current=='/') { 
		sprintf(rpath,"%s%s", var, (current) ? current : "");
	    } else {
		sprintf(rpath,"%s/%s", var, (current) ? current : "");
	    }
	    if (realpath(rpath,resolved) != NULL) {
		(*list)[index] = strdup(resolved);
	    } else {
		(*list)[index] = strdup(rpath);
	    }
	}
	break;
      case '$':
	current++;
	var = getenv(current);
	if ((next = strchr(current, '/')) != NULL) {
	    current = next+1;
	} else {
	    current = NULL;
	}
	if (var != NULL) {
	    if (current && *current=='/') { 
		sprintf(rpath,"%s%s", var, (current) ? current : "");
	    } else {
		sprintf(rpath,"%s/%s", var, (current) ? current : "");
	    }
	    if (realpath(rpath,resolved) != NULL) {
		(*list)[index] = strdup(resolved);
	    } else {
		(*list)[index] = strdup(rpath);
	    }
	}
	break;
      default:
	if (realpath(current,resolved) != NULL) {
	    (*list)[index] = strdup(resolved);
	} else {
	    (*list)[index] = strdup(current);
	}
    }
}

static void
initACL(char *path, char *defaultPath, char ***list)
{
    char	*next;
    char	*current;
    int		length;
    int		i;
    
    if (*list != 0 && classLoadersInStack()) {
	/* refuse to change the ACL if it is already set */
	SignalError(0, JAVAPKG "IOException","permission error");
	return;
    }

    if (path == 0) {
	path = defaultPath;
    }

    if (*path == 0) {
	/* disallow all accesses to this list */
	*list = (char **)malloc(sizeof(char *));
	(*list)[0] = 0;
	return;
    }

    /* Disable any checking on this ACL if the first element is */
    /* wildcard */
    if (*path == '*') {
	return;
    }

    /* count how many elements in the path */
    length = 0;
    current = path;
    while (current && (next = strchr(current,PATH_SEPARATOR))) {
	length++;
	current = (next + 1);
    }
    
    *list = (char **)malloc((length + 2) * sizeof(char *));
    for (current = path, next=strchr(current,PATH_SEPARATOR), i=0;
	 next != NULL;
	 current=next, next=strchr(current,PATH_SEPARATOR), i++) {
	*next = 0;
	resolveEntry(current, list, i);
	next++;
    }
    resolveEntry(current, list, i++);
    (*list)[i] = 0;
}


void
initReadACL(Hjava_lang_String *path)
{
    char *f;
    char *s = 0;
    char *s2 = 0;
    char *defaultPath = DEFAULT_READ_PATH;
    char *cpath = 0;

    if (f=getenv("HOTJAVA_HOME")) {
	s = (char *)malloc(strlen(f) + strlen(defaultPath) + 2);
	sprintf(s,"%s%c%s", f, PATH_SEPARATOR, defaultPath);
	defaultPath = s;
    }
    if (path != 0) {
	cpath = makeCString(path);
	if (*cpath == '+') {
	    cpath++;
	    if (*cpath == PATH_SEPARATOR) {
		cpath++;
		s2 = (char *)malloc(strlen(cpath) + strlen(defaultPath) + 2);
		if (f) {
		    sprintf(s2,"%s%c%s",
			    f,
			    PATH_SEPARATOR,
			    cpath );
		} else {
		    sprintf(s2,"%s%c%s", DEFAULT_READ_PATH,PATH_SEPARATOR,cpath);
		}
		cpath = s2;
	    } else {
		cpath = defaultPath;
	    }
	}
	rACL = strdup(cpath);
    } else {
	rACL = strdup(defaultPath);
    }
    initACL(cpath, defaultPath, &rList);
    if (s) {
	free(s);
    }
    if (s2) {
	free(s2);
    }
}

char *
getReadACL()
{
    return rACL;
}

char *
getWriteACL()
{
    return wACL;
}

void
initWriteACL(Hjava_lang_String *path)
{
    char *cpath;
    char *s = 0;
    char *defaultPath = DEFAULT_WRITE_PATH;

    if (path != 0) {
	cpath = makeCString(path);
	if (*cpath == '+') {
	    cpath++;
	    s = (char *)malloc(strlen(cpath) + strlen(DEFAULT_WRITE_PATH) + 2);
	    sprintf(s,"%s%c%s",DEFAULT_WRITE_PATH,PATH_SEPARATOR,cpath);
	    defaultPath = s;
	} else {
	    defaultPath = cpath;
	}
	wACL = strdup(defaultPath);
    } else {
	wACL = strdup(DEFAULT_WRITE_PATH);
    }
    initACL(defaultPath, DEFAULT_WRITE_PATH, &wList);
    if (s) {
	free(s);
    }
}

void
initExecACL(Hjava_lang_String *path)
{
    initACL((path != 0) ? makeCString(path) : 0, "", &eList);
}

void
setAccessErrorHandler(Hjava_io_AccessErrorHandler *handler)
{
    /* only allow setting the handler if not already set */
    if (errorHandler == 0) {
	errorHandler = handler;
    } else {
	SignalError(0, JAVAPKG "IOException","permission error");
	return;
    }
}
    

long
checkWithHandler(char *method, Hjava_lang_String *hfile, char ***list)
{
    long eVal;
    int	len;

    if (errorHandler != 0 && hfile != 0) {
	eVal = execute_java_dynamic_method(EE(),
					  (HObject *)errorHandler,
					  method,
					  "(Ljava/lang/String;)I",
					  hfile);
	switch (eVal) {
	  case java_io_AccessErrorHandler_ALLOW_ACCESS:
	    /* add this file to the path */
	    /* count up how many elements in the list */
	    for (len=0; (*list)[len]; len++) {
	    }
	    *list = (char **)realloc(*list, sizeof(char *) * (len + 2));
	    (*list)[len] = strdup(makeCString(hfile));
	    (*list)[len+1] = 0;
	    return 1;
	  case java_io_AccessErrorHandler_ALLOW_THIS_ACCESS:
	    return 1;
	  case java_io_AccessErrorHandler_DENY_ACCESS:
	  default:
	    return 0;
	}
    } else {
	return 0;
    }
}


long
checkForReadAccess(char *name, Hjava_lang_String *hfile)
{
    int i;
    int l;
    char *s;
    char resolved[MAXPATHLEN];

    if (rList == (char **)0 || !classLoadersInStack()) {
	return 1;
    }
    if (realpath(name, resolved) == NULL) {
	sprintf(resolved,"%s",name);
    }

    for (i = 0; rList[i]; i++) {
	s = rList[i];
	l = strlen(s);
	if (strncmp(s, resolved, l) == 0) {
	    return 1;
	}
    }

    return checkWithHandler("readException", hfile, &rList);
}

long
checkWriteAccess(char *name)
{
    int i;
    int l;
    char *s;
    char resolved[MAXPATHLEN];

    if (wList == (char **)0) {
	return 1;
    }
    if (realpath(name, resolved) == NULL) {
	sprintf(resolved,"%s",name);
    }
    for (i = 0; wList[i]; i++) {
	s = wList[i];
	l = strlen(s);

	if (strncmp(s, resolved, l) == 0) {
	    return 1;
	}
    }

    return 0;
}

long
checkForWriteAccess(char *name, Hjava_lang_String *hfile)
{
    if (wList == (char **)0 || !classLoadersInStack()) {
	return 1;
    }

    return (checkWriteAccess(name) ||
	    checkWithHandler("writeException", hfile, &wList)); 
}

#endif				/* DO_FILE_ACL */
