/*
 * @(#)itimer.c	1.11 95/01/31  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * The clock interface used when the Green threads API is on Unix.
 */

#include <threads_md.h>
#include <monitor_md.h>
#include <machapi.h>

#include <time.h>
#include <sys/time.h>

extern int gettimeofday(struct timeval *, struct timezone *);
extern int setitimer(int, struct itimerval *, struct itimerval *);

/*
 * The minimum clock resolution in uSeconds.
 */
#define CLOCK_RESOLUTION    1000

timeval_t
currentTime()
{
    struct timeval tv;
    timeval_t time;

    (void) gettimeofday(&tv, (struct timezone *) 0);
    time.tv_sec = tv.tv_sec;
    time.tv_usec = tv.tv_usec;
    return time;
}

unsigned int
hostClockInit()
{
    asyncNotifier(ASYNC_REGISTER, SYS_ASYNC_MON_ALARM);
    return SYS_ASYNC_MON_ALARM;
}

void
scheduleAlarm(timeval_t when)
{
    struct itimerval itv;

    if (when.tv_sec == 0 && when.tv_usec < CLOCK_RESOLUTION)
	when.tv_usec = CLOCK_RESOLUTION;

    itv.it_value.tv_sec = when.tv_sec;
    itv.it_value.tv_usec = when.tv_usec;
    itv.it_interval.tv_sec = 0;
    itv.it_interval.tv_usec = 0;
    setitimer(ITIMER_REAL, &itv, (struct itimerval *) 0);
    return;
}


void
cancelAlarm()
{
    struct itimerval itv;

    itv.it_value.tv_sec = 0;
    itv.it_value.tv_usec = 0;
    itv.it_interval.tv_sec = 0;
    itv.it_interval.tv_usec = 0;
    setitimer(ITIMER_REAL, &itv, (struct itimerval *) 0);
    return;
}
