/*
 * @(#)linker_md.c	1.7 95/01/31  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Machine Dependent implementation of the dynamic linking support
 * for java.  This routine is Solaris specific.
 */

#include <stdio.h>
#include <dlfcn.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "sys_api.h"
#include "installpath.h"
#include "path_md.h"

static	int	linkerinitialized = 0;
static	int	useddlsegments = 0;
static	int	sizedlsegments = 0;

/* 
 * The shared libraries array 
 */
static struct dlsegment {
    char *fname;
    void *handle;
} *dlsegment = 0;


/*
 * Save the ld path
 */

void
sysSaveLDPath(char * ldp)
{
    putenv(ldp);
    return;
}


/*
 * return the local directory separator
 */

int
sysGetSeparator()
{
    return(':');
}


/*
 * create a string for the dynamic lib open call by adding the
 * appropriate pre and extensions to a filename and the path
 *
 * WARNING - doesn't check sizes, relies on stub to make sure "holder"
 * 	     is large enough to hold path, ext's and filename
 */

void
sysBuildLibName(char *holder, char *pname, char *fname)
{
    sprintf(holder,"%s/lib%s.so", pname, fname);
    return;
}


/*
 * Initialize the linker
 */
char *
sysInitializeLinker()
{
    char *ldpathbuf, *ldpath;

    /*
     * setup linker search paths
     */
    ldpath = getenv("LD_LIBRARY_PATH");
    if (ldpath != 0) {
        ldpathbuf = (char *)malloc(strlen(ldpath) +
                                   strlen(INSTALLPATH) +
                                   strlen(INSTALLPLATFORM) + 32);
        sprintf(ldpathbuf, "LD_LIBRARY_PATH=%s:%s%clib%c%s",
                ldpath, INSTALLPATH, LOCAL_DIR_SEPARATOR, LOCAL_DIR_SEPARATOR,
                INSTALLPLATFORM);
    } else {
        ldpathbuf = (char *)malloc(strlen(INSTALLPATH) +
                                   strlen(INSTALLPLATFORM) + 32);
        sprintf(ldpathbuf, "LD_LIBRARY_PATH=%s%clib%c%s",
                INSTALLPATH, LOCAL_DIR_SEPARATOR, LOCAL_DIR_SEPARATOR, 
		INSTALLPLATFORM);
    }
    if (linkerinitialized) {
	return(ldpathbuf);
    }

    dlsegment = (struct dlsegment *)
	malloc(sizeof(struct dlsegment) * (sizedlsegments = 10));
    dlsegment[useddlsegments].fname = "<main>";
    if ((dlsegment[useddlsegments].handle = dlopen(0, 1)) == 0)
	fprintf(stderr, "Unable to dlopen main!\n");
    else
	useddlsegments++;
    linkerinitialized = 1;
    return(ldpathbuf);
}

/* 
 * Add a shared library to the list
 */
int
sysAddDLSegment(char *fn)
{
    int    i;

#ifdef DEBUG
    strcpy(fn + strlen(fn) - 3, "_g.so");
#endif
    for (i = 0; i < useddlsegments; i++) {
	if (strcmp(dlsegment[i].fname, fn) == 0) {
	    return 1;
	}
    }

    if (access(fn, R_OK) != 0) {
	return 0;
    }

    if (useddlsegments >= sizedlsegments) {
	sizedlsegments += 10;
	dlsegment = (struct dlsegment *)
	    realloc(dlsegment, sizeof(struct dlsegment) * sizedlsegments);
    }

    if (dlsegment == 0) {
	void out_of_memory();
	out_of_memory();
    }

    if ((dlsegment[useddlsegments].handle = dlopen(fn, 1)) == 0) {
	char *err = (char *) dlerror();
	if (err) {
	    fprintf(stderr, "%s (%s)\n", err, fn);
	}
	return 0;
    }

    dlsegment[useddlsegments++].fname = strdup(fn);
    return 1;
}


long
sysDynamicLink(char *sym_p)
{
    void *f = 0;
    int    i;

    /*
     * we need this kludge, because the java interpreter internals need
     * to access this before the Linker class is loaded and calls
     * the static initializers
     */
    if (!linkerinitialized) {
	(void)sysInitializeLinker();
    }
    
    if ((f = dlsym(RTLD_NEXT, sym_p)) == 0) {
	for (i = useddlsegments; --i >= 0;) {
	    if ((f = dlsym(dlsegment[i].handle, sym_p))) {
		return((long)f);
	    }
	}
    }
    return 0;
}

