/*
 * @(#)schedule.c	1.10 95/01/31  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Thread scheduling utilities and data structures.
 */

#include <threads_md.h>
#include <queue.h>
#include <machapi.h>
#include <schedule.h>
#include <log.h>
#include <debug.h>

int _scheduling_lock;

queue_t runnable_queue;

void
reschedule(preemption_t preemption)
{
    sys_thread_t *tid;

    assert(SCHED_LOCKED());
    assert(_scheduling_lock == 1);
    tid = queueRemoveHead(&runnable_queue, tid);
    assert(tid->state == RUNNABLE);
    Log3(5, "reschedule: from TID 0x%x to TID 0x%x preemption %d\n",
    	 sysThreadSelf(), tid, preemption);
    setCurrentThread(tid);
    if (tid->full_switch) {
	fullSwitchContext(CONTEXT(tid), preemption);
    } else if (preemption == ASYNCHRONOUS) {
	returnFromTrap(CONTEXT(tid));
    } else {
	switchContext(CONTEXT(tid));
    }
    /* NOTREACHED */
}
