/*
 * @(#)audio.c	1.6 95/03/14 Eugene Kuerner
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */


#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/errno.h>

#include "oobj.h"
#include "interpreter.h"

#include "browser_audio_AudioPlayer.h"


static 	int	audioFd = -1;

long
browser_audio_AudioPlayer_audioOpen(struct Hbrowser_audio_AudioPlayer *aP)
{
    extern int errno;

    if (audioFd < 0) {
	errno = 0;
  	if ((audioFd = open("/dev/audio", O_WRONLY | O_NONBLOCK)) < 0) {
	    switch (errno) {
	      case ENODEV:
	      case EACCES:
		/* Dont bother, there is no audio device */
		return -1;
	    }
	    /* Try again later, probably busy */
	    return 0;
	}
    }
    return 1;
}


void
browser_audio_AudioPlayer_audioClose(struct Hbrowser_audio_AudioPlayer *aP)
{
    close(audioFd);
    audioFd = -1;
    return;
}


void
browser_audio_AudioPlayer_audioWrite(struct Hbrowser_audio_AudioPlayer *aP,
				       HArrayOfByte *buf,
				       long len)
{
    char *data;
    long dataLen;

    if (buf == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if (len <= 0) {
	return;
    }

    data = unhand(buf)->body;
    dataLen = obj_length(buf);

    if (dataLen < len) {
	len = dataLen;
    }
    write(audioFd, data, len);
    return;
}
