/*
 * @(#)firewall.c	1.11 95/05/15 Sami Shaio
 *
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include <errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/systeminfo.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdio.h>
#include <oobj.h>
#include <interpreter.h>
#include <javaString.h>
#include <iomgr.h>
#include <net_InetAddress.h>
#include <net_Socket.h>
#include <net_SocketOutputStream.h>
#include <net_SocketInputStream.h>
#include <fcntl.h>
#include <sys/fcntl.h>
#include <sys/filio.h>
#include <sys/ioctl.h>
#include "javaString.h"
#include "net_www_html_URL.h"
#include "net_www_html_WWWClassLoader.h"
#include "net_Firewall.h"
#include "java_lang_ClassLoader.h"

/*
 * ASSUMPTIONS:
 * 1. the names of the trusted Java classes haven't been changed.
 * 2. the names of the trusted Java classes are trustworthy (i.e. you
 *    can't create an arbitrary class and have us think its name
 *    is one of the trusted names).
 * 3. the bytecode verifier is enabled and is checking accesses
 *    correctly.
 * 4. The firewall list is configured properly.
 * 5. None of the Java classes from which information is retrieved to
 *    process security decisions have been compromised. 
 */

extern long classLoadersInStack();


Hjava_lang_String *
net_Firewall_getDomainName(Hnet_Firewall *this)
{
    FILE *fstr;
    static char h[MAXHOSTNAMELEN];
    static int hSet = 0;

    if (!hSet) {
	if ((fstr = fopen("/etc/defaultdomain", "r")) == 0) {
	    return (Hjava_lang_String *)0;
	}
	fscanf(fstr, "%s", h);
	fclose(fstr);
	hSet = 1;
    }

    return makeJavaString(h, strlen(h));
}



Hjava_lang_String *
net_Firewall_getInetAddressHost(Hnet_Firewall *this, Hnet_InetAddress *addr)
{
    Classnet_InetAddress	*addrP;

    if (addr == NULL) {
	return NULL;
    }
    addrP = unhand(addr);

    return addrP->pHost;
}


Hnet_www_html_URL *
net_Firewall_getSourceURL(Hnet_Firewall *this,
			  Hnet_www_html_WWWClassLoader *loader)
{
    Classnet_www_html_WWWClassLoader *loaderPtr;

    if (loader == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    loaderPtr = unhand(loader);

    return loaderPtr->source;
}


HObject *
net_Firewall_getClassLoader(Hnet_Firewall *this)
{
    struct javaframe	*frame;
    HClass		*cl;
    ClassClass		*cb;

    /*
     * Find the class loader, if any, on the stack.
     */
    for (frame = EE()->current_frame ; frame != 0 ; frame = frame->prev) {
	if (frame->current_method != 0) {
	    cl = (HClass *)(fieldclass(&frame->current_method->fb)->HandleToSelf);
	    if (cl != 0 && (cb=unhand(cl)) && cb->loader != 0) {
		return cb->loader;
	    }
	} 
    }

    return (HObject *)0;
}

long
net_Firewall_compareAddresses(Hnet_Firewall *this,
			      Hnet_InetAddress *hA,
			      Hnet_InetAddress *hB)
{
    Classnet_InetAddress *a = unhand(hA);
    Classnet_InetAddress *b = unhand(hB);

    return a->pAddress == b->pAddress;
}

long
net_Firewall_isWriteableByApplets(Hnet_Firewall *this,
				  Hjava_lang_String *file)
{
    char	*cpath;
    extern	 checkWriteAccess(char *f); /* defined in file_acl.c */

    if (file == 0) {
	return 1;
    }
    cpath = makeCString(file);
    
    return checkWriteAccess(cpath);
}

/*
 * Returns 1 if there is a trusted class on the stack. It is presumed
 * that all such trusted classes are declared "final" and thus can't be
 * subclassed.
 */
long
net_Firewall_isTrustedClass(Hnet_Firewall *this)
{
    struct javaframe *frame;
    HClass	*cl;
    ClassClass	*cb;
    char	clname[256];

    if (! classLoadersInStack()) {
	return 1;
    }

    for (frame = EE()->current_frame ; frame != 0 ; frame = frame->prev) {
	if (frame->current_method != 0) {
	    cl = (HClass *)(fieldclass(&frame->current_method->fb)->HandleToSelf);
	    if (cl != 0 && (cb = unhand(cl))) {
		classname2string(classname(cb), clname, sizeof(clname));
		if (!strcmp(clname,"net.www.http.HttpClient") ||
		    !strcmp(clname,"net.ftp.FtpClient")) {
		    return 1;
		}
	    }
	} 
    }

    return 0;
}

